@@ -1,4 +1,4 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,7 @@
 
 // Following tests are ported and extended tests from libcpp for std::set.
 // They can be found here:
-// https://github.com/llvm-mirror/libcxx/tree/master/test/std/containers/associative/set
+// https://github.com/llvm/llvm-project/tree/main/libcxx/test/std/containers/associative/set
 //
 // Not ported tests:
 // * No tests with PrivateConstructor and std::less<> changed to std::less<T>
@@ -28,6 +28,7 @@
 // * No tests with min_allocator and no tests counting allocations.
 //   Flat sets currently don't support allocators.
 
+#include <deque>
 #include <forward_list>
 #include <functional>
 #include <iterator>
@@ -35,7 +36,6 @@
 #include <string>
 #include <vector>
 
-#include "base/functional/identity.h"
 #include "base/ranges/algorithm.h"
 #include "base/template_util.h"
 #include "base/test/gtest_util.h"
@@ -151,28 +151,28 @@ struct LessByFirst {
 // Common test trees.
 template <typename ContainerT>
 using TypedTree = flat_tree<typename ContainerT::value_type,
-                            base::identity,
+                            std::identity,
                             std::less<>,
                             ContainerT>;
 using IntTree = TypedTree<std::vector<int>>;
 using IntPair = std::pair<int, int>;
 using IntPairTree = flat_tree<IntPair,
-                              base::identity,
+                              std::identity,
                               LessByFirst<IntPair>,
                               std::vector<IntPair>>;
 using MoveOnlyTree = flat_tree<MoveOnlyInt,
-                               base::identity,
+                               std::identity,
                                std::less<>,
                                std::vector<MoveOnlyInt>>;
 using EmplaceableTree = flat_tree<Emplaceable,
-                                  base::identity,
+                                  std::identity,
                                   std::less<>,
                                   std::vector<Emplaceable>>;
 using ReversedTree =
-    flat_tree<int, base::identity, std::greater<int>, std::vector<int>>;
+    flat_tree<int, std::identity, std::greater<int>, std::vector<int>>;
 
 using TreeWithStrangeCompare = flat_tree<int,
-                                         base::identity,
+                                         std::identity,
                                          NonDefaultConstructibleCompare,
                                          std::vector<int>>;
 
@@ -185,20 +185,6 @@ template <typename T>
 class FlatTreeTest : public testing::Test {};
 TYPED_TEST_SUITE_P(FlatTreeTest);
 
-TEST(FlatTree, IsMultipass) {
-  static_assert(!is_multipass<std::istream_iterator<int>>(),
-                "InputIterator is not multipass");
-  static_assert(!is_multipass<std::ostream_iterator<int>>(),
-                "OutputIterator is not multipass");
-
-  static_assert(is_multipass<std::forward_list<int>::iterator>(),
-                "ForwardIterator is multipass");
-  static_assert(is_multipass<std::list<int>::iterator>(),
-                "BidirectionalIterator is multipass");
-  static_assert(is_multipass<std::vector<int>::iterator>(),
-                "RandomAccessIterator is multipass");
-}
-
 // Tests that the compiler generated move operators propagrate noexcept
 // specifiers.
 TEST(FlatTree, NoExcept) {
@@ -207,17 +193,17 @@ TEST(FlatTree, NoExcept) {
     MoveThrows& operator=(MoveThrows&&) noexcept(false) { return *this; }
   };
 
-  using MoveThrowsTree = flat_tree<MoveThrows, base::identity, std::less<>,
+  using MoveThrowsTree = flat_tree<MoveThrows, std::identity, std::less<>,
                                    std::array<MoveThrows, 1>>;
 
-  static_assert(std::is_nothrow_move_constructible<IntTree>::value,
+  static_assert(std::is_nothrow_move_constructible_v<IntTree>,
                 "Error: IntTree is not nothrow move constructible");
-  static_assert(std::is_nothrow_move_assignable<IntTree>::value,
+  static_assert(std::is_nothrow_move_assignable_v<IntTree>,
                 "Error: IntTree is not nothrow move assignable");
 
-  static_assert(!std::is_nothrow_move_constructible<MoveThrowsTree>::value,
+  static_assert(!std::is_nothrow_move_constructible_v<MoveThrowsTree>,
                 "Error: MoveThrowsTree is nothrow move constructible");
-  static_assert(!std::is_nothrow_move_assignable<MoveThrowsTree>::value,
+  static_assert(!std::is_nothrow_move_assignable_v<MoveThrowsTree>,
                 "Error: MoveThrowsTree is nothrow move assignable");
 }
 
@@ -229,7 +215,7 @@ TEST(FlatTree, NoExcept) {
 
 TEST(FlatTree, IncompleteType) {
   struct A {
-    using Tree = flat_tree<A, base::identity, std::less<A>, std::vector<A>>;
+    using Tree = flat_tree<A, std::identity, std::less<A>, std::vector<A>>;
     int data;
     Tree set_with_incomplete_type;
     Tree::iterator it;
@@ -245,7 +231,7 @@ TEST(FlatTree, Stability) {
   using Pair = std::pair<int, int>;
 
   using Tree =
-      flat_tree<Pair, base::identity, LessByFirst<Pair>, std::vector<Pair>>;
+      flat_tree<Pair, std::identity, LessByFirst<Pair>, std::vector<Pair>>;
 
   // Constructors are stable.
   Tree cont({{0, 0}, {1, 0}, {0, 1}, {2, 0}, {0, 2}, {1, 1}});
@@ -290,14 +276,13 @@ TEST(FlatTree, Stability) {
 
 TEST(FlatTree, Types) {
   // These are guaranteed to be portable.
-  static_assert((std::is_same<int, IntTree::key_type>::value), "");
-  static_assert((std::is_same<int, IntTree::value_type>::value), "");
-  static_assert((std::is_same<std::less<>, IntTree::key_compare>::value), "");
-  static_assert((std::is_same<int&, IntTree::reference>::value), "");
-  static_assert((std::is_same<const int&, IntTree::const_reference>::value),
-                "");
-  static_assert((std::is_same<int*, IntTree::pointer>::value), "");
-  static_assert((std::is_same<const int*, IntTree::const_pointer>::value), "");
+  static_assert((std::is_same_v<int, IntTree::key_type>), "");
+  static_assert((std::is_same_v<int, IntTree::value_type>), "");
+  static_assert((std::is_same_v<std::less<>, IntTree::key_compare>), "");
+  static_assert((std::is_same_v<int&, IntTree::reference>), "");
+  static_assert((std::is_same_v<const int&, IntTree::const_reference>), "");
+  static_assert((std::is_same_v<int*, IntTree::pointer>), "");
+  static_assert((std::is_same_v<const int*, IntTree::const_pointer>), "");
 }
 
 // ----------------------------------------------------------------------------
@@ -394,7 +379,7 @@ TEST(FlatTree, ContainerMoveConstructor) {
   storage.push_back(Pair(2, MoveOnlyInt(1)));
 
   using Tree =
-      flat_tree<Pair, base::identity, LessByFirst<Pair>, std::vector<Pair>>;
+      flat_tree<Pair, std::identity, LessByFirst<Pair>, std::vector<Pair>>;
   Tree tree(std::move(storage));
 
   // The list should be two items long, with only the first "2" saved.
@@ -477,7 +462,7 @@ TEST(FlatTree, SortedUniqueVectorMoveConstructor) {
   storage.push_back(Pair(2, MoveOnlyInt(0)));
 
   using Tree =
-      flat_tree<Pair, base::identity, LessByFirst<Pair>, std::vector<Pair>>;
+      flat_tree<Pair, std::identity, LessByFirst<Pair>, std::vector<Pair>>;
   Tree tree(sorted_unique, std::move(storage));
 
   ASSERT_EQ(2u, tree.size());
@@ -1004,7 +989,7 @@ TYPED_TEST_P(FlatTreeTest, ErasePosition) {
   {
     using T = TemplateConstructor;
 
-    flat_tree<T, base::identity, std::less<>, std::vector<T>> cont;
+    flat_tree<T, std::identity, std::less<>, std::vector<T>> cont;
     T v(0);
 
     auto it = cont.find(v);
@@ -1095,8 +1080,7 @@ TEST(FlatTree, KeyComp) {
 
   EXPECT_TRUE(ranges::is_sorted(cont, cont.key_comp()));
   int new_elements[] = {6, 7, 8, 9, 10};
-  std::copy(std::begin(new_elements), std::end(new_elements),
-            std::inserter(cont, cont.end()));
+  ranges::copy(new_elements, std::inserter(cont, cont.end()));
   EXPECT_TRUE(ranges::is_sorted(cont, cont.key_comp()));
 }
 
@@ -1107,8 +1091,7 @@ TEST(FlatTree, ValueComp) {
 
   EXPECT_TRUE(ranges::is_sorted(cont, cont.value_comp()));
   int new_elements[] = {6, 7, 8, 9, 10};
-  std::copy(std::begin(new_elements), std::end(new_elements),
-            std::inserter(cont, cont.end()));
+  ranges::copy(new_elements, std::inserter(cont, cont.end()));
   EXPECT_TRUE(ranges::is_sorted(cont, cont.value_comp()));
 }
 
