@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,29 +6,33 @@
 
 #import <UIKit/UIKit.h>
 
-namespace base {
-namespace internal {
+namespace base::internal {
 
 ImmediateCriticalClosure::ImmediateCriticalClosure(StringPiece task_name,
                                                    OnceClosure closure)
-    : critical_action_(task_name), closure_(std::move(closure)) {}
+    : critical_action_(task_name), closure_(std::move(closure)) {
+  CHECK(!closure_.is_null());
+}
 
 ImmediateCriticalClosure::~ImmediateCriticalClosure() {}
 
 void ImmediateCriticalClosure::Run() {
+  CHECK(!closure_.is_null());
   std::move(closure_).Run();
 }
 
 PendingCriticalClosure::PendingCriticalClosure(StringPiece task_name,
                                                OnceClosure closure)
-    : task_name_(task_name), closure_(std::move(closure)) {}
+    : task_name_(task_name), closure_(std::move(closure)) {
+  CHECK(!closure_.is_null());
+}
 
 PendingCriticalClosure::~PendingCriticalClosure() {}
 
 void PendingCriticalClosure::Run() {
+  CHECK(!closure_.is_null());
   critical_action_.emplace(task_name_);
   std::move(closure_).Run();
 }
 
-}  // namespace internal
-}  // namespace base
+}  // namespace base::internal
