@@ -1,4 +1,4 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,6 +6,9 @@
 
 #include <stddef.h>
 
+#include <ostream>
+#include <set>
+#include <string>
 #include <utility>
 #include <vector>
 
@@ -13,6 +16,7 @@
 #include "base/format_macros.h"
 #include "base/memory/read_only_shared_memory_region.h"
 #include "base/metrics/field_trial.h"
+#include "base/metrics/field_trial_param_associator.h"
 #include "base/metrics/persistent_memory_allocator.h"
 #include "base/ranges/algorithm.h"
 #include "base/strings/strcat.h"
@@ -20,22 +24,24 @@
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/test/scoped_feature_list.h"
-#include "base/test/scoped_field_trial_list_resetter.h"
+#include "build/chromeos_buildflags.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+#include "base/feature_visitor.h"
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+
 namespace base {
 
 namespace {
 
 constexpr char kFeatureOnByDefaultName[] = "OnByDefault";
-struct Feature kFeatureOnByDefault {
-  kFeatureOnByDefaultName, FEATURE_ENABLED_BY_DEFAULT
-};
+constinit Feature kFeatureOnByDefault(kFeatureOnByDefaultName,
+                                      FEATURE_ENABLED_BY_DEFAULT);
 
 constexpr char kFeatureOffByDefaultName[] = "OffByDefault";
-struct Feature kFeatureOffByDefault {
-  kFeatureOffByDefaultName, FEATURE_DISABLED_BY_DEFAULT
-};
+constinit Feature kFeatureOffByDefault(kFeatureOffByDefaultName,
+                                       FEATURE_DISABLED_BY_DEFAULT);
 
 std::string SortFeatureListString(const std::string& feature_list) {
   std::vector<base::StringPiece> features =
@@ -65,7 +71,7 @@ TEST_F(FeatureListTest, DefaultStates) {
   EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOffByDefault));
 }
 
-TEST_F(FeatureListTest, InitializeFromCommandLine) {
+TEST_F(FeatureListTest, InitFromCommandLine) {
   struct {
     const char* enable_features;
     const char* disable_features;
@@ -88,8 +94,8 @@ TEST_F(FeatureListTest, InitializeFromCommandLine) {
                                     test_case.disable_features));
 
     auto feature_list = std::make_unique<FeatureList>();
-    feature_list->InitializeFromCommandLine(test_case.enable_features,
-                                            test_case.disable_features);
+    feature_list->InitFromCommandLine(test_case.enable_features,
+                                      test_case.disable_features);
     test::ScopedFeatureList scoped_feature_list;
     scoped_feature_list.InitWithFeatureList(std::move(feature_list));
 
@@ -99,10 +105,20 @@ TEST_F(FeatureListTest, InitializeFromCommandLine) {
     EXPECT_EQ(test_case.expected_feature_off_state,
               FeatureList::IsEnabled(kFeatureOffByDefault))
         << i;
+
+    // Reading the state of each feature again will pull it from their
+    // respective caches instead of performing the full lookup, which should
+    // yield the same result.
+    EXPECT_EQ(test_case.expected_feature_on_state,
+              FeatureList::IsEnabled(kFeatureOnByDefault))
+        << i;
+    EXPECT_EQ(test_case.expected_feature_off_state,
+              FeatureList::IsEnabled(kFeatureOffByDefault))
+        << i;
   }
 }
 
-TEST_F(FeatureListTest, InitializeFromCommandLineWithFeatureParams) {
+TEST_F(FeatureListTest, InitFromCommandLineWithFeatureParams) {
   struct {
     const std::string enable_features;
     const std::string expected_field_trial_created;
@@ -115,21 +131,26 @@ TEST_F(FeatureListTest, InitializeFromCommandLineWithFeatureParams) {
        {{"x", "test"}, {"y", "uma"}, {"z", "ukm"}}},
   };
 
-  const Feature kFeature = {"Feature", FEATURE_DISABLED_BY_DEFAULT};
+  // Clear global state so that repeated runs of this test don't flake.
+  // When https://crrev.com/c/3694674 is submitted, we should be able to remove
+  // this.
+  base::FieldTrialParamAssociator::GetInstance()->ClearAllParamsForTesting();
+
+  static BASE_FEATURE(kFeature, "Feature", FEATURE_DISABLED_BY_DEFAULT);
   for (const auto& test_case : test_cases) {
     SCOPED_TRACE(test_case.enable_features);
 
     auto feature_list = std::make_unique<FeatureList>();
-    feature_list->InitializeFromCommandLine(test_case.enable_features, "");
+    feature_list->InitFromCommandLine(test_case.enable_features, "");
     test::ScopedFeatureList scoped_feature_list;
     scoped_feature_list.InitWithFeatureList(std::move(feature_list));
 
     EXPECT_TRUE(FeatureList::IsEnabled(kFeature));
     EXPECT_TRUE(
         FieldTrialList::IsTrialActive(test_case.expected_field_trial_created));
-    std::map<std::string, std::string> actualParams;
-    EXPECT_TRUE(GetFieldTrialParamsByFeature(kFeature, &actualParams));
-    EXPECT_EQ(test_case.expected_feature_params, actualParams);
+    std::map<std::string, std::string> actual_params;
+    EXPECT_TRUE(GetFieldTrialParamsByFeature(kFeature, &actual_params));
+    EXPECT_EQ(test_case.expected_feature_params, actual_params);
   }
 }
 
@@ -176,8 +197,9 @@ TEST_F(FeatureListTest, FieldTrialOverrides) {
     const auto& test_case = test_cases[i];
     SCOPED_TRACE(base::StringPrintf("Test[%" PRIuS "]", i));
 
-    test::ScopedFieldTrialListResetter resetter;
-    FieldTrialList field_trial_list(nullptr);
+    test::ScopedFeatureList outer_scope;
+    outer_scope.InitWithEmptyFeatureAndFieldTrialLists();
+
     auto feature_list = std::make_unique<FeatureList>();
 
     FieldTrial* trial1 = FieldTrialList::CreateFieldTrial("TrialExample1", "A");
@@ -242,7 +264,7 @@ TEST_F(FeatureListTest, CommandLineEnableTakesPrecedenceOverFieldTrial) {
   auto feature_list = std::make_unique<FeatureList>();
 
   // The feature is explicitly enabled on the command-line.
-  feature_list->InitializeFromCommandLine(kFeatureOffByDefaultName, "");
+  feature_list->InitFromCommandLine(kFeatureOffByDefaultName, "");
 
   // But the FieldTrial would set the feature to disabled.
   FieldTrial* trial = FieldTrialList::CreateFieldTrial("TrialExample2", "A");
@@ -264,7 +286,7 @@ TEST_F(FeatureListTest, CommandLineDisableTakesPrecedenceOverFieldTrial) {
   auto feature_list = std::make_unique<FeatureList>();
 
   // The feature is explicitly disabled on the command-line.
-  feature_list->InitializeFromCommandLine("", kFeatureOffByDefaultName);
+  feature_list->InitFromCommandLine("", kFeatureOffByDefaultName);
 
   // But the FieldTrial would set the feature to enabled.
   FieldTrial* trial = FieldTrialList::CreateFieldTrial("TrialExample2", "A");
@@ -327,7 +349,7 @@ TEST_F(FeatureListTest, IsFeatureOverriddenFromCommandLine) {
       kFeatureOffByDefaultName, FeatureList::OVERRIDE_ENABLE_FEATURE));
 
   // Now, enable |kFeatureOffByDefaultName| via the command-line.
-  feature_list->InitializeFromCommandLine(kFeatureOffByDefaultName, "");
+  feature_list->InitFromCommandLine(kFeatureOffByDefaultName, "");
 
   // It should now be overridden for the enabled group.
   EXPECT_TRUE(feature_list->IsFeatureOverridden(kFeatureOffByDefaultName));
@@ -396,11 +418,12 @@ TEST_F(FeatureListTest, AssociateReportingFieldTrial) {
                                     test_case.enable_features,
                                     test_case.disable_features));
 
-    test::ScopedFieldTrialListResetter resetter;
-    FieldTrialList field_trial_list(nullptr);
+    test::ScopedFeatureList outer_scope;
+    outer_scope.InitWithEmptyFeatureAndFieldTrialLists();
+
     auto feature_list = std::make_unique<FeatureList>();
-    feature_list->InitializeFromCommandLine(test_case.enable_features,
-                                            test_case.disable_features);
+    feature_list->InitFromCommandLine(test_case.enable_features,
+                                      test_case.disable_features);
 
     FieldTrial* enable_trial = nullptr;
     if (feature_list->IsFeatureOverriddenFromCommandLine(
@@ -454,10 +477,10 @@ TEST_F(FeatureListTest, RegisterExtraFeatureOverrides) {
   EXPECT_TRUE(FeatureList::IsEnabled(kFeatureOffByDefault));
 }
 
-TEST_F(FeatureListTest, InitializeFromCommandLineThenRegisterExtraOverrides) {
+TEST_F(FeatureListTest, InitFromCommandLineThenRegisterExtraOverrides) {
   auto feature_list = std::make_unique<FeatureList>();
-  feature_list->InitializeFromCommandLine(kFeatureOnByDefaultName,
-                                          kFeatureOffByDefaultName);
+  feature_list->InitFromCommandLine(kFeatureOnByDefaultName,
+                                    kFeatureOffByDefaultName);
   std::vector<FeatureList::FeatureOverrideInfo> overrides;
   overrides.push_back({std::cref(kFeatureOnByDefault),
                        FeatureList::OverrideState::OVERRIDE_DISABLE_FEATURE});
@@ -467,7 +490,7 @@ TEST_F(FeatureListTest, InitializeFromCommandLineThenRegisterExtraOverrides) {
   test::ScopedFeatureList scoped_feature_list;
   scoped_feature_list.InitWithFeatureList(std::move(feature_list));
 
-  // The InitializeFromCommandLine supersedes the RegisterExtraFeatureOverrides
+  // The InitFromCommandLine supersedes the RegisterExtraFeatureOverrides
   // because it was called first.
   EXPECT_TRUE(FeatureList::IsEnabled(kFeatureOnByDefault));
   EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOffByDefault));
@@ -482,7 +505,7 @@ TEST_F(FeatureListTest, InitializeFromCommandLineThenRegisterExtraOverrides) {
 
 TEST_F(FeatureListTest, GetFeatureOverrides) {
   auto feature_list = std::make_unique<FeatureList>();
-  feature_list->InitializeFromCommandLine("A,X", "D");
+  feature_list->InitFromCommandLine("A,X", "D");
 
   Feature feature_b = {"B", FEATURE_ENABLED_BY_DEFAULT};
   Feature feature_c = {"C", FEATURE_DISABLED_BY_DEFAULT};
@@ -516,7 +539,7 @@ TEST_F(FeatureListTest, GetFeatureOverrides) {
 
 TEST_F(FeatureListTest, GetFeatureOverrides_UseDefault) {
   auto feature_list = std::make_unique<FeatureList>();
-  feature_list->InitializeFromCommandLine("A,X", "D");
+  feature_list->InitFromCommandLine("A,X", "D");
 
   FieldTrial* trial = FieldTrialList::CreateFieldTrial("Trial", "Group");
   feature_list->RegisterFieldTrialOverride(
@@ -545,10 +568,10 @@ TEST_F(FeatureListTest, GetFieldTrial) {
   EXPECT_EQ(nullptr, FeatureList::GetFieldTrial(kFeatureOffByDefault));
 }
 
-TEST_F(FeatureListTest, InitializeFromCommandLine_WithFieldTrials) {
+TEST_F(FeatureListTest, InitFromCommandLine_WithFieldTrials) {
   FieldTrialList::CreateFieldTrial("Trial", "Group");
   auto feature_list = std::make_unique<FeatureList>();
-  feature_list->InitializeFromCommandLine("A,OffByDefault<Trial,X", "D");
+  feature_list->InitFromCommandLine("A,OffByDefault<Trial,X", "D");
   test::ScopedFeatureList scoped_feature_list;
   scoped_feature_list.InitWithFeatureList(std::move(feature_list));
 
@@ -557,12 +580,12 @@ TEST_F(FeatureListTest, InitializeFromCommandLine_WithFieldTrials) {
   EXPECT_TRUE(FieldTrialList::IsTrialActive("Trial"));
 }
 
-TEST_F(FeatureListTest, InitializeFromCommandLine_UseDefault) {
+TEST_F(FeatureListTest, InitFromCommandLine_UseDefault) {
   FieldTrialList::CreateFieldTrial("T1", "Group");
   FieldTrialList::CreateFieldTrial("T2", "Group");
   auto feature_list = std::make_unique<FeatureList>();
-  feature_list->InitializeFromCommandLine(
-      "A,*OffByDefault<T1,*OnByDefault<T2,X", "D");
+  feature_list->InitFromCommandLine("A,*OffByDefault<T1,*OnByDefault<T2,X",
+                                    "D");
   test::ScopedFeatureList scoped_feature_list;
   scoped_feature_list.InitWithFeatureList(std::move(feature_list));
 
@@ -575,7 +598,7 @@ TEST_F(FeatureListTest, InitializeFromCommandLine_UseDefault) {
   EXPECT_TRUE(FieldTrialList::IsTrialActive("T2"));
 }
 
-TEST_F(FeatureListTest, InitializeInstance) {
+TEST_F(FeatureListTest, InitInstance) {
   std::unique_ptr<base::FeatureList> feature_list(new base::FeatureList);
   test::ScopedFeatureList scoped_feature_list;
   scoped_feature_list.InitWithFeatureList(std::move(feature_list));
@@ -584,12 +607,12 @@ TEST_F(FeatureListTest, InitializeInstance) {
   EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOffByDefault));
 
   // Initialize from command line if we haven't yet.
-  FeatureList::InitializeInstance("", kFeatureOnByDefaultName);
+  FeatureList::InitInstance("", kFeatureOnByDefaultName);
   EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOnByDefault));
   EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOffByDefault));
 
   // Do not initialize from commandline if we have already.
-  FeatureList::InitializeInstance(kFeatureOffByDefaultName, "");
+  FeatureList::InitInstance(kFeatureOffByDefaultName, "");
   EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOnByDefault));
   EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOffByDefault));
 }
@@ -635,7 +658,7 @@ TEST_F(FeatureListTest, StoreAndRetrieveFeaturesFromSharedMemory) {
   EXPECT_FALSE(feature_list2->IsFeatureOverriddenFromCommandLine(
       kFeatureOnByDefaultName, FeatureList::OVERRIDE_DISABLE_FEATURE));
 
-  feature_list2->InitializeFromSharedMemory(&allocator);
+  feature_list2->InitFromSharedMemory(&allocator);
   // Check that the new feature list now has 2 overrides.
   EXPECT_TRUE(feature_list2->IsFeatureOverriddenFromCommandLine(
       kFeatureOffByDefaultName, FeatureList::OVERRIDE_ENABLE_FEATURE));
@@ -663,7 +686,7 @@ TEST_F(FeatureListTest, StoreAndRetrieveAssociatedFeaturesFromSharedMemory) {
   feature_list->AddFeaturesToAllocator(&allocator);
 
   std::unique_ptr<base::FeatureList> feature_list2(new base::FeatureList);
-  feature_list2->InitializeFromSharedMemory(&allocator);
+  feature_list2->InitFromSharedMemory(&allocator);
   feature_list2->FinalizeInitialization();
 
   // Check that the field trials are still associated.
@@ -675,6 +698,40 @@ TEST_F(FeatureListTest, StoreAndRetrieveAssociatedFeaturesFromSharedMemory) {
   EXPECT_EQ(associated_trial2, trial2);
 }
 
+TEST_F(FeatureListTest, SetEarlyAccessInstance_AllowList) {
+  test::ScopedFeatureList clear_feature_list;
+  clear_feature_list.InitWithNullFeatureAndFieldTrialLists();
+
+  auto early_access_feature_list = std::make_unique<FeatureList>();
+  early_access_feature_list->InitFromCommandLine("OffByDefault", "OnByDefault");
+  FeatureList::SetEarlyAccessInstance(std::move(early_access_feature_list),
+                                      {"DcheckIsFatal", "OnByDefault"});
+  EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOnByDefault));
+  EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOffByDefault));
+  EXPECT_EQ(&kFeatureOffByDefault,
+            FeatureList::GetEarlyAccessedFeatureForTesting());
+  FeatureList::ResetEarlyFeatureAccessTrackerForTesting();
+}
+
+TEST_F(FeatureListTest, SetEarlyAccessInstance_ReplaceByRealList) {
+  test::ScopedFeatureList clear_feature_list;
+  clear_feature_list.InitWithNullFeatureAndFieldTrialLists();
+
+  auto early_access_feature_list = std::make_unique<FeatureList>();
+  early_access_feature_list->InitFromCommandLine("OffByDefault", "OnByDefault");
+  FeatureList::SetEarlyAccessInstance(
+      std::move(early_access_feature_list),
+      {"DcheckIsFatal", "OffByDefault", "OnByDefault"});
+  EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOnByDefault));
+  EXPECT_TRUE(FeatureList::IsEnabled(kFeatureOffByDefault));
+
+  auto feature_list = std::make_unique<FeatureList>();
+  feature_list->InitFromCommandLine("", "");
+  FeatureList::SetInstance(std::move(feature_list));
+  EXPECT_TRUE(FeatureList::IsEnabled(kFeatureOnByDefault));
+  EXPECT_FALSE(FeatureList::IsEnabled(kFeatureOffByDefault));
+}
+
 #if BUILDFLAG(ENABLE_BANNED_BASE_FEATURE_PREFIX) && \
     defined(GTEST_HAS_DEATH_TEST)
 using FeatureListDeathTest = FeatureListTest;
@@ -690,9 +747,9 @@ TEST_F(FeatureListDeathTest, DiesWithBadFeatureName) {
         // defined(GTEST_HAS_DEATH_TEST)
 
 TEST(FeatureListAccessorTest, DefaultStates) {
+  test::ScopedFeatureList scoped_feature_list;
   auto feature_list = std::make_unique<FeatureList>();
   auto feature_list_accessor = feature_list->ConstructAccessor();
-  test::ScopedFeatureList scoped_feature_list;
   scoped_feature_list.InitWithFeatureList(std::move(feature_list));
 
   EXPECT_EQ(feature_list_accessor->GetOverrideStateByFeatureName(
@@ -703,7 +760,7 @@ TEST(FeatureListAccessorTest, DefaultStates) {
             FeatureList::OVERRIDE_USE_DEFAULT);
 }
 
-TEST(FeatureListAccessorTest, InitializeFromCommandLine) {
+TEST(FeatureListAccessorTest, InitFromCommandLine) {
   struct {
     const char* enable_features;
     const char* disable_features;
@@ -732,12 +789,11 @@ TEST(FeatureListAccessorTest, InitializeFromCommandLine) {
                                     test_case.enable_features,
                                     test_case.disable_features));
 
+    test::ScopedFeatureList scoped_feature_list;
     auto feature_list = std::make_unique<FeatureList>();
     auto feature_list_accessor = feature_list->ConstructAccessor();
-
-    feature_list->InitializeFromCommandLine(test_case.enable_features,
-                                            test_case.disable_features);
-    test::ScopedFeatureList scoped_feature_list;
+    feature_list->InitFromCommandLine(test_case.enable_features,
+                                      test_case.disable_features);
     scoped_feature_list.InitWithFeatureList(std::move(feature_list));
 
     EXPECT_EQ(test_case.expected_feature_on_state,
@@ -751,4 +807,169 @@ TEST(FeatureListAccessorTest, InitializeFromCommandLine) {
   }
 }
 
+TEST(FeatureListAccessorTest, InitFromCommandLineWithFeatureParams) {
+  struct {
+    const std::string enable_features;
+    const std::map<std::string, std::string> expected_feature_params;
+  } test_cases[] = {
+      {"Feature:x/100/y/test", {{"x", "100"}, {"y", "test"}}},
+      {"Feature<Trial:asdf/ghjkl/y/123", {{"asdf", "ghjkl"}, {"y", "123"}}},
+  };
+
+  // Clear global state so that repeated runs of this test don't flake.
+  // When https://crrev.com/c/3694674 is submitted, we should be able to remove
+  // this.
+  base::FieldTrialParamAssociator::GetInstance()->ClearAllParamsForTesting();
+
+  for (size_t i = 0; i < std::size(test_cases); ++i) {
+    const auto& test_case = test_cases[i];
+    SCOPED_TRACE(test_case.enable_features);
+
+    test::ScopedFeatureList scoped_feature_list;
+    auto feature_list = std::make_unique<FeatureList>();
+    auto feature_list_accessor = feature_list->ConstructAccessor();
+    feature_list->InitFromCommandLine(test_case.enable_features, "");
+    scoped_feature_list.InitWithFeatureList(std::move(feature_list));
+
+    EXPECT_EQ(FeatureList::OVERRIDE_ENABLE_FEATURE,
+              feature_list_accessor->GetOverrideStateByFeatureName("Feature"))
+        << i;
+    std::map<std::string, std::string> actual_params;
+    EXPECT_TRUE(feature_list_accessor->GetParamsByFeatureName("Feature",
+                                                              &actual_params))
+        << i;
+    EXPECT_EQ(test_case.expected_feature_params, actual_params) << i;
+  }
+}
+
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+// Test only class to verify correctness of
+// FeatureList::VisitFeaturesAndParams().
+class TestFeatureVisitor : public FeatureVisitor {
+ public:
+  TestFeatureVisitor() = default;
+
+  TestFeatureVisitor(const TestFeatureVisitor&) = delete;
+  TestFeatureVisitor& operator=(const TestFeatureVisitor&) = delete;
+
+  ~TestFeatureVisitor() override = default;
+
+  struct VisitedFeatureState {
+    auto operator<=>(const VisitedFeatureState&) const = default;
+
+    std::string feature_name;
+    const base::FeatureList::OverrideState override_state;
+    base::FieldTrialParams params;
+    std::string trial_name;
+    std::string group_name;
+  };
+
+  void Visit(const std::string& feature_name,
+             FeatureList::OverrideState override_state,
+             const FieldTrialParams& params,
+             const std::string& trial_name,
+             const std::string& group_name) override {
+    feature_state_.insert(TestFeatureVisitor::VisitedFeatureState{
+        feature_name, override_state, params, trial_name, group_name});
+  }
+
+  const std::multiset<TestFeatureVisitor::VisitedFeatureState>&
+  feature_state() {
+    return feature_state_;
+  }
+
+ private:
+  std::multiset<VisitedFeatureState> feature_state_;
+};
+
+// Makes test output human readable.
+std::ostream& operator<<(std::ostream& out,
+                         const TestFeatureVisitor::VisitedFeatureState& state) {
+  out << ".feature_name='" << state.feature_name
+      << "', .override_state=" << state.override_state << ", .params={";
+
+  for (const auto& param : state.params) {
+    out << param.first << "=" << param.second << ", ";
+  }
+
+  out << "}, .trial_name='" << state.trial_name << "', .group_name='"
+      << state.group_name << "'";
+  return out;
+}
+
+TEST(TestFeatureVisitor, FeatureWithNoFieldTrial) {
+  base::test::ScopedFeatureList outer_scope;
+  outer_scope.InitWithEmptyFeatureAndFieldTrialLists();
+
+  base::test::ScopedFeatureList feature_list;
+  feature_list.InitWithFeatures(/*enabled_features=*/{kFeatureOffByDefault},
+                                /*disabled_features=*/{kFeatureOnByDefault});
+
+  TestFeatureVisitor visitor;
+  base::FeatureList::VisitFeaturesAndParams(visitor);
+  std::multiset<TestFeatureVisitor::VisitedFeatureState> actual_feature_state =
+      visitor.feature_state();
+
+  std::multiset<TestFeatureVisitor::VisitedFeatureState>
+      expected_feature_state = {
+          {"OnByDefault", FeatureList::OverrideState::OVERRIDE_DISABLE_FEATURE,
+           FieldTrialParams{}, "", ""},
+          {"OffByDefault", FeatureList::OverrideState::OVERRIDE_ENABLE_FEATURE,
+           FieldTrialParams{}, "", ""},
+      };
+
+  EXPECT_EQ(actual_feature_state, expected_feature_state);
+}
+
+TEST(TestFeatureVisitor, FeatureOverrideUseDefault) {
+  base::test::ScopedFeatureList outer_scope;
+  outer_scope.InitWithEmptyFeatureAndFieldTrialLists();
+
+  auto feature_list = std::make_unique<base::FeatureList>();
+  base::FieldTrial* trial =
+      base::FieldTrialList::CreateFieldTrial("TrialExample", "A");
+  feature_list->RegisterFieldTrialOverride(
+      "TestFeature", base::FeatureList::OVERRIDE_USE_DEFAULT, trial);
+
+  base::test::ScopedFeatureList initialized_feature_list;
+  initialized_feature_list.InitWithFeatureList(std::move(feature_list));
+
+  TestFeatureVisitor visitor;
+  base::FeatureList::VisitFeaturesAndParams(visitor);
+  std::multiset<TestFeatureVisitor::VisitedFeatureState> actual_feature_state =
+      visitor.feature_state();
+
+  std::multiset<TestFeatureVisitor::VisitedFeatureState>
+      expected_feature_state = {
+          {"TestFeature", FeatureList::OverrideState::OVERRIDE_USE_DEFAULT,
+           FieldTrialParams{}, "TrialExample", "A"}};
+
+  EXPECT_EQ(actual_feature_state, expected_feature_state);
+}
+
+TEST(TestFeatureVisitor, FeatureHasParams) {
+  base::test::ScopedFeatureList outer_scope;
+  outer_scope.InitWithEmptyFeatureAndFieldTrialLists();
+
+  base::test::ScopedFeatureList initialized_feature_list;
+
+  initialized_feature_list.InitFromCommandLine(
+      /*enabled_features=*/"TestFeature<foo.bar:k1/v1/k2/v2",
+      /*disabled_features=*/"");
+
+  TestFeatureVisitor visitor;
+  base::FeatureList::VisitFeaturesAndParams(visitor);
+  std::multiset<TestFeatureVisitor::VisitedFeatureState> actual_feature_state =
+      visitor.feature_state();
+
+  std::multiset<TestFeatureVisitor::VisitedFeatureState>
+      expected_feature_state = {
+          {"TestFeature", FeatureList::OverrideState::OVERRIDE_ENABLE_FEATURE,
+           FieldTrialParams{{"k1", "v1"}, {"k2", "v2"}}, "foo", "bar"},
+      };
+
+  EXPECT_EQ(actual_feature_state, expected_feature_state);
+}
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+
 }  // namespace base
