@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,43 +10,62 @@
 #include <memory>
 #include <sstream>
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/files/file.h"
+#include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
-#include "base/macros.h"
+#include "base/functional/bind.h"
 #include "base/memory/shared_memory_mapping.h"
 #include "base/memory/writable_shared_memory_region.h"
+#include "base/process/launch.h"
+#include "base/process/process.h"
+#include "base/process/process_handle.h"
 #include "base/ranges/algorithm.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/system/sys_info.h"
+#include "base/test/gtest_util.h"
 #include "base/test/multiprocess_test.h"
+#include "base/test/test_timeouts.h"
 #include "base/threading/thread.h"
+#include "build/blink_buildflags.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "testing/multiprocess_func_list.h"
 
-#if defined(OS_APPLE)
+#if BUILDFLAG(IS_APPLE)
 #include <sys/mman.h>
 #endif
 
-#if defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
-#include "base/process/internal_linux.h"
+#if BUILDFLAG(IS_MAC) || (BUILDFLAG(IS_IOS) && BUILDFLAG(USE_BLINK))
+#include <mach/mach.h>
+
+#include "base/apple/mach_logging.h"
+#include "base/apple/scoped_mach_port.h"
+#include "base/mac/mach_port_rendezvous.h"
+#include "base/process/port_provider_mac.h"
 #endif
 
-namespace base {
-namespace debug {
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) ||      \
+    BUILDFLAG(IS_CHROMEOS_LACROS) || BUILDFLAG(IS_WIN) || \
+    BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_FUCHSIA) || BUILDFLAG(IS_APPLE)
+#define ENABLE_CPU_TESTS 1
+#else
+#define ENABLE_CPU_TESTS 0
+#endif
+
+namespace base::debug {
 
-#if defined(OS_LINUX) || defined(OS_CHROMEOS) || \
-    BUILDFLAG(IS_CHROMEOS_LACROS) || defined(OS_WIN) || defined(OS_ANDROID)
 namespace {
 
+#if ENABLE_CPU_TESTS
+
 void BusyWork(std::vector<std::string>* vec) {
   int64_t test_value = 0;
   for (int i = 0; i < 100000; ++i) {
@@ -55,10 +74,223 @@ void BusyWork(std::vector<std::string>* vec) {
   }
 }
 
+TimeDelta TestCumulativeCPU(ProcessMetrics* metrics, TimeDelta prev_cpu_usage) {
+  const TimeDelta current_cpu_usage = metrics->GetCumulativeCPUUsage();
+  EXPECT_GE(current_cpu_usage, prev_cpu_usage);
+  EXPECT_GE(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+  return current_cpu_usage;
+}
+
+TimeDelta TestPreciseCumulativeCPU(ProcessMetrics* metrics,
+                                   TimeDelta prev_cpu_usage) {
+#if BUILDFLAG(IS_WIN)
+  const TimeDelta current_cpu_usage = metrics->GetPreciseCumulativeCPUUsage();
+  EXPECT_GE(current_cpu_usage, prev_cpu_usage);
+  EXPECT_GE(metrics->GetPreciseCPUUsage(), 0.0);
+  return current_cpu_usage;
+#else
+  // Do nothing. Not supported on this platform.
+  return base::TimeDelta();
+#endif
+}
+
+#endif  // ENABLE_CPU_TESTS
+
+using ::testing::AssertionFailure;
+using ::testing::AssertionResult;
+using ::testing::AssertionSuccess;
+
+// Helper to deal with Mac process launching complexity. On other platforms this
+// is just a thin wrapper around SpawnMultiProcessTestChild.
+class TestChildLauncher {
+ public:
+  TestChildLauncher() = default;
+  ~TestChildLauncher() = default;
+
+  TestChildLauncher(const TestChildLauncher&) = delete;
+  TestChildLauncher& operator=(const TestChildLauncher&) = delete;
+
+  // Returns a reference to the command line for the child process. This can be
+  // used to add extra parameters before calling SpawnChildProcess().
+  CommandLine& command_line() { return command_line_; }
+
+  // Returns a reference to the child process object, which will be invalid
+  // until SpawnChildProcess() is called.
+  Process& child_process() { return child_process_; }
+
+  // Spawns a multiprocess test child to execute the function `procname`.
+  AssertionResult SpawnChildProcess(const std::string& procname);
+
+  // Returns a ProcessMetrics object for the child process created by
+  // SpawnChildProcess().
+  std::unique_ptr<ProcessMetrics> CreateChildProcessMetrics();
+
+  // Terminates the child process created by SpawnChildProcess(). Returns true
+  // if the process successfully terminates within the allowed time.
+  bool TerminateChildProcess();
+
+  // Called from the child process to send data back to the parent if needed.
+  static void NotifyParent();
+
+ private:
+  CommandLine command_line_ = GetMultiProcessTestChildBaseCommandLine();
+  Process child_process_;
+
+#if BUILDFLAG(IS_MAC) || (BUILDFLAG(IS_IOS) && BUILDFLAG(USE_BLINK))
+  class TestChildPortProvider;
+  std::unique_ptr<TestChildPortProvider> port_provider_;
+#endif
+};
+
+#if BUILDFLAG(IS_MAC) || (BUILDFLAG(IS_IOS) && BUILDFLAG(USE_BLINK))
+
+// Adapted from base/mac/mach_port_rendezvous_unittest.cc and
+// https://mw.foldr.org/posts/computers/macosx/task-info-fun-with-mach/
+
+constexpr MachPortsForRendezvous::key_type kTestChildRendezvousKey = 'test';
+
+// A PortProvider that tracks child processes spawned by TestChildLauncher.
+class TestChildLauncher::TestChildPortProvider final : public PortProvider {
+ public:
+  TestChildPortProvider(ProcessHandle handle, apple::ScopedMachSendRight port)
+      : handle_(handle), port_(std::move(port)) {}
+
+  ~TestChildPortProvider() final = default;
+
+  TestChildPortProvider(const TestChildPortProvider&) = delete;
+  TestChildPortProvider& operator=(const TestChildPortProvider&) = delete;
+
+  mach_port_t TaskForPid(ProcessHandle process) const final {
+    return process == handle_ ? port_.get() : MACH_PORT_NULL;
+  }
+
+ private:
+  ProcessHandle handle_;
+  apple::ScopedMachSendRight port_;
+};
+
+AssertionResult TestChildLauncher::SpawnChildProcess(
+    const std::string& procname) {
+  // Allocate a port for the parent to receive details from the child process.
+  apple::ScopedMachReceiveRight receive_port;
+  if (!apple::CreateMachPort(&receive_port, nullptr)) {
+    return AssertionFailure() << "Failed to allocate receive port";
+  }
+
+  // Pass the sending end of the port to the child.
+  LaunchOptions options = LaunchOptionsForTest();
+  options.mach_ports_for_rendezvous.emplace(
+      kTestChildRendezvousKey,
+      MachRendezvousPort(receive_port.get(), MACH_MSG_TYPE_MAKE_SEND));
+  child_process_ =
+      SpawnMultiProcessTestChild(procname, command_line_, std::move(options));
+  if (!child_process_.IsValid()) {
+    return AssertionFailure() << "Failed to launch child process.";
+  }
+
+  // Wait for the child to send back its mach_task_self().
+  struct : mach_msg_base_t {
+    mach_msg_port_descriptor_t task_port;
+    mach_msg_trailer_t trailer;
+  } msg{};
+  kern_return_t kr =
+      mach_msg(&msg.header, MACH_RCV_MSG | MACH_RCV_TIMEOUT, 0, sizeof(msg),
+               receive_port.get(),
+               TestTimeouts::action_timeout().InMilliseconds(), MACH_PORT_NULL);
+  if (kr != KERN_SUCCESS) {
+    return AssertionFailure()
+           << "Failed to read mach_task_self from child process: "
+           << mach_error_string(kr);
+  }
+  port_provider_ = std::make_unique<TestChildPortProvider>(
+      child_process_.Handle(), apple::ScopedMachSendRight(msg.task_port.name));
+  return AssertionSuccess();
+}
+
+std::unique_ptr<ProcessMetrics> TestChildLauncher::CreateChildProcessMetrics() {
+#if BUILDFLAG(IS_MAC)
+  return ProcessMetrics::CreateProcessMetrics(child_process_.Handle(),
+                                              port_provider_.get());
+#else
+  return ProcessMetrics::CreateProcessMetrics(child_process_.Handle());
+#endif
+}
+
+bool TestChildLauncher::TerminateChildProcess() {
+  return TerminateMultiProcessTestChild(child_process_, /*exit_code=*/0,
+                                        /*wait=*/true);
+}
+
+// static
+void TestChildLauncher::NotifyParent() {
+  auto* client = MachPortRendezvousClient::GetInstance();
+  ASSERT_TRUE(client);
+  apple::ScopedMachSendRight send_port =
+      client->TakeSendRight(kTestChildRendezvousKey);
+  ASSERT_TRUE(send_port.is_valid());
+
+  // Send mach_task_self to the parent process so that it can use the port to
+  // create ProcessMetrics.
+  struct : mach_msg_base_t {
+    mach_msg_port_descriptor_t task_port;
+  } msg{};
+  msg.header.msgh_bits =
+      MACH_MSGH_BITS_REMOTE(MACH_MSG_TYPE_COPY_SEND) | MACH_MSGH_BITS_COMPLEX;
+  msg.header.msgh_remote_port = send_port.get();
+  msg.header.msgh_size = sizeof(msg);
+  msg.body.msgh_descriptor_count = 1;
+  msg.task_port.name = mach_task_self();
+  msg.task_port.disposition = MACH_MSG_TYPE_COPY_SEND;
+  msg.task_port.type = MACH_MSG_PORT_DESCRIPTOR;
+  kern_return_t kr =
+      mach_msg(&msg.header, MACH_SEND_MSG, msg.header.msgh_size, 0,
+               MACH_PORT_NULL, MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL);
+  MACH_CHECK(kr == KERN_SUCCESS, kr);
+}
+
+#else
+
+AssertionResult TestChildLauncher::SpawnChildProcess(
+    const std::string& procname) {
+  child_process_ = SpawnMultiProcessTestChild(procname, command_line_,
+                                              LaunchOptionsForTest());
+  return child_process_.IsValid()
+             ? AssertionSuccess()
+             : AssertionFailure() << "Failed to launch child process.";
+}
+
+std::unique_ptr<ProcessMetrics> TestChildLauncher::CreateChildProcessMetrics() {
+  return ProcessMetrics::CreateProcessMetrics(child_process_.Handle());
+}
+
+bool TestChildLauncher::TerminateChildProcess() {
+  [[maybe_unused]] const ProcessHandle child_handle = child_process_.Handle();
+  if (!TerminateMultiProcessTestChild(child_process_, /*exit_code=*/0,
+                                      /*wait=*/true)) {
+    return false;
+  }
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_ANDROID)
+  // After the process exits, ProcessMetrics races to read /proc/<pid>/stat
+  // before it's deleted. Wait until it's definitely gone.
+  const auto stat_path = FilePath(FILE_PATH_LITERAL("/proc"))
+                             .AppendASCII(NumberToString(child_handle))
+                             .Append(FILE_PATH_LITERAL("stat"));
+
+  while (PathExists(stat_path)) {
+    PlatformThread::Sleep(TestTimeouts::tiny_timeout());
+  }
+#endif
+  return true;
+}
+
+// static
+void TestChildLauncher::NotifyParent() {
+  // Do nothing.
+}
+
+#endif  // BUILDFLAG(IS_MAC)
+
 }  // namespace
-#endif  // defined(OS_LINUX) || defined(OS_CHROMEOS) ||
-        // BUILDFLAG(IS_CHROMEOS_LACROS) || defined(OS_WIN) ||
-        // defined(OS_ANDROID)
 
 // Tests for SystemMetrics.
 // Exists as a class so it can be a friend of SystemMetrics.
@@ -66,11 +298,11 @@ class SystemMetricsTest : public testing::Test {
  public:
   SystemMetricsTest() = default;
 
- private:
-  DISALLOW_COPY_AND_ASSIGN(SystemMetricsTest);
+  SystemMetricsTest(const SystemMetricsTest&) = delete;
+  SystemMetricsTest& operator=(const SystemMetricsTest&) = delete;
 };
 
-#if defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_ANDROID)
 TEST_F(SystemMetricsTest, IsValidDiskName) {
   const char invalid_input1[] = "";
   const char invalid_input2[] = "s";
@@ -201,15 +433,15 @@ TEST_F(SystemMetricsTest, ParseMeminfo) {
   EXPECT_EQ(meminfo.swap_free, 3672368);
   EXPECT_EQ(meminfo.dirty, 184);
   EXPECT_EQ(meminfo.reclaimable, 30936);
-#if BUILDFLAG(IS_CHROMEOS_ASH) || BUILDFLAG(IS_CHROMEOS_LACROS)
+#if BUILDFLAG(IS_CHROMEOS)
   EXPECT_EQ(meminfo.shmem, 140204);
   EXPECT_EQ(meminfo.slab, 54212);
 #endif
-  EXPECT_EQ(355725,
+  EXPECT_EQ(355725u,
             base::SysInfo::AmountOfAvailablePhysicalMemory(meminfo) / 1024);
   // Simulate as if there is no MemAvailable.
   meminfo.available = 0;
-  EXPECT_EQ(374448,
+  EXPECT_EQ(374448u,
             base::SysInfo::AmountOfAvailablePhysicalMemory(meminfo) / 1024);
   meminfo = {};
   EXPECT_TRUE(ParseProcMeminfo(valid_input2, &meminfo));
@@ -221,7 +453,7 @@ TEST_F(SystemMetricsTest, ParseMeminfo) {
   EXPECT_EQ(meminfo.swap_total, 524280);
   EXPECT_EQ(meminfo.swap_free, 524200);
   EXPECT_EQ(meminfo.dirty, 4);
-  EXPECT_EQ(69936,
+  EXPECT_EQ(69936u,
             base::SysInfo::AmountOfAvailablePhysicalMemory(meminfo) / 1024);
 }
 
@@ -340,20 +572,22 @@ TEST_F(SystemMetricsTest, ParseVmstat) {
   const char empty_input[] = "";
   EXPECT_FALSE(ParseProcVmstat(empty_input, &vmstat));
 }
-#endif  // defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
-
-#if defined(OS_LINUX) || defined(OS_CHROMEOS) || \
-    BUILDFLAG(IS_CHROMEOS_LACROS) || defined(OS_WIN)
+#endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) ||
+        // BUILDFLAG(IS_ANDROID)
 
+#if ENABLE_CPU_TESTS
 // Test that ProcessMetrics::GetPlatformIndependentCPUUsage() doesn't return
 // negative values when the number of threads running on the process decreases
 // between two successive calls to it.
 TEST_F(SystemMetricsTest, TestNoNegativeCpuUsage) {
-  ProcessHandle handle = GetCurrentProcessHandle();
-  std::unique_ptr<ProcessMetrics> metrics(
-      ProcessMetrics::CreateProcessMetrics(handle));
+  std::unique_ptr<ProcessMetrics> metrics =
+      ProcessMetrics::CreateCurrentProcessMetrics();
 
   EXPECT_GE(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+#if BUILDFLAG(IS_WIN)
+  EXPECT_GE(metrics->GetPreciseCPUUsage(), 0.0);
+#endif
+
   Thread thread1("thread1");
   Thread thread2("thread2");
   Thread thread3("thread3");
@@ -374,32 +608,113 @@ TEST_F(SystemMetricsTest, TestNoNegativeCpuUsage) {
   thread2.task_runner()->PostTask(FROM_HERE, BindOnce(&BusyWork, &vec2));
   thread3.task_runner()->PostTask(FROM_HERE, BindOnce(&BusyWork, &vec3));
 
-  TimeDelta prev_cpu_usage = metrics->GetCumulativeCPUUsage();
-  EXPECT_GE(prev_cpu_usage, TimeDelta());
-  EXPECT_GE(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+  TimeDelta prev_cpu_usage = TestCumulativeCPU(metrics.get(), TimeDelta());
+  TimeDelta prev_precise_cpu_usage =
+      TestPreciseCumulativeCPU(metrics.get(), TimeDelta());
 
   thread1.Stop();
-  TimeDelta current_cpu_usage = metrics->GetCumulativeCPUUsage();
-  EXPECT_GE(current_cpu_usage, prev_cpu_usage);
-  prev_cpu_usage = current_cpu_usage;
-  EXPECT_GE(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+  prev_cpu_usage = TestCumulativeCPU(metrics.get(), prev_cpu_usage);
+  prev_precise_cpu_usage =
+      TestPreciseCumulativeCPU(metrics.get(), prev_precise_cpu_usage);
 
   thread2.Stop();
-  current_cpu_usage = metrics->GetCumulativeCPUUsage();
-  EXPECT_GE(current_cpu_usage, prev_cpu_usage);
-  prev_cpu_usage = current_cpu_usage;
-  EXPECT_GE(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+  prev_cpu_usage = TestCumulativeCPU(metrics.get(), prev_cpu_usage);
+  prev_precise_cpu_usage =
+      TestPreciseCumulativeCPU(metrics.get(), prev_precise_cpu_usage);
 
   thread3.Stop();
-  current_cpu_usage = metrics->GetCumulativeCPUUsage();
-  EXPECT_GE(current_cpu_usage, prev_cpu_usage);
-  EXPECT_GE(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+  prev_cpu_usage = TestCumulativeCPU(metrics.get(), prev_cpu_usage);
+  prev_precise_cpu_usage =
+      TestPreciseCumulativeCPU(metrics.get(), prev_precise_cpu_usage);
 }
 
-#endif  // defined(OS_LINUX) || defined(OS_CHROMEOS) ||
-        // BUILDFLAG(IS_CHROMEOS_LACROS) || defined(OS_WIN)
+#if !BUILDFLAG(IS_APPLE) || BUILDFLAG(USE_BLINK)
 
-#if BUILDFLAG(IS_CHROMEOS_ASH) || BUILDFLAG(IS_CHROMEOS_LACROS)
+// Subprocess to test the child CPU usage.
+MULTIPROCESS_TEST_MAIN(CPUUsageChildMain) {
+  TestChildLauncher::NotifyParent();
+  // Busy wait until terminated.
+  while (true) {
+    std::vector<std::string> vec;
+    BusyWork(&vec);
+  }
+}
+
+TEST_F(SystemMetricsTest, MeasureChildCpuUsage) {
+  TestChildLauncher child_launcher;
+  ASSERT_TRUE(child_launcher.SpawnChildProcess("CPUUsageChildMain"));
+  std::unique_ptr<ProcessMetrics> metrics =
+      child_launcher.CreateChildProcessMetrics();
+
+  const TimeDelta cpu_usage1 = TestCumulativeCPU(metrics.get(), TimeDelta());
+  PlatformThread::Sleep(TestTimeouts::tiny_timeout());
+
+  const TimeDelta cpu_usage2 = TestCumulativeCPU(metrics.get(), cpu_usage1);
+  EXPECT_TRUE(cpu_usage2.is_positive());
+
+  ASSERT_TRUE(child_launcher.TerminateChildProcess());
+
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_FUCHSIA)
+  // Windows and Fuchsia return final measurements of a process after it exits.
+  TestCumulativeCPU(metrics.get(), cpu_usage2);
+#elif BUILDFLAG(IS_APPLE)
+  // Mac and iOS return 0 on error. GetPlatformIndependentCPUUsage subtracts
+  // from this to get a negative.
+  EXPECT_EQ(metrics->GetCumulativeCPUUsage(), TimeDelta());
+  EXPECT_LT(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+#else
+  // All other platforms return a negative time value to indicate an error.
+  EXPECT_LT(metrics->GetCumulativeCPUUsage(), TimeDelta());
+  EXPECT_LT(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+#endif
+}
+
+#endif  // !BUILDFLAG(IS_APPLE) || BUILDFLAG(USE_BLINK)
+
+#if BUILDFLAG(IS_FUCHSIA)
+
+// Fuchsia CHECK's when measuring an invalid procses.
+using SystemMetricsDeathTest = SystemMetricsTest;
+TEST_F(SystemMetricsDeathTest, InvalidProcessCpuUsage) {
+  std::unique_ptr<ProcessMetrics> metrics =
+      ProcessMetrics::CreateProcessMetrics(kNullProcessHandle);
+  EXPECT_CHECK_DEATH(
+      { [[maybe_unused]] TimeDelta usage = metrics->GetCumulativeCPUUsage(); });
+  EXPECT_CHECK_DEATH({
+    [[maybe_unused]] double usage = metrics->GetPlatformIndependentCPUUsage();
+  });
+}
+
+#else  // !BUILDFLAG(IS_FUCHSIA)
+
+// Windows, Mac and iOS also return 0 on error. All other platforms return a
+// negative value to indicate an error.
+TEST_F(SystemMetricsTest, InvalidProcessCpuUsage) {
+#if BUILDFLAG(IS_MAC)
+  std::unique_ptr<ProcessMetrics> metrics =
+      ProcessMetrics::CreateProcessMetrics(kNullProcessHandle, nullptr);
+#else
+  std::unique_ptr<ProcessMetrics> metrics =
+      ProcessMetrics::CreateProcessMetrics(kNullProcessHandle);
+#endif
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_APPLE)
+  EXPECT_EQ(metrics->GetCumulativeCPUUsage(), TimeDelta());
+#else
+  EXPECT_LT(metrics->GetCumulativeCPUUsage(), TimeDelta());
+#endif
+
+  // The first call always caches the result of GetCumulativeCPUUsage() and
+  // returns 0, even if it's an error value. The second call returns the delta
+  // with the cached result, which is 0 when the second GetCumulativeCPUUsage()
+  // call returns the same error value.
+  EXPECT_EQ(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+  EXPECT_EQ(metrics->GetPlatformIndependentCPUUsage(), 0.0);
+}
+#endif  // !BUILDFLAG(IS_FUCHSIA)
+
+#endif  // ENABLE_CPU_TESTS
+
+#if BUILDFLAG(IS_CHROMEOS)
 TEST_F(SystemMetricsTest, ParseZramMmStat) {
   SwapInfo swapinfo;
 
@@ -434,59 +749,62 @@ TEST_F(SystemMetricsTest, ParseZramStat) {
   EXPECT_EQ(299ULL, swapinfo.num_reads);
   EXPECT_EQ(1ULL, swapinfo.num_writes);
 }
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH) || BUILDFLAG(IS_CHROMEOS_LACROS)
+#endif  // BUILDFLAG(IS_CHROMEOS)
 
-#if defined(OS_WIN) || defined(OS_APPLE) || defined(OS_LINUX) || \
-    defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_APPLE) || BUILDFLAG(IS_LINUX) || \
+    BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_ANDROID)
 TEST(SystemMetrics2Test, GetSystemMemoryInfo) {
   SystemMemoryInfoKB info;
   EXPECT_TRUE(GetSystemMemoryInfo(&info));
 
   // Ensure each field received a value.
   EXPECT_GT(info.total, 0);
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   EXPECT_GT(info.avail_phys, 0);
 #else
   EXPECT_GT(info.free, 0);
 #endif
-#if defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_ANDROID)
   EXPECT_GT(info.buffers, 0);
   EXPECT_GT(info.cached, 0);
   EXPECT_GT(info.active_anon + info.inactive_anon, 0);
   EXPECT_GT(info.active_file + info.inactive_file, 0);
-#endif  // defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) ||
+        // BUILDFLAG(IS_ANDROID)
 
   // All the values should be less than the total amount of memory.
-#if !defined(OS_WIN) && !defined(OS_IOS)
+#if !BUILDFLAG(IS_WIN) && !BUILDFLAG(IS_IOS)
   // TODO(crbug.com/711450): re-enable the following assertion on iOS.
   EXPECT_LT(info.free, info.total);
 #endif
-#if defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_ANDROID)
   EXPECT_LT(info.buffers, info.total);
   EXPECT_LT(info.cached, info.total);
   EXPECT_LT(info.active_anon, info.total);
   EXPECT_LT(info.inactive_anon, info.total);
   EXPECT_LT(info.active_file, info.total);
   EXPECT_LT(info.inactive_file, info.total);
-#endif  // defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) ||
+        // BUILDFLAG(IS_ANDROID)
 
-#if defined(OS_APPLE)
+#if BUILDFLAG(IS_APPLE)
   EXPECT_GT(info.file_backed, 0);
 #endif
 
-#if BUILDFLAG(IS_CHROMEOS_ASH) || BUILDFLAG(IS_CHROMEOS_LACROS)
+#if BUILDFLAG(IS_CHROMEOS)
   // Chrome OS exposes shmem.
   EXPECT_GT(info.shmem, 0);
   EXPECT_LT(info.shmem, info.total);
 #endif
 }
-#endif  // defined(OS_WIN) || defined(OS_APPLE) || defined(OS_LINUX) ||
-        // defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#endif  // BUILDFLAG(IS_WIN) || BUILDFLAG(IS_APPLE) || BUILDFLAG(IS_LINUX) ||
+        // BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_ANDROID)
 
-#if defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_ANDROID)
 TEST(ProcessMetricsTest, ParseProcStatCPU) {
   // /proc/self/stat for a process running "top".
-  const char kTopStat[] = "960 (top) S 16230 960 16230 34818 960 "
+  const char kTopStat[] =
+      "960 (top) S 16230 960 16230 34818 960 "
       "4202496 471 0 0 0 "
       "12 16 0 0 "  // <- These are the goods.
       "20 0 1 0 121946157 15077376 314 18446744073709551615 4194304 "
@@ -495,7 +813,8 @@ TEST(ProcessMetricsTest, ParseProcStatCPU) {
   EXPECT_EQ(12 + 16, ParseProcStatCPU(kTopStat));
 
   // cat /proc/self/stat on a random other machine I have.
-  const char kSelfStat[] = "5364 (cat) R 5354 5364 5354 34819 5364 "
+  const char kSelfStat[] =
+      "5364 (cat) R 5354 5364 5354 34819 5364 "
       "0 142 0 0 0 "
       "0 0 0 0 "  // <- No CPU, apparently.
       "16 0 1 0 1676099790 2957312 114 4294967295 134512640 134528148 "
@@ -505,7 +824,8 @@ TEST(ProcessMetricsTest, ParseProcStatCPU) {
 
   // Some weird long-running process with a weird name that I created for the
   // purposes of this test.
-  const char kWeirdNameStat[] = "26115 (Hello) You ()))  ) R 24614 26115 24614"
+  const char kWeirdNameStat[] =
+      "26115 (Hello) You ()))  ) R 24614 26115 24614"
       " 34839 26115 4218880 227 0 0 0 "
       "5186 11 0 0 "
       "20 0 1 0 36933953 4296704 90 18446744073709551615 4194304 4196116 "
@@ -514,91 +834,16 @@ TEST(ProcessMetricsTest, ParseProcStatCPU) {
       "140735857770737 140735857774557 0";
   EXPECT_EQ(5186 + 11, ParseProcStatCPU(kWeirdNameStat));
 }
-
-TEST(ProcessMetricsTest, ParseProcTimeInState) {
-  ProcessHandle handle = GetCurrentProcessHandle();
-  std::unique_ptr<ProcessMetrics> metrics(
-      ProcessMetrics::CreateProcessMetrics(handle));
-  ProcessMetrics::TimeInStatePerThread time_in_state;
-
-  const char kStatThread123[] =
-      "cpu0\n"
-      "100000 4\n"
-      "200000 5\n"
-      "300000 0\n"
-      "cpu4\n"
-      "400000 3\n"
-      "500000 2\n";
-  EXPECT_TRUE(
-      metrics->ParseProcTimeInState(kStatThread123, 123, time_in_state));
-
-  // Zero-valued entry should not exist.
-  ASSERT_EQ(time_in_state.size(), 4u);
-
-  EXPECT_EQ(time_in_state[0].thread_id, 123);
-  EXPECT_EQ(time_in_state[0].cluster_core_index, 0u);
-  EXPECT_EQ(time_in_state[0].core_frequency_khz, 100000u);
-  EXPECT_EQ(time_in_state[0].cumulative_cpu_time,
-            base::TimeDelta::FromMilliseconds(40));
-  EXPECT_EQ(time_in_state[1].thread_id, 123);
-  EXPECT_EQ(time_in_state[1].cluster_core_index, 0u);
-  EXPECT_EQ(time_in_state[1].core_frequency_khz, 200000u);
-  EXPECT_EQ(time_in_state[1].cumulative_cpu_time,
-            base::TimeDelta::FromMilliseconds(50));
-  EXPECT_EQ(time_in_state[2].thread_id, 123);
-  EXPECT_EQ(time_in_state[2].cluster_core_index, 4u);
-  EXPECT_EQ(time_in_state[2].core_frequency_khz, 400000u);
-  EXPECT_EQ(time_in_state[2].cumulative_cpu_time,
-            base::TimeDelta::FromMilliseconds(30));
-  EXPECT_EQ(time_in_state[3].thread_id, 123);
-  EXPECT_EQ(time_in_state[3].cluster_core_index, 4u);
-  EXPECT_EQ(time_in_state[3].core_frequency_khz, 500000u);
-  EXPECT_EQ(time_in_state[3].cumulative_cpu_time,
-            base::TimeDelta::FromMilliseconds(20));
-
-  // Calling ParseProcTimeInState again adds to the vector.
-  const char kStatThread456[] =
-      "cpu0\n"
-      "\n"           // extra empty line is fine.
-      "1000000 10";  // missing "\n" at end is fine.
-  EXPECT_TRUE(
-      metrics->ParseProcTimeInState(kStatThread456, 456, time_in_state));
-
-  ASSERT_EQ(time_in_state.size(), 5u);
-  EXPECT_EQ(time_in_state[4].thread_id, 456);
-  EXPECT_EQ(time_in_state[4].cluster_core_index, 0u);
-  EXPECT_EQ(time_in_state[4].core_frequency_khz, 1000000u);
-  EXPECT_EQ(time_in_state[4].cumulative_cpu_time,
-            base::TimeDelta::FromMilliseconds(100));
-
-  // Calling ParseProcTimeInState with invalid data returns false.
-  EXPECT_FALSE(
-      metrics->ParseProcTimeInState("100000 5\n"  // no header
-                                    "200000 6\n",
-                                    123, time_in_state));
-  EXPECT_FALSE(
-      metrics->ParseProcTimeInState("cpu0\n"
-                                    "100000\n",  // no time value
-                                    123, time_in_state));
-  EXPECT_FALSE(
-      metrics->ParseProcTimeInState("header0\n"  // invalid header / line
-                                    "100000 5\n",
-                                    123, time_in_state));
-  EXPECT_FALSE(
-      metrics->ParseProcTimeInState("cpu0\n"
-                                    "100000 5\n"
-                                    "invalid334 4\n",  // invalid header / line
-                                    123, time_in_state));
-}
-#endif  // defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_ANDROID)
+#endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) ||
+        // BUILDFLAG(IS_ANDROID)
 
 // Disable on Android because base_unittests runs inside a Dalvik VM that
 // starts and stop threads (crbug.com/175563).
-#if defined(OS_LINUX) || defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
 // http://crbug.com/396455
 TEST(ProcessMetricsTest, DISABLED_GetNumberOfThreads) {
   const ProcessHandle current = GetCurrentProcessHandle();
-  const int initial_threads = GetNumberOfThreads(current);
+  const int64_t initial_threads = GetNumberOfThreads(current);
   ASSERT_GT(initial_threads, 0);
   const int kNumAdditionalThreads = 10;
   {
@@ -612,9 +857,10 @@ TEST(ProcessMetricsTest, DISABLED_GetNumberOfThreads) {
   // The Thread destructor will stop them.
   ASSERT_EQ(initial_threads, GetNumberOfThreads(current));
 }
-#endif  // defined(OS_LINUX) || defined(OS_CHROMEOS)
+#endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
 
-#if defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_MAC)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) || \
+    (BUILDFLAG(IS_APPLE) && BUILDFLAG(USE_BLINK))
 namespace {
 
 // Keep these in sync so the GetChildOpenFdCount test can refer to correct test
@@ -648,12 +894,15 @@ bool CheckEvent(const FilePath& signal_dir, const char* signal_file) {
 
 // Busy-wait for an event to be signaled.
 void WaitForEvent(const FilePath& signal_dir, const char* signal_file) {
-  while (!CheckEvent(signal_dir, signal_file))
-    PlatformThread::Sleep(TimeDelta::FromMilliseconds(10));
+  while (!CheckEvent(signal_dir, signal_file)) {
+    PlatformThread::Sleep(Milliseconds(10));
+  }
 }
 
 // Subprocess to test the number of open file descriptors.
 MULTIPROCESS_TEST_MAIN(ChildMain) {
+  TestChildLauncher::NotifyParent();
+
   CommandLine* command_line = CommandLine::ForCurrentProcess();
   const FilePath temp_path = command_line->GetSwitchValuePath(kTempDirFlag);
   CHECK(DirectoryExists(temp_path));
@@ -675,9 +924,9 @@ MULTIPROCESS_TEST_MAIN(ChildMain) {
   CHECK(SignalEvent(temp_path, kSignalClosed));
 
   // Wait to be terminated.
-  while (true)
-    PlatformThread::Sleep(TimeDelta::FromSeconds(1));
-  return 0;
+  while (true) {
+    PlatformThread::Sleep(Seconds(1));
+  }
 }
 
 }  // namespace
@@ -686,20 +935,15 @@ TEST(ProcessMetricsTest, GetChildOpenFdCount) {
   ScopedTempDir temp_dir;
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
   const FilePath temp_path = temp_dir.GetPath();
-  CommandLine child_command_line(GetMultiProcessTestChildBaseCommandLine());
-  child_command_line.AppendSwitchPath(kTempDirFlag, temp_path);
-  Process child = SpawnMultiProcessTestChild(
-      ChildMainString, child_command_line, LaunchOptions());
-  ASSERT_TRUE(child.IsValid());
+
+  TestChildLauncher child_launcher;
+  child_launcher.command_line().AppendSwitchPath(kTempDirFlag, temp_path);
+  ASSERT_TRUE(child_launcher.SpawnChildProcess(ChildMainString));
 
   WaitForEvent(temp_path, kSignalReady);
 
   std::unique_ptr<ProcessMetrics> metrics =
-#if defined(OS_APPLE)
-      ProcessMetrics::CreateProcessMetrics(child.Handle(), nullptr);
-#else
-      ProcessMetrics::CreateProcessMetrics(child.Handle());
-#endif  // defined(OS_APPLE)
+      child_launcher.CreateChildProcessMetrics();
 
   const int fd_count = metrics->GetOpenFdCount();
   EXPECT_GE(fd_count, 0);
@@ -714,17 +958,12 @@ TEST(ProcessMetricsTest, GetChildOpenFdCount) {
 
   EXPECT_EQ(fd_count, metrics->GetOpenFdCount());
 
-  ASSERT_TRUE(child.Terminate(0, true));
+  ASSERT_TRUE(child_launcher.TerminateChildProcess());
 }
 
 TEST(ProcessMetricsTest, GetOpenFdCount) {
-  base::ProcessHandle process = base::GetCurrentProcessHandle();
-  std::unique_ptr<base::ProcessMetrics> metrics =
-#if defined(OS_APPLE)
-      ProcessMetrics::CreateProcessMetrics(process, nullptr);
-#else
-      ProcessMetrics::CreateProcessMetrics(process);
-#endif  // defined(OS_APPLE)
+  std::unique_ptr<ProcessMetrics> metrics =
+      ProcessMetrics::CreateCurrentProcessMetrics();
 
   ScopedTempDir temp_dir;
   ASSERT_TRUE(temp_dir.CreateUniqueTempDir());
@@ -737,15 +976,13 @@ TEST(ProcessMetricsTest, GetOpenFdCount) {
   EXPECT_GT(new_fd_count, 0);
   EXPECT_EQ(new_fd_count, fd_count + 1);
 }
+#endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_APPLE)
 
-#endif  // defined(OS_LINUX) || defined(OS_CHROMEOS) || defined(OS_MAC)
-
-#if defined(OS_ANDROID) || defined(OS_LINUX) || defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
 
 TEST(ProcessMetricsTestLinux, GetPageFaultCounts) {
-  std::unique_ptr<base::ProcessMetrics> process_metrics(
-      base::ProcessMetrics::CreateProcessMetrics(
-          base::GetCurrentProcessHandle()));
+  std::unique_ptr<ProcessMetrics> process_metrics =
+      ProcessMetrics::CreateCurrentProcessMetrics();
 
   PageFaultCounts counts;
   ASSERT_TRUE(process_metrics->GetPageFaultCounts(&counts));
@@ -774,9 +1011,8 @@ TEST(ProcessMetricsTestLinux, GetPageFaultCounts) {
 }
 
 TEST(ProcessMetricsTestLinux, GetCumulativeCPUUsagePerThread) {
-  ProcessHandle handle = GetCurrentProcessHandle();
-  std::unique_ptr<ProcessMetrics> metrics(
-      ProcessMetrics::CreateProcessMetrics(handle));
+  std::unique_ptr<ProcessMetrics> metrics =
+      ProcessMetrics::CreateCurrentProcessMetrics();
 
   Thread thread1("thread1");
   thread1.StartAndWaitForTesting();
@@ -827,64 +1063,12 @@ TEST(ProcessMetricsTestLinux, GetCumulativeCPUUsagePerThread) {
           return entry.first == prev_entry.first;
         });
 
-    if (prev_it != prev_thread_times.end())
+    if (prev_it != prev_thread_times.end()) {
       EXPECT_GE(entry.second, prev_it->second);
+    }
   }
 }
+#endif  // BUILDFLAG(IS_ANDROID) || BUILDFLAG(IS_LINUX) ||
+        // BUILDFLAG(IS_CHROMEOS)
 
-TEST(ProcessMetricsTestLinux, GetPerThreadCumulativeCPUTimeInState) {
-  ProcessHandle handle = GetCurrentProcessHandle();
-  std::unique_ptr<ProcessMetrics> metrics(
-      ProcessMetrics::CreateProcessMetrics(handle));
-
-  // Only some test systems will support GetPerThreadCumulativeCPUTimeInState,
-  // as it relies on /proc/pid/task/tid/time_in_state support in the kernel. In
-  // Android, this is only supported in newer kernels with a patch such as this:
-  // https://android-review.googlesource.com/c/kernel/common/+/610460/.
-  bool expect_success;
-  std::string contents;
-  {
-    FilePath time_in_state_path = FilePath("/proc")
-                                      .Append(NumberToString(handle))
-                                      .Append("task")
-                                      .Append(NumberToString(handle))
-                                      .Append("time_in_state");
-    expect_success = ReadFileToString(time_in_state_path, &contents) &&
-                     StartsWith(contents, "cpu", CompareCase::SENSITIVE);
-  }
-
-  ProcessMetrics::TimeInStatePerThread prev_thread_times;
-  EXPECT_EQ(metrics->GetPerThreadCumulativeCPUTimeInState(prev_thread_times),
-            expect_success)
-      << "time_in_state example contents: \n"
-      << contents;
-
-  // Only non-zero entries are reported.
-  for (const auto& entry : prev_thread_times) {
-    EXPECT_NE(entry.thread_id, 0);
-    EXPECT_GT(entry.cumulative_cpu_time, base::TimeDelta());
-  }
-
-  ProcessMetrics::TimeInStatePerThread current_thread_times;
-  EXPECT_EQ(metrics->GetPerThreadCumulativeCPUTimeInState(current_thread_times),
-            expect_success);
-
-  // Reported times should not decrease.
-  for (const auto& entry : current_thread_times) {
-    auto prev_it = ranges::find_if(
-        prev_thread_times,
-        [&entry](const ProcessMetrics::ThreadTimeInState& prev_entry) {
-          return entry.thread_id == prev_entry.thread_id &&
-                 entry.core_type == prev_entry.core_type &&
-                 entry.core_frequency_khz == prev_entry.core_frequency_khz;
-        });
-
-    if (prev_it != prev_thread_times.end())
-      EXPECT_GE(entry.cumulative_cpu_time, prev_it->cumulative_cpu_time);
-  }
-}
-
-#endif  // defined(OS_ANDROID) || defined(OS_LINUX) || defined(OS_CHROMEOS)
-
-}  // namespace debug
-}  // namespace base
+}  // namespace base::debug
