@@ -1,4 +1,4 @@
-// Copyright 2016 The Chromium Authors. All rights reserved.
+// Copyright 2016 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,10 +11,10 @@
 #include <vector>
 
 #include "base/barrier_closure.h"
-#include "base/bind.h"
-#include "base/callback.h"
-#include "base/callback_helpers.h"
 #include "base/check_op.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
+#include "base/functional/callback_helpers.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/raw_ptr.h"
 #include "base/memory/ref_counted.h"
@@ -36,10 +36,8 @@
 #include "base/test/test_waitable_event.h"
 #include "base/threading/platform_thread.h"
 #include "base/threading/scoped_blocking_call.h"
-#include "base/threading/sequenced_task_runner_handle.h"
 #include "base/threading/simple_thread.h"
 #include "base/threading/thread_restrictions.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -79,7 +77,7 @@ class ThreadPostingAndRunningTask : public SimpleThread {
     WILL_POST_AND_RUN,
   };
 
-  // |action| must be either WILL_POST of WILL_POST_AND_RUN.
+  // |action| must be either WILL_POST or WILL_POST_AND_RUN.
   // |task| will be pushed to |sequence| and |sequence| will be registered. If
   // |action| is WILL_POST_AND_RUN, a task from |sequence| will run.
   ThreadPostingAndRunningTask(TaskTracker* tracker,
@@ -122,7 +120,11 @@ class ThreadPostingAndRunningTask : public SimpleThread {
 
       post_and_queue_succeeded =
           tracker_->WillPostTask(&task_, sequence_->shutdown_behavior());
-      sequence_->BeginTransaction().PushTask(std::move(task_));
+      {
+        auto transaction = sequence_->BeginTransaction();
+        transaction.WillPushImmediateTask();
+        transaction.PushImmediateTask(std::move(task_));
+      }
       task_source_ = tracker_->RegisterTaskSource(std::move(sequence_));
 
       post_and_queue_succeeded &= !!task_source_;
@@ -333,6 +335,49 @@ TEST_P(ThreadPoolTaskTrackerTest, WillPostAndRunLongTaskBeforeShutdown) {
     WAIT_FOR_ASYNC_SHUTDOWN_COMPLETED();
 }
 
+// Posting a BLOCK_SHUTDOWN task after shutdown must be allowed from a
+// CONTINUE_ON_SHUTDOWN. Ref. https://crbug.com/1499644#c9 - #c16.
+// Note: This test can't be TEST_P as non-CONTINUE_ON_SHUTDOWN `poster` would
+// hang in CompleteShutdown().
+TEST_F(ThreadPoolTaskTrackerTest, PostAfterShutdownFromContinueOnShutdown) {
+  // Dummy.
+  Task task{CreateTask()};
+
+  // Create a task that verifies the properties of this test.
+  TestWaitableEvent task_running;
+  TestWaitableEvent task_barrier;
+  Task poster(FROM_HERE, BindLambdaForTesting([&]() {
+                task_running.Signal();
+                task_barrier.Wait();
+
+                // No death when posting BLOCK_SHUTDOWN from
+                // CONTINUE_ON_SHUTDOWN.
+                EXPECT_TRUE(tracker_.IsShutdownComplete());
+                EXPECT_FALSE(tracker_.WillPostTask(
+                    &task, TaskShutdownBehavior::BLOCK_SHUTDOWN));
+              }),
+              TimeTicks::Now(), TimeDelta());
+
+  // Inform |task_tracker_| that |blocked_task| will be posted.
+  auto sequence = WillPostTaskAndQueueTaskSource(
+      std::move(poster), TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN);
+  EXPECT_TRUE(sequence);
+
+  // Create a thread to run the task. Wait until the task starts running.
+  ThreadPostingAndRunningTask thread_running_task(&tracker_,
+                                                  std::move(sequence));
+  thread_running_task.Start();
+  task_running.Wait();
+
+  // Fully shutdown `tracker_` Make sure it's complete before releasing the task
+  // to perform its test for CONTINUE_ON_SHUTDOWN.
+  test::ShutdownTaskTracker(&tracker_);
+
+  // Unblock the task and wait for it to perform its test.
+  task_barrier.Signal();
+  thread_running_task.Join();
+}
+
 // Verify that an undelayed task whose sequence wasn't queued does not block
 // shutdown, regardless of its shutdown behavior.
 TEST_P(ThreadPoolTaskTrackerTest, WillPostBeforeShutdownQueueDuringShutdown) {
@@ -470,6 +515,15 @@ TEST_P(ThreadPoolTaskTrackerTest, WillPostAfterShutdown) {
 
   // |task_tracker_| shouldn't allow a task to be posted after shutdown.
   if (GetParam() == TaskShutdownBehavior::BLOCK_SHUTDOWN) {
+    // When the task tracker is allowed to fizzle block shutdown tasks,
+    // WillPostTask will return false and leak the task.
+    tracker_.BeginFizzlingBlockShutdownTasks();
+    EXPECT_FALSE(tracker_.WillPostTask(&task, GetParam()));
+    tracker_.EndFizzlingBlockShutdownTasks();
+
+    // If a BLOCK_SHUTDOWN task is posted after shutdown without explicitly
+    // allowing BLOCK_SHUTDOWN task fizzling, WillPostTask DCHECKs to find
+    // ordering bugs.
     EXPECT_DCHECK_DEATH(tracker_.WillPostTask(&task, GetParam()));
   } else {
     EXPECT_FALSE(tracker_.WillPostTask(&task, GetParam()));
@@ -525,90 +579,95 @@ TEST_P(ThreadPoolTaskTrackerTest, IOAllowed) {
   RunAndPopNextTask(std::move(sequence_without_may_block));
 }
 
-static void RunTaskRunnerHandleVerificationTask(
+static void RunTaskRunnerCurrentDefaultHandleVerificationTask(
     TaskTracker* tracker,
     Task verify_task,
     TaskTraits traits,
-    scoped_refptr<TaskRunner> task_runner,
+    scoped_refptr<SequencedTaskRunner> task_runner,
     TaskSourceExecutionMode execution_mode) {
   // Pretend |verify_task| is posted to respect TaskTracker's contract.
   EXPECT_TRUE(tracker->WillPostTask(&verify_task, traits.shutdown_behavior()));
 
-  // Confirm that the test conditions are right (no TaskRunnerHandles set
-  // already).
-  EXPECT_FALSE(ThreadTaskRunnerHandle::IsSet());
-  EXPECT_FALSE(SequencedTaskRunnerHandle::IsSet());
+  // Confirm that the test conditions are right (no
+  // task runner CurrentDefaultHandles set already).
+  EXPECT_FALSE(SingleThreadTaskRunner::HasCurrentDefault());
+  EXPECT_FALSE(SequencedTaskRunner::HasCurrentDefault());
 
   test::QueueAndRunTaskSource(
       tracker,
       test::CreateSequenceWithTask(std::move(verify_task), traits,
                                    std::move(task_runner), execution_mode));
 
-  // TaskRunnerHandle state is reset outside of task's scope.
-  EXPECT_FALSE(ThreadTaskRunnerHandle::IsSet());
-  EXPECT_FALSE(SequencedTaskRunnerHandle::IsSet());
+  // task runner CurrentDefaultHandle state is reset outside of task's scope.
+  EXPECT_FALSE(SingleThreadTaskRunner::HasCurrentDefault());
+  EXPECT_FALSE(SequencedTaskRunner::HasCurrentDefault());
 }
 
-static void VerifyNoTaskRunnerHandle() {
-  EXPECT_FALSE(ThreadTaskRunnerHandle::IsSet());
-  EXPECT_FALSE(SequencedTaskRunnerHandle::IsSet());
+static void VerifyNoTaskRunnerCurrentDefaultHandle() {
+  EXPECT_FALSE(SingleThreadTaskRunner::HasCurrentDefault());
+  EXPECT_FALSE(SequencedTaskRunner::HasCurrentDefault());
 }
 
 TEST_P(ThreadPoolTaskTrackerTest, TaskRunnerHandleIsNotSetOnParallel) {
   // Create a task that will verify that TaskRunnerHandles are not set in its
   // scope per no TaskRunner ref being set to it.
-  Task verify_task(FROM_HERE, BindOnce(&VerifyNoTaskRunnerHandle),
+  Task verify_task(FROM_HERE, BindOnce(&VerifyNoTaskRunnerCurrentDefaultHandle),
                    TimeTicks::Now(), TimeDelta());
 
-  RunTaskRunnerHandleVerificationTask(&tracker_, std::move(verify_task),
-                                      TaskTraits(GetParam()), nullptr,
-                                      TaskSourceExecutionMode::kParallel);
+  RunTaskRunnerCurrentDefaultHandleVerificationTask(
+      &tracker_, std::move(verify_task), TaskTraits(GetParam()), nullptr,
+      TaskSourceExecutionMode::kParallel);
 }
 
-static void VerifySequencedTaskRunnerHandle(
+static void VerifySequencedTaskRunnerCurrentDefaultHandle(
     const SequencedTaskRunner* expected_task_runner) {
-  EXPECT_FALSE(ThreadTaskRunnerHandle::IsSet());
-  EXPECT_TRUE(SequencedTaskRunnerHandle::IsSet());
-  EXPECT_EQ(expected_task_runner, SequencedTaskRunnerHandle::Get());
+  EXPECT_FALSE(SingleThreadTaskRunner::HasCurrentDefault());
+  EXPECT_TRUE(SequencedTaskRunner::HasCurrentDefault());
+  EXPECT_EQ(expected_task_runner, SequencedTaskRunner::GetCurrentDefault());
 }
 
-TEST_P(ThreadPoolTaskTrackerTest, SequencedTaskRunnerHandleIsSetOnSequenced) {
+TEST_P(ThreadPoolTaskTrackerTest,
+       SequencedTaskRunnerHasCurrentDefaultOnSequenced) {
   scoped_refptr<SequencedTaskRunner> test_task_runner(new TestSimpleTaskRunner);
 
-  // Create a task that will verify that SequencedTaskRunnerHandle is properly
-  // set to |test_task_runner| in its scope per |sequenced_task_runner_ref|
-  // being set to it.
+  // Create a task that will verify that
+  // SequencedTaskRunner::CurrentDefaultHandle is properly set to
+  // |test_task_runner| in its scope per |sequenced_task_runner_ref| being set
+  // to it.
   Task verify_task(FROM_HERE,
-                   BindOnce(&VerifySequencedTaskRunnerHandle,
+                   BindOnce(&VerifySequencedTaskRunnerCurrentDefaultHandle,
                             Unretained(test_task_runner.get())),
                    TimeTicks::Now(), TimeDelta());
 
-  RunTaskRunnerHandleVerificationTask(
+  RunTaskRunnerCurrentDefaultHandleVerificationTask(
       &tracker_, std::move(verify_task), TaskTraits(GetParam()),
       std::move(test_task_runner), TaskSourceExecutionMode::kSequenced);
 }
 
-static void VerifyThreadTaskRunnerHandle(
+static void VerifySingleThreadTaskRunnerCurrentDefaultHandle(
     const SingleThreadTaskRunner* expected_task_runner) {
-  EXPECT_TRUE(ThreadTaskRunnerHandle::IsSet());
-  // SequencedTaskRunnerHandle inherits ThreadTaskRunnerHandle for thread.
-  EXPECT_TRUE(SequencedTaskRunnerHandle::IsSet());
-  EXPECT_EQ(expected_task_runner, ThreadTaskRunnerHandle::Get());
+  EXPECT_TRUE(SingleThreadTaskRunner::HasCurrentDefault());
+  // SequencedTaskRunner::CurrentDefaultHandle inherits
+  // SingleThreadTaskRunner::CurrentDefaultHandle for thread.
+  EXPECT_TRUE(SequencedTaskRunner::HasCurrentDefault());
+  EXPECT_EQ(expected_task_runner, SingleThreadTaskRunner::GetCurrentDefault());
 }
 
-TEST_P(ThreadPoolTaskTrackerTest, ThreadTaskRunnerHandleIsSetOnSingleThreaded) {
+TEST_P(ThreadPoolTaskTrackerTest,
+       SingleThreadTaskRunnerCurrentDefaultHandleIsSetOnSingleThreaded) {
   scoped_refptr<SingleThreadTaskRunner> test_task_runner(
       new TestSimpleTaskRunner);
 
-  // Create a task that will verify that ThreadTaskRunnerHandle is properly set
-  // to |test_task_runner| in its scope per |single_thread_task_runner_ref|
-  // being set on it.
+  // Create a task that will verify that
+  // SingleThreadTaskRunner::CurrentDefaultHandle is properly set to
+  // |test_task_runner| in its scope per |single_thread_task_runner_ref| being
+  // set on it.
   Task verify_task(FROM_HERE,
-                   BindOnce(&VerifyThreadTaskRunnerHandle,
+                   BindOnce(&VerifySingleThreadTaskRunnerCurrentDefaultHandle,
                             Unretained(test_task_runner.get())),
                    TimeTicks::Now(), TimeDelta());
 
-  RunTaskRunnerHandleVerificationTask(
+  RunTaskRunnerCurrentDefaultHandleVerificationTask(
       &tracker_, std::move(verify_task), TaskTraits(GetParam()),
       std::move(test_task_runner), TaskSourceExecutionMode::kSingleThread);
 }
@@ -958,13 +1017,14 @@ TEST_F(ThreadPoolTaskTrackerTest, CurrentSequenceToken) {
 
   {
     Sequence::Transaction sequence_transaction(sequence->BeginTransaction());
-    sequence_transaction.PushTask(std::move(task));
+    sequence_transaction.WillPushImmediateTask();
+    sequence_transaction.PushImmediateTask(std::move(task));
 
-    EXPECT_FALSE(SequenceToken::GetForCurrentThread().IsValid());
+    EXPECT_NE(SequenceToken::GetForCurrentThread(), sequence_token);
   }
 
   test::QueueAndRunTaskSource(&tracker_, std::move(sequence));
-  EXPECT_FALSE(SequenceToken::GetForCurrentThread().IsValid());
+  EXPECT_NE(SequenceToken::GetForCurrentThread(), sequence_token);
 }
 
 TEST_F(ThreadPoolTaskTrackerTest, LoadWillPostAndRunBeforeShutdown) {
@@ -1158,7 +1218,11 @@ TEST_F(ThreadPoolTaskTrackerTest,
 
   scoped_refptr<Sequence> sequence =
       test::CreateSequenceWithTask(std::move(task_1), default_traits);
-  sequence->BeginTransaction().PushTask(std::move(task_2));
+  {
+    auto transaction = sequence->BeginTransaction();
+    transaction.WillPushImmediateTask();
+    transaction.PushImmediateTask(std::move(task_2));
+  }
   EXPECT_EQ(sequence,
             test::QueueAndRunTaskSource(&tracker_, sequence).Unregister());
 }
@@ -1224,7 +1288,7 @@ TEST(ThreadPoolTaskTrackerWaitAllowedTest, WaitAllowed) {
   // Run the test on the separate thread since it is not possible to reset the
   // "wait allowed" bit of a thread without being a friend of
   // ThreadRestrictions.
-  testing::GTEST_FLAG(death_test_style) = "threadsafe";
+  GTEST_FLAG_SET(death_test_style, "threadsafe");
   WaitAllowedTestThread wait_allowed_test_thread;
   wait_allowed_test_thread.Start();
   wait_allowed_test_thread.Join();
