@@ -1,4 +1,4 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,9 +7,9 @@
 #include <vector>
 
 #include "base/barrier_closure.h"
-#include "base/bind.h"
-#include "base/callback.h"
-#include "base/callback_helpers.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
+#include "base/functional/callback_helpers.h"
 #include "base/memory/raw_ptr.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/test/bind.h"
@@ -119,7 +119,7 @@ class ThreadLocalStoragePerfTest : public testing::Test {
                   base::BindLambdaForTesting([&]() {
                     volatile intptr_t total = 0;
                     for (size_t i = 0; i < num_operation; ++i)
-                      total += read();
+                      total = total + read();
                   }),
                   num_operation, num_threads);
 
