@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,141 +17,12 @@ namespace base {
 
 namespace {
 
-class ThreadLocalTesterBase : public DelegateSimpleThreadPool::Delegate {
- public:
-  typedef ThreadLocalPointer<char> TLPType;
-
-  ThreadLocalTesterBase(TLPType* tlp, WaitableEvent* done)
-      : tlp_(tlp), done_(done) {}
-  ~ThreadLocalTesterBase() override = default;
-
- protected:
-  raw_ptr<TLPType> tlp_;
-  raw_ptr<WaitableEvent> done_;
-};
-
-class SetThreadLocal : public ThreadLocalTesterBase {
- public:
-  SetThreadLocal(TLPType* tlp, WaitableEvent* done)
-      : ThreadLocalTesterBase(tlp, done), val_(nullptr) {}
-  ~SetThreadLocal() override = default;
-
-  void set_value(char* val) { val_ = val; }
-
-  void Run() override {
-    DCHECK(!done_->IsSignaled());
-    tlp_->Set(val_.get());
-    done_->Signal();
-  }
-
- private:
-  raw_ptr<char> val_;
-};
-
-class GetThreadLocal : public ThreadLocalTesterBase {
- public:
-  GetThreadLocal(TLPType* tlp, WaitableEvent* done)
-      : ThreadLocalTesterBase(tlp, done), ptr_(nullptr) {}
-  ~GetThreadLocal() override = default;
-
-  void set_ptr(char** ptr) { ptr_ = ptr; }
-
-  void Run() override {
-    DCHECK(!done_->IsSignaled());
-    *ptr_ = tlp_->Get();
-    done_->Signal();
-  }
-
- private:
-  raw_ptr<char*> ptr_;
-};
-
-}  // namespace
-
-// In this test, we start 2 threads which will access a ThreadLocalPointer.  We
-// make sure the default is NULL, and the pointers are unique to the threads.
-TEST(ThreadLocalTest, Pointer) {
-  DelegateSimpleThreadPool tp1("ThreadLocalTest tp1", 1);
-  DelegateSimpleThreadPool tp2("ThreadLocalTest tp1", 1);
-  tp1.Start();
-  tp2.Start();
-
-  ThreadLocalPointer<char> tlp;
-
-  static char* const kBogusPointer = reinterpret_cast<char*>(0x1234);
-
-  char* tls_val;
-  WaitableEvent done(WaitableEvent::ResetPolicy::MANUAL,
-                     WaitableEvent::InitialState::NOT_SIGNALED);
-
-  GetThreadLocal getter(&tlp, &done);
-  getter.set_ptr(&tls_val);
-
-  // Check that both threads defaulted to NULL.
-  tls_val = kBogusPointer;
-  done.Reset();
-  tp1.AddWork(&getter);
-  done.Wait();
-  EXPECT_EQ(static_cast<char*>(nullptr), tls_val);
-
-  tls_val = kBogusPointer;
-  done.Reset();
-  tp2.AddWork(&getter);
-  done.Wait();
-  EXPECT_EQ(static_cast<char*>(nullptr), tls_val);
-
-  SetThreadLocal setter(&tlp, &done);
-  setter.set_value(kBogusPointer);
-
-  // Have thread 1 set their pointer value to kBogusPointer.
-  done.Reset();
-  tp1.AddWork(&setter);
-  done.Wait();
-
-  tls_val = nullptr;
-  done.Reset();
-  tp1.AddWork(&getter);
-  done.Wait();
-  EXPECT_EQ(kBogusPointer, tls_val);
-
-  // Make sure thread 2 is still NULL
-  tls_val = kBogusPointer;
-  done.Reset();
-  tp2.AddWork(&getter);
-  done.Wait();
-  EXPECT_EQ(static_cast<char*>(nullptr), tls_val);
-
-  // Set thread 2 to kBogusPointer + 1.
-  setter.set_value(kBogusPointer + 1);
-
-  done.Reset();
-  tp2.AddWork(&setter);
-  done.Wait();
-
-  tls_val = nullptr;
-  done.Reset();
-  tp2.AddWork(&getter);
-  done.Wait();
-  EXPECT_EQ(kBogusPointer + 1, tls_val);
-
-  // Make sure thread 1 is still kBogusPointer.
-  tls_val = nullptr;
-  done.Reset();
-  tp1.AddWork(&getter);
-  done.Wait();
-  EXPECT_EQ(kBogusPointer, tls_val);
-
-  tp1.JoinAll();
-  tp2.JoinAll();
-}
-
-namespace {
-
 // A simple helper which sets the given boolean to true on destruction.
 class SetTrueOnDestruction {
  public:
-  SetTrueOnDestruction(bool* was_destroyed) : was_destroyed_(was_destroyed) {
-    CHECK(was_destroyed != nullptr);
+  explicit SetTrueOnDestruction(bool* was_destroyed)
+      : was_destroyed_(was_destroyed) {
+    CHECK_NE(was_destroyed, nullptr);
   }
 
   SetTrueOnDestruction(const SetTrueOnDestruction&) = delete;
@@ -255,7 +126,7 @@ TEST(ThreadLocalTest, ThreadLocalOwnedPointerCleansUpMainThreadOnDestruction) {
 }
 
 TEST(ThreadLocalTest, ThreadLocalOwnedPointerDeathIfDestroyedWithActiveThread) {
-  testing::FLAGS_gtest_death_test_style = "threadsafe";
+  GTEST_FLAG_SET(death_test_style, "threadsafe");
 
   absl::optional<ThreadLocalOwnedPointer<int>> tls_owned_pointer(
       absl::in_place);
@@ -332,23 +203,4 @@ TEST(ThreadLocalTest, ThreadLocalOwnedPointerMultiThreadedAndStaticStorage) {
   EXPECT_FALSE(main_thread_was_destroyed);
 }
 
-TEST(ThreadLocalTest, Boolean) {
-  {
-    ThreadLocalBoolean tlb;
-    EXPECT_FALSE(tlb.Get());
-
-    tlb.Set(false);
-    EXPECT_FALSE(tlb.Get());
-
-    tlb.Set(true);
-    EXPECT_TRUE(tlb.Get());
-  }
-
-  // Our slot should have been freed, we're all reset.
-  {
-    ThreadLocalBoolean tlb;
-    EXPECT_FALSE(tlb.Get());
-  }
-}
-
 }  // namespace base
