@@ -1,4 +1,4 @@
-// Copyright 2020 The Chromium Authors. All rights reserved.
+// Copyright 2020 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,17 +6,58 @@
 
 #include <vector>
 
+#include "base/memory/raw_ptr.h"
 #include "base/synchronization/lock.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/time/time.h"
+#include "build/chromeos_buildflags.h"
 #include "cc/animation/animation_host.h"
+#include "cc/metrics/custom_metrics_recorder.h"
 #include "cc/test/fake_content_layer_client.h"
+#include "cc/test/fake_frame_info.h"
 #include "cc/test/fake_picture_layer.h"
 #include "cc/test/layer_tree_test.h"
 
 namespace cc {
 namespace {
 
+using SmoothnessStrategy = DroppedFrameCounter::SmoothnessStrategy;
+
+FrameInfo CreateStubFrameInfo(bool is_dropped) {
+  return CreateFakeFrameInfo(is_dropped
+                                 ? FrameInfo::FrameFinalState::kDropped
+                                 : FrameInfo::FrameFinalState::kPresentedAll);
+}
+
+class TestCustomMetricsRecorder : public CustomMetricRecorder {
+ public:
+  TestCustomMetricsRecorder() = default;
+  ~TestCustomMetricsRecorder() override = default;
+
+  // CustomMetricRecorder:
+  void ReportPercentDroppedFramesInOneSecondWindow2(double percent) override {
+    ++report_count_;
+    last_percent_dropped_frames_ = percent;
+  }
+  void ReportEventLatency(
+      std::vector<EventLatencyTracker::LatencyData> latencies) override {}
+
+  void Reset() {
+    report_count_ = 0u;
+    last_percent_dropped_frames_ = 0;
+  }
+
+  int report_count() const { return report_count_; }
+
+  double last_percent_dropped_frames() const {
+    return last_percent_dropped_frames_;
+  }
+
+ private:
+  int report_count_ = 0u;
+  double last_percent_dropped_frames_ = 0;
+};
+
 class DroppedFrameCounterTestBase : public LayerTreeTest {
  public:
   DroppedFrameCounterTestBase() = default;
@@ -188,7 +229,7 @@ class DroppedFrameCounterTestBase : public LayerTreeTest {
   // The |wait_| event is used when the test wants to deliberately force the
   // main-thread to block while processing begin-main-frames.
   base::Lock wait_lock_;
-  base::WaitableEvent* wait_ = nullptr;
+  raw_ptr<base::WaitableEvent> wait_ = nullptr;
 
   // These fields are populated in the compositor thread when the desired number
   // of frames have been processed. These fields are subsequently compared
@@ -255,7 +296,9 @@ class DroppedFrameCounterMainDropsSmoothnessTest
 
 class DroppedFrameCounterTest : public testing::Test {
  public:
-  DroppedFrameCounterTest() {
+  explicit DroppedFrameCounterTest(SmoothnessStrategy smoothness_strategy =
+                                       SmoothnessStrategy::kDefaultStrategy)
+      : smoothness_strategy_(smoothness_strategy) {
     dropped_frame_counter_.set_total_counter(&total_frame_counter_);
     dropped_frame_counter_.OnFcpReceived();
   }
@@ -266,21 +309,54 @@ class DroppedFrameCounterTest : public testing::Test {
     for (int i = 0; i < repeat; i++) {
       for (auto is_dropped : frame_states) {
         viz::BeginFrameArgs args_ = SimulateBeginFrameArgs();
-        dropped_frame_counter_.OnBeginFrame(args_, /*is_scroll_active=*/false);
-        dropped_frame_counter_.OnEndFrame(args_, is_dropped);
+        dropped_frame_counter_.OnBeginFrame(args_);
+        dropped_frame_counter_.OnEndFrame(args_,
+                                          CreateStubFrameInfo(is_dropped));
         sequence_number_++;
         frame_time_ += interval_;
       }
     }
   }
 
-  void SimulatePendingFrame(int repeat) {
+  // Make a sequence of frame states where the first |dropped_frames| out of
+  // |total_frames| are dropped.
+  std::vector<bool> MakeFrameSequence(int dropped_frames, int total_frames) {
+    std::vector<bool> frame_states(total_frames, false);
+    for (int i = 0; i < dropped_frames; i++) {
+      frame_states[i] = true;
+    }
+    return frame_states;
+  }
+
+  std::vector<viz::BeginFrameArgs> SimulatePendingFrame(int repeat) {
+    std::vector<viz::BeginFrameArgs> args(repeat);
     for (int i = 0; i < repeat; i++) {
-      viz::BeginFrameArgs args_ = SimulateBeginFrameArgs();
-      dropped_frame_counter_.OnBeginFrame(args_, /*is_scroll_active=*/false);
+      args[i] = SimulateBeginFrameArgs();
+      dropped_frame_counter_.OnBeginFrame(args[i]);
       sequence_number_++;
       frame_time_ += interval_;
     }
+    return args;
+  }
+
+  // Simulate a main and impl thread update on the same frame.
+  void SimulateForkedFrame(bool main_dropped, bool impl_dropped) {
+    viz::BeginFrameArgs args_ = SimulateBeginFrameArgs();
+    dropped_frame_counter_.OnBeginFrame(args_);
+    dropped_frame_counter_.OnBeginFrame(args_);
+
+    // End the 'main thread' arm of the fork.
+    auto main_info = CreateStubFrameInfo(main_dropped);
+    main_info.main_thread_response = FrameInfo::MainThreadResponse::kIncluded;
+    dropped_frame_counter_.OnEndFrame(args_, main_info);
+
+    // End the 'compositor thread' arm of the fork.
+    auto impl_info = CreateStubFrameInfo(impl_dropped);
+    impl_info.main_thread_response = FrameInfo::MainThreadResponse::kMissing;
+    dropped_frame_counter_.OnEndFrame(args_, impl_info);
+
+    sequence_number_++;
+    frame_time_ += interval_;
   }
 
   void AdvancetimeByIntervals(int interval_count) {
@@ -291,18 +367,79 @@ class DroppedFrameCounterTest : public testing::Test {
     return dropped_frame_counter_.sliding_window_max_percent_dropped();
   }
 
-  double PercentDroppedFrame95Percentile() {
-    return dropped_frame_counter_.SlidingWindow95PercentilePercentDropped();
+  double MaxPercentDroppedFrameAfter1Sec() {
+    auto percent_dropped =
+        dropped_frame_counter_.max_percent_dropped_After_1_sec();
+    EXPECT_TRUE(percent_dropped.has_value());
+    return percent_dropped.value();
   }
 
-  double GetTotalFramesInWindow() {
-    return base::TimeDelta::FromSeconds(1) / interval_;
+  double MaxPercentDroppedFrameAfter2Sec() {
+    auto percent_dropped =
+        dropped_frame_counter_.max_percent_dropped_After_2_sec();
+    EXPECT_TRUE(percent_dropped.has_value());
+    return percent_dropped.value();
   }
 
+  double MaxPercentDroppedFrameAfter5Sec() {
+    auto percent_dropped =
+        dropped_frame_counter_.max_percent_dropped_After_5_sec();
+    EXPECT_TRUE(percent_dropped.has_value());
+    return percent_dropped.value();
+  }
+
+  double PercentDroppedFrame95Percentile() {
+    return dropped_frame_counter_.SlidingWindow95PercentilePercentDropped(
+        smoothness_strategy_);
+  }
+
+  double PercentDroppedFrameMedian() {
+    return dropped_frame_counter_.SlidingWindowMedianPercentDropped(
+        smoothness_strategy_);
+  }
+
+  double PercentDroppedFrameVariance() {
+    return dropped_frame_counter_.SlidingWindowPercentDroppedVariance(
+        smoothness_strategy_);
+  }
+
+  const DroppedFrameCounter::SlidingWindowHistogram*
+  GetSlidingWindowHistogram() {
+    return dropped_frame_counter_.GetSlidingWindowHistogram(
+        smoothness_strategy_);
+  }
+
+  double GetTotalFramesInWindow() { return base::Seconds(1) / interval_; }
+
   void SetInterval(base::TimeDelta interval) { interval_ = interval; }
 
   base::TimeTicks GetNextFrameTime() const { return frame_time_ + interval_; }
 
+  // Wrap calls with EXPECT_TRUE. Logs the buckets and returns false if they
+  // don't match (within a given epsilon).
+  bool CheckSmoothnessBuckets(std::vector<double> expected_buckets) {
+    constexpr double epsilon = 0.001;
+    bool buckets_match = true;
+    std::vector<double> buckets =
+        GetSlidingWindowHistogram()->GetPercentDroppedFrameBuckets();
+    if (buckets.size() != expected_buckets.size()) {
+      buckets_match = false;
+    } else {
+      for (size_t i = 0; i < buckets.size(); i++) {
+        if (std::abs(buckets[i] - expected_buckets[i]) > epsilon) {
+          buckets_match = false;
+          break;
+        }
+      }
+    }
+    if (!buckets_match) {
+      LOG(ERROR) << "Smoothness buckets do not match!";
+      LOG(ERROR) << "Expected: " << testing::PrintToString(expected_buckets);
+      LOG(ERROR) << "  Actual: " << testing::PrintToString(buckets);
+    }
+    return buckets_match;
+  }
+
  public:
   DroppedFrameCounter dropped_frame_counter_;
 
@@ -310,10 +447,12 @@ class DroppedFrameCounterTest : public testing::Test {
   TotalFrameCounter total_frame_counter_;
   uint64_t sequence_number_ = 1;
   uint64_t source_id_ = 1;
-  const base::TickClock* tick_clock_ = base::DefaultTickClock::GetInstance();
+  raw_ptr<const base::TickClock> tick_clock_ =
+      base::DefaultTickClock::GetInstance();
   base::TimeTicks frame_time_ = tick_clock_->NowTicks();
-  base::TimeDelta interval_ =
-      base::TimeDelta::FromMicroseconds(16667);  // 16.667 ms
+  base::TimeDelta interval_ = base::Microseconds(16667);  // 16.667 ms
+
+  SmoothnessStrategy smoothness_strategy_;
 
   viz::BeginFrameArgs SimulateBeginFrameArgs() {
     viz::BeginFrameId current_id_(source_id_, sequence_number_);
@@ -325,7 +464,30 @@ class DroppedFrameCounterTest : public testing::Test {
   }
 };
 
-TEST_F(DroppedFrameCounterTest, SimplePattern1) {
+// Test class that supports parameterized tests for each of the different
+// SmoothnessStrategy.
+//
+// TODO(jonross): when we build the other strategies parameterize the
+// expectations.
+class SmoothnessStrategyDroppedFrameCounterTest
+    : public DroppedFrameCounterTest,
+      public testing::WithParamInterface<SmoothnessStrategy> {
+ public:
+  SmoothnessStrategyDroppedFrameCounterTest()
+      : DroppedFrameCounterTest(GetParam()) {}
+  ~SmoothnessStrategyDroppedFrameCounterTest() override = default;
+  SmoothnessStrategyDroppedFrameCounterTest(
+      const SmoothnessStrategyDroppedFrameCounterTest&) = delete;
+  SmoothnessStrategyDroppedFrameCounterTest& operator=(
+      const SmoothnessStrategyDroppedFrameCounterTest&) = delete;
+};
+
+INSTANTIATE_TEST_SUITE_P(
+    DefaultStrategy,
+    SmoothnessStrategyDroppedFrameCounterTest,
+    ::testing::Values(SmoothnessStrategy::kDefaultStrategy));
+
+TEST_P(SmoothnessStrategyDroppedFrameCounterTest, SimplePattern1) {
   // 2 out of every 3 frames are dropped (In total 80 frames out of 120).
   SimulateFrameSequence({true, true, true, false, true, false}, 20);
 
@@ -334,36 +496,46 @@ TEST_F(DroppedFrameCounterTest, SimplePattern1) {
   // Which means a max of 67 dropped frames.
   EXPECT_EQ(std::round(MaxPercentDroppedFrame()), 67);
   EXPECT_EQ(PercentDroppedFrame95Percentile(), 67);  // all values are in the
-  // 67th bucket, and as a result 95th percentile is also 67.
+  // 65th-67th bucket, and as a result 95th percentile is also 67.
+  EXPECT_EQ(PercentDroppedFrameMedian(), 65);
+  EXPECT_LE(PercentDroppedFrameVariance(), 1);
 }
 
-TEST_F(DroppedFrameCounterTest, SimplePattern2) {
+TEST_P(SmoothnessStrategyDroppedFrameCounterTest, SimplePattern2) {
   // 1 out of every 5 frames are dropped (In total 24 frames out of 120).
   SimulateFrameSequence({false, false, false, false, true}, 24);
 
   double expected_percent_dropped_frame = (12 / GetTotalFramesInWindow()) * 100;
   EXPECT_FLOAT_EQ(MaxPercentDroppedFrame(), expected_percent_dropped_frame);
-  EXPECT_EQ(PercentDroppedFrame95Percentile(), 20);  // all values are in the
+  EXPECT_EQ(PercentDroppedFrame95Percentile(),
+            20);  // all values are in the
   // 20th bucket, and as a result 95th percentile is also 20.
+  EXPECT_EQ(PercentDroppedFrameMedian(), 20);
+  EXPECT_LE(PercentDroppedFrameVariance(), 1);
 }
 
-TEST_F(DroppedFrameCounterTest, IncompleteWindow) {
-  // There are only 5 frames submitted and both Max and 95pct should report
-  // zero.
+TEST_P(SmoothnessStrategyDroppedFrameCounterTest, IncompleteWindow) {
+  // There are only 5 frames submitted, so Max, 95pct, median and variance
+  // should report zero.
   SimulateFrameSequence({false, false, false, false, true}, 1);
   EXPECT_EQ(MaxPercentDroppedFrame(), 0.0);
   EXPECT_EQ(PercentDroppedFrame95Percentile(), 0);
+  EXPECT_EQ(PercentDroppedFrameMedian(), 0);
+  EXPECT_LE(PercentDroppedFrameVariance(), 1);
 }
 
-TEST_F(DroppedFrameCounterTest, MaxPercentDroppedChanges) {
+TEST_P(SmoothnessStrategyDroppedFrameCounterTest, MaxPercentDroppedChanges) {
   // First 60 frames have 20% dropped.
   SimulateFrameSequence({false, false, false, false, true}, 12);
 
   double expected_percent_dropped_frame1 =
       (12 / GetTotalFramesInWindow()) * 100;
   EXPECT_EQ(MaxPercentDroppedFrame(), expected_percent_dropped_frame1);
-  EXPECT_FLOAT_EQ(PercentDroppedFrame95Percentile(), 20);  // There is only one
+  EXPECT_FLOAT_EQ(PercentDroppedFrame95Percentile(),
+                  20);  // There is only one
   // element in the histogram and that is 20.
+  EXPECT_EQ(PercentDroppedFrameMedian(), 20);
+  EXPECT_LE(PercentDroppedFrameVariance(), 1);
 
   // 30 new frames are added that have 18 dropped frames.
   // and the 30 frame before that had 6 dropped frames.
@@ -405,25 +577,25 @@ TEST_F(DroppedFrameCounterTest, MaxPercentDroppedWithIdleFrames) {
 }
 
 TEST_F(DroppedFrameCounterTest, NoCrashForIntervalLargerThanWindow) {
-  SetInterval(base::TimeDelta::FromMilliseconds(1000));
+  SetInterval(base::Milliseconds(1000));
   SimulateFrameSequence({false, false}, 1);
 }
 
-TEST_F(DroppedFrameCounterTest, Percentile95WithIdleFrames) {
+TEST_P(SmoothnessStrategyDroppedFrameCounterTest, Percentile95WithIdleFrames) {
   // Test scenario:
   //  . 4s of 20% dropped frames.
   //  . 96s of idle time.
   // The 96%ile dropped-frame metric should be 0.
 
   // Set an interval that rounds up nicely with 1 second.
-  constexpr auto kInterval = base::TimeDelta::FromMilliseconds(10);
-  constexpr size_t kFps = base::TimeDelta::FromSeconds(1) / kInterval;
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
   static_assert(
       kFps % 5 == 0,
       "kFps must be a multiple of 5 because this test depends on it.");
   SetInterval(kInterval);
 
-  const auto* histogram = dropped_frame_counter_.GetSlidingWindowHistogram();
+  const auto* histogram = GetSlidingWindowHistogram();
 
   // First 4 seconds with 20% dropped frames.
   SimulateFrameSequence({false, false, false, false, true}, (kFps / 5) * 4);
@@ -441,7 +613,8 @@ TEST_F(DroppedFrameCounterTest, Percentile95WithIdleFrames) {
   EXPECT_GT(histogram->GetPercentDroppedFramePercentile(0.97), 0u);
 }
 
-TEST_F(DroppedFrameCounterTest, Percentile95WithIdleFramesWhileHidden) {
+TEST_P(SmoothnessStrategyDroppedFrameCounterTest,
+       Percentile95WithIdleFramesWhileHidden) {
   // The test scenario is the same as |Percentile95WithIdleFrames| test:
   //  . 4s of 20% dropped frames.
   //  . 96s of idle time.
@@ -450,14 +623,14 @@ TEST_F(DroppedFrameCounterTest, Percentile95WithIdleFramesWhileHidden) {
   // contribute to the sliding window.
 
   // Set an interval that rounds up nicely with 1 second.
-  constexpr auto kInterval = base::TimeDelta::FromMilliseconds(10);
-  constexpr size_t kFps = base::TimeDelta::FromSeconds(1) / kInterval;
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
   static_assert(
       kFps % 5 == 0,
       "kFps must be a multiple of 5 because this test depends on it.");
   SetInterval(kInterval);
 
-  const auto* histogram = dropped_frame_counter_.GetSlidingWindowHistogram();
+  const auto* histogram = GetSlidingWindowHistogram();
 
   // First 4 seconds with 20% dropped frames.
   SimulateFrameSequence({false, false, false, false, true}, (kFps / 5) * 4);
@@ -474,7 +647,8 @@ TEST_F(DroppedFrameCounterTest, Percentile95WithIdleFramesWhileHidden) {
   EXPECT_EQ(histogram->GetPercentDroppedFramePercentile(0.95), 20u);
 }
 
-TEST_F(DroppedFrameCounterTest, Percentile95WithIdleFramesThenHide) {
+TEST_P(SmoothnessStrategyDroppedFrameCounterTest,
+       Percentile95WithIdleFramesThenHide) {
   // The test scenario is the same as |Percentile95WithIdleFramesWhileHidden|:
   //  . 4s of 20% dropped frames.
   //  . 96s of idle time.
@@ -483,14 +657,14 @@ TEST_F(DroppedFrameCounterTest, Percentile95WithIdleFramesThenHide) {
   // contribute to the sliding window.
 
   // Set an interval that rounds up nicely with 1 second.
-  constexpr auto kInterval = base::TimeDelta::FromMilliseconds(10);
-  constexpr size_t kFps = base::TimeDelta::FromSeconds(1) / kInterval;
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
   static_assert(
       kFps % 5 == 0,
       "kFps must be a multiple of 5 because this test depends on it.");
   SetInterval(kInterval);
 
-  const auto* histogram = dropped_frame_counter_.GetSlidingWindowHistogram();
+  const auto* histogram = GetSlidingWindowHistogram();
 
   // First 4 seconds with 20% dropped frames.
   SimulateFrameSequence({false, false, false, false, true}, (kFps / 5) * 4);
@@ -510,7 +684,7 @@ TEST_F(DroppedFrameCounterTest, Percentile95WithIdleFramesThenHide) {
 
 // Tests that when ResetPendingFrames updates the sliding window, that the max
 // PercentDroppedFrames is also updated accordingly. (https://crbug.com/1225307)
-TEST_F(DroppedFrameCounterTest,
+TEST_P(SmoothnessStrategyDroppedFrameCounterTest,
        ResetPendingFramesUpdatesMaxPercentDroppedFrames) {
   // This tests a scenario where gaps in frame production lead to having
   // leftover frames in the sliding window for calculations of
@@ -524,8 +698,8 @@ TEST_F(DroppedFrameCounterTest,
   // a new GPU Process. (https://crbug.com/1164647)
 
   // Set an interval that rounds up nicely with 1 second.
-  constexpr auto kInterval = base::TimeDelta::FromMilliseconds(10);
-  constexpr size_t kFps = base::TimeDelta::FromSeconds(1) / kInterval;
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
   SetInterval(kInterval);
 
   // One good frame
@@ -543,19 +717,20 @@ TEST_F(DroppedFrameCounterTest,
 
   // Advance 1s so that we will attempt to update the window when resetting the
   // pending frames. The pending dropped frame above should be calculated here,
-  // and both the max and 95th percentile should be updated.
+  // and the max percentile should be updated.
   AdvancetimeByIntervals(kFps);
   dropped_frame_counter_.ResetPendingFrames(GetNextFrameTime());
-
-  EXPECT_EQ(dropped_frame_counter_.sliding_window_max_percent_dropped(),
-            dropped_frame_counter_.SlidingWindow95PercentilePercentDropped());
   EXPECT_GT(dropped_frame_counter_.sliding_window_max_percent_dropped(), 0u);
+
+  // There should be enough sliding windows reported with 0 dropped frames that
+  // the 95th percentile stays at 0.
+  EXPECT_EQ(PercentDroppedFrame95Percentile(), 0u);
 }
 
 TEST_F(DroppedFrameCounterTest, ResetPendingFramesAccountingForPendingFrames) {
   // Set an interval that rounds up nicely with 1 second.
-  constexpr auto kInterval = base::TimeDelta::FromMilliseconds(10);
-  constexpr size_t kFps = base::TimeDelta::FromSeconds(1) / kInterval;
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
   SetInterval(kInterval);
 
   // First 2 seconds with 20% dropped frames.
@@ -578,8 +753,8 @@ TEST_F(DroppedFrameCounterTest, ResetPendingFramesAccountingForPendingFrames) {
 
 TEST_F(DroppedFrameCounterTest, Reset) {
   // Set an interval that rounds up nicely with 1 second.
-  constexpr auto kInterval = base::TimeDelta::FromMilliseconds(10);
-  constexpr size_t kFps = base::TimeDelta::FromSeconds(1) / kInterval;
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
   SetInterval(kInterval);
 
   // First 2 seconds with 20% dropped frames.
@@ -600,5 +775,205 @@ TEST_F(DroppedFrameCounterTest, Reset) {
   EXPECT_EQ(MaxPercentDroppedFrame(), 0u);
 }
 
+TEST_F(DroppedFrameCounterTest, ConsistentSmoothnessRatings) {
+  // Set an interval that rounds up nicely with 1 second.
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
+  static_assert(kFps == 100,
+                "kFps must be 100 because this test depends on it.");
+  SetInterval(kInterval);
+
+  // Add 5 seconds with 2% dropped frames. This should be in the first bucket.
+  SimulateFrameSequence(MakeFrameSequence(1, 50), (kFps / 50) * 5);
+  EXPECT_TRUE(CheckSmoothnessBuckets({100, 0, 0, 0, 0, 0, 0}));
+
+  // Add 5 seconds with 5% dropped frames. This should be in the second bucket.
+  dropped_frame_counter_.Reset();
+  dropped_frame_counter_.OnFcpReceived();
+  SimulateFrameSequence(MakeFrameSequence(1, 20), (kFps / 20) * 5);
+  EXPECT_TRUE(CheckSmoothnessBuckets({0, 100, 0, 0, 0, 0, 0}));
+
+  // Add 5 seconds with 10% dropped frames. This should be in the third bucket.
+  dropped_frame_counter_.Reset();
+  dropped_frame_counter_.OnFcpReceived();
+  SimulateFrameSequence(MakeFrameSequence(1, 10), (kFps / 10) * 5);
+  EXPECT_TRUE(CheckSmoothnessBuckets({0, 0, 100, 0, 0, 0, 0}));
+
+  // Add 5 seconds with 20% dropped frames. This should be in the fourth bucket.
+  dropped_frame_counter_.Reset();
+  dropped_frame_counter_.OnFcpReceived();
+  SimulateFrameSequence({false, false, false, false, true}, (kFps / 5) * 5);
+  EXPECT_TRUE(CheckSmoothnessBuckets({0, 0, 0, 100, 0, 0, 0}));
+
+  // Add 5 seconds with 40% dropped frames. This should be in the fifth bucket.
+  dropped_frame_counter_.Reset();
+  dropped_frame_counter_.OnFcpReceived();
+  SimulateFrameSequence({false, false, false, true, true}, (kFps / 5) * 5);
+  EXPECT_TRUE(CheckSmoothnessBuckets({0, 0, 0, 0, 100, 0, 0}));
+
+  // Add 5 seconds with 60% dropped frames. This should be in the sixth bucket.
+  dropped_frame_counter_.Reset();
+  dropped_frame_counter_.OnFcpReceived();
+  SimulateFrameSequence({false, false, true, true, true}, (kFps / 5) * 5);
+  EXPECT_TRUE(CheckSmoothnessBuckets({0, 0, 0, 0, 0, 100, 0}));
+
+  // Add 5 seconds with 80% dropped frames. This should be in the last bucket.
+  dropped_frame_counter_.Reset();
+  dropped_frame_counter_.OnFcpReceived();
+  SimulateFrameSequence({false, true, true, true, true}, (kFps / 5) * 5);
+  EXPECT_TRUE(CheckSmoothnessBuckets({0, 0, 0, 0, 0, 0, 100}));
+}
+
+TEST_F(DroppedFrameCounterTest, MovingSmoothnessRatings) {
+  // Set an interval that rounds up nicely with 1 second.
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
+  static_assert(kFps == 100,
+                "kFps must be 100 because this test depends on it.");
+  SetInterval(kInterval);
+
+  // Add a second with 40% dropped frames. Nothing should be added to the
+  // histogram yet.
+  SimulateFrameSequence({false, false, false, true, true}, kFps / 5);
+  EXPECT_TRUE(CheckSmoothnessBuckets({0, 0, 0, 0, 0, 0, 0}));
+
+  // Add a second with 80% dropped frames. All very bad buckets should have some
+  // entries.
+  SimulateFrameSequence({false, true, true, true, true}, kFps / 5);
+  EXPECT_TRUE(CheckSmoothnessBuckets({0, 0, 0, 0, 22, 64, 14}));
+
+  // Add a second with 10% dropped frames. Should be mostly very bad, with a few
+  // bad and okay windows.
+  SimulateFrameSequence(MakeFrameSequence(1, 10), kFps / 10);
+  EXPECT_TRUE(CheckSmoothnessBuckets({0, 0, 1, 9, 29, 50, 11}));
+
+  // Add a second with 5% dropped frames, and a second with no dropped frames.
+  // The sliding window should shift from ok to very good over time.
+  SimulateFrameSequence(MakeFrameSequence(1, 20), kFps / 20);
+  SimulateFrameSequence({false}, kFps);
+  EXPECT_TRUE(CheckSmoothnessBuckets({15, 12.5, 23, 4.5, 14.5, 25, 5.5}));
+
+  // Clear the counter, then add a second with 100% dropped frames and a second
+  // with 0% dropped frames. As the sliding window shifts each integer percent
+  // (other than 100%) should be reported once, exactly matching the size of
+  // each bucket.
+  dropped_frame_counter_.Reset();
+  dropped_frame_counter_.OnFcpReceived();
+  SimulateFrameSequence({true}, kFps);
+  SimulateFrameSequence({false}, kFps);
+  EXPECT_TRUE(CheckSmoothnessBuckets({3, 3, 6, 13, 25, 25, 25}));
+}
+
+TEST_F(DroppedFrameCounterTest, FramesInFlightWhenFcpReceived) {
+  // Start five frames in flight.
+  std::vector<viz::BeginFrameArgs> pending_frames = SimulatePendingFrame(5);
+
+  // Set that FCP was received after the third frame starts, but before it ends.
+  base::TimeTicks time_fcp_sent =
+      pending_frames[2].frame_time + pending_frames[2].interval / 2;
+  dropped_frame_counter_.SetTimeFcpReceivedForTesting(time_fcp_sent);
+
+  // End each of the frames as dropped. The first three should not count for
+  // smoothness, only the last two.
+  for (const auto& frame : pending_frames) {
+    dropped_frame_counter_.OnEndFrame(frame, CreateStubFrameInfo(true));
+  }
+  EXPECT_EQ(dropped_frame_counter_.total_smoothness_dropped(), 2u);
+}
+
+TEST_F(DroppedFrameCounterTest, ForkedCompositorFrameReporter) {
+  // Run different combinations of main and impl threads dropping, make sure
+  // only one frame is counted as dropped each time.
+  SimulateForkedFrame(false, false);
+  EXPECT_EQ(dropped_frame_counter_.total_smoothness_dropped(), 0u);
+
+  SimulateForkedFrame(true, false);
+  EXPECT_EQ(dropped_frame_counter_.total_smoothness_dropped(), 1u);
+
+  SimulateForkedFrame(false, true);
+  EXPECT_EQ(dropped_frame_counter_.total_smoothness_dropped(), 2u);
+
+  SimulateForkedFrame(true, true);
+  EXPECT_EQ(dropped_frame_counter_.total_smoothness_dropped(), 3u);
+}
+
+TEST_F(DroppedFrameCounterTest, WorstSmoothnessTiming) {
+  // Set an interval that rounds up nicely with 1 second.
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
+  static_assert(
+      kFps % 5 == 0,
+      "kFps must be a multiple of 5 because this test depends on it.");
+  SetInterval(kInterval);
+
+  // Prepare a second of pending frames, and send FCP after the last of these
+  // frames.
+  dropped_frame_counter_.Reset();
+  std::vector<viz::BeginFrameArgs> pending_frames = SimulatePendingFrame(kFps);
+  const auto& last_frame = pending_frames.back();
+  base::TimeTicks time_fcp_sent =
+      last_frame.frame_time + last_frame.interval / 2;
+  dropped_frame_counter_.OnFcpReceived();
+  dropped_frame_counter_.SetTimeFcpReceivedForTesting(time_fcp_sent);
+
+  // End each of the pending frames as dropped. These shouldn't affect any of
+  // the metrics.
+  for (const auto& frame : pending_frames) {
+    dropped_frame_counter_.OnEndFrame(frame, CreateStubFrameInfo(true));
+  }
+
+  // After FCP time, add a second each of 80% and 60%, and three seconds of 40%
+  // dropped frames. This should be five seconds total.
+  SimulateFrameSequence({false, true, true, true, true}, kFps / 5);
+  SimulateFrameSequence({false, false, true, true, true}, kFps / 5);
+  SimulateFrameSequence({false, false, false, true, true}, (kFps / 5) * 3);
+
+  // Next two seconds are 20% dropped frames.
+  SimulateFrameSequence({false, false, false, false, true}, (kFps / 5) * 2);
+
+  // The first 1, 2, and 5 seconds shouldn't be recorded in the corresponding
+  // max dropped after N seconds metrics.
+  EXPECT_FLOAT_EQ(MaxPercentDroppedFrame(), 80);
+  EXPECT_FLOAT_EQ(MaxPercentDroppedFrameAfter1Sec(), 60);
+  EXPECT_FLOAT_EQ(MaxPercentDroppedFrameAfter2Sec(), 40);
+  EXPECT_FLOAT_EQ(MaxPercentDroppedFrameAfter5Sec(), 20);
+
+  // Next second is 100% dropped frames, all metrics should include this.
+  SimulateFrameSequence({true}, kFps);
+  EXPECT_FLOAT_EQ(MaxPercentDroppedFrame(), 100);
+  EXPECT_FLOAT_EQ(MaxPercentDroppedFrameAfter1Sec(), 100);
+  EXPECT_FLOAT_EQ(MaxPercentDroppedFrameAfter2Sec(), 100);
+  EXPECT_FLOAT_EQ(MaxPercentDroppedFrameAfter5Sec(), 100);
+}
+
+TEST_F(DroppedFrameCounterTest, ReportOnEveryFrameForUI) {
+  constexpr auto kInterval = base::Milliseconds(10);
+  constexpr int kFps = base::Seconds(1).IntDiv(kInterval);
+  static_assert(
+      kFps % 5 == 0,
+      "kFps must be a multiple of 5 because this test depends on it.");
+  SetInterval(kInterval);
+
+  dropped_frame_counter_.EnableReporForUI();
+  TestCustomMetricsRecorder recorder;
+
+  // 4 seconds with 20% dropped frames.
+  SimulateFrameSequence({false, false, false, false, true}, (kFps / 5) * 4);
+
+  // Recorded (kFps * 3) samples of 20% dropped frame percentage. Only 3 seconds
+  // of frames reported because there is no reports for the very 1st second.
+  EXPECT_EQ(recorder.report_count(), kFps * 3);
+  EXPECT_FLOAT_EQ(recorder.last_percent_dropped_frames(), 20.0f);
+
+  recorder.Reset();
+
+  // 4 seconds with 0 dropped frames.
+  SimulateFrameSequence({false, false, false, false, false}, (kFps / 5) * 4);
+
+  // Recorded (kFps * 4) samples of 0% dropped frame percentage.
+  EXPECT_EQ(recorder.report_count(), kFps * 4);
+  EXPECT_FLOAT_EQ(recorder.last_percent_dropped_frames(), 0.0f);
+}
+
 }  // namespace
 }  // namespace cc
