@@ -1,4 +1,4 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,9 +9,9 @@
 #include <string>
 #include <utility>
 
-#include "base/bind.h"
 #include "base/containers/contains.h"
 #include "base/containers/cxx20_erase.h"
+#include "base/functional/bind.h"
 #include "base/logging.h"
 #include "base/metrics/histogram.h"
 #include "base/metrics/histogram_macros.h"
@@ -74,18 +74,18 @@ const char* FrameSequenceTracker::GetFrameSequenceTrackerTypeName(
       return "CanvasAnimation";
     case FrameSequenceTrackerType::kJSAnimation:
       return "JSAnimation";
+    case FrameSequenceTrackerType::kSETMainThreadAnimation:
+      return "SETMainThreadAnimation";
+    case FrameSequenceTrackerType::kSETCompositorAnimation:
+      return "SETCompositorAnimation";
     case FrameSequenceTrackerType::kMaxType:
       return "";
   }
 }
 
-FrameSequenceTracker::FrameSequenceTracker(
-    FrameSequenceTrackerType type,
-    ThroughputUkmReporter* throughput_ukm_reporter)
+FrameSequenceTracker::FrameSequenceTracker(FrameSequenceTrackerType type)
     : custom_sequence_id_(-1),
-      metrics_(
-          std::make_unique<FrameSequenceMetrics>(type,
-                                                 throughput_ukm_reporter)) {
+      metrics_(std::make_unique<FrameSequenceMetrics>(type)) {
   DCHECK_LT(type, FrameSequenceTrackerType::kMaxType);
   DCHECK(type != FrameSequenceTrackerType::kCustom);
   // TODO(crbug.com/1158439): remove the trace event once the validation is
@@ -100,8 +100,7 @@ FrameSequenceTracker::FrameSequenceTracker(
     FrameSequenceMetrics::CustomReporter custom_reporter)
     : custom_sequence_id_(custom_sequence_id),
       metrics_(std::make_unique<FrameSequenceMetrics>(
-          FrameSequenceTrackerType::kCustom,
-          /*ukm_reporter=*/nullptr)) {
+          FrameSequenceTrackerType::kCustom)) {
   DCHECK_GT(custom_sequence_id_, 0);
   metrics_->SetCustomReporter(std::move(custom_reporter));
 }
@@ -117,10 +116,12 @@ FrameSequenceTracker::~FrameSequenceTracker() {
 void FrameSequenceTracker::ScheduleTerminate() {
   // If the last frame has ended and there is no frame awaiting presentation,
   // then it is ready to terminate.
-  if (!is_inside_frame_ && last_submitted_frame_ == 0)
+  if (!is_inside_frame_ && last_ended_frame_id_.sequence_number <=
+                               last_sorted_frame_id_.sequence_number) {
     termination_status_ = TerminationStatus::kReadyForTermination;
-  else
+  } else {
     termination_status_ = TerminationStatus::kScheduledForTermination;
+  }
 }
 
 void FrameSequenceTracker::ReportBeginImplFrame(
@@ -157,9 +158,6 @@ void FrameSequenceTracker::ReportBeginImplFrame(
                          args.frames_throttled_since_last);
   impl_throughput().frames_expected +=
       begin_impl_frame_data_.previous_sequence_delta;
-#if DCHECK_IS_ON()
-  ++impl_throughput().frames_received;
-#endif
 
   if (first_frame_timestamp_.is_null())
     first_frame_timestamp_ = args.frame_time;
@@ -254,35 +252,12 @@ void FrameSequenceTracker::ReportSubmitFrame(
     const viz::BeginFrameAck& ack,
     const viz::BeginFrameArgs& origin_args) {
   DCHECK_NE(termination_status_, TerminationStatus::kReadyForTermination);
-
-  // TODO(crbug.com/1072482): find a proper way to terminate a tracker.
-  // Right now, we define a magical number |frames_to_terminate_tracker| = 3,
-  // which means that if this frame_token is more than 3 frames compared with
-  // the last submitted frame, then we assume that the last submitted frame is
-  // not going to be presented, and thus terminate this tracker.
-  const uint32_t frames_to_terminate_tracker = 3;
-  if (termination_status_ == TerminationStatus::kScheduledForTermination &&
-      last_submitted_frame_ != 0 &&
-      viz::FrameTokenGT(frame_token,
-                        last_submitted_frame_ + frames_to_terminate_tracker)) {
-    termination_status_ = TerminationStatus::kReadyForTermination;
-    return;
-  }
-
   if (ShouldIgnoreBeginFrameSource(ack.frame_id.source_id) ||
       ShouldIgnoreSequence(ack.frame_id.sequence_number)) {
     ignored_frame_tokens_.insert(frame_token);
     return;
   }
 
-#if DCHECK_IS_ON()
-  DCHECK(is_inside_frame_) << TRACKER_DCHECK_MSG;
-  DCHECK_LT(impl_throughput().frames_processed,
-            impl_throughput().frames_received)
-      << TRACKER_DCHECK_MSG;
-  ++impl_throughput().frames_processed;
-#endif
-
   last_processed_impl_sequence_ = ack.frame_id.sequence_number;
   if (first_submitted_frame_ == 0)
     first_submitted_frame_ = frame_token;
@@ -326,10 +301,6 @@ void FrameSequenceTracker::ReportSubmitFrame(
           << TRACKER_DCHECK_MSG;
     }
   }
-
-  if (has_missing_content) {
-    checkerboarding_.frames.push_back(frame_token);
-  }
 }
 
 void FrameSequenceTracker::ReportFrameEnd(
@@ -382,44 +353,26 @@ void FrameSequenceTracker::ReportFrameEnd(
   // is ignored.
   if (frame_had_no_compositor_damage_ && !compositor_frame_submitted_) {
     DCHECK_GT(impl_throughput().frames_expected, 0u) << TRACKER_DCHECK_MSG;
-    DCHECK_GT(impl_throughput().frames_expected,
-              impl_throughput().frames_produced)
-        << TRACKER_DCHECK_MSG;
-    DCHECK_GE(impl_throughput().frames_produced,
-              impl_throughput().frames_ontime)
-        << TRACKER_DCHECK_MSG;
     --impl_throughput().frames_expected;
     metrics()->NotifyNoUpdateForJankReporter(
         FrameInfo::SmoothEffectDrivingThread::kCompositor,
         args.frame_id.sequence_number, args.interval);
-#if DCHECK_IS_ON()
-    ++impl_throughput().frames_processed;
-    // If these two are the same, it means that each impl frame is either
-    // no-damage or submitted. That's expected, so we don't need those in the
-    // output of DCHECK.
-    if (impl_throughput().frames_processed == impl_throughput().frames_received)
-      ignored_trace_char_count_ = frame_sequence_trace_.str().size();
-    else
-      NOTREACHED() << TRACKER_DCHECK_MSG;
-#endif
     begin_impl_frame_data_.previous_sequence = 0;
   }
-  // last_submitted_frame_ == 0 means the last impl frame has been presented.
-  if (termination_status_ == TerminationStatus::kScheduledForTermination &&
-      last_submitted_frame_ == 0)
-    termination_status_ = TerminationStatus::kReadyForTermination;
 
   frame_had_no_compositor_damage_ = false;
   compositor_frame_submitted_ = false;
   submitted_frame_had_new_main_content_ = false;
   last_processed_main_sequence_latency_ = 0;
-
-  DCHECK(is_inside_frame_) << TRACKER_DCHECK_MSG;
   is_inside_frame_ = false;
 
   DCHECK_EQ(last_started_impl_sequence_, last_processed_impl_sequence_)
       << TRACKER_DCHECK_MSG;
   last_started_impl_sequence_ = 0;
+
+  if (termination_status_ == TerminationStatus::kActive) {
+    last_ended_frame_id_ = args.frame_id;
+  }
 }
 
 void FrameSequenceTracker::ReportFramePresented(
@@ -436,15 +389,6 @@ void FrameSequenceTracker::ReportFramePresented(
   // 0 such that the tracker can be terminated.
   if (last_submitted_frame_ && frame_token_acks_last_frame)
     last_submitted_frame_ = 0;
-  // Update termination status if this is scheduled for termination, and it is
-  // not waiting for any frames, or it has received the presentation-feedback
-  // for the latest frame it is tracking.
-  //
-  // We should always wait for an impl frame to end, that is, ReportFrameEnd.
-  if (termination_status_ == TerminationStatus::kScheduledForTermination &&
-      last_submitted_frame_ == 0 && !is_inside_frame_) {
-    termination_status_ = TerminationStatus::kReadyForTermination;
-  }
 
   if (first_submitted_frame_ == 0 ||
       viz::FrameTokenGT(first_submitted_frame_, frame_token)) {
@@ -465,25 +409,11 @@ void FrameSequenceTracker::ReportFramePresented(
       (feedback.interval.is_zero() ? viz::BeginFrameArgs::DefaultInterval()
                                    : feedback.interval);
   DCHECK(!vsync_interval.is_zero()) << TRACKER_DCHECK_MSG;
-  base::TimeTicks safe_deadline_for_frame =
-      last_frame_presentation_timestamp_ + vsync_interval * 1.5;
-
   const bool was_presented = !feedback.failed();
   if (was_presented && submitted_frame_since_last_presentation) {
-    if (!last_frame_presentation_timestamp_.is_null() &&
-        (safe_deadline_for_frame < feedback.timestamp)) {
-      DCHECK_LE(impl_throughput().frames_ontime,
-                impl_throughput().frames_produced)
-          << TRACKER_DCHECK_MSG;
-      ++impl_throughput().frames_ontime;
-    }
-
-    DCHECK_LT(impl_throughput().frames_produced,
-              impl_throughput().frames_expected)
-        << TRACKER_DCHECK_MSG;
     ++impl_throughput().frames_produced;
     if (metrics()->GetEffectiveThread() == ThreadType::kCompositor) {
-      metrics()->AdvanceTrace(feedback.timestamp);
+      metrics()->AdvanceTrace(feedback.timestamp, frame_token);
     }
 
     metrics()->ComputeJank(FrameInfo::SmoothEffectDrivingThread::kCompositor,
@@ -499,59 +429,14 @@ void FrameSequenceTracker::ReportFramePresented(
       main_frames_.pop_front();
     }
     if (main_frames_.size() < size_before_erase) {
-      DCHECK_LT(main_throughput().frames_produced,
-                main_throughput().frames_expected)
-          << TRACKER_DCHECK_MSG;
       ++main_throughput().frames_produced;
       if (metrics()->GetEffectiveThread() == ThreadType::kMain) {
-        metrics()->AdvanceTrace(feedback.timestamp);
+        metrics()->AdvanceTrace(feedback.timestamp, frame_token);
       }
 
       metrics()->ComputeJank(FrameInfo::SmoothEffectDrivingThread::kMain,
                              frame_token, feedback.timestamp, vsync_interval);
     }
-    if (main_frames_.size() < size_before_erase) {
-      if (!last_frame_presentation_timestamp_.is_null() &&
-          (safe_deadline_for_frame < feedback.timestamp)) {
-        DCHECK_LE(main_throughput().frames_ontime,
-                  main_throughput().frames_produced)
-            << TRACKER_DCHECK_MSG;
-        ++main_throughput().frames_ontime;
-      }
-    }
-    last_frame_presentation_timestamp_ = feedback.timestamp;
-
-    if (checkerboarding_.last_frame_had_checkerboarding) {
-      DCHECK(!checkerboarding_.last_frame_timestamp.is_null())
-          << TRACKER_DCHECK_MSG;
-      DCHECK(!feedback.timestamp.is_null()) << TRACKER_DCHECK_MSG;
-
-      // |feedback.timestamp| is the timestamp when the latest frame was
-      // presented. |checkerboarding_.last_frame_timestamp| is the timestamp
-      // when the previous frame (which had checkerboarding) was presented. Use
-      // |feedback.interval| to compute the number of vsyncs that have passed
-      // between the two frames (since that is how many times the user saw that
-      // checkerboarded frame).
-      base::TimeDelta difference =
-          feedback.timestamp - checkerboarding_.last_frame_timestamp;
-      const auto& interval = feedback.interval.is_zero()
-                                 ? viz::BeginFrameArgs::DefaultInterval()
-                                 : feedback.interval;
-      DCHECK(!interval.is_zero()) << TRACKER_DCHECK_MSG;
-      constexpr base::TimeDelta kEpsilon = base::Milliseconds(1);
-      int64_t frames = (difference + kEpsilon).IntDiv(interval);
-      metrics_->add_checkerboarded_frames(frames);
-    }
-
-    const bool frame_had_checkerboarding =
-        base::Contains(checkerboarding_.frames, frame_token);
-    checkerboarding_.last_frame_had_checkerboarding = frame_had_checkerboarding;
-    checkerboarding_.last_frame_timestamp = feedback.timestamp;
-  }
-
-  while (!checkerboarding_.frames.empty() &&
-         !viz::FrameTokenGT(checkerboarding_.frames.front(), frame_token)) {
-    checkerboarding_.frames.pop_front();
   }
 }
 
@@ -630,8 +515,6 @@ void FrameSequenceTracker::ReportMainFrameCausedNoDamage(
   DCHECK_GT(main_throughput().frames_expected,
             main_throughput().frames_produced)
       << TRACKER_DCHECK_MSG;
-  DCHECK_GE(main_throughput().frames_produced, main_throughput().frames_ontime)
-      << TRACKER_DCHECK_MSG;
   last_no_main_damage_sequence_ = args.frame_id.sequence_number;
   --main_throughput().frames_expected;
   metrics()->NotifyNoUpdateForJankReporter(
@@ -668,8 +551,22 @@ void FrameSequenceTracker::UpdateTrackedFrameData(
       frame_data->previous_source == source_id) {
     uint32_t current_latency =
         sequence_number - frame_data->previous_sequence - throttled_frame_count;
-    DCHECK_GT(current_latency, 0u) << TRACKER_DCHECK_MSG;
-    frame_data->previous_sequence_delta = current_latency;
+    if (current_latency > 0u) {
+      frame_data->previous_sequence_delta = current_latency;
+    } else {
+      // It is possible for the `current_latency` to be 0. This can occur when
+      // the Renderer or VideoFrameSubmitter is detached from Viz as a client.
+      // Before being re-attached during the same frame. This can be caused by
+      // re-embedding this client. Such as when triggering Picture-in-Picture
+      // mode.
+      //
+      // These special-case re-embedding lead to the `source_id` staying the
+      // same. As it is the same instance of Viz.
+      //
+      // We do not want to error on such a transition, so we treat the
+      // transition as a single frame delta.
+      frame_data->previous_sequence_delta = 1;
+    }
   } else {
     frame_data->previous_sequence_delta = 1;
   }
@@ -706,11 +603,6 @@ bool FrameSequenceTracker::ShouldReportMetricsNow(
 }
 
 std::unique_ptr<FrameSequenceMetrics> FrameSequenceTracker::TakeMetrics() {
-#if DCHECK_IS_ON()
-  DCHECK_EQ(impl_throughput().frames_received,
-            impl_throughput().frames_processed)
-      << frame_sequence_trace_.str().substr(ignored_trace_char_count_);
-#endif
   return std::move(metrics_);
 }
 
@@ -721,11 +613,42 @@ void FrameSequenceTracker::CleanUp() {
 
 void FrameSequenceTracker::AddSortedFrame(const viz::BeginFrameArgs& args,
                                           const FrameInfo& frame_info) {
+  // Do not process any frames once we are terminated.
+  if (termination_status_ == TerminationStatus::kReadyForTermination) {
+    return;
+  }
+  last_sorted_frame_id_ = args.frame_id;
+  // For trackers that scheduled for termination, only proceed to update
+  // metrics for the frame if it is before the last ended frame.
+  if (termination_status_ == TerminationStatus::kScheduledForTermination) {
+    if (last_ended_frame_id_.source_id != args.frame_id.source_id) {
+      // Frame source changed so we will no longer receive updates for frames
+      // we submitted to the old source.
+      termination_status_ = TerminationStatus::kReadyForTermination;
+      return;
+    }
+    // We only terminate when the content is no longer dropped.
+    if (frame_info.final_state != FrameInfo::FrameFinalState::kDropped) {
+      if (last_ended_frame_id_.sequence_number <
+          args.frame_id.sequence_number) {
+        termination_status_ = TerminationStatus::kReadyForTermination;
+        return;
+      } else if (last_ended_frame_id_.sequence_number ==
+                 args.frame_id.sequence_number) {
+        // We still report the final `sequence_number`, but need to mark for
+        // termination.
+        termination_status_ = TerminationStatus::kReadyForTermination;
+      }
+    }
+  }
+
+  // Don't count drops from after the sequence has begun termination.
+  if (frame_info.final_state == FrameInfo::FrameFinalState::kDropped &&
+      last_ended_frame_id_.sequence_number < args.frame_id.sequence_number) {
+    return;
+  }
   if (metrics_)
     metrics_->AddSortedFrame(args, frame_info);
 }
 
-FrameSequenceTracker::CheckerboardingData::CheckerboardingData() = default;
-FrameSequenceTracker::CheckerboardingData::~CheckerboardingData() = default;
-
 }  // namespace cc
