@@ -1,4 +1,4 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,18 +10,24 @@
 #include <string>
 #include <utility>
 
-#include "base/bind.h"
 #include "base/debug/stack_trace.h"
 #include "base/format_macros.h"
+#include "base/functional/bind.h"
+#include "base/memory/raw_ptr.h"
+#include "base/memory/raw_ref.h"
 #include "base/metrics/histogram_macros.h"
+#include "base/numerics/ostream_operators.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/stringprintf.h"
-#include "base/threading/thread_task_runner_handle.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/trace_event/memory_dump_manager.h"
 #include "cc/base/devtools_instrumentation.h"
+#include "cc/base/features.h"
 #include "cc/base/histograms.h"
 #include "cc/raster/tile_task.h"
 #include "cc/tiles/mipmap_util.h"
-#include "ui/gfx/skia_util.h"
+#include "components/miracle_parameter/common/public/miracle_parameter.h"
+#include "ui/gfx/geometry/skia_conversions.h"
 
 using base::trace_event::MemoryAllocatorDump;
 using base::trace_event::MemoryDumpLevelOfDetail;
@@ -29,12 +35,18 @@ using base::trace_event::MemoryDumpLevelOfDetail;
 namespace cc {
 namespace {
 
+BASE_FEATURE(kNormalMaxItemsInCacheForSoftwareFeature,
+             "NormalMaxItemsInCacheForSoftwareFeature",
+             base::FEATURE_ENABLED_BY_DEFAULT);
 
 // The number of entries to keep around in the cache. This limit can be breached
 // if more items are locked. That is, locked items ignore this limit.
 // Depending on the memory state of the system, we limit the amount of items
 // differently.
-const size_t kNormalMaxItemsInCacheForSoftware = 1000;
+MIRACLE_PARAMETER_FOR_INT(GetNormalMaxItemsInCacheForSoftware,
+                          kNormalMaxItemsInCacheForSoftwareFeature,
+                          "NormalMaxItemsInCacheForSoftware",
+                          1000)
 
 class AutoRemoveKeyFromTaskMap {
  public:
@@ -44,13 +56,14 @@ class AutoRemoveKeyFromTaskMap {
                          SoftwareImageDecodeCache::CacheKeyHash>* task_map,
       const SoftwareImageDecodeCache::CacheKey& key)
       : task_map_(task_map), key_(key) {}
-  ~AutoRemoveKeyFromTaskMap() { task_map_->erase(key_); }
+  ~AutoRemoveKeyFromTaskMap() { task_map_->erase(*key_); }
 
  private:
-  std::unordered_map<SoftwareImageDecodeCache::CacheKey,
-                     scoped_refptr<TileTask>,
-                     SoftwareImageDecodeCache::CacheKeyHash>* task_map_;
-  const SoftwareImageDecodeCache::CacheKey& key_;
+  raw_ptr<std::unordered_map<SoftwareImageDecodeCache::CacheKey,
+                             scoped_refptr<TileTask>,
+                             SoftwareImageDecodeCache::CacheKeyHash>>
+      task_map_;
+  const raw_ref<const SoftwareImageDecodeCache::CacheKey> key_;
 };
 
 class SoftwareImageDecodeTaskImpl : public TileTask {
@@ -62,7 +75,10 @@ class SoftwareImageDecodeTaskImpl : public TileTask {
       SoftwareImageDecodeCache::DecodeTaskType task_type,
       const ImageDecodeCache::TracingInfo& tracing_info)
       : TileTask(TileTask::SupportsConcurrentExecution::kYes,
-                 TileTask::SupportsBackgroundThreadPriority::kYes),
+                 (base::FeatureList::IsEnabled(
+                      features::kNormalPriorityImageDecoding)
+                      ? TileTask::SupportsBackgroundThreadPriority::kNo
+                      : TileTask::SupportsBackgroundThreadPriority::kYes)),
         cache_(cache),
         image_key_(image_key),
         paint_image_(paint_image),
@@ -84,7 +100,7 @@ class SoftwareImageDecodeTaskImpl : public TileTask {
         image_metadata ? image_metadata->image_type : ImageType::kInvalid;
     devtools_instrumentation::ScopedImageDecodeTask image_decode_task(
         paint_image_.GetSwSkImage().get(),
-        devtools_instrumentation::ScopedImageDecodeTask::kSoftware,
+        devtools_instrumentation::ScopedImageDecodeTask::DecodeType::kSoftware,
         ImageDecodeCache::ToScopedTaskType(tracing_info_.task_type),
         ImageDecodeCache::ToScopedImageType(image_type));
     SoftwareImageDecodeCache::TaskProcessingResult result =
@@ -100,11 +116,18 @@ class SoftwareImageDecodeTaskImpl : public TileTask {
     cache_->OnImageDecodeTaskCompleted(image_key_, task_type_);
   }
 
+  // Overridden from TileTask:
+  bool TaskContainsLCPCandidateImages() const override {
+    if (!HasCompleted() && paint_image_.may_be_lcp_candidate())
+      return true;
+    return TileTask::TaskContainsLCPCandidateImages();
+  }
+
  protected:
   ~SoftwareImageDecodeTaskImpl() override = default;
 
  private:
-  SoftwareImageDecodeCache* cache_;
+  raw_ptr<SoftwareImageDecodeCache, AcrossTasksDanglingUntriaged> cache_;
   SoftwareImageDecodeCache::CacheKey image_key_;
   PaintImage paint_image_;
   SoftwareImageDecodeCache::DecodeTaskType task_type_;
@@ -139,19 +162,19 @@ PaintFlags::FilterQuality GetDecodedFilterQuality(
 
 SoftwareImageDecodeCache::SoftwareImageDecodeCache(
     SkColorType color_type,
-    size_t locked_memory_limit_bytes,
-    PaintImage::GeneratorClientId generator_client_id)
-    : decoded_images_(ImageMRUCache::NO_AUTO_EVICT),
+    size_t locked_memory_limit_bytes)
+    : decoded_images_(ImageLRUCache::NO_AUTO_EVICT),
       locked_images_budget_(locked_memory_limit_bytes),
       color_type_(color_type),
-      generator_client_id_(generator_client_id),
-      max_items_in_cache_(kNormalMaxItemsInCacheForSoftware) {
-  // In certain cases, ThreadTaskRunnerHandle isn't set (Android Webview).
-  // Don't register a dump provider in these cases.
-  if (base::ThreadTaskRunnerHandle::IsSet()) {
+      generator_client_id_(PaintImage::GetNextGeneratorClientId()),
+      max_items_in_cache_(GetNormalMaxItemsInCacheForSoftware()) {
+  DCHECK_NE(generator_client_id_, PaintImage::kDefaultGeneratorClientId);
+  // In certain cases, SingleThreadTaskRunner::CurrentDefaultHandle isn't set
+  // (Android Webview).  Don't register a dump provider in these cases.
+  if (base::SingleThreadTaskRunner::HasCurrentDefault()) {
     base::trace_event::MemoryDumpManager::GetInstance()->RegisterDumpProvider(
         this, "cc::SoftwareImageDecodeCache",
-        base::ThreadTaskRunnerHandle::Get());
+        base::SingleThreadTaskRunner::GetCurrentDefault());
   }
 }
 
@@ -162,16 +185,22 @@ SoftwareImageDecodeCache::~SoftwareImageDecodeCache() {
 }
 
 ImageDecodeCache::TaskResult SoftwareImageDecodeCache::GetTaskForImageAndRef(
+    ClientId client_id,
     const DrawImage& image,
     const TracingInfo& tracing_info) {
   DCHECK_EQ(tracing_info.task_type, TaskType::kInRaster);
+  DCHECK_EQ(client_id, ImageDecodeCache::kDefaultClientId)
+      << "SoftwareImageDecodeCache cannot be shared between multiple clients.";
   return GetTaskForImageAndRefInternal(image, tracing_info,
                                        DecodeTaskType::USE_IN_RASTER_TASKS);
 }
 
 ImageDecodeCache::TaskResult
 SoftwareImageDecodeCache::GetOutOfRasterDecodeTaskForImageAndRef(
+    ClientId client_id,
     const DrawImage& image) {
+  DCHECK_EQ(client_id, ImageDecodeCache::kDefaultClientId)
+      << "SoftwareImageDecodeCache cannot be shared between multiple clients.";
   return GetTaskForImageAndRefInternal(
       image, TracingInfo(0, TilePriority::NOW, TaskType::kOutOfRaster),
       DecodeTaskType::USE_OUT_OF_RASTER_TASKS);
@@ -183,7 +212,7 @@ SoftwareImageDecodeCache::GetTaskForImageAndRefInternal(
     const TracingInfo& tracing_info,
     DecodeTaskType task_type) {
   CacheKey key = CacheKey::FromDrawImage(
-      image, GetColorTypeForPaintImage(image.target_color_space(),
+      image, GetColorTypeForPaintImage(image.target_color_params(),
                                        image.paint_image()));
   TRACE_EVENT1(TRACE_DISABLED_BY_DEFAULT("cc.debug"),
                "SoftwareImageDecodeCache::GetTaskForImageAndRefInternal", "key",
@@ -279,7 +308,7 @@ void SoftwareImageDecodeCache::RemoveBudgetForImage(const CacheKey& key,
 
 void SoftwareImageDecodeCache::UnrefImage(const DrawImage& image) {
   const CacheKey& key = CacheKey::FromDrawImage(
-      image, GetColorTypeForPaintImage(image.target_color_space(),
+      image, GetColorTypeForPaintImage(image.target_color_params(),
                                        image.paint_image()));
   TRACE_EVENT1(TRACE_DISABLED_BY_DEFAULT("cc.debug"),
                "SoftwareImageDecodeCache::UnrefImage", "key", key.ToString());
@@ -298,6 +327,8 @@ void SoftwareImageDecodeCache::UnrefImage(const CacheKey& key) {
       RemoveBudgetForImage(key, entry);
     if (entry->is_locked)
       entry->Unlock();
+
+    ReduceCacheUsageUntilWithinLimit(max_items_in_cache_);
   }
 }
 
@@ -354,14 +385,14 @@ SoftwareImageDecodeCache::DecodeImageIfNecessary(const CacheKey& key,
     base::AutoUnlock release(lock_);
     local_cache_entry = Utils::DoDecodeImage(
         key, paint_image,
-        GetColorTypeForPaintImage(key.target_color_space(), paint_image),
+        GetColorTypeForPaintImage(key.target_color_params(), paint_image),
         generator_client_id_,
         base::BindOnce(&SoftwareImageDecodeCache::ClearCache,
                        base::Unretained(this)));
   } else {
     // Attempt to find a cached decode to generate a scaled/subrected decode
     // from.
-    absl::optional<CacheKey> candidate_key = FindCachedCandidate(key);
+    std::optional<CacheKey> candidate_key = FindCachedCandidate(key);
 
     SkISize desired_size = gfx::SizeToSkISize(key.target_size());
     const bool should_decode_to_scale =
@@ -386,7 +417,7 @@ SoftwareImageDecodeCache::DecodeImageIfNecessary(const CacheKey& key,
       base::AutoUnlock release(lock_);
       local_cache_entry = Utils::DoDecodeImage(
           key, paint_image,
-          GetColorTypeForPaintImage(key.target_color_space(), paint_image),
+          GetColorTypeForPaintImage(key.target_color_params(), paint_image),
           generator_client_id_,
           base::BindOnce(&SoftwareImageDecodeCache::ClearCache,
                          base::Unretained(this)));
@@ -416,10 +447,10 @@ SoftwareImageDecodeCache::DecodeImageIfNecessary(const CacheKey& key,
               : gfx::RectToSkIRect(key.src_rect());
       DrawImage candidate_draw_image(
           paint_image, false, src_rect, PaintFlags::FilterQuality::kNone,
-          SkM44(), key.frame_key().frame_index(), key.target_color_space());
+          SkM44(), key.frame_key().frame_index(), key.target_color_params());
       candidate_key.emplace(CacheKey::FromDrawImage(
           candidate_draw_image,
-          GetColorTypeForPaintImage(key.target_color_space(), paint_image)));
+          GetColorTypeForPaintImage(key.target_color_params(), paint_image)));
     }
 
     if (candidate_key) {
@@ -437,7 +468,7 @@ SoftwareImageDecodeCache::DecodeImageIfNecessary(const CacheKey& key,
         local_cache_entry = Utils::GenerateCacheEntryFromCandidate(
             key, decoded_draw_image,
             candidate_key->type() == CacheKey::kOriginal,
-            GetColorTypeForPaintImage(key.target_color_space(), paint_image));
+            GetColorTypeForPaintImage(key.target_color_params(), paint_image));
       }
 
       // Unref to balance the GetDecodedImageForDrawInternal() call.
@@ -469,7 +500,7 @@ SoftwareImageDecodeCache::DecodeImageIfNecessary(const CacheKey& key,
   return TaskProcessingResult::kFullDecode;
 }
 
-absl::optional<SoftwareImageDecodeCache::CacheKey>
+std::optional<SoftwareImageDecodeCache::CacheKey>
 SoftwareImageDecodeCache::FindCachedCandidate(const CacheKey& key) {
   auto image_keys_it = frame_key_to_image_keys_.find(key.frame_key());
   // We know that we must have at least our own |entry| in this list, so it
@@ -503,7 +534,7 @@ SoftwareImageDecodeCache::FindCachedCandidate(const CacheKey& key) {
     }
   }
 
-  return absl::nullopt;
+  return std::nullopt;
 }
 
 bool SoftwareImageDecodeCache::UseCacheForDrawImage(
@@ -530,15 +561,24 @@ bool SoftwareImageDecodeCache::UseCacheForDrawImage(
   return false;
 }
 
+ImageDecodeCache::ClientId SoftwareImageDecodeCache::GenerateClientId() {
+  ClientId next_client_id = ImageDecodeCache::GenerateClientId();
+  // The software decode cache cannot be shared between multiple clients. Thus,
+  // this DCHECK helps us to verify the software cache has only a single client
+  // that generated a client id for itself only oce.
+  DCHECK_EQ(ImageDecodeCache::kDefaultClientId, next_client_id);
+  return next_client_id;
+}
+
 DecodedDrawImage SoftwareImageDecodeCache::GetDecodedImageForDraw(
     const DrawImage& draw_image) {
   DCHECK(UseCacheForDrawImage(draw_image));
 
   base::AutoLock hold(lock_);
   return GetDecodedImageForDrawInternal(
-      CacheKey::FromDrawImage(
-          draw_image, GetColorTypeForPaintImage(draw_image.target_color_space(),
-                                                draw_image.paint_image())),
+      CacheKey::FromDrawImage(draw_image, GetColorTypeForPaintImage(
+                                              draw_image.target_color_params(),
+                                              draw_image.paint_image())),
       draw_image.paint_image());
 }
 
@@ -579,7 +619,7 @@ void SoftwareImageDecodeCache::DrawWithImageFinished(
   TRACE_EVENT1(TRACE_DISABLED_BY_DEFAULT("cc.debug"),
                "SoftwareImageDecodeCache::DrawWithImageFinished", "key",
                CacheKey::FromDrawImage(
-                   image, GetColorTypeForPaintImage(image.target_color_space(),
+                   image, GetColorTypeForPaintImage(image.target_color_params(),
                                                     image.paint_image()))
                    .ToString());
   UnrefImage(image);
@@ -597,8 +637,7 @@ void SoftwareImageDecodeCache::ReduceCacheUsageUntilWithinLimit(size_t limit) {
 
     const CacheKey& key = it->first;
     auto vector_it = frame_key_to_image_keys_.find(key.frame_key());
-    auto item_it =
-        std::find(vector_it->second.begin(), vector_it->second.end(), key);
+    auto item_it = base::ranges::find(vector_it->second, key);
     DCHECK(item_it != vector_it->second.end());
     vector_it->second.erase(item_it);
     if (vector_it->second.empty())
@@ -630,6 +669,8 @@ void SoftwareImageDecodeCache::OnImageDecodeTaskCompleted(
   auto image_it = decoded_images_.Peek(key);
   DCHECK(image_it != decoded_images_.end());
   CacheEntry* cache_entry = image_it->second.get();
+  UMA_HISTOGRAM_BOOLEAN("Compositing.DecodeLCPCandidateImage.Software",
+                        key.may_be_lcp_candidate());
   auto& task = task_type == DecodeTaskType::USE_IN_RASTER_TASKS
                    ? cache_entry->in_raster_task
                    : cache_entry->out_of_raster_task;
@@ -643,7 +684,7 @@ bool SoftwareImageDecodeCache::OnMemoryDump(
     base::trace_event::ProcessMemoryDump* pmd) {
   base::AutoLock lock(lock_);
 
-  if (args.level_of_detail == MemoryDumpLevelOfDetail::BACKGROUND) {
+  if (args.level_of_detail == MemoryDumpLevelOfDetail::kBackground) {
     std::string dump_name = base::StringPrintf(
         "cc/image_memory/cache_0x%" PRIXPTR, reinterpret_cast<uintptr_t>(this));
     MemoryAllocatorDump* dump = pmd->CreateAllocatorDump(dump_name);
@@ -693,19 +734,24 @@ size_t SoftwareImageDecodeCache::GetNumCacheEntriesForTesting() {
   base::AutoLock lock(lock_);
   return decoded_images_.size();
 }
+size_t SoftwareImageDecodeCache::GetMaxNumCacheEntriesForTesting() {
+  return GetNormalMaxItemsInCacheForSoftware();
+}
 
 SkColorType SoftwareImageDecodeCache::GetColorTypeForPaintImage(
-    const gfx::ColorSpace& target_color_space,
+    const TargetColorParams& target_color_params,
     const PaintImage& paint_image) {
-  // Decode HDR images to half float when targeting HDR.
-  //
+  const gfx::ColorSpace& target_color_space = target_color_params.color_space;
   // TODO(crbug.com/1076568): Once we have access to the display's buffer format
   // via gfx::DisplayColorSpaces, we should also do this for HBD images.
-  if (paint_image.GetContentColorUsage() == gfx::ContentColorUsage::kHDR &&
+  // Do not decode an image to F16 unless the PaintImage reports that its type
+  // is F16. Otherwise, image decode will fail.
+  // https://crbug.com/1488786
+  if (paint_image.GetColorType() == kRGBA_F16_SkColorType &&
+      paint_image.GetContentColorUsage() == gfx::ContentColorUsage::kHDR &&
       target_color_space.IsHDR()) {
     return kRGBA_F16_SkColorType;
   }
-
   return color_type_;
 }
 
