@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,24 +6,25 @@
 
 #include <map>
 
-#include "base/bind.h"
-#include "base/callback_helpers.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback_helpers.h"
 #include "base/json/json_reader.h"
 #include "base/memory/ptr_util.h"
 #include "base/metrics/user_metrics.h"
+#include "base/strings/escape.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
+#include "chrome/browser/browser_features.h"
 #include "chrome/browser/devtools/devtools_window.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/devtools_agent_host.h"
 #include "content/public/browser/devtools_external_agent_proxy.h"
 #include "content/public/browser/devtools_external_agent_proxy_delegate.h"
-#include "net/base/escape.h"
 
 using content::BrowserThread;
 using content::DevToolsAgentHost;
@@ -33,7 +34,7 @@ using RemotePage = DevToolsDeviceDiscovery::RemotePage;
 
 namespace {
 
-const char kPageListRequest[] = "/json";
+const char kPageListRequest[] = "/json/list";
 const char kVersionRequest[] = "/json/version";
 const char kClosePageRequest[] = "/json/close/%s";
 const char kActivatePageRequest[] = "/json/activate/%s";
@@ -46,8 +47,7 @@ const char kWebViewSocketPrefix[] = "webview_devtools_remote";
 
 static void ScheduleTaskDefault(base::OnceClosure task) {
   content::GetUIThreadTaskRunner({})->PostDelayedTask(
-      FROM_HERE, std::move(task),
-      base::TimeDelta::FromMilliseconds(kPollingIntervalMs));
+      FROM_HERE, std::move(task), base::Milliseconds(kPollingIntervalMs));
 }
 
 // ProtocolCommand ------------------------------------------------------------
@@ -60,6 +60,9 @@ class ProtocolCommand
                   const std::string& target_path,
                   const std::string& command);
 
+  ProtocolCommand(const ProtocolCommand&) = delete;
+  ProtocolCommand& operator=(const ProtocolCommand&) = delete;
+
  private:
   void OnSocketOpened() override;
   void OnFrameRead(const std::string& message) override;
@@ -68,8 +71,6 @@ class ProtocolCommand
 
   const std::string command_;
   std::unique_ptr<AndroidDeviceManager::AndroidWebSocket> web_socket_;
-
-  DISALLOW_COPY_AND_ASSIGN(ProtocolCommand);
 };
 
 ProtocolCommand::ProtocolCommand(
@@ -92,8 +93,7 @@ void ProtocolCommand::OnSocketClosed() {
   delete this;
 }
 
-ProtocolCommand::~ProtocolCommand() {
-}
+ProtocolCommand::~ProtocolCommand() {}
 
 // AgentHostDelegate ----------------------------------------------------------
 
@@ -102,21 +102,26 @@ class WebSocketProxy : public AndroidDeviceManager::AndroidWebSocket::Delegate {
   explicit WebSocketProxy(content::DevToolsExternalAgentProxy* proxy)
       : socket_opened_(false), proxy_(proxy) {}
 
+  WebSocketProxy(const WebSocketProxy&) = delete;
+  WebSocketProxy& operator=(const WebSocketProxy&) = delete;
+
   void WebSocketCreated(AndroidDeviceManager::AndroidWebSocket* web_socket) {
     web_socket_.reset(web_socket);
   }
 
   void SendMessageToBackend(std::string message) {
-    if (socket_opened_)
+    if (socket_opened_) {
       web_socket_->SendFrame(std::move(message));
-    else
+    } else {
       pending_messages_.push_back(std::move(message));
+    }
   }
 
   void OnSocketOpened() override {
     socket_opened_ = true;
-    for (std::string& message : pending_messages_)
+    for (std::string& message : pending_messages_) {
       SendMessageToBackend(std::move(message));
+    }
     pending_messages_.clear();
   }
 
@@ -140,7 +145,6 @@ class WebSocketProxy : public AndroidDeviceManager::AndroidWebSocket::Delegate {
   std::vector<std::string> pending_messages_;
   std::unique_ptr<AndroidDeviceManager::AndroidWebSocket> web_socket_;
   content::DevToolsExternalAgentProxy* proxy_;
-  DISALLOW_COPY_AND_ASSIGN(WebSocketProxy);
 };
 
 class AgentHostDelegate : public content::DevToolsExternalAgentProxyDelegate {
@@ -152,7 +156,11 @@ class AgentHostDelegate : public content::DevToolsExternalAgentProxyDelegate {
       const std::string& local_id,
       const std::string& target_path,
       const std::string& type,
-      base::Value* value);
+      const base::Value::Dict* value);
+
+  AgentHostDelegate(const AgentHostDelegate&) = delete;
+  AgentHostDelegate& operator=(const AgentHostDelegate&) = delete;
+
   ~AgentHostDelegate() override;
 
  private:
@@ -162,7 +170,7 @@ class AgentHostDelegate : public content::DevToolsExternalAgentProxyDelegate {
                     const std::string& local_id,
                     const std::string& target_path,
                     const std::string& type,
-                    base::Value* value);
+                    const base::Value::Dict* value);
   // DevToolsExternalAgentProxyDelegate overrides.
   void Attach(content::DevToolsExternalAgentProxy* proxy) override;
   void Detach(content::DevToolsExternalAgentProxy* proxy) override;
@@ -194,42 +202,45 @@ class AgentHostDelegate : public content::DevToolsExternalAgentProxyDelegate {
   std::map<content::DevToolsExternalAgentProxy*,
            std::unique_ptr<WebSocketProxy>>
       proxies_;
-  DISALLOW_COPY_AND_ASSIGN(AgentHostDelegate);
 };
 
-static std::string GetStringProperty(const base::Value& value,
+static std::string GetStringProperty(const base::Value::Dict& value,
                                      const std::string& name) {
-  const std::string* result = value.FindStringKey(name);
+  const std::string* result = value.FindString(name);
   return result ? *result : std::string();
 }
 
 static std::string BuildUniqueTargetId(const std::string& serial,
                                        const std::string& browser_id,
-                                       const base::Value& value) {
-  return base::StringPrintf("%s:%s:%s", serial.c_str(),
-      browser_id.c_str(), GetStringProperty(value, "id").c_str());
+                                       const base::Value::Dict& value) {
+  return base::StringPrintf("%s:%s:%s", serial.c_str(), browser_id.c_str(),
+                            GetStringProperty(value, "id").c_str());
 }
 
-static std::string GetFrontendURLFromValue(const base::Value& value,
+static std::string GetFrontendURLFromValue(const base::Value::Dict& value,
                                            const std::string& browser_version) {
   std::string frontend_url = GetStringProperty(value, "devtoolsFrontendUrl");
   size_t ws_param = frontend_url.find("?ws");
-  if (ws_param != std::string::npos)
+  if (ws_param != std::string::npos) {
     frontend_url = frontend_url.substr(0, ws_param);
-  if (base::StartsWith(frontend_url, "http:", base::CompareCase::SENSITIVE))
+  }
+  if (base::StartsWith(frontend_url, "http:", base::CompareCase::SENSITIVE)) {
     frontend_url = "https:" + frontend_url.substr(5);
-  if (!browser_version.empty())
+  }
+  if (!browser_version.empty()) {
     frontend_url += "?remoteVersion=" + browser_version;
+  }
   return frontend_url;
 }
 
-static std::string GetTargetPath(const base::Value& value) {
+static std::string GetTargetPath(const base::Value::Dict& value) {
   std::string target_path = GetStringProperty(value, "webSocketDebuggerUrl");
 
   if (base::StartsWith(target_path, "ws://", base::CompareCase::SENSITIVE)) {
     size_t pos = target_path.find("/", 5);
-    if (pos == std::string::npos)
+    if (pos == std::string::npos) {
       pos = 5;
+    }
     target_path = target_path.substr(pos);
   } else {
     target_path = std::string();
@@ -246,17 +257,18 @@ AgentHostDelegate::GetOrCreateAgentHost(
     const std::string& local_id,
     const std::string& target_path,
     const std::string& type,
-    base::Value* value) {
+    const base::Value::Dict* value) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   scoped_refptr<DevToolsAgentHost> result =
       DevToolsAgentHost::GetForId(local_id);
-  if (result)
+  if (result) {
     return result;
+  }
 
   AgentHostDelegate* delegate = new AgentHostDelegate(
       device, browser_id, browser_version, local_id, target_path, type, value);
-  result = content::DevToolsAgentHost::Forward(
-      local_id, base::WrapUnique(delegate));
+  result =
+      content::DevToolsAgentHost::Forward(local_id, base::WrapUnique(delegate));
   delegate->agent_host_ = result.get();
   return result;
 }
@@ -268,7 +280,7 @@ AgentHostDelegate::AgentHostDelegate(
     const std::string& local_id,
     const std::string& target_path,
     const std::string& type,
-    base::Value* value)
+    const base::Value::Dict* value)
     : device_(device),
       browser_id_(browser_id),
       local_id_(local_id),
@@ -277,7 +289,7 @@ AgentHostDelegate::AgentHostDelegate(
       remote_id_(value ? GetStringProperty(*value, "id") : ""),
       frontend_url_(value ? GetFrontendURLFromValue(*value, browser_version)
                           : ""),
-      title_(value ? base::UTF16ToUTF8(net::UnescapeForHTML(
+      title_(value ? base::UTF16ToUTF8(base::UnescapeForHTML(
                          base::UTF8ToUTF16(GetStringProperty(*value, "title"))))
                    : ""),
       description_(value ? GetStringProperty(*value, "description") : ""),
@@ -285,8 +297,7 @@ AgentHostDelegate::AgentHostDelegate(
       favicon_url_(GURL(value ? GetStringProperty(*value, "faviconUrl") : "")),
       agent_host_(nullptr) {}
 
-AgentHostDelegate::~AgentHostDelegate() {
-}
+AgentHostDelegate::~AgentHostDelegate() = default;
 
 void AgentHostDelegate::Attach(content::DevToolsExternalAgentProxy* proxy) {
   std::unique_ptr<WebSocketProxy> ws_proxy(new WebSocketProxy(proxy));
@@ -329,22 +340,23 @@ std::string AgentHostDelegate::GetFrontendURL() {
 }
 
 bool AgentHostDelegate::Activate() {
-  std::string request = base::StringPrintf(kActivatePageRequest,
-                                           remote_id_.c_str());
+  std::string request =
+      base::StringPrintf(kActivatePageRequest, remote_id_.c_str());
   device_->SendJsonRequest(browser_id_, request, base::DoNothing());
   return true;
 }
 
 void AgentHostDelegate::Reload() {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  if (target_path_.empty())
+  if (target_path_.empty()) {
     return;
+  }
   new ProtocolCommand(device_, browser_id_, target_path_, kPageReloadCommand);
 }
 
 bool AgentHostDelegate::Close() {
-  std::string request = base::StringPrintf(kClosePageRequest,
-                                           remote_id_.c_str());
+  std::string request =
+      base::StringPrintf(kClosePageRequest, remote_id_.c_str());
   device_->SendJsonRequest(browser_id_, request, base::DoNothing());
   return true;
 }
@@ -358,8 +370,9 @@ void AgentHostDelegate::SendMessageToBackend(
     base::span<const uint8_t> message) {
   auto it = proxies_.find(proxy);
   // We could have detached due to physical connection being closed.
-  if (it == proxies_.end())
+  if (it == proxies_.end()) {
     return;
+  }
   it->second->SendMessageToBackend(std::string(message.begin(), message.end()));
 }
 
@@ -392,6 +405,9 @@ class DevToolsDeviceDiscovery::DiscoveryRequest
                      scoped_refptr<RemoteBrowser>,
                      int result,
                      const std::string& response);
+  void ParseBrowserInfo(scoped_refptr<RemoteBrowser> browser,
+                        const std::string& version_response,
+                        bool& is_chrome);
 
   base::OnceCallback<void(const CompleteDevices&)> callback_;
   DevToolsDeviceDiscovery::CompleteDevices complete_devices_;
@@ -443,6 +459,38 @@ void DevToolsDeviceDiscovery::DiscoveryRequest::ReceivedDeviceInfo(
   }
 }
 
+void DevToolsDeviceDiscovery::DiscoveryRequest::ParseBrowserInfo(
+    scoped_refptr<RemoteBrowser> browser,
+    const std::string& version_response,
+    bool& is_chrome) {
+  // Parse version, append to package name if available,
+  std::optional<base::Value::Dict> value_dict =
+      base::JSONReader::ReadDict(version_response);
+  if (!value_dict) {
+    return;
+  }
+  const std::string* browser_name = value_dict->FindString("Browser");
+  if (browser_name) {
+    std::vector<base::StringPiece> parts = base::SplitStringPiece(
+        *browser_name, "/", base::KEEP_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
+    if (parts.size() == 2) {
+      browser->version_ = parts[1];
+      is_chrome = parts[0] == "Chrome" || parts[0] == "HeadlessChrome";
+    } else {
+      browser->version_ = *browser_name;
+    }
+  }
+  browser->browser_target_id_ = GetTargetPath(*value_dict);
+  if (browser->browser_target_id_.empty()) {
+    browser->browser_target_id_ = kBrowserTargetSocket;
+  }
+  const std::string* package = value_dict->FindString("Android-Package");
+  if (package) {
+    browser->display_name_ =
+        AndroidDeviceManager::GetBrowserName(browser->socket(), *package);
+  }
+}
+
 void DevToolsDeviceDiscovery::DiscoveryRequest::ReceivedVersion(
     scoped_refptr<AndroidDeviceManager::Device> device,
     scoped_refptr<RemoteBrowser> browser,
@@ -450,33 +498,18 @@ void DevToolsDeviceDiscovery::DiscoveryRequest::ReceivedVersion(
     const std::string& response) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
-  device->SendJsonRequest(
-      browser->socket(), kPageListRequest,
-      base::BindOnce(&DiscoveryRequest::ReceivedPages, this, device, browser));
-
-  if (result < 0)
-    return;
-  // Parse version, append to package name if available,
-  base::Optional<base::Value> value = base::JSONReader::Read(response);
-  if (value && value->is_dict()) {
-    const std::string* browser_name = value->FindStringKey("Browser");
-    if (browser_name) {
-      std::vector<std::string> parts = base::SplitString(
-          *browser_name, "/", base::KEEP_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
-      if (parts.size() == 2)
-        browser->version_ = parts[1];
-      else
-        browser->version_ = *browser_name;
-    }
-    browser->browser_target_id_ = GetTargetPath(*value);
-    if (browser->browser_target_id_.empty())
-      browser->browser_target_id_ = kBrowserTargetSocket;
-    const std::string* package = value->FindStringKey("Android-Package");
-    if (package) {
-      browser->display_name_ =
-          AndroidDeviceManager::GetBrowserName(browser->socket(), *package);
-    }
+  std::string url = kPageListRequest;
+  bool is_chrome = false;
+  if (result >= 0) {
+    ParseBrowserInfo(browser, response, is_chrome);
   }
+  if (base::FeatureList::IsEnabled(::features::kDevToolsTabTarget) &&
+      is_chrome) {
+    url += "?for_tab";
+  }
+  device->SendJsonRequest(
+      browser->socket(), url,
+      base::BindOnce(&DiscoveryRequest::ReceivedPages, this, device, browser));
 }
 
 void DevToolsDeviceDiscovery::DiscoveryRequest::ReceivedPages(
@@ -485,15 +518,22 @@ void DevToolsDeviceDiscovery::DiscoveryRequest::ReceivedPages(
     int result,
     const std::string& response) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  if (result < 0)
+  if (result < 0) {
     return;
-  base::Optional<base::Value> value = base::JSONReader::Read(response);
-  if (value && value->is_list()) {
-    for (base::Value& page_value : value->GetList()) {
-      if (page_value.is_dict())
-        browser->pages_.push_back(new RemotePage(device, browser->browser_id_,
-                                                 browser->version_,
-                                                 std::move(page_value)));
+  }
+  std::optional<base::Value> value = base::JSONReader::Read(response);
+  if (!value) {
+    return;
+  }
+  auto* value_list = value->GetIfList();
+  if (!value_list) {
+    return;
+  }
+  for (base::Value& page_value : *value_list) {
+    if (page_value.is_dict()) {
+      browser->pages_.push_back(
+          new RemotePage(device, browser->browser_id_, browser->version_,
+                         std::move(page_value).TakeDict()));
     }
   }
 }
@@ -504,14 +544,13 @@ DevToolsDeviceDiscovery::RemotePage::RemotePage(
     scoped_refptr<AndroidDeviceManager::Device> device,
     const std::string& browser_id,
     const std::string& browser_version,
-    base::Value dict)
+    base::Value::Dict dict)
     : device_(device),
       browser_id_(browser_id),
       browser_version_(browser_version),
       dict_(std::move(dict)) {}
 
-DevToolsDeviceDiscovery::RemotePage::~RemotePage() {
-}
+DevToolsDeviceDiscovery::RemotePage::~RemotePage() {}
 
 scoped_refptr<content::DevToolsAgentHost>
 DevToolsDeviceDiscovery::RemotePage::CreateTarget() {
@@ -521,12 +560,12 @@ DevToolsDeviceDiscovery::RemotePage::CreateTarget() {
   std::string type = GetStringProperty(dict_, "type");
 
   std::string port_num = browser_id_;
-  if (type == "node")
+  if (type == "node") {
     port_num = GURL(GetStringProperty(dict_, "webSocketDebuggerUrl")).port();
+  }
 
   agent_host_ = AgentHostDelegate::GetOrCreateAgentHost(
-      device_, port_num, browser_version_, local_id, target_path, type,
-      &dict_);
+      device_, port_num, browser_version_, local_id, target_path, type, &dict_);
   return agent_host_;
 }
 
@@ -539,8 +578,7 @@ DevToolsDeviceDiscovery::RemoteBrowser::RemoteBrowser(
       browser_id_(browser_info.socket_name),
       display_name_(browser_info.display_name),
       user_(browser_info.user),
-      type_(browser_info.type) {
-}
+      type_(browser_info.type) {}
 
 bool DevToolsDeviceDiscovery::RemoteBrowser::IsChrome() {
   return type_ == AndroidDeviceManager::BrowserInfo::kTypeChrome;
@@ -553,8 +591,7 @@ std::string DevToolsDeviceDiscovery::RemoteBrowser::GetId() {
 DevToolsDeviceDiscovery::RemoteBrowser::ParsedVersion
 DevToolsDeviceDiscovery::RemoteBrowser::GetParsedVersion() {
   ParsedVersion result;
-  for (const base::StringPiece& part :
-       base::SplitStringPiece(
+  for (const base::StringPiece& part : base::SplitStringPiece(
            version_, ".", base::KEEP_WHITESPACE, base::SPLIT_WANT_NONEMPTY)) {
     int value = 0;
     base::StringToInt(part, &value);
@@ -563,8 +600,7 @@ DevToolsDeviceDiscovery::RemoteBrowser::GetParsedVersion() {
   return result;
 }
 
-DevToolsDeviceDiscovery::RemoteBrowser::~RemoteBrowser() {
-}
+DevToolsDeviceDiscovery::RemoteBrowser::~RemoteBrowser() {}
 
 // DevToolsDeviceDiscovery::RemoteDevice --------------------------------------
 
@@ -581,8 +617,7 @@ DevToolsDeviceDiscovery::RemoteDevice::RemoteDevice(
   }
 }
 
-DevToolsDeviceDiscovery::RemoteDevice::~RemoteDevice() {
-}
+DevToolsDeviceDiscovery::RemoteDevice::~RemoteDevice() {}
 
 // DevToolsDeviceDiscovery ----------------------------------------------------
 
