@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,27 +7,32 @@
 #include <set>
 #include <vector>
 
-#include "base/bind.h"
-#include "base/callback.h"
 #include "base/files/file_util.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/hash/md5.h"
+#include "base/json/values_util.h"
 #include "base/lazy_instance.h"
-#include "base/macros.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/task/post_task.h"
+#include "base/task/sequenced_task_runner.h"
 #include "base/task/thread_pool.h"
-#include "base/threading/sequenced_task_runner_handle.h"
-#include "base/util/values/values_util.h"
+#if !defined(TOOLKIT_QT)
 #include "chrome/browser/browser_process.h"
+#endif  // !defined(TOOLKIT_QT)
 #include "chrome/browser/devtools/devtools_file_watcher.h"
+#if !defined(TOOLKIT_QT)
 #include "chrome/browser/download/download_prefs.h"
+#endif  // !defined(TOOLKIT_QT)
 #include "chrome/browser/profiles/profile.h"
+#if !defined(TOOLKIT_QT)
 #include "chrome/browser/ui/chrome_select_file_policy.h"
+#endif  // !defined(TOOLKIT_QT)
 #include "chrome/common/pref_names.h"
 #include "chrome/grit/generated_resources.h"
 #include "components/prefs/pref_service.h"
 #include "components/prefs/scoped_user_pref_update.h"
 #include "content/public/browser/browser_context.h"
+#include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/child_process_security_policy.h"
 #include "content/public/browser/download_manager.h"
@@ -39,10 +44,21 @@
 #include "storage/browser/file_system/file_system_url.h"
 #include "storage/browser/file_system/isolated_context.h"
 #include "storage/common/file_system/file_system_util.h"
+#include "third_party/blink/public/common/storage_key/storage_key.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/shell_dialogs/select_file_dialog.h"
+#include "ui/shell_dialogs/selected_file_info.h"
+#include "url/gurl.h"
+#include "url/origin.h"
 
-using base::Bind;
+#if defined(TOOLKIT_QT)
+#include "content/public/browser/content_browser_client.h"
+#include "content/public/browser/download_manager_delegate.h"
+#include "ui/shell_dialogs/select_file_policy.h"
+namespace content {
+extern ContentClient* GetContentClient();
+}
+#endif  // !defined(TOOLKIT_QT)
 using content::BrowserContext;
 using content::BrowserThread;
 using content::DownloadManager;
@@ -64,11 +80,15 @@ typedef base::OnceCallback<void(void)> CanceledCallback;
 
 class SelectFileDialog : public ui::SelectFileDialog::Listener {
  public:
+  SelectFileDialog(const SelectFileDialog&) = delete;
+  SelectFileDialog& operator=(const SelectFileDialog&) = delete;
+
   static void Show(SelectedCallback selected_callback,
                    CanceledCallback canceled_callback,
                    WebContents* web_contents,
                    ui::SelectFileDialog::Type type,
                    const base::FilePath& default_path) {
+    // `dialog` is self-deleting.
     auto* dialog = new SelectFileDialog();
     dialog->ShowDialog(std::move(selected_callback),
                        std::move(canceled_callback), web_contents, type,
@@ -76,28 +96,23 @@ class SelectFileDialog : public ui::SelectFileDialog::Listener {
   }
 
   // ui::SelectFileDialog::Listener implementation.
-  void FileSelected(const base::FilePath& path,
+  void FileSelected(const ui::SelectedFileInfo& file,
                     int index,
                     void* params) override {
-    std::move(selected_callback_).Run(path);
+    std::move(selected_callback_).Run(file.path());
     delete this;
   }
 
-  void MultiFilesSelected(const std::vector<base::FilePath>& files,
-                          void* params) override {
-    delete this;
-    NOTREACHED() << "Should not be able to select multiple files";
-  }
-
   void FileSelectionCanceled(void* params) override {
-    if (!canceled_callback_.is_null())
+    if (canceled_callback_) {
       std::move(canceled_callback_).Run();
+    }
     delete this;
   }
 
  private:
   SelectFileDialog() = default;
-  ~SelectFileDialog() override = default;
+  ~SelectFileDialog() override { select_file_dialog_->ListenerDestroyed(); }
 
   void ShowDialog(SelectedCallback selected_callback,
                   CanceledCallback canceled_callback,
@@ -106,8 +121,16 @@ class SelectFileDialog : public ui::SelectFileDialog::Listener {
                   const base::FilePath& default_path) {
     selected_callback_ = std::move(selected_callback);
     canceled_callback_ = std::move(canceled_callback);
+#if !defined(TOOLKIT_QT)
     select_file_dialog_ = ui::SelectFileDialog::Create(
         this, std::make_unique<ChromeSelectFilePolicy>(web_contents));
+#else
+    select_file_dialog_ = ui::SelectFileDialog::Create(
+        this,
+        std::unique_ptr<ui::SelectFilePolicy>(
+            content::GetContentClient()->browser()->CreateSelectFilePolicy(
+                web_contents)));
+#endif  // !defined(TOOLKIT_QT)
     base::FilePath::StringType ext;
     ui::SelectFileDialog::FileTypeInfo file_type_info;
     if (type == ui::SelectFileDialog::SELECT_SAVEAS_FILE &&
@@ -117,27 +140,29 @@ class SelectFileDialog : public ui::SelectFileDialog::Listener {
       file_type_info.extensions[0].push_back(ext);
     }
     select_file_dialog_->SelectFile(
-        type, base::string16(), default_path, &file_type_info, 0, ext,
+        type, std::u16string(), default_path, &file_type_info, 0, ext,
+#if !defined(TOOLKIT_QT)
         platform_util::GetTopLevel(web_contents->GetNativeView()), nullptr);
+#else
+        nullptr, nullptr);
+#endif  // !defined(TOOLKIT_QT)
   }
 
   scoped_refptr<ui::SelectFileDialog> select_file_dialog_;
   SelectedCallback selected_callback_;
   CanceledCallback canceled_callback_;
-
-  DISALLOW_COPY_AND_ASSIGN(SelectFileDialog);
 };
 
 void WriteToFile(const base::FilePath& path, const std::string& content) {
   DCHECK(!path.empty());
 
-  base::WriteFile(path, content.c_str(), content.length());
+  base::WriteFile(path, content);
 }
 
 void AppendToFile(const base::FilePath& path, const std::string& content) {
   DCHECK(!path.empty());
 
-  base::AppendToFile(path, content.c_str(), content.size());
+  base::AppendToFile(path, content);
 }
 
 storage::IsolatedContext* isolated_context() {
@@ -151,7 +176,8 @@ storage::IsolatedContext* isolated_context() {
 std::string RegisterFileSystem(WebContents* web_contents,
                                const base::FilePath& path) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  CHECK(web_contents->GetURL().SchemeIs(content::kChromeDevToolsScheme));
+  CHECK(web_contents->GetLastCommittedURL().SchemeIs(
+      content::kChromeDevToolsScheme));
   std::string root_name(kRootName);
   storage::IsolatedContext::ScopedFSHandle file_system =
       isolated_context()->RegisterFileSystemForPath(
@@ -160,7 +186,7 @@ std::string RegisterFileSystem(WebContents* web_contents,
   content::ChildProcessSecurityPolicy* policy =
       content::ChildProcessSecurityPolicy::GetInstance();
   RenderViewHost* render_view_host =
-      web_contents->GetMainFrame()->GetRenderViewHost();
+      web_contents->GetPrimaryMainFrame()->GetRenderViewHost();
   int renderer_id = render_view_host->GetProcess()->GetID();
   policy->GrantReadFileSystem(renderer_id, file_system.id());
   policy->GrantWriteFileSystem(renderer_id, file_system.id());
@@ -180,7 +206,8 @@ DevToolsFileHelper::FileSystem CreateFileSystemStruct(
     const std::string& type,
     const std::string& file_system_id,
     const std::string& file_system_path) {
-  const GURL origin = web_contents->GetURL().GetOrigin();
+  const GURL origin =
+      web_contents->GetLastCommittedURL().DeprecatedGetOriginAsURL();
   std::string file_system_name =
       storage::GetIsolatedFileSystemName(origin, file_system_id);
   std::string root_url = storage::GetIsolatedFileSystemRootURIString(
@@ -191,14 +218,13 @@ DevToolsFileHelper::FileSystem CreateFileSystemStruct(
 
 using PathToType = std::map<std::string, std::string>;
 PathToType GetAddedFileSystemPaths(Profile* profile) {
-  const base::DictionaryValue* file_systems_paths_value =
-      profile->GetPrefs()->GetDictionary(prefs::kDevToolsFileSystemPaths);
+  const base::Value::Dict& file_systems_paths_value =
+      profile->GetPrefs()->GetDict(prefs::kDevToolsFileSystemPaths);
   PathToType result;
-  for (base::DictionaryValue::Iterator it(*file_systems_paths_value);
-       !it.IsAtEnd(); it.Advance()) {
+  for (auto pair : file_systems_paths_value) {
     std::string type =
-        it.value().is_string() ? it.value().GetString() : std::string();
-    result[it.key()] = type;
+        pair.second.is_string() ? pair.second.GetString() : std::string();
+    result[pair.first] = type;
   }
   return result;
 }
@@ -244,31 +270,52 @@ void DevToolsFileHelper::Save(const std::string& url,
     return;
   }
 
-  const base::DictionaryValue* file_map =
-      profile_->GetPrefs()->GetDictionary(prefs::kDevToolsEditedFiles);
+  const base::Value::Dict& file_map =
+      profile_->GetPrefs()->GetDict(prefs::kDevToolsEditedFiles);
   base::FilePath initial_path;
 
-  const base::Value* path_value;
-  if (file_map->Get(base::MD5String(url), &path_value)) {
-    base::Optional<base::FilePath> path = util::ValueToFilePath(*path_value);
+  if (const base::Value* path_value = file_map.Find(base::MD5String(url))) {
+    std::optional<base::FilePath> path = base::ValueToFilePath(*path_value);
     if (path)
       initial_path = std::move(*path);
   }
 
   if (initial_path.empty()) {
     GURL gurl(url);
-    std::string suggested_file_name = gurl.is_valid() ?
-        gurl.ExtractFileName() : url;
-
+    std::string suggested_file_name;
+    if (gurl.is_valid()) {
+      url::RawCanonOutputW<1024> unescaped_content;
+      std::string escaped_content = gurl.ExtractFileName();
+      url::DecodeURLEscapeSequences(escaped_content,
+                                    url::DecodeURLMode::kUTF8OrIsomorphic,
+                                    &unescaped_content);
+      // TODO(crbug.com/1324254): Due to filename encoding on Windows we can't
+      // expect to always be able to convert to UTF8 and back
+      std::string unescaped_content_string =
+          base::UTF16ToUTF8(unescaped_content.view());
+      suggested_file_name = unescaped_content_string;
+    } else {
+      suggested_file_name = url;
+    }
+    // TODO(crbug.com/1324254): Truncate a UTF8 string in a better way
     if (suggested_file_name.length() > 64)
       suggested_file_name = suggested_file_name.substr(0, 64);
-
+    // TODO(crbug.com/1324254): Ensure suggested_file_name is an ASCII string
     if (!g_last_save_path.Pointer()->empty()) {
       initial_path = g_last_save_path.Pointer()->DirName().AppendASCII(
           suggested_file_name);
     } else {
-      base::FilePath download_path = DownloadPrefs::FromDownloadManager(
-          BrowserContext::GetDownloadManager(profile_))->DownloadPath();
+#if !defined(TOOLKIT_QT)
+      base::FilePath download_path =
+          DownloadPrefs::FromDownloadManager(profile_->GetDownloadManager())
+              ->DownloadPath();
+#else
+      base::FilePath download_path;
+      base::FilePath website_save_directory;  // Unused
+      profile_->GetDownloadManagerDelegate()->GetSaveDir(
+          web_contents_->GetBrowserContext(), &website_save_directory,
+          &download_path);
+#endif  // !defined(TOOLKIT_QT)
       initial_path = download_path.AppendASCII(suggested_file_name);
     }
   }
@@ -299,10 +346,10 @@ void DevToolsFileHelper::SaveAsFileSelected(const std::string& url,
   *g_last_save_path.Pointer() = path;
   saved_files_[url] = path;
 
-  DictionaryPrefUpdate update(profile_->GetPrefs(),
+  ScopedDictPrefUpdate update(profile_->GetPrefs(),
                               prefs::kDevToolsEditedFiles);
-  base::DictionaryValue* files_map = update.Get();
-  files_map->SetKey(base::MD5String(url), util::FilePathToValue(path));
+  base::Value::Dict& files_map = update.Get();
+  files_map.Set(base::MD5String(url), base::FilePathToValue(path));
   std::string file_system_path = path.AsUTF8Unsafe();
   std::move(callback).Run(file_system_path);
   file_task_runner_->PostTask(FROM_HERE, BindOnce(&WriteToFile, path, content));
@@ -322,8 +369,9 @@ void DevToolsFileHelper::AddFileSystem(
 void DevToolsFileHelper::UpgradeDraggedFileSystemPermissions(
     const std::string& file_system_url,
     const ShowInfoBarCallback& show_info_bar_callback) {
-  storage::FileSystemURL root_url =
-      isolated_context()->CrackURL(GURL(file_system_url));
+  const GURL gurl(file_system_url);
+  storage::FileSystemURL root_url = isolated_context()->CrackURL(
+      gurl, blink::StorageKey::CreateFirstParty(url::Origin::Create(gurl)));
   if (!root_url.is_valid() || !root_url.path().empty())
     return;
 
@@ -346,13 +394,17 @@ void DevToolsFileHelper::InnerAddFileSystem(
   if (IsFileSystemAdded(file_system_path))
     RemoveFileSystem(file_system_path);
 
+#if !defined(TOOLKIT_QT)
   std::string path_display_name = path.AsEndingWithSeparator().AsUTF8Unsafe();
-  base::string16 message = l10n_util::GetStringFUTF16(
-      IDS_DEV_TOOLS_CONFIRM_ADD_FILE_SYSTEM_MESSAGE,
-      base::UTF8ToUTF16(path_display_name));
+  std::u16string message =
+      l10n_util::GetStringFUTF16(IDS_DEV_TOOLS_CONFIRM_ADD_FILE_SYSTEM_MESSAGE,
+                                 base::UTF8ToUTF16(path_display_name));
   show_info_bar_callback.Run(
       message, BindOnce(&DevToolsFileHelper::AddUserConfirmedFileSystem,
                         weak_factory_.GetWeakPtr(), type, path));
+#else
+  AddUserConfirmedFileSystem(type, path, true);
+#endif  // !defined(TOOLKIT_QT)
 }
 
 void DevToolsFileHelper::AddUserConfirmedFileSystem(const std::string& type,
@@ -366,17 +418,24 @@ void DevToolsFileHelper::AddUserConfirmedFileSystem(const std::string& type,
   std::string file_system_id = RegisterFileSystem(web_contents_, path);
   std::string file_system_path = path.AsUTF8Unsafe();
 
-  DictionaryPrefUpdate update(profile_->GetPrefs(),
+  ScopedDictPrefUpdate update(profile_->GetPrefs(),
                               prefs::kDevToolsFileSystemPaths);
-  base::DictionaryValue* file_systems_paths_value = update.Get();
-  file_systems_paths_value->SetWithoutPathExpansion(
-      file_system_path, std::make_unique<base::Value>(type));
+  base::Value::Dict& file_systems_paths_value = update.Get();
+  file_systems_paths_value.Set(file_system_path, type);
 }
 
 void DevToolsFileHelper::FailedToAddFileSystem(const std::string& error) {
   delegate_->FileSystemAdded(error, nullptr);
 }
 
+namespace {
+
+void RunOnUIThread(base::OnceClosure callback) {
+  content::GetUIThreadTaskRunner({})->PostTask(FROM_HERE, std::move(callback));
+}
+
+}  // namespace
+
 std::vector<DevToolsFileHelper::FileSystem>
 DevToolsFileHelper::GetFileSystems() {
   file_system_paths_ = GetAddedFileSystemPaths(profile_);
@@ -385,11 +444,13 @@ DevToolsFileHelper::GetFileSystems() {
     file_watcher_.reset(new DevToolsFileWatcher(
         base::BindRepeating(&DevToolsFileHelper::FilePathsChanged,
                             weak_factory_.GetWeakPtr()),
-        base::SequencedTaskRunnerHandle::Get()));
+        base::SequencedTaskRunner::GetCurrentDefault()));
+    auto change_handler_on_ui = base::BindRepeating(
+        &DevToolsFileHelper::FileSystemPathsSettingChangedOnUI,
+        weak_factory_.GetWeakPtr());
     pref_change_registrar_.Add(
         prefs::kDevToolsFileSystemPaths,
-        base::BindRepeating(&DevToolsFileHelper::FileSystemPathsSettingChanged,
-                            base::Unretained(this)));
+        base::BindRepeating(RunOnUIThread, change_handler_on_ui));
   }
   for (auto file_system_path : file_system_paths_) {
     base::FilePath path =
@@ -409,21 +470,22 @@ void DevToolsFileHelper::RemoveFileSystem(const std::string& file_system_path) {
   base::FilePath path = base::FilePath::FromUTF8Unsafe(file_system_path);
   isolated_context()->RevokeFileSystemByPath(path);
 
-  DictionaryPrefUpdate update(profile_->GetPrefs(),
+  ScopedDictPrefUpdate update(profile_->GetPrefs(),
                               prefs::kDevToolsFileSystemPaths);
-  base::DictionaryValue* file_systems_paths_value = update.Get();
-  file_systems_paths_value->RemoveKey(file_system_path);
+  base::Value::Dict& file_systems_paths_value = update.Get();
+  file_systems_paths_value.Remove(file_system_path);
 }
 
 bool DevToolsFileHelper::IsFileSystemAdded(
     const std::string& file_system_path) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
-  const base::DictionaryValue* file_systems_paths_value =
-      profile_->GetPrefs()->GetDictionary(prefs::kDevToolsFileSystemPaths);
-  return file_systems_paths_value->HasKey(file_system_path);
+  const base::Value::Dict& file_systems_paths_value =
+      profile_->GetPrefs()->GetDict(prefs::kDevToolsFileSystemPaths);
+  return file_systems_paths_value.Find(file_system_path);
 }
 
+#if !defined(TOOLKIT_QT)
 void DevToolsFileHelper::OnOpenItemComplete(
     const base::FilePath& path,
     platform_util::OpenOperationResult result) {
@@ -442,8 +504,10 @@ void DevToolsFileHelper::ShowItemInFolder(const std::string& file_system_path) {
       base::BindOnce(&DevToolsFileHelper::OnOpenItemComplete,
                      weak_factory_.GetWeakPtr(), path));
 }
+#endif  // !defined(TOOLKIT_QT)
 
-void DevToolsFileHelper::FileSystemPathsSettingChanged() {
+void DevToolsFileHelper::FileSystemPathsSettingChangedOnUI() {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
   PathToType remaining;
   remaining.swap(file_system_paths_);
   DCHECK(file_watcher_.get());
