@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,6 +8,8 @@
 #include <tuple>
 #include <utility>
 
+#include "base/memory/raw_ptr_exclusion.h"
+#include "base/test/gmock_callback_support.h"
 #include "base/test/gmock_move_support.h"
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "content/public/test/browser_test.h"
@@ -25,10 +27,12 @@
 #include "extensions/test/result_catcher.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
+using base::test::RunOnceCallback;
 using device::BluetoothAdapter;
 using device::BluetoothDevice;
 using device::BluetoothGattConnection;
 using device::BluetoothGattNotifySession;
+using device::BluetoothGattService;
 using device::BluetoothRemoteGattCharacteristic;
 using device::BluetoothRemoteGattDescriptor;
 using device::BluetoothRemoteGattService;
@@ -197,7 +201,10 @@ class BluetoothLowEnergyApiTest : public extensions::ExtensionApiTest {
         ->event_router();
   }
 
-  testing::StrictMock<MockBluetoothAdapter>* mock_adapter_;
+  // This field is not a raw_ptr<> because problems related to passing to a
+  // templated && parameter, which is later forwarded to something that doesn't
+  // vibe with raw_ptr<T>.
+  RAW_PTR_EXCLUSION testing::StrictMock<MockBluetoothAdapter>* mock_adapter_;
   std::unique_ptr<testing::NiceMock<MockBluetoothDevice>> device0_;
   std::unique_ptr<testing::NiceMock<MockBluetoothDevice>> device1_;
   std::unique_ptr<testing::NiceMock<MockBluetoothGattService>> service0_;
@@ -224,19 +231,30 @@ ACTION_TEMPLATE(InvokeCallbackArgument,
   std::move(std::get<k>(args)).Run(p0);
 }
 
+ACTION_TEMPLATE(InvokeCallbackArgument,
+                HAS_1_TEMPLATE_PARAMS(int, k),
+                AND_2_VALUE_PARAMS(p0, p1)) {
+  std::move(std::get<k>(args)).Run(p0, p1);
+}
+
 ACTION_TEMPLATE(InvokeCallbackWithScopedPtrArg,
                 HAS_2_TEMPLATE_PARAMS(int, k, typename, T),
                 AND_1_VALUE_PARAMS(p0)) {
   std::move(std::get<k>(args)).Run(std::unique_ptr<T>(p0));
 }
 
-BluetoothGattConnection* CreateGattConnection(
+ACTION_TEMPLATE(InvokeCallbackWithScopedPtrArg,
+                HAS_2_TEMPLATE_PARAMS(int, k, typename, T),
+                AND_2_VALUE_PARAMS(p0, p1)) {
+  std::move(std::get<k>(args)).Run(std::unique_ptr<T>(p0), p1);
+}
+
+std::unique_ptr<BluetoothGattConnection> CreateGattConnection(
     scoped_refptr<device::BluetoothAdapter> adapter,
     const std::string& device_address,
     bool expect_disconnect) {
-  testing::NiceMock<MockBluetoothGattConnection>* conn =
-      new testing::NiceMock<MockBluetoothGattConnection>(adapter,
-                                                         device_address);
+  auto conn = std::make_unique<testing::NiceMock<MockBluetoothGattConnection>>(
+      adapter, device_address);
   EXPECT_CALL(*conn, Disconnect()).Times(expect_disconnect ? 1 : 0);
   return conn;
 }
@@ -251,7 +269,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetServices) {
 
   EXPECT_CALL(*mock_adapter_, GetDevice(_))
       .Times(3)
-      .WillOnce(Return(static_cast<BluetoothDevice*>(NULL)))
+      .WillOnce(Return(static_cast<BluetoothDevice*>(nullptr)))
       .WillRepeatedly(Return(device0_.get()));
 
   EXPECT_CALL(*device0_, GetGattServices())
@@ -260,7 +278,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetServices) {
       .WillOnce(Return(services));
 
   // Load and wait for setup.
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("bluetooth_low_energy/get_services")));
@@ -280,16 +298,16 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetService) {
 
   EXPECT_CALL(*mock_adapter_, GetDevice(_))
       .Times(3)
-      .WillOnce(Return(static_cast<BluetoothDevice*>(NULL)))
+      .WillOnce(Return(static_cast<BluetoothDevice*>(nullptr)))
       .WillRepeatedly(Return(device0_.get()));
 
   EXPECT_CALL(*device0_, GetGattService(kTestServiceId0))
       .Times(2)
-      .WillOnce(Return(static_cast<BluetoothRemoteGattService*>(NULL)))
+      .WillOnce(Return(static_cast<BluetoothRemoteGattService*>(nullptr)))
       .WillOnce(Return(service0_.get()));
 
   // Load and wait for setup.
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("bluetooth_low_energy/get_service")));
@@ -308,7 +326,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, ServiceEvents) {
   catcher.RestrictToBrowserContext(browser()->profile());
 
   // Load the extension and let it set up.
-  ExtensionTestMessageListener listener(true);
+  ExtensionTestMessageListener listener(ReplyBehavior::kWillReply);
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("bluetooth_low_energy/service_events")));
   EXPECT_TRUE(listener.WaitUntilSatisfied());
@@ -356,7 +374,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetRemovedService) {
       .Times(1)
       .WillOnce(Return(service0_.get()));
 
-  ExtensionTestMessageListener get_service_success_listener(false);
+  ExtensionTestMessageListener get_service_success_listener;
 
   event_router()->GattServiceAdded(
       mock_adapter_, device0_.get(), service0_.get());
@@ -373,7 +391,8 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetRemovedService) {
   EXPECT_CALL(*mock_adapter_, GetDevice(_)).Times(0);
   EXPECT_CALL(*device0_, GetGattService(kTestServiceId0)).Times(0);
 
-  ExtensionTestMessageListener get_service_fail_listener(true);
+  ExtensionTestMessageListener get_service_fail_listener(
+      ReplyBehavior::kWillReply);
 
   event_router()->GattServiceRemoved(
       mock_adapter_, device0_.get(), service0_.get());
@@ -394,7 +413,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetIncludedServices) {
   catcher.RestrictToBrowserContext(browser()->profile());
 
   // Wait for initial call to end with failure as there is no mapping.
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
 
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
@@ -448,14 +467,14 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetCharacteristics) {
       Return(device0_.get()));
   EXPECT_CALL(*device0_, GetGattService(kTestServiceId0))
       .Times(3)
-      .WillOnce(Return(static_cast<BluetoothRemoteGattService*>(NULL)))
+      .WillOnce(Return(static_cast<BluetoothRemoteGattService*>(nullptr)))
       .WillRepeatedly(Return(service0_.get()));
   EXPECT_CALL(*service0_, GetCharacteristics())
       .Times(2)
       .WillOnce(Return(std::vector<BluetoothRemoteGattCharacteristic*>()))
       .WillOnce(Return(characteristics));
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("bluetooth_low_energy/get_characteristics")));
   EXPECT_TRUE(listener.WaitUntilSatisfied());
@@ -477,21 +496,22 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetCharacteristic) {
 
   EXPECT_CALL(*mock_adapter_, GetDevice(_))
       .Times(4)
-      .WillOnce(Return(static_cast<BluetoothDevice*>(NULL)))
+      .WillOnce(Return(static_cast<BluetoothDevice*>(nullptr)))
       .WillRepeatedly(Return(device0_.get()));
 
   EXPECT_CALL(*device0_, GetGattService(kTestServiceId0))
       .Times(3)
-      .WillOnce(Return(static_cast<BluetoothRemoteGattService*>(NULL)))
+      .WillOnce(Return(static_cast<BluetoothRemoteGattService*>(nullptr)))
       .WillRepeatedly(Return(service0_.get()));
 
   EXPECT_CALL(*service0_, GetCharacteristic(kTestCharacteristicId0))
       .Times(2)
-      .WillOnce(Return(static_cast<BluetoothRemoteGattCharacteristic*>(NULL)))
+      .WillOnce(
+          Return(static_cast<BluetoothRemoteGattCharacteristic*>(nullptr)))
       .WillOnce(Return(chrc0_.get()));
 
   // Load the extension and wait for first test.
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("bluetooth_low_energy/get_characteristic")));
@@ -554,7 +574,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, CharacteristicProperties) {
           BluetoothRemoteGattCharacteristic::PROPERTY_RELIABLE_WRITE |
           BluetoothRemoteGattCharacteristic::PROPERTY_WRITABLE_AUXILIARIES));
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/characteristic_properties")));
@@ -587,7 +607,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetRemovedCharacteristic) {
       mock_adapter_, device0_.get(), service0_.get());
   event_router()->GattCharacteristicAdded(mock_adapter_, chrc0_.get());
 
-  ExtensionTestMessageListener listener(true);
+  ExtensionTestMessageListener listener(ReplyBehavior::kWillReply);
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/get_removed_characteristic")));
 
@@ -659,7 +679,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, CharacteristicValueChanged) {
       .WillOnce(InvokeCallbackWithScopedPtrArg<0, BluetoothGattNotifySession>(
           session1));
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/characteristic_value_changed")));
 
@@ -703,13 +723,14 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, ReadCharacteristicValue) {
       .WillRepeatedly(Return(chrc0_.get()));
 
   std::vector<uint8_t> value;
-  EXPECT_CALL(*chrc0_, ReadRemoteCharacteristic_(_, _))
+  EXPECT_CALL(*chrc0_, ReadRemoteCharacteristic_(_))
       .Times(2)
-      .WillOnce(InvokeCallbackArgument<1>(
-          BluetoothRemoteGattService::GATT_ERROR_FAILED))
-      .WillOnce(InvokeCallbackArgument<0>(value));
+      .WillOnce(InvokeCallbackArgument<0>(
+          BluetoothGattService::GattErrorCode::kFailed,
+          /*value=*/std::vector<uint8_t>()))
+      .WillOnce(InvokeCallbackArgument<0>(std::nullopt, value));
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/read_characteristic_value")));
@@ -748,12 +769,12 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, WriteCharacteristicValue) {
   EXPECT_CALL(*chrc0_, DeprecatedWriteRemoteCharacteristic_(_, _, _))
       .Times(2)
       .WillOnce(InvokeCallbackArgument<2>(
-          BluetoothRemoteGattService::GATT_ERROR_FAILED))
+          BluetoothGattService::GattErrorCode::kFailed))
       .WillOnce(DoAll(SaveArg<0>(&write_value), InvokeCallbackArgument<1>()));
 
   EXPECT_CALL(*chrc0_, GetValue()).Times(1).WillOnce(ReturnRef(write_value));
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/write_characteristic_value")));
@@ -788,14 +809,15 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetDescriptors) {
       .WillRepeatedly(Return(service0_.get()));
   EXPECT_CALL(*service0_, GetCharacteristic(kTestCharacteristicId0))
       .Times(3)
-      .WillOnce(Return(static_cast<BluetoothRemoteGattCharacteristic*>(NULL)))
+      .WillOnce(
+          Return(static_cast<BluetoothRemoteGattCharacteristic*>(nullptr)))
       .WillRepeatedly(Return(chrc0_.get()));
   EXPECT_CALL(*chrc0_, GetDescriptors())
       .Times(2)
       .WillOnce(Return(std::vector<BluetoothRemoteGattDescriptor*>()))
       .WillOnce(Return(descriptors));
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("bluetooth_low_energy/get_descriptors")));
@@ -821,26 +843,27 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetDescriptor) {
 
   EXPECT_CALL(*mock_adapter_, GetDevice(_))
       .Times(5)
-      .WillOnce(Return(static_cast<BluetoothDevice*>(NULL)))
+      .WillOnce(Return(static_cast<BluetoothDevice*>(nullptr)))
       .WillRepeatedly(Return(device0_.get()));
 
   EXPECT_CALL(*device0_, GetGattService(kTestServiceId0))
       .Times(4)
-      .WillOnce(Return(static_cast<BluetoothRemoteGattService*>(NULL)))
+      .WillOnce(Return(static_cast<BluetoothRemoteGattService*>(nullptr)))
       .WillRepeatedly(Return(service0_.get()));
 
   EXPECT_CALL(*service0_, GetCharacteristic(kTestCharacteristicId0))
       .Times(3)
-      .WillOnce(Return(static_cast<BluetoothRemoteGattCharacteristic*>(NULL)))
+      .WillOnce(
+          Return(static_cast<BluetoothRemoteGattCharacteristic*>(nullptr)))
       .WillRepeatedly(Return(chrc0_.get()));
 
   EXPECT_CALL(*chrc0_, GetDescriptor(kTestDescriptorId0))
       .Times(2)
-      .WillOnce(Return(static_cast<BluetoothRemoteGattDescriptor*>(NULL)))
+      .WillOnce(Return(static_cast<BluetoothRemoteGattDescriptor*>(nullptr)))
       .WillOnce(Return(desc0_.get()));
 
   // Load the extension and wait for first test.
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("bluetooth_low_energy/get_descriptor")));
@@ -878,7 +901,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GetRemovedDescriptor) {
   event_router()->GattCharacteristicAdded(mock_adapter_, chrc0_.get());
   event_router()->GattDescriptorAdded(mock_adapter_, desc0_.get());
 
-  ExtensionTestMessageListener listener(true);
+  ExtensionTestMessageListener listener(ReplyBehavior::kWillReply);
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/get_removed_descriptor")));
 
@@ -920,7 +943,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, DescriptorValueChanged) {
   event_router()->GattDescriptorAdded(mock_adapter_, desc1_.get());
 
   // Load the extension and let it set up.
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/descriptor_value_changed")));
   EXPECT_TRUE(listener.WaitUntilSatisfied());
@@ -968,25 +991,32 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, ReadDescriptorValue) {
       .WillRepeatedly(Return(desc0_.get()));
 
   std::vector<uint8_t> value;
-  EXPECT_CALL(*desc0_, ReadRemoteDescriptor_(_, _))
+  EXPECT_CALL(*desc0_, ReadRemoteDescriptor_(_))
       .Times(8)
-      .WillOnce(InvokeCallbackArgument<1>(
-          BluetoothRemoteGattService::GATT_ERROR_FAILED))
-      .WillOnce(InvokeCallbackArgument<1>(
-          BluetoothRemoteGattService::GATT_ERROR_INVALID_LENGTH))
-      .WillOnce(InvokeCallbackArgument<1>(
-          BluetoothRemoteGattService::GATT_ERROR_NOT_PERMITTED))
-      .WillOnce(InvokeCallbackArgument<1>(
-          BluetoothRemoteGattService::GATT_ERROR_NOT_AUTHORIZED))
-      .WillOnce(InvokeCallbackArgument<1>(
-          BluetoothRemoteGattService::GATT_ERROR_NOT_PAIRED))
-      .WillOnce(InvokeCallbackArgument<1>(
-          BluetoothRemoteGattService::GATT_ERROR_NOT_SUPPORTED))
-      .WillOnce(InvokeCallbackArgument<1>(
-          BluetoothRemoteGattService::GATT_ERROR_IN_PROGRESS))
-      .WillOnce(InvokeCallbackArgument<0>(value));
+      .WillOnce(InvokeCallbackArgument<0>(
+          BluetoothGattService::GattErrorCode::kFailed,
+          /*value=*/std::vector<uint8_t>()))
+      .WillOnce(InvokeCallbackArgument<0>(
+          BluetoothGattService::GattErrorCode::kInvalidLength,
+          /*value=*/std::vector<uint8_t>()))
+      .WillOnce(InvokeCallbackArgument<0>(
+          BluetoothGattService::GattErrorCode::kNotPermitted,
+          /*value=*/std::vector<uint8_t>()))
+      .WillOnce(InvokeCallbackArgument<0>(
+          BluetoothGattService::GattErrorCode::kNotAuthorized,
+          /*value=*/std::vector<uint8_t>()))
+      .WillOnce(InvokeCallbackArgument<0>(
+          BluetoothGattService::GattErrorCode::kNotPaired,
+          /*value=*/std::vector<uint8_t>()))
+      .WillOnce(InvokeCallbackArgument<0>(
+          BluetoothGattService::GattErrorCode::kNotSupported,
+          /*value=*/std::vector<uint8_t>()))
+      .WillOnce(InvokeCallbackArgument<0>(
+          BluetoothGattService::GattErrorCode::kInProgress,
+          /*value=*/std::vector<uint8_t>()))
+      .WillOnce(InvokeCallbackArgument<0>(/*error_code=*/std::nullopt, value));
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/read_descriptor_value")));
@@ -1031,12 +1061,12 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, WriteDescriptorValue) {
   EXPECT_CALL(*desc0_, WriteRemoteDescriptor_(_, _, _))
       .Times(2)
       .WillOnce(InvokeCallbackArgument<2>(
-          BluetoothRemoteGattService::GATT_ERROR_FAILED))
+          BluetoothGattService::GattErrorCode::kFailed))
       .WillOnce(DoAll(SaveArg<0>(&write_value), InvokeCallbackArgument<1>()));
 
   EXPECT_CALL(*desc0_, GetValue()).Times(1).WillOnce(ReturnRef(write_value));
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/write_descriptor_value")));
@@ -1097,7 +1127,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, UuidPermissionEvents) {
   ResultCatcher catcher;
   catcher.RestrictToBrowserContext(browser()->profile());
 
-  ExtensionTestMessageListener listener(true);
+  ExtensionTestMessageListener listener(ReplyBehavior::kWillReply);
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/uuid_permission_events")));
 
@@ -1130,35 +1160,44 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, GattConnection) {
   catcher.RestrictToBrowserContext(browser()->profile());
 
   EXPECT_CALL(*mock_adapter_, GetDevice(_))
-      .WillRepeatedly(Return(static_cast<BluetoothDevice*>(NULL)));
+      .WillRepeatedly(Return(static_cast<BluetoothDevice*>(nullptr)));
   EXPECT_CALL(*mock_adapter_, GetDevice(kTestLeDeviceAddress0))
       .WillRepeatedly(Return(device0_.get()));
   EXPECT_CALL(*mock_adapter_, GetDevice(kTestLeDeviceAddress1))
       .WillRepeatedly(Return(device1_.get()));
   static_assert(
-      BluetoothDevice::NUM_CONNECT_ERROR_CODES == 8,
+      BluetoothDevice::NUM_CONNECT_ERROR_CODES == 14,
       "Update required if the number of BluetoothDevice enums changes.");
-  EXPECT_CALL(*device0_, CreateGattConnection_(_, _))
+  EXPECT_CALL(*device0_, CreateGattConnection(_, _))
       .Times(9)
-      .WillOnce(InvokeCallbackArgument<1>(BluetoothDevice::ERROR_FAILED))
-      .WillOnce(InvokeCallbackArgument<1>(BluetoothDevice::ERROR_INPROGRESS))
-      .WillOnce(InvokeCallbackArgument<1>(BluetoothDevice::ERROR_AUTH_FAILED))
-      .WillOnce(InvokeCallbackArgument<1>(BluetoothDevice::ERROR_AUTH_REJECTED))
-      .WillOnce(InvokeCallbackArgument<1>(BluetoothDevice::ERROR_AUTH_CANCELED))
-      .WillOnce(InvokeCallbackArgument<1>(BluetoothDevice::ERROR_AUTH_TIMEOUT))
-      .WillOnce(
-          InvokeCallbackArgument<1>(BluetoothDevice::ERROR_UNSUPPORTED_DEVICE))
-      .WillOnce(InvokeCallbackWithScopedPtrArg<0, BluetoothGattConnection>(
+      .WillOnce(RunOnceCallback<0>(/*connection=*/nullptr,
+                                   BluetoothDevice::ERROR_FAILED))
+      .WillOnce(RunOnceCallback<0>(/*connection=*/nullptr,
+                                   BluetoothDevice::ERROR_INPROGRESS))
+      .WillOnce(RunOnceCallback<0>(/*connection=*/nullptr,
+                                   BluetoothDevice::ERROR_AUTH_FAILED))
+      .WillOnce(RunOnceCallback<0>(/*connection=*/nullptr,
+                                   BluetoothDevice::ERROR_AUTH_REJECTED))
+      .WillOnce(RunOnceCallback<0>(/*connection=*/nullptr,
+                                   BluetoothDevice::ERROR_AUTH_CANCELED))
+      .WillOnce(RunOnceCallback<0>(/*connection=*/nullptr,
+                                   BluetoothDevice::ERROR_AUTH_TIMEOUT))
+      .WillOnce(RunOnceCallback<0>(
+          /*connection=*/nullptr, BluetoothDevice::ERROR_UNSUPPORTED_DEVICE))
+      .WillOnce(RunOnceCallback<0>(
           CreateGattConnection(mock_adapter_, kTestLeDeviceAddress0,
-                               true /* expect_disconnect */)))
-      .WillOnce(InvokeCallbackWithScopedPtrArg<0, BluetoothGattConnection>(
+                               /*expect_disconnect=*/true),
+          /*error_code=*/std::nullopt))
+      .WillOnce(RunOnceCallback<0>(
           CreateGattConnection(mock_adapter_, kTestLeDeviceAddress0,
-                               false /* expect_disconnect */)));
-  EXPECT_CALL(*device1_, CreateGattConnection_(_, _))
+                               /*expect_disconnect=*/false),
+          /*error_code=*/std::nullopt));
+  EXPECT_CALL(*device1_, CreateGattConnection(_, _))
       .Times(1)
-      .WillOnce(InvokeCallbackWithScopedPtrArg<0, BluetoothGattConnection>(
+      .WillOnce(RunOnceCallback<0>(
           CreateGattConnection(mock_adapter_, kTestLeDeviceAddress1,
-                               true /* expect_disconnect */)));
+                               /*expect_disconnect=*/true),
+          /*error_code=*/std::nullopt));
 
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("bluetooth_low_energy/gatt_connection")));
@@ -1172,21 +1211,23 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, ReconnectAfterDisconnected) {
   EXPECT_CALL(*mock_adapter_, GetDevice(kTestLeDeviceAddress0))
       .WillRepeatedly(Return(device0_.get()));
 
-  MockBluetoothGattConnection* first_conn =
-      static_cast<MockBluetoothGattConnection*>(CreateGattConnection(
-          mock_adapter_, kTestLeDeviceAddress0, false /* expect_disconnect */));
-  EXPECT_CALL(*first_conn, IsConnected())
+  auto first_conn = CreateGattConnection(mock_adapter_, kTestLeDeviceAddress0,
+                                         /*expect_disconnect=*/false);
+  MockBluetoothGattConnection* first_conn_ptr =
+      static_cast<MockBluetoothGattConnection*>(first_conn.get());
+  EXPECT_CALL(*first_conn_ptr, IsConnected())
       .Times(2)
       .WillOnce(Return(true))
       .WillOnce(Return(false));
 
-  EXPECT_CALL(*device0_, CreateGattConnection_(_, _))
+  EXPECT_CALL(*device0_, CreateGattConnection(_, _))
       .Times(2)
-      .WillOnce(InvokeCallbackWithScopedPtrArg<0, BluetoothGattConnection>(
-          first_conn))
-      .WillOnce(InvokeCallbackWithScopedPtrArg<0, BluetoothGattConnection>(
+      .WillOnce(RunOnceCallback<0>(std::move(first_conn),
+                                   /*error_code=*/std::nullopt))
+      .WillOnce(RunOnceCallback<0>(
           CreateGattConnection(mock_adapter_, kTestLeDeviceAddress0,
-                               false /* expect_disconnect */)));
+                               /*expect_disconnect=*/false),
+          /*error_code=*/std::nullopt));
 
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/reconnect_after_disconnected")));
@@ -1208,11 +1249,11 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, ConnectInProgress) {
   std::unique_ptr<BluetoothGattConnection> conn_ptr(conn);
   EXPECT_CALL(*conn, Disconnect()).Times(1);
 
-  EXPECT_CALL(*device0_, CreateGattConnection_(_, _))
+  EXPECT_CALL(*device0_, CreateGattConnection(_, _))
       .Times(1)
       .WillOnce(MoveArg<0>(&connect_callback));
 
-  ExtensionTestMessageListener listener(false);
+  ExtensionTestMessageListener listener;
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/connect_in_progress")));
 
@@ -1222,7 +1263,8 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, ConnectInProgress) {
       << listener.message();
   listener.Reset();
 
-  std::move(connect_callback).Run(std::move(conn_ptr));
+  std::move(connect_callback)
+      .Run(std::move(conn_ptr), /*error_code=*/std::nullopt);
   EXPECT_TRUE(listener.WaitUntilSatisfied());
   ASSERT_EQ("After 2nd call to disconnect.", listener.message())
       << listener.message();
@@ -1272,7 +1314,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, StartStopNotifications) {
   EXPECT_CALL(*chrc0_, StartNotifySession_(_, _))
       .Times(2)
       .WillOnce(InvokeCallbackArgument<1>(
-          BluetoothRemoteGattService::GATT_ERROR_FAILED))
+          BluetoothGattService::GattErrorCode::kFailed))
       .WillOnce(InvokeCallbackWithScopedPtrArg<0, BluetoothGattNotifySession>(
           session0));
   EXPECT_CALL(*chrc1_, StartNotifySession_(_, _))
@@ -1280,7 +1322,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, StartStopNotifications) {
       .WillOnce(InvokeCallbackWithScopedPtrArg<0, BluetoothGattNotifySession>(
           session1));
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
       "bluetooth_low_energy/start_stop_notifications")));
@@ -1328,7 +1370,7 @@ IN_PROC_BROWSER_TEST_F(BluetoothLowEnergyApiTest, AddressChange) {
                                        kTestLeDeviceAddress0);
 
   // Load and wait for setup.
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   listener.set_failure_message("fail");
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("bluetooth_low_energy/address_change")));
