@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,20 +11,25 @@
 #include "base/memory/raw_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/path_service.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/stringprintf.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/test/metrics/histogram_tester.h"
 #include "base/test/scoped_mock_time_message_loop_task_runner.h"
 #include "base/test/simple_test_tick_clock.h"
+#include "base/test/to_vector.h"
 #include "base/test/values_test_util.h"
+#include "base/values.h"
 #include "build/chromeos_buildflags.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/extensions/api/debugger/debugger_api.h"
 #include "chrome/browser/extensions/api/debugger/extension_dev_tools_infobar_delegate.h"
 #include "chrome/browser/extensions/extension_apitest.h"
-#include "chrome/browser/extensions/extension_function_test_utils.h"
 #include "chrome/browser/extensions/extension_management_test_util.h"
+#include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/profiles/profile_destroyer.h"
 #include "chrome/browser/profiles/profile_manager.h"
+#include "chrome/browser/profiles/profile_test_util.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_switches.h"
@@ -36,18 +41,30 @@
 #include "components/infobars/core/infobar_delegate.h"
 #include "components/policy/core/common/mock_configuration_policy_provider.h"
 #include "components/prefs/pref_service.h"
+#include "components/security_interstitials/content/security_interstitial_controller_client.h"
+#include "components/security_interstitials/content/security_interstitial_page.h"
+#include "components/security_interstitials/content/security_interstitial_tab_helper.h"
+#include "components/security_interstitials/content/settings_page_helper.h"
+#include "components/security_interstitials/core/metrics_helper.h"
 #include "components/sessions/content/session_tab_helper.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/browser_test_utils.h"
+#include "content/public/test/mock_navigation_handle.h"
 #include "content/public/test/no_renderer_crashes_assertion.h"
+#include "extensions/browser/api_test_utils.h"
 #include "extensions/browser/extension_function.h"
 #include "extensions/common/extension.h"
 #include "extensions/common/extension_builder.h"
 #include "extensions/common/manifest_constants.h"
 #include "extensions/common/switches.h"
-#include "extensions/common/value_builder.h"
 #include "extensions/test/test_extension_dir.h"
 #include "net/dns/mock_host_resolver.h"
+#include "pdf/buildflags.h"
+
+#if BUILDFLAG(ENABLE_PDF)
+#include "base/test/with_feature_override.h"
+#include "pdf/pdf_features.h"
+#endif  // BUILDFLAG(ENABLE_PDF)
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
 #include "chrome/browser/ash/profiles/profile_helper.h"
@@ -69,6 +86,9 @@ class DebuggerApiTest : public ExtensionApiTest {
   // to succeed.
   testing::AssertionResult RunAttachFunction(const GURL& url,
                                              const std::string& expected_error);
+  testing::AssertionResult RunAttachFunction(
+      const content::WebContents* web_contents,
+      const std::string& expected_error);
 
   const Extension* extension() const { return extension_.get(); }
   base::CommandLine* command_line() const { return command_line_; }
@@ -116,9 +136,13 @@ void DebuggerApiTest::SetUpOnMainThread() {
 testing::AssertionResult DebuggerApiTest::RunAttachFunction(
     const GURL& url, const std::string& expected_error) {
   EXPECT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
-  content::WebContents* web_contents =
-      browser()->tab_strip_model()->GetActiveWebContents();
+  return RunAttachFunction(browser()->tab_strip_model()->GetActiveWebContents(),
+                           expected_error);
+}
 
+testing::AssertionResult DebuggerApiTest::RunAttachFunction(
+    const content::WebContents* web_contents,
+    const std::string& expected_error) {
   // Attach by tabId.
   int tab_id = sessions::SessionTabHelper::IdForTab(web_contents).id();
   std::string debugee_by_tab = base::StringPrintf("{\"tabId\": %d}", tab_id);
@@ -130,20 +154,19 @@ testing::AssertionResult DebuggerApiTest::RunAttachFunction(
   // Attach by targetId.
   scoped_refptr<DebuggerGetTargetsFunction> get_targets_function =
       new DebuggerGetTargetsFunction();
-  std::unique_ptr<base::Value> value(
-      extension_function_test_utils::RunFunctionAndReturnSingleResult(
-          get_targets_function.get(), "[]", browser()));
+  std::optional<base::Value> value(
+      api_test_utils::RunFunctionAndReturnSingleResult(
+          get_targets_function.get(), "[]", profile()));
   EXPECT_TRUE(value->is_list());
-  const base::ListValue& targets = base::Value::AsListValue(*value);
 
   std::string debugger_target_id;
-  for (const base::Value& target_value : targets.GetListDeprecated()) {
+  for (const base::Value& target_value : value->GetList()) {
     EXPECT_TRUE(target_value.is_dict());
-    absl::optional<int> id = target_value.FindIntKey("tabId");
+    std::optional<int> id = target_value.GetDict().FindInt("tabId");
     if (id == tab_id) {
-      const base::DictionaryValue& target_dict =
-          base::Value::AsDictionaryValue(target_value);
-      EXPECT_TRUE(target_dict.GetString("id", &debugger_target_id));
+      const std::string* id_str = target_value.GetDict().FindString("id");
+      EXPECT_TRUE(id_str);
+      debugger_target_id = *id_str;
       break;
     }
   }
@@ -161,20 +184,19 @@ testing::AssertionResult DebuggerApiTest::RunAttachFunctionOnTarget(
   attach_function->set_extension(extension_.get());
 
   std::string actual_error;
-  if (!extension_function_test_utils::RunFunction(
+  if (!api_test_utils::RunFunction(
           attach_function.get(),
           base::StringPrintf("[%s, \"1.1\"]", debuggee_target.c_str()),
-          browser(), api_test_utils::NONE)) {
+          profile())) {
     actual_error = attach_function->GetError();
   } else {
     // Clean up and detach.
     scoped_refptr<DebuggerDetachFunction> detach_function =
         new DebuggerDetachFunction();
     detach_function->set_extension(extension_.get());
-    if (!extension_function_test_utils::RunFunction(
+    if (!api_test_utils::RunFunction(
             detach_function.get(),
-            base::StringPrintf("[%s]", debuggee_target.c_str()), browser(),
-            api_test_utils::NONE)) {
+            base::StringPrintf("[%s]", debuggee_target.c_str()), profile())) {
       return testing::AssertionFailure() << "Could not detach from "
           << debuggee_target << " : " << detach_function->GetError();
     }
@@ -229,6 +251,60 @@ IN_PROC_BROWSER_TEST_F(DebuggerApiTest,
   EXPECT_TRUE(RunExtensionTest("debugger_file_access")) << message_;
 }
 
+class TestInterstitialPage
+    : public security_interstitials::SecurityInterstitialPage {
+ public:
+  TestInterstitialPage(content::WebContents* web_contents,
+                       const GURL& request_url)
+      : SecurityInterstitialPage(
+            web_contents,
+            request_url,
+            std::make_unique<
+                security_interstitials::SecurityInterstitialControllerClient>(
+                web_contents,
+                CreateTestMetricsHelper(web_contents),
+                nullptr,
+                base::i18n::GetConfiguredLocale(),
+                GURL(),
+                /* settings_page_helper*/ nullptr)) {}
+
+  ~TestInterstitialPage() override = default;
+  void OnInterstitialClosing() override {}
+
+ protected:
+  void PopulateInterstitialStrings(base::Value::Dict& load_time_data) override {
+  }
+
+  std::unique_ptr<security_interstitials::MetricsHelper>
+  CreateTestMetricsHelper(content::WebContents* web_contents) {
+    security_interstitials::MetricsHelper::ReportDetails report_details;
+    report_details.metric_prefix = "test_blocking_page";
+    return std::make_unique<security_interstitials::MetricsHelper>(
+        GURL(), report_details, nullptr);
+  }
+};
+
+IN_PROC_BROWSER_TEST_F(DebuggerApiTest,
+                       DebuggerNotAllowedOnSecirutyInterstitials) {
+  content::WebContents* web_contents =
+      browser()->tab_strip_model()->GetActiveWebContents();
+  std::unique_ptr<content::MockNavigationHandle> navigation_handle =
+      std::make_unique<content::MockNavigationHandle>(
+          GURL("https://google.com/"), web_contents->GetPrimaryMainFrame());
+  navigation_handle->set_has_committed(true);
+  navigation_handle->set_is_same_document(false);
+  EXPECT_TRUE(RunAttachFunction(web_contents, ""));
+
+  security_interstitials::SecurityInterstitialTabHelper::AssociateBlockingPage(
+      navigation_handle.get(),
+      std::make_unique<TestInterstitialPage>(web_contents, GURL()));
+  security_interstitials::SecurityInterstitialTabHelper::FromWebContents(
+      web_contents)
+      ->DidFinishNavigation(navigation_handle.get());
+
+  EXPECT_TRUE(RunAttachFunction(web_contents, "Cannot attach to this target."));
+}
+
 IN_PROC_BROWSER_TEST_F(DebuggerApiTest, InfoBar) {
   int tab_id = sessions::SessionTabHelper::IdForTab(
                    browser()->tab_strip_model()->GetActiveWebContents())
@@ -257,104 +333,102 @@ IN_PROC_BROWSER_TEST_F(DebuggerApiTest, InfoBar) {
   // Attaching to one tab should create infobars in both browsers.
   attach_function = new DebuggerAttachFunction();
   attach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       attach_function.get(),
-      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), browser(),
-      api_test_utils::NONE));
-  EXPECT_EQ(1u, manager1->infobar_count());
-  EXPECT_EQ(1u, manager2->infobar_count());
-  EXPECT_EQ(1u, manager3->infobar_count());
+      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), profile()));
+  EXPECT_EQ(1u, manager1->infobars().size());
+  EXPECT_EQ(1u, manager2->infobars().size());
+  EXPECT_EQ(1u, manager3->infobars().size());
 
   // Attaching to another tab should not create more infobars.
   attach_function = new DebuggerAttachFunction();
   attach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       attach_function.get(),
-      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id2), browser(),
-      api_test_utils::NONE));
-  EXPECT_EQ(1u, manager1->infobar_count());
-  EXPECT_EQ(1u, manager2->infobar_count());
-  EXPECT_EQ(1u, manager3->infobar_count());
+      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id2), profile()));
+  EXPECT_EQ(1u, manager1->infobars().size());
+  EXPECT_EQ(1u, manager2->infobars().size());
+  EXPECT_EQ(1u, manager3->infobars().size());
 
   // Detaching from one of the tabs should not remove infobars.
   detach_function = new DebuggerDetachFunction();
   detach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       detach_function.get(), base::StringPrintf("[{\"tabId\": %d}]", tab_id2),
-      browser(), api_test_utils::NONE));
-  EXPECT_EQ(1u, manager1->infobar_count());
-  EXPECT_EQ(1u, manager2->infobar_count());
-  EXPECT_EQ(1u, manager3->infobar_count());
+      profile()));
+  EXPECT_EQ(1u, manager1->infobars().size());
+  EXPECT_EQ(1u, manager2->infobars().size());
+  EXPECT_EQ(1u, manager3->infobars().size());
 
   // Detaching from the other tab also should not remove infobars, since even
   // though there is no longer an extension attached, the infobar can only be
   // dismissed by explicit user action.
   detach_function = new DebuggerDetachFunction();
   detach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       detach_function.get(), base::StringPrintf("[{\"tabId\": %d}]", tab_id),
-      browser(), api_test_utils::NONE));
-  EXPECT_EQ(1u, manager1->infobar_count());
-  EXPECT_EQ(1u, manager2->infobar_count());
-  EXPECT_EQ(1u, manager3->infobar_count());
+      profile()));
+  EXPECT_EQ(1u, manager1->infobars().size());
+  EXPECT_EQ(1u, manager2->infobars().size());
+  EXPECT_EQ(1u, manager3->infobars().size());
 
   // Attach again; should not create infobars.
   attach_function = new DebuggerAttachFunction();
   attach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       attach_function.get(),
-      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), browser(),
-      api_test_utils::NONE));
-  EXPECT_EQ(1u, manager1->infobar_count());
-  EXPECT_EQ(1u, manager2->infobar_count());
-  EXPECT_EQ(1u, manager3->infobar_count());
+      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), profile()));
+  EXPECT_EQ(1u, manager1->infobars().size());
+  EXPECT_EQ(1u, manager2->infobars().size());
+  EXPECT_EQ(1u, manager3->infobars().size());
 
   // Remove the global infobar by simulating what happens when the user clicks
   // the close button (see InfoBarView::ButtonPressed()).  The
   // InfoBarDismissed() call will remove the infobars everywhere except on
   // |manager2| itself; the RemoveSelf() call removes that one.
-  manager2->infobar_at(0)->delegate()->InfoBarDismissed();
-  manager2->infobar_at(0)->RemoveSelf();
-  EXPECT_EQ(0u, manager1->infobar_count());
-  EXPECT_EQ(0u, manager2->infobar_count());
-  EXPECT_EQ(0u, manager3->infobar_count());
+  manager2->infobars()[0]->delegate()->InfoBarDismissed();
+  manager2->infobars()[0]->RemoveSelf();
+  EXPECT_EQ(0u, manager1->infobars().size());
+  EXPECT_EQ(0u, manager2->infobars().size());
+  EXPECT_EQ(0u, manager3->infobars().size());
   detach_function = new DebuggerDetachFunction();
   detach_function->set_extension(extension());
   // Cannot detach again.
-  ASSERT_FALSE(extension_function_test_utils::RunFunction(
+  ASSERT_FALSE(api_test_utils::RunFunction(
       detach_function.get(), base::StringPrintf("[{\"tabId\": %d}]", tab_id),
-      browser(), api_test_utils::NONE));
+      profile()));
 
   // Attaching once again should create a new infobar.
   attach_function = new DebuggerAttachFunction();
   attach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       attach_function.get(),
-      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), browser(),
-      api_test_utils::NONE));
-  EXPECT_EQ(1u, manager1->infobar_count());
-  EXPECT_EQ(1u, manager2->infobar_count());
-  EXPECT_EQ(1u, manager3->infobar_count());
+      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), profile()));
+  EXPECT_EQ(1u, manager1->infobars().size());
+  EXPECT_EQ(1u, manager2->infobars().size());
+  EXPECT_EQ(1u, manager3->infobars().size());
 
   // Closing tab should not affect anything.
-  ASSERT_TRUE(another_browser->tab_strip_model()->CloseWebContentsAt(1, 0));
+  EXPECT_EQ(2, another_browser->tab_strip_model()->count());
+  another_browser->tab_strip_model()->CloseWebContentsAt(1, 0);
+  EXPECT_EQ(1, another_browser->tab_strip_model()->count());
   manager3 = nullptr;
-  EXPECT_EQ(1u, manager1->infobar_count());
-  EXPECT_EQ(1u, manager2->infobar_count());
+  EXPECT_EQ(1u, manager1->infobars().size());
+  EXPECT_EQ(1u, manager2->infobars().size());
 
   // Closing browser should not affect anything.
   CloseBrowserSynchronously(another_browser);
   manager2 = nullptr;
   another_browser = nullptr;
-  EXPECT_EQ(1u, manager1->infobar_count());
+  EXPECT_EQ(1u, manager1->infobars().size());
 
   // Detach should not affect anything.
   detach_function = new DebuggerDetachFunction();
   detach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       detach_function.get(), base::StringPrintf("[{\"tabId\": %d}]", tab_id),
-      browser(), api_test_utils::NONE));
-  EXPECT_EQ(1u, manager1->infobar_count());
+      profile()));
+  EXPECT_EQ(1u, manager1->infobars().size());
 }
 
 IN_PROC_BROWSER_TEST_F(DebuggerApiTest, InfoBarIsRemovedAfterFiveSeconds) {
@@ -368,33 +442,118 @@ IN_PROC_BROWSER_TEST_F(DebuggerApiTest, InfoBarIsRemovedAfterFiveSeconds) {
   // Attaching to the tab should create an infobar.
   auto attach_function = base::MakeRefCounted<DebuggerAttachFunction>();
   attach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       attach_function.get(),
-      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), browser(),
-      api_test_utils::NONE));
-  EXPECT_EQ(1u, manager->infobar_count());
+      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), profile()));
+  EXPECT_EQ(1u, manager->infobars().size());
 
   // Detaching from the tab should remove the infobar after 5 seconds.
   auto detach_function = base::MakeRefCounted<DebuggerDetachFunction>();
   detach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       detach_function.get(), base::StringPrintf("[{\"tabId\": %d}]", tab_id),
-      browser(), api_test_utils::NONE));
+      profile()));
 
   // Even though the extension detached, the infobar should not detach
   // immediately, and should remain visible for 5 seconds to ensure the user
   // has an opportunity to see it.
   base::RunLoop run_loop;
-  base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+  base::SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
       FROM_HERE, run_loop.QuitClosure(),
       ExtensionDevToolsInfoBarDelegate::kAutoCloseDelay);
-  EXPECT_EQ(1u, manager->infobar_count());  // Infobar is still shown.
+  EXPECT_EQ(1u, manager->infobars().size());  // Infobar is still shown.
 
   // Advance the clock by 5 seconds, and verify the infobar is removed.
   AdvanceClock(ExtensionDevToolsInfoBarDelegate::kAutoCloseDelay);
   run_loop.Run();
 
-  EXPECT_EQ(0u, manager->infobar_count());
+  EXPECT_EQ(0u, manager->infobars().size());
+}
+
+IN_PROC_BROWSER_TEST_F(DebuggerApiTest,
+                       InfoBarIsNotRemovedWhenAnotherDebuggerAttached) {
+  const int tab_id1 = sessions::SessionTabHelper::IdForTab(
+                          browser()->tab_strip_model()->GetActiveWebContents())
+                          .id();
+  infobars::ContentInfoBarManager* manager =
+      infobars::ContentInfoBarManager::FromWebContents(
+          browser()->tab_strip_model()->GetActiveWebContents());
+
+  ASSERT_TRUE(embedded_test_server()->Start());
+  ASSERT_TRUE(ui_test_utils::NavigateToURLWithDisposition(
+      browser(), embedded_test_server()->GetURL("/simple.html"),
+      WindowOpenDisposition::NEW_FOREGROUND_TAB,
+      ui_test_utils::BROWSER_TEST_WAIT_FOR_LOAD_STOP));
+  const int tab_id2 = sessions::SessionTabHelper::IdForTab(
+                          browser()->tab_strip_model()->GetActiveWebContents())
+                          .id();
+
+  // Attaching to a tab should create an infobar.
+  {
+    auto attach_function = base::MakeRefCounted<DebuggerAttachFunction>();
+    attach_function->set_extension(extension());
+    ASSERT_TRUE(api_test_utils::RunFunction(
+        attach_function.get(),
+        base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id1), profile()));
+  }
+
+  EXPECT_EQ(1u, manager->infobars().size());
+
+  // Attaching to a 2nd tab, to have another attached debugger.
+  {
+    auto attach_function = base::MakeRefCounted<DebuggerAttachFunction>();
+    attach_function->set_extension(extension());
+    ASSERT_TRUE(api_test_utils::RunFunction(
+        attach_function.get(),
+        base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id2), profile()));
+  }
+
+  EXPECT_EQ(1u, manager->infobars().size());
+
+  // Detaching from the tab should not remove the infobar after 5 seconds, as
+  // another debugger is still attached.
+  {
+    auto detach_function = base::MakeRefCounted<DebuggerDetachFunction>();
+    detach_function->set_extension(extension());
+    ASSERT_TRUE(api_test_utils::RunFunction(
+        detach_function.get(), base::StringPrintf("[{\"tabId\": %d}]", tab_id1),
+        profile()));
+  }
+
+  // Advance the clock by 5 seconds.
+  {
+    base::RunLoop run_loop;
+    base::SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
+        FROM_HERE, run_loop.QuitClosure(),
+        ExtensionDevToolsInfoBarDelegate::kAutoCloseDelay);
+    AdvanceClock(ExtensionDevToolsInfoBarDelegate::kAutoCloseDelay);
+    run_loop.Run();
+  }
+
+  // Verify inforbar not removed.
+  EXPECT_EQ(1u, manager->infobars().size());
+
+  // Now detach the last debugger.
+  {
+    auto detach_function = base::MakeRefCounted<DebuggerDetachFunction>();
+    detach_function->set_extension(extension());
+    ASSERT_TRUE(api_test_utils::RunFunction(
+        detach_function.get(), base::StringPrintf("[{\"tabId\": %d}]", tab_id2),
+        profile()));
+  }
+
+  // Advance the clock by 5 seconds, once again.
+  {
+    base::RunLoop run_loop;
+    base::SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
+        FROM_HERE, run_loop.QuitClosure(),
+        ExtensionDevToolsInfoBarDelegate::kAutoCloseDelay);
+    AdvanceClock(ExtensionDevToolsInfoBarDelegate::kAutoCloseDelay);
+    run_loop.Run();
+  }
+
+  // Verify inforbar removed.
+  EXPECT_EQ(0u, manager->infobars().size());
 }
 
 class CrossProfileDebuggerApiTest : public DebuggerApiTest {
@@ -420,30 +579,19 @@ class CrossProfileDebuggerApiTest : public DebuggerApiTest {
     DebuggerApiTest::SetUpOnMainThread();
     profile_manager_ = g_browser_process->profile_manager();
 
-    base::RunLoop run_loop;
-    profile_manager_->CreateProfileAsync(
-        profile_manager_->GenerateNextProfileDirectoryPath(),
-        base::BindRepeating(
-            [](CrossProfileDebuggerApiTest* self, base::RunLoop* run_loop,
-               Profile* profile, Profile::CreateStatus status) {
-              if (status == Profile::CREATE_STATUS_INITIALIZED) {
-                self->other_profile_ = profile;
-                run_loop->Quit();
-              }
-            },
-            this, &run_loop));
-    run_loop.Run();
+    other_profile_ = &profiles::testing::CreateProfileSync(
+        profile_manager_, profile_manager_->GenerateNextProfileDirectoryPath());
     otr_profile_ = profile()->GetPrimaryOTRProfile(true);
   }
 
   void TearDownOnMainThread() override {
-    ProfileDestroyer::DestroyProfileWhenAppropriate(otr_profile_);
+    ProfileDestroyer::DestroyOTRProfileWhenAppropriate(otr_profile_);
     DebuggerApiTest::TearDownOnMainThread();
   }
 
-  ProfileManager* profile_manager_ = nullptr;
-  Profile* other_profile_ = nullptr;
-  Profile* otr_profile_ = nullptr;
+  raw_ptr<ProfileManager, DanglingUntriaged> profile_manager_ = nullptr;
+  raw_ptr<Profile, DanglingUntriaged> other_profile_ = nullptr;
+  raw_ptr<Profile, DanglingUntriaged> otr_profile_ = nullptr;
 };
 
 IN_PROC_BROWSER_TEST_F(CrossProfileDebuggerApiTest, GetTargets) {
@@ -457,31 +605,29 @@ IN_PROC_BROWSER_TEST_F(CrossProfileDebuggerApiTest, GetTargets) {
   {
     auto get_targets_function =
         base::MakeRefCounted<DebuggerGetTargetsFunction>();
-    base::Value value = std::move(
-        *extension_function_test_utils::RunFunctionAndReturnSingleResult(
-            get_targets_function.get(), "[]", browser()));
+    base::Value value =
+        std::move(*api_test_utils::RunFunctionAndReturnSingleResult(
+            get_targets_function.get(), "[]", profile()));
 
     ASSERT_TRUE(value.is_list());
-    const base::Value::List targets = std::move(value.GetList());
+    const base::Value::List targets = std::move(value).TakeList();
     ASSERT_THAT(targets, testing::SizeIs(1));
-    EXPECT_THAT(targets[0], base::test::DictionaryHasValue(
-                                "url", base::Value("about:blank")));
+    EXPECT_THAT(targets[0].GetDict(), base::test::DictionaryHasValue(
+                                          "url", base::Value("about:blank")));
   }
 
   {
     auto get_targets_function =
         base::MakeRefCounted<DebuggerGetTargetsFunction>();
-    base::Value value = std::move(
-        *extension_function_test_utils::RunFunctionAndReturnSingleResult(
-            get_targets_function.get(), "[]", browser(),
-            api_test_utils::RunFunctionFlags::INCLUDE_INCOGNITO));
+    base::Value value =
+        std::move(*api_test_utils::RunFunctionAndReturnSingleResult(
+            get_targets_function.get(), "[]", profile(),
+            api_test_utils::FunctionMode::kIncognito));
 
     ASSERT_TRUE(value.is_list());
-    const base::Value::List targets = std::move(value.GetList());
-    std::vector<std::string> urls;
-    std::transform(
-        targets.begin(), targets.end(), std::back_inserter(urls),
-        [](const base::Value& value) -> std::string {
+    const base::Value::List targets = std::move(value).TakeList();
+    std::vector<std::string> urls =
+        base::test::ToVector(targets, [](const base::Value& value) {
           GURL::Replacements remove_port;
           remove_port.ClearPort();
           const std::string* url = value.GetDict().FindString("url");
@@ -506,17 +652,16 @@ IN_PROC_BROWSER_TEST_F(CrossProfileDebuggerApiTest, Attach) {
     auto debugger_attach_function =
         base::MakeRefCounted<DebuggerAttachFunction>();
     debugger_attach_function->set_extension(extension());
-    EXPECT_FALSE(extension_function_test_utils::RunFunction(
-        debugger_attach_function.get(), target_in_other_profile, browser(),
-        api_test_utils::NONE));
+    EXPECT_FALSE(api_test_utils::RunFunction(
+        debugger_attach_function.get(), target_in_other_profile, profile()));
   }
   {
     auto debugger_attach_function =
         base::MakeRefCounted<DebuggerAttachFunction>();
     debugger_attach_function->set_extension(extension());
-    EXPECT_FALSE(extension_function_test_utils::RunFunction(
+    EXPECT_FALSE(api_test_utils::RunFunction(
         debugger_attach_function.get(), target_in_other_profile.c_str(),
-        browser(), api_test_utils::INCLUDE_INCOGNITO));
+        profile(), api_test_utils::FunctionMode::kIncognito));
   }
 
   auto wc2 = CreateTabWithProfileAndNavigate(
@@ -530,17 +675,17 @@ IN_PROC_BROWSER_TEST_F(CrossProfileDebuggerApiTest, Attach) {
     auto debugger_attach_function =
         base::MakeRefCounted<DebuggerAttachFunction>();
     debugger_attach_function->set_extension(extension());
-    EXPECT_FALSE(extension_function_test_utils::RunFunction(
-        debugger_attach_function.get(), target_in_otr_profile.c_str(),
-        browser(), api_test_utils::NONE));
+    EXPECT_FALSE(api_test_utils::RunFunction(debugger_attach_function.get(),
+                                             target_in_otr_profile.c_str(),
+                                             profile()));
   }
   {
     auto debugger_attach_function =
         base::MakeRefCounted<DebuggerAttachFunction>();
     debugger_attach_function->set_extension(extension());
-    EXPECT_TRUE(extension_function_test_utils::RunFunction(
+    EXPECT_TRUE(api_test_utils::RunFunction(
         debugger_attach_function.get(), target_in_otr_profile.c_str(),
-        browser(), api_test_utils::INCLUDE_INCOGNITO));
+        profile(), api_test_utils::FunctionMode::kIncognito));
   }
 }
 
@@ -556,39 +701,37 @@ IN_PROC_BROWSER_TEST_F(DebuggerApiTest,
   // Attaching to the tab should create an infobar.
   auto attach_function = base::MakeRefCounted<DebuggerAttachFunction>();
   attach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       attach_function.get(),
-      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), browser(),
-      api_test_utils::NONE));
-  EXPECT_EQ(1u, manager->infobar_count());
+      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), profile()));
+  EXPECT_EQ(1u, manager->infobars().size());
 
   // Detaching from the tab and attaching it again before 5 seconds should not
   // remove the infobar.
   auto detach_function = base::MakeRefCounted<DebuggerDetachFunction>();
   detach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       detach_function.get(), base::StringPrintf("[{\"tabId\": %d}]", tab_id),
-      browser(), api_test_utils::NONE));
-  EXPECT_EQ(1u, manager->infobar_count());
+      profile()));
+  EXPECT_EQ(1u, manager->infobars().size());
 
   attach_function = base::MakeRefCounted<DebuggerAttachFunction>();
   attach_function->set_extension(extension());
-  ASSERT_TRUE(extension_function_test_utils::RunFunction(
+  ASSERT_TRUE(api_test_utils::RunFunction(
       attach_function.get(),
-      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), browser(),
-      api_test_utils::NONE));
+      base::StringPrintf("[{\"tabId\": %d}, \"1.1\"]", tab_id), profile()));
   // Verify that only one infobar is created.
-  EXPECT_EQ(1u, manager->infobar_count());
+  EXPECT_EQ(1u, manager->infobars().size());
 
   // Verify that infobar is not closed after 5 seconds.
   base::RunLoop run_loop;
-  base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+  base::SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
       FROM_HERE, run_loop.QuitClosure(),
       ExtensionDevToolsInfoBarDelegate::kAutoCloseDelay);
   AdvanceClock(ExtensionDevToolsInfoBarDelegate::kAutoCloseDelay);
   run_loop.Run();
 
-  EXPECT_EQ(1u, manager->infobar_count());
+  EXPECT_EQ(1u, manager->infobars().size());
 }
 
 // Tests that policy blocked hosts supersede the `debugger`
@@ -623,6 +766,12 @@ IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, ParentTargetPermissions) {
   ASSERT_TRUE(RunExtensionTest("parent_target_permissions")) << message_;
 }
 
+IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, ReloadAndResetHistory) {
+  // Run test with file access disabled.
+  ASSERT_TRUE(RunExtensionTest("debugger_reload_and_reset_history"))
+      << message_;
+}
+
 // Tests that an extension is not allowed to inspect a worker through the
 // inspectWorker debugger command.
 // Regression test for https://crbug.com/1059577.
@@ -640,19 +789,54 @@ IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, AttachToEmptyUrls) {
   ASSERT_TRUE(RunExtensionTest("debugger_attach_to_empty_urls")) << message_;
 }
 
-IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, AttachToPdf) {
+#if BUILDFLAG(ENABLE_PDF)
+class DebuggerExtensionApiPdfTest : public base::test::WithFeatureOverride,
+                                    public DebuggerExtensionApiTest {
+ public:
+  DebuggerExtensionApiPdfTest()
+      : base::test::WithFeatureOverride(chrome_pdf::features::kPdfOopif) {}
+};
+
+IN_PROC_BROWSER_TEST_P(DebuggerExtensionApiPdfTest, AttachToPdf) {
+  // TODO(crbug.com/1445746): Remove this once the test passes for OOPIF PDF.
+  if (IsParamFeatureEnabled()) {
+    GTEST_SKIP();
+  }
+
   ASSERT_TRUE(RunExtensionTest("debugger_attach_to_pdf")) << message_;
 }
 
+// TODO(crbug.com/1445746): Stop testing both modes after OOPIF PDF viewer
+// launches.
+INSTANTIATE_FEATURE_OVERRIDE_TEST_SUITE(DebuggerExtensionApiPdfTest);
+#endif  // BUILDFLAG(ENABLE_PDF)
+
 // Tests that navigation to a forbidden URL is properly denied and
 // does not cause a crash.
 // This is a regression test for https://crbug.com/1188889.
-IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, NavigateToForbiddenUrl) {
+// TODO(crbug.com/1517512): Re-enable this test.
+#if BUILDFLAG(IS_WIN)
+#define MAYBE_NavigateToForbiddenUrl DISABLED_NavigateToForbiddenUrl
+#else
+#define MAYBE_NavigateToForbiddenUrl NavigateToForbiddenUrl
+#endif
+IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, MAYBE_NavigateToForbiddenUrl) {
   content::ScopedAllowRendererCrashes scoped_allow_renderer_crashes;
   ASSERT_TRUE(RunExtensionTest("debugger_navigate_to_forbidden_url"))
       << message_;
 }
 
+IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, NavigateToUntrustedWebUIUrl) {
+  ASSERT_TRUE(RunExtensionTest("debugger_navigate_to_untrusted_webui_url"))
+      << message_;
+}
+
+// Tests that Target.createTarget to WebUI origins are blocked.
+IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, CreateTargetToUntrustedWebUI) {
+  ASSERT_TRUE(RunExtensionTest("debugger_create_target_to_untrusted_webui"))
+      << message_;
+}
+
 IN_PROC_BROWSER_TEST_F(DebuggerExtensionApiTest, IsDeveloperModeTrueHistogram) {
   profile()->GetPrefs()->SetBoolean(prefs::kExtensionsUIDeveloperMode, true);
   base::HistogramTester histograms;
@@ -693,8 +877,8 @@ IN_PROC_BROWSER_TEST_F(SitePerProcessDebuggerExtensionApiTest, Debugger) {
   content::TestNavigationManager navigation_manager_iframe(tab, iframe_url);
   tab->GetController().LoadURL(url, content::Referrer(),
                                ui::PAGE_TRANSITION_LINK, std::string());
-  navigation_manager.WaitForNavigationFinished();
-  navigation_manager_iframe.WaitForNavigationFinished();
+  ASSERT_TRUE(navigation_manager.WaitForNavigationFinished());
+  ASSERT_TRUE(navigation_manager_iframe.WaitForNavigationFinished());
   EXPECT_TRUE(content::WaitForLoadStop(tab));
 
   ASSERT_TRUE(RunExtensionTest("debugger",
