@@ -1,4 +1,4 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,8 +8,8 @@
 #include <string>
 #include <utility>
 
-#include "base/callback.h"
 #include "base/callback_list.h"
+#include "base/functional/callback.h"
 #include "base/lazy_instance.h"
 #include "base/memory/ptr_util.h"
 #include "base/strings/utf_string_conversions.h"
@@ -61,18 +61,6 @@ ExtensionDevToolsInfoBarDelegate::~ExtensionDevToolsInfoBarDelegate() {
   DCHECK(erased);
 }
 
-void ExtensionDevToolsInfoBarDelegate::NotifyExtensionDetached(
-    const std::string& extension_id) {
-  const Delegates& delegates = g_delegates.Get();
-  const auto iter = delegates.find(extension_id);
-  if (iter != delegates.cend()) {
-    // Infobar_ was set in Create() which makes the following access safe.
-    iter->second->timer_.Start(FROM_HERE, kAutoCloseDelay,
-                               iter->second->infobar_.get(),
-                               &GlobalConfirmInfoBar::Close);
-  }
-}
-
 infobars::InfoBarDelegate::InfoBarIdentifier
 ExtensionDevToolsInfoBarDelegate::GetIdentifier() const {
   return EXTENSION_DEV_TOOLS_INFOBAR_DELEGATE;
@@ -83,9 +71,18 @@ bool ExtensionDevToolsInfoBarDelegate::ShouldExpire(
   return false;
 }
 
+// When rendering the text is being truncated to a 10000 characters (see
+// https://source.chromium.org/chromium/chromium/src/+/main:ui/gfx/render_text_harfbuzz.cc;l=70-71;drc=736ed6e5da7f81505b547d4b03d31b30ed025a46).
+// If the extension name is longer then this limit, the part of the text saying
+// that "... is debugging the browser" would be truncated. To work this around
+// enforce more modest limit on the name length here, so that the user will see
+// '<end of extension name> is debugging the browser.
+const size_t kMaxExtensionNameLength = 1000;
+
 std::u16string ExtensionDevToolsInfoBarDelegate::GetMessageText() const {
-  return l10n_util::GetStringFUTF16(IDS_DEV_TOOLS_INFOBAR_LABEL,
-                                    extension_name_);
+  return l10n_util::GetStringFUTF16(
+      IDS_DEV_TOOLS_INFOBAR_LABEL,
+      extension_name_.substr(0, kMaxExtensionNameLength));
 }
 
 gfx::ElideBehavior ExtensionDevToolsInfoBarDelegate::GetMessageElideBehavior()
@@ -93,6 +90,7 @@ gfx::ElideBehavior ExtensionDevToolsInfoBarDelegate::GetMessageElideBehavior()
   // The important part of the message text above is at the end:
   // "... is debugging the browser". If the extension name is very long,
   // we'd rather truncate it instead. See https://crbug.com/823194.
+  // See also the comment above for the kMaxExtensionNameLength
   return gfx::ELIDE_HEAD;
 }
 
@@ -104,7 +102,11 @@ ExtensionDevToolsInfoBarDelegate::ExtensionDevToolsInfoBarDelegate(
     std::string extension_id,
     const std::string& extension_name)
     : extension_id_(std::move(extension_id)),
-      extension_name_(base::UTF8ToUTF16(extension_name)) {}
+      extension_name_(base::UTF8ToUTF16(extension_name)) {
+  callback_list_.set_removal_callback(base::BindRepeating(
+      &ExtensionDevToolsInfoBarDelegate::MaybeStartAutocloseTimer,
+      base::Unretained(this)));
+}
 
 base::CallbackListSubscription
 ExtensionDevToolsInfoBarDelegate::RegisterDestroyedCallback(
@@ -112,4 +114,12 @@ ExtensionDevToolsInfoBarDelegate::RegisterDestroyedCallback(
   return callback_list_.Add(std::move(destroyed_callback));
 }
 
+void ExtensionDevToolsInfoBarDelegate::MaybeStartAutocloseTimer() {
+  if (callback_list_.empty()) {
+    // infobar_ was set in Create() which makes the following access safe.
+    timer_.Start(FROM_HERE, kAutoCloseDelay, infobar_.get(),
+                 &GlobalConfirmInfoBar::Close);
+  }
+}
+
 }  // namespace extensions
