@@ -1,4 +1,4 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,7 +10,7 @@
 #include <vector>
 
 #include "base/containers/contains.h"
-#include "base/macros.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/strings/utf_string_conversions.h"
 #include "chrome/browser/bookmarks/bookmark_model_factory.h"
@@ -19,6 +19,7 @@
 #include "chrome/test/base/testing_profile.h"
 #include "components/bookmarks/browser/bookmark_model.h"
 #include "components/bookmarks/browser/scoped_group_bookmark_actions.h"
+#include "components/bookmarks/common/bookmark_metrics.h"
 #include "components/bookmarks/test/bookmark_test_helpers.h"
 #include "content/public/browser/navigation_controller.h"
 #include "content/public/browser/web_contents.h"
@@ -32,17 +33,16 @@ namespace {
 
 scoped_refptr<const Extension> CreateExtensionWithBookmarksPermission(
     bool include_bookmarks) {
-  ListBuilder permissions;
+  base::Value::List permissions;
   permissions.Append("declarativeContent");
   if (include_bookmarks)
     permissions.Append("bookmarks");
   return ExtensionBuilder()
-      .SetManifest(DictionaryBuilder()
+      .SetManifest(base::Value::Dict()
                        .Set("name", "Test extension")
                        .Set("version", "1.0")
                        .Set("manifest_version", 2)
-                       .Set("permissions", permissions.Build())
-                       .Build())
+                       .Set("permissions", std::move(permissions)))
       .Build();
 }
 
@@ -68,11 +68,20 @@ using testing::UnorderedElementsAreArray;
 
 class DeclarativeContentIsBookmarkedConditionTrackerTest
     : public DeclarativeContentConditionTrackerTest {
+ public:
+  DeclarativeContentIsBookmarkedConditionTrackerTest(
+      const DeclarativeContentIsBookmarkedConditionTrackerTest&) = delete;
+  DeclarativeContentIsBookmarkedConditionTrackerTest& operator=(
+      const DeclarativeContentIsBookmarkedConditionTrackerTest&) = delete;
+
  protected:
   class Delegate : public ContentPredicateEvaluator::Delegate {
    public:
     Delegate() {}
 
+    Delegate(const Delegate&) = delete;
+    Delegate& operator=(const Delegate&) = delete;
+
     std::set<content::WebContents*>& evaluation_requests() {
       return evaluation_requests_;
     }
@@ -90,8 +99,6 @@ class DeclarativeContentIsBookmarkedConditionTrackerTest
 
    private:
     std::set<content::WebContents*> evaluation_requests_;
-
-    DISALLOW_COPY_AND_ASSIGN(Delegate);
   };
 
   DeclarativeContentIsBookmarkedConditionTrackerTest() {
@@ -101,9 +108,8 @@ class DeclarativeContentIsBookmarkedConditionTrackerTest
     bookmarks::test::WaitForBookmarkModelToLoad(
         BookmarkModelFactory::GetForBrowserContext(profile()));
     bookmark_model_ = BookmarkModelFactory::GetForBrowserContext(profile());
-    tracker_.reset(new DeclarativeContentIsBookmarkedConditionTracker(
-        profile(),
-        &delegate_));
+    tracker_ = std::make_unique<DeclarativeContentIsBookmarkedConditionTracker>(
+        profile(), &delegate_);
     extension_ = CreateExtensionWithBookmarksPermission(true);
     is_bookmarked_predicate_ = CreatePredicate(tracker_.get(), extension_.get(),
                                                true);
@@ -147,16 +153,13 @@ class DeclarativeContentIsBookmarkedConditionTrackerTest
   }
 
   Delegate delegate_;
-  bookmarks::BookmarkModel* bookmark_model_;
+  raw_ptr<bookmarks::BookmarkModel> bookmark_model_;
   std::unique_ptr<DeclarativeContentIsBookmarkedConditionTracker> tracker_;
   scoped_refptr<const Extension> extension_;
   std::unique_ptr<DeclarativeContentIsBookmarkedPredicate>
       is_bookmarked_predicate_;
   std::unique_ptr<DeclarativeContentIsBookmarkedPredicate>
       is_not_bookmarked_predicate_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(DeclarativeContentIsBookmarkedConditionTrackerTest);
 };
 
 
@@ -211,8 +214,7 @@ TEST_F(DeclarativeContentIsBookmarkedConditionTrackerTest,
   LoadURL(tab.get(), GURL("http://bookmarked/"));
   EXPECT_TRUE(delegate_.evaluation_requests().empty());
 
-  bookmark_model_->AddURL(bookmark_model_->other_node(), 0,
-                          base::ASCIIToUTF16("title"),
+  bookmark_model_->AddURL(bookmark_model_->other_node(), 0, u"title",
                           GURL("http://bookmarked/"));
 
   tracker_->TrackForWebContents(tab.get());
@@ -246,10 +248,8 @@ TEST_F(DeclarativeContentIsBookmarkedConditionTrackerTest,
 
   // Bookmark the first tab's URL.
   delegate_.evaluation_requests().clear();
-  const bookmarks::BookmarkNode* node =
-      bookmark_model_->AddURL(bookmark_model_->other_node(), 0,
-                              base::ASCIIToUTF16("title"),
-                              GURL("http://bookmarked/"));
+  const bookmarks::BookmarkNode* node = bookmark_model_->AddURL(
+      bookmark_model_->other_node(), 0, u"title", GURL("http://bookmarked/"));
   EXPECT_THAT(delegate_.evaluation_requests(),
               UnorderedElementsAre(tabs[0].get()));
   EXPECT_TRUE(CheckPredicates(tabs[0].get(), true));
@@ -257,7 +257,7 @@ TEST_F(DeclarativeContentIsBookmarkedConditionTrackerTest,
 
   // Remove the bookmark.
   delegate_.evaluation_requests().clear();
-  bookmark_model_->Remove(node);
+  bookmark_model_->Remove(node, bookmarks::metrics::BookmarkEditSource::kOther);
   EXPECT_THAT(delegate_.evaluation_requests(),
               UnorderedElementsAre(tabs[0].get()));
   EXPECT_TRUE(CheckPredicates(tabs[0].get(), false));
@@ -292,10 +292,8 @@ TEST_F(DeclarativeContentIsBookmarkedConditionTrackerTest, ExtensiveChanges) {
     // added nodes.
     delegate_.evaluation_requests().clear();
     bookmark_model_->BeginExtensiveChanges();
-    const bookmarks::BookmarkNode* node =
-        bookmark_model_->AddURL(bookmark_model_->other_node(), 0,
-                                base::ASCIIToUTF16("title"),
-                                GURL("http://bookmarked/"));
+    const bookmarks::BookmarkNode* node = bookmark_model_->AddURL(
+        bookmark_model_->other_node(), 0, u"title", GURL("http://bookmarked/"));
     EXPECT_TRUE(delegate_.evaluation_requests().empty());
     EXPECT_TRUE(CheckPredicates(tabs[0].get(), false));
     EXPECT_TRUE(CheckPredicates(tabs[1].get(), false));
@@ -309,7 +307,8 @@ TEST_F(DeclarativeContentIsBookmarkedConditionTrackerTest, ExtensiveChanges) {
     // removed nodes.
     delegate_.evaluation_requests().clear();
     bookmark_model_->BeginExtensiveChanges();
-    bookmark_model_->Remove(node);
+    bookmark_model_->Remove(node,
+                            bookmarks::metrics::BookmarkEditSource::kOther);
     EXPECT_TRUE(delegate_.evaluation_requests().empty());
     EXPECT_TRUE(CheckPredicates(tabs[0].get(), true));
     EXPECT_TRUE(CheckPredicates(tabs[1].get(), false));
@@ -327,8 +326,7 @@ TEST_F(DeclarativeContentIsBookmarkedConditionTrackerTest, ExtensiveChanges) {
     const bookmarks::BookmarkNode* node = nullptr;
     {
       bookmarks::ScopedGroupBookmarkActions scoped_group(bookmark_model_);
-      node = bookmark_model_->AddURL(bookmark_model_->other_node(), 0,
-                                     base::ASCIIToUTF16("title"),
+      node = bookmark_model_->AddURL(bookmark_model_->other_node(), 0, u"title",
                                      GURL("http://bookmarked/"));
       EXPECT_TRUE(delegate_.evaluation_requests().empty());
       EXPECT_TRUE(CheckPredicates(tabs[0].get(), false));
@@ -344,7 +342,8 @@ TEST_F(DeclarativeContentIsBookmarkedConditionTrackerTest, ExtensiveChanges) {
     delegate_.evaluation_requests().clear();
     {
       bookmarks::ScopedGroupBookmarkActions scoped_group(bookmark_model_);
-      bookmark_model_->Remove(node);
+      bookmark_model_->Remove(node,
+                              bookmarks::metrics::BookmarkEditSource::kOther);
       EXPECT_TRUE(delegate_.evaluation_requests().empty());
       EXPECT_TRUE(CheckPredicates(tabs[0].get(), true));
       EXPECT_TRUE(CheckPredicates(tabs[1].get(), false));
@@ -361,11 +360,9 @@ TEST_F(DeclarativeContentIsBookmarkedConditionTrackerTest, ExtensiveChanges) {
 TEST_F(DeclarativeContentIsBookmarkedConditionTrackerTest, Navigation) {
   // Bookmark two URLs.
   delegate_.evaluation_requests().clear();
-  bookmark_model_->AddURL(bookmark_model_->other_node(), 0,
-                          base::ASCIIToUTF16("title"),
+  bookmark_model_->AddURL(bookmark_model_->other_node(), 0, u"title",
                           GURL("http://bookmarked1/"));
-  bookmark_model_->AddURL(bookmark_model_->other_node(), 0,
-                          base::ASCIIToUTF16("title"),
+  bookmark_model_->AddURL(bookmark_model_->other_node(), 0, u"title",
                           GURL("http://bookmarked2/"));
 
   // Create two tabs.
