@@ -80,7 +80,7 @@ std::string ValidateCrosapi(int min_version, content::BrowserContext* context) {
   if (!service || !service->IsAvailable<crosapi::mojom::KeystoreService>())
     return kUnsupportedByAsh;
 
-  int version = service->GetInterfaceVersion(KeystoreService::Uuid_);
+  int version = service->GetInterfaceVersion<KeystoreService>();
   if (version < min_version)
     return kUnsupportedByAsh;
 
@@ -95,13 +95,13 @@ std::string ValidateCrosapi(int min_version, content::BrowserContext* context) {
   return "";
 }
 
-absl::optional<crosapi::mojom::KeystoreType> KeystoreTypeFromString(
+std::optional<crosapi::mojom::KeystoreType> KeystoreTypeFromString(
     const std::string& input) {
   if (input == "user")
     return crosapi::mojom::KeystoreType::kUser;
   if (input == "system")
     return crosapi::mojom::KeystoreType::kDevice;
-  return absl::nullopt;
+  return std::nullopt;
 }
 
 // Validates that |token_id| is well-formed. Converts |token_id| into the output
@@ -110,7 +110,7 @@ absl::optional<crosapi::mojom::KeystoreType> KeystoreTypeFromString(
 // extension termination.
 std::string ValidateInput(const std::string& token_id,
                           crosapi::mojom::KeystoreType* keystore) {
-  absl::optional<crosapi::mojom::KeystoreType> keystore_type =
+  std::optional<crosapi::mojom::KeystoreType> keystore_type =
       KeystoreTypeFromString(token_id);
   if (!keystore_type)
     return platform_keys::kErrorInvalidToken;
@@ -147,8 +147,8 @@ EnterprisePlatformKeysInternalGenerateKeyFunction::
 
 ExtensionFunction::ResponseAction
 EnterprisePlatformKeysInternalGenerateKeyFunction::Run() {
-  std::unique_ptr<api_epki::GenerateKey::Params> params(
-      api_epki::GenerateKey::Params::Create(args()));
+  std::optional<api_epki::GenerateKey::Params> params =
+      api_epki::GenerateKey::Params::Create(args());
 
 #if BUILDFLAG(IS_CHROMEOS_LACROS)
   // TODO(b/191958380): Lift the restriction when *.platformKeys.* APIs are
@@ -158,7 +158,7 @@ EnterprisePlatformKeysInternalGenerateKeyFunction::Run() {
 #endif  // BUILDFLAG(IS_CHROMEOS_LACROS)
 
   EXTENSION_FUNCTION_VALIDATE(params);
-  absl::optional<chromeos::platform_keys::TokenId> platform_keys_token_id =
+  std::optional<chromeos::platform_keys::TokenId> platform_keys_token_id =
       platform_keys::ApiIdToPlatformKeysTokenId(params->token_id);
   if (!platform_keys_token_id)
     return RespondNow(Error(platform_keys::kErrorInvalidToken));
@@ -195,7 +195,7 @@ EnterprisePlatformKeysInternalGenerateKeyFunction::Run() {
 
 void EnterprisePlatformKeysInternalGenerateKeyFunction::OnGeneratedKey(
     std::vector<uint8_t> public_key_der,
-    absl::optional<crosapi::mojom::KeystoreError> error) {
+    std::optional<crosapi::mojom::KeystoreError> error) {
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
   if (!error) {
     Respond(ArgumentList(
@@ -210,8 +210,8 @@ void EnterprisePlatformKeysInternalGenerateKeyFunction::OnGeneratedKey(
 
 ExtensionFunction::ResponseAction
 EnterprisePlatformKeysGetCertificatesFunction::Run() {
-  std::unique_ptr<api_epk::GetCertificates::Params> params(
-      api_epk::GetCertificates::Params::Create(args()));
+  std::optional<api_epk::GetCertificates::Params> params =
+      api_epk::GetCertificates::Params::Create(args());
   EXTENSION_FUNCTION_VALIDATE(params);
 
   std::string error = ValidateCrosapi(
@@ -256,8 +256,8 @@ void EnterprisePlatformKeysGetCertificatesFunction::OnGetCertificates(
 
 ExtensionFunction::ResponseAction
 EnterprisePlatformKeysImportCertificateFunction::Run() {
-  std::unique_ptr<api_epk::ImportCertificate::Params> params(
-      api_epk::ImportCertificate::Params::Create(args()));
+  std::optional<api_epk::ImportCertificate::Params> params =
+      api_epk::ImportCertificate::Params::Create(args());
   EXTENSION_FUNCTION_VALIDATE(params);
 
   std::string error = ValidateCrosapi(
@@ -291,8 +291,8 @@ void EnterprisePlatformKeysImportCertificateFunction::OnAddCertificate(
 
 ExtensionFunction::ResponseAction
 EnterprisePlatformKeysRemoveCertificateFunction::Run() {
-  std::unique_ptr<api_epk::RemoveCertificate::Params> params(
-      api_epk::RemoveCertificate::Params::Create(args()));
+  std::optional<api_epk::RemoveCertificate::Params> params =
+      api_epk::RemoveCertificate::Params::Create(args());
   EXTENSION_FUNCTION_VALIDATE(params);
 
   std::string error = ValidateCrosapi(
@@ -373,8 +373,8 @@ void EnterprisePlatformKeysInternalGetTokensFunction::OnGetKeyStores(
 
 ExtensionFunction::ResponseAction
 EnterprisePlatformKeysChallengeMachineKeyFunction::Run() {
-  std::unique_ptr<api_epk::ChallengeMachineKey::Params> params(
-      api_epk::ChallengeMachineKey::Params::Create(args()));
+  std::optional<api_epk::ChallengeMachineKey::Params> params =
+      api_epk::ChallengeMachineKey::Params::Create(args());
   EXTENSION_FUNCTION_VALIDATE(params);
 
   const std::string error = ValidateCrosapi(
@@ -422,8 +422,8 @@ void EnterprisePlatformKeysChallengeMachineKeyFunction::
 
 ExtensionFunction::ResponseAction
 EnterprisePlatformKeysChallengeUserKeyFunction::Run() {
-  std::unique_ptr<api_epk::ChallengeUserKey::Params> params(
-      api_epk::ChallengeUserKey::Params::Create(args()));
+  std::optional<api_epk::ChallengeUserKey::Params> params =
+      api_epk::ChallengeUserKey::Params::Create(args());
   EXTENSION_FUNCTION_VALIDATE(params);
 
   const std::string error = ValidateCrosapi(
@@ -472,8 +472,8 @@ const uint64_t kChallengeKeystoreAlgorithmParameterMinVersion = 17;
 
 ExtensionFunction::ResponseAction
 EnterprisePlatformKeysChallengeKeyFunction::Run() {
-  std::unique_ptr<api_epk::ChallengeKey::Params> params(
-      api_epk::ChallengeKey::Params::Create(args()));
+  std::optional<api_epk::ChallengeKey::Params> params =
+      api_epk::ChallengeKey::Params::Create(args());
   EXTENSION_FUNCTION_VALIDATE(params);
 
   std::string error = ValidateCrosapi(
@@ -491,15 +491,15 @@ EnterprisePlatformKeysChallengeKeyFunction::Run() {
   crosapi::mojom::KeystoreType keystore_type =
       crosapi::mojom::KeystoreType::kDevice;
   EXTENSION_FUNCTION_VALIDATE(params->options.scope !=
-                              api::enterprise_platform_keys::SCOPE_NONE);
+                              api::enterprise_platform_keys::Scope::kNone);
   switch (params->options.scope) {
-    case api::enterprise_platform_keys::SCOPE_USER:
+    case api::enterprise_platform_keys::Scope::kUser:
       keystore_type = crosapi::mojom::KeystoreType::kUser;
       break;
-    case api::enterprise_platform_keys::SCOPE_MACHINE:
+    case api::enterprise_platform_keys::Scope::kMachine:
       keystore_type = crosapi::mojom::KeystoreType::kDevice;
       break;
-    case api::enterprise_platform_keys::SCOPE_NONE:
+    case api::enterprise_platform_keys::Scope::kNone:
       NOTREACHED();
   }
 
@@ -507,14 +507,15 @@ EnterprisePlatformKeysChallengeKeyFunction::Run() {
   crosapi::mojom::KeystoreSigningAlgorithmName algorithm =
       crosapi::mojom::KeystoreSigningAlgorithmName::kRsassaPkcs115;
   if (params->options.register_key.has_value()) {
-    EXTENSION_FUNCTION_VALIDATE(params->options.register_key->algorithm !=
-                                api::enterprise_platform_keys::ALGORITHM_NONE);
+    EXTENSION_FUNCTION_VALIDATE(
+        params->options.register_key->algorithm !=
+        api::enterprise_platform_keys::Algorithm::kNone);
     switch (params->options.register_key->algorithm) {
-      case api::enterprise_platform_keys::ALGORITHM_RSA:
+      case api::enterprise_platform_keys::Algorithm::kRsa:
         algorithm =
             crosapi::mojom::KeystoreSigningAlgorithmName::kRsassaPkcs115;
         break;
-      case api::enterprise_platform_keys::ALGORITHM_ECDSA: {
+      case api::enterprise_platform_keys::Algorithm::kEcdsa: {
         // Older versions of Ash default to RSA. If ECDSA is specified but the
         // Keystore would use RSA instead, return an error.
         const std::string version_error = ValidateCrosapi(
@@ -525,7 +526,7 @@ EnterprisePlatformKeysChallengeKeyFunction::Run() {
         algorithm = crosapi::mojom::KeystoreSigningAlgorithmName::kEcdsa;
         break;
       }
-      case api::enterprise_platform_keys::ALGORITHM_NONE:
+      case api::enterprise_platform_keys::Algorithm::kNone:
         NOTREACHED();
     }
   }
