@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,7 +9,6 @@
 #include "base/files/scoped_temp_dir.h"
 #include "base/strings/string_piece.h"
 #include "base/strings/stringprintf.h"
-#include "base/test/metrics/histogram_tester.h"
 #include "base/threading/thread_restrictions.h"
 #include "build/build_config.h"
 #include "chrome/browser/download/download_prefs.h"
@@ -35,7 +34,6 @@
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/prefs/pref_service.h"
 #include "content/public/browser/browser_context.h"
-#include "content/public/browser/notification_service.h"
 #include "content/public/browser/overlay_window.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/video_picture_in_picture_window_controller.h"
@@ -59,14 +57,15 @@
 #include "extensions/test/result_catcher.h"
 #include "net/test/embedded_test_server/embedded_test_server.h"
 #include "testing/gmock/include/gmock/gmock.h"
-#include "ui/base/layout.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/base/resource/resource_scale_factor.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/size.h"
 #include "ui/gfx/geometry/skia_conversions.h"
 #include "ui/gfx/image/canvas_image_source.h"
 #include "ui/gfx/image/image_skia.h"
 #include "ui/gfx/image/image_skia_operations.h"
+#include "ui/gfx/image/image_skia_rep.h"
 #include "ui/gfx/image/image_unittest_util.h"
 
 using content::WebContents;
@@ -163,7 +162,8 @@ class BrowserActionApiTestWithContextType
 
  protected:
   void RunUpdateTest(base::StringPiece path, bool expect_failure) {
-    ExtensionTestMessageListener ready_listener("ready", true);
+    ExtensionTestMessageListener ready_listener("ready",
+                                                ReplyBehavior::kWillReply);
     ASSERT_TRUE(embedded_test_server()->Start());
     const Extension* extension =
         LoadExtension(test_data_dir_.AppendASCII(path));
@@ -202,7 +202,8 @@ class BrowserActionApiTestWithContextType
   }
 
   void RunEnableTest(base::StringPiece path, bool start_enabled) {
-    ExtensionTestMessageListener ready_listener("ready", true);
+    ExtensionTestMessageListener ready_listener("ready",
+                                                ReplyBehavior::kWillReply);
     const Extension* extension =
         LoadExtension(test_data_dir_.AppendASCII(path));
     ASSERT_TRUE(extension) << message_;
@@ -234,7 +235,7 @@ class BrowserActionApiTestWithContextType
 };
 
 IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, Basic) {
-  ExtensionTestMessageListener ready_listener("ready", false);
+  ExtensionTestMessageListener ready_listener("ready");
   ASSERT_TRUE(embedded_test_server()->Start());
   const Extension* extension =
       LoadExtension(test_data_dir_.AppendASCII("browser_action/basics"));
@@ -290,12 +291,9 @@ IN_PROC_BROWSER_TEST_F(BrowserActionApiCanvasTest, DynamicBrowserAction) {
   ASSERT_TRUE(extension) << message_;
 
 #if BUILDFLAG(IS_MAC)
-  // We need this on mac so we don't loose 2x representations from browser icon
+  // We need this on mac so we don't lose 2x representations from browser icon
   // in transformations gfx::ImageSkia -> NSImage -> gfx::ImageSkia.
-  std::vector<ui::ResourceScaleFactor> supported_scale_factors;
-  supported_scale_factors.push_back(ui::k100Percent);
-  supported_scale_factors.push_back(ui::k200Percent);
-  ui::SetSupportedResourceScaleFactors(supported_scale_factors);
+  ui::SetSupportedResourceScaleFactors({ui::k100Percent, ui::k200Percent});
 #endif
 
   // We should not be creating icons asynchronously, so we don't need an
@@ -321,8 +319,12 @@ IN_PROC_BROWSER_TEST_F(BrowserActionApiCanvasTest, DynamicBrowserAction) {
   // these may be generated from the provided scales.
   float kSmallIconScale = 21.f / ExtensionAction::ActionIconSize();
   float kLargeIconScale = 42.f / ExtensionAction::ActionIconSize();
-  ASSERT_FALSE(ui::IsSupportedScale(kSmallIconScale));
-  ASSERT_FALSE(ui::IsSupportedScale(kLargeIconScale));
+  ASSERT_NE(ui::GetScaleForResourceScaleFactor(
+                ui::GetSupportedResourceScaleFactor(kSmallIconScale)),
+            kSmallIconScale);
+  ASSERT_NE(ui::GetScaleForResourceScaleFactor(
+                ui::GetSupportedResourceScaleFactor(kLargeIconScale)),
+            kLargeIconScale);
 
   // Tell the extension to update the icon using ImageData object.
   ResultCatcher catcher;
@@ -485,39 +487,23 @@ IN_PROC_BROWSER_TEST_F(BrowserActionApiCanvasTest, InvisibleIconBrowserAction) {
           extension->id());
   ASSERT_TRUE(background_page);
 
-  static constexpr char kScript[] =
-      "setIcon(%s).then(function(arg) {"
-      "  domAutomationController.send(arg);"
-      "});";
+  static constexpr char kScript[] = "setIcon(%s);";
 
-  const std::string histogram_name =
-      "Extensions.DynamicExtensionActionIconWasVisible";
   {
-    base::HistogramTester histogram_tester;
-    std::string result;
-    EXPECT_TRUE(ExecuteScriptAndExtractString(
-        background_page->host_contents(),
-        base::StringPrintf(kScript, "invisibleImageData"), &result));
-    EXPECT_EQ("Icon not sufficiently visible.", result);
+    EXPECT_EQ("Icon not sufficiently visible.",
+              EvalJs(background_page->host_contents(),
+                     base::StringPrintf(kScript, "invisibleImageData")));
     // The icon should not have changed.
     EXPECT_TRUE(gfx::test::AreImagesEqual(
         initial_bar_icon, GetBrowserActionsBar()->GetIcon(extension->id())));
-    EXPECT_THAT(histogram_tester.GetAllSamples(histogram_name),
-                testing::ElementsAre(base::Bucket(0, 1)));
   }
 
   {
-    base::HistogramTester histogram_tester;
-    std::string result;
-    EXPECT_TRUE(ExecuteScriptAndExtractString(
-        background_page->host_contents(),
-        base::StringPrintf(kScript, "visibleImageData"), &result));
-    EXPECT_EQ("", result);
+    EXPECT_EQ("", EvalJs(background_page->host_contents(),
+                         base::StringPrintf(kScript, "visibleImageData")));
     // The icon should have changed.
     EXPECT_FALSE(gfx::test::AreImagesEqual(
         initial_bar_icon, GetBrowserActionsBar()->GetIcon(extension->id())));
-    EXPECT_THAT(histogram_tester.GetAllSamples(histogram_name),
-                testing::ElementsAre(base::Bucket(1, 1)));
   }
 }
 
@@ -545,7 +531,8 @@ IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType,
 
   // Go back to first tab, changed title should reappear.
   browser()->tab_strip_model()->ActivateTabAt(
-      0, {TabStripModel::GestureType::kOther});
+      0, TabStripUserGestureDetails(
+             TabStripUserGestureDetails::GestureType::kOther));
   EXPECT_EQ("Showing icon 2",
             GetBrowserActionsBar()->GetTooltip(extension->id()));
 
@@ -678,7 +665,7 @@ IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, RemovePopup) {
 }
 
 IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, IncognitoBasic) {
-  ExtensionTestMessageListener ready_listener("ready", false);
+  ExtensionTestMessageListener ready_listener("ready");
   ASSERT_TRUE(embedded_test_server()->Start());
   scoped_refptr<const Extension> extension =
       LoadExtension(test_data_dir_.AppendASCII("browser_action/basics"));
@@ -700,7 +687,7 @@ IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, IncognitoBasic) {
   // action shows up.
   // SetIsIncognitoEnabled() requires a reload of the extension, so we have to
   // wait for it.
-  ExtensionTestMessageListener incognito_ready_listener("ready", false);
+  ExtensionTestMessageListener incognito_ready_listener("ready");
   TestExtensionRegistryObserver registry_observer(
       ExtensionRegistry::Get(profile()), extension->id());
   extensions::util::SetIsIncognitoEnabled(
@@ -728,7 +715,7 @@ IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, IncognitoBasic) {
 IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, IncognitoUpdate) {
   ASSERT_TRUE(embedded_test_server()->Start());
   ExtensionTestMessageListener incognito_not_allowed_listener(
-      "incognito not allowed", false);
+      "incognito not allowed");
   scoped_refptr<const Extension> extension =
       LoadExtension(test_data_dir_.AppendASCII("browser_action/update"));
   ASSERT_TRUE(extension) << message_;
@@ -750,8 +737,8 @@ IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, IncognitoUpdate) {
   // execution until the transition is completed, since the script will
   // start and stop multiple times during the initial load of the extension
   // and the enabling of incognito mode.
-  ExtensionTestMessageListener incognito_allowed_listener("incognito allowed",
-                                                          true);
+  ExtensionTestMessageListener incognito_allowed_listener(
+      "incognito allowed", ReplyBehavior::kWillReply);
   // Now enable the extension in incognito mode, and test that the browser
   // action shows up. SetIsIncognitoEnabled() requires a reload of the
   // extension, so we have to wait for it to finish.
@@ -789,8 +776,8 @@ IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, IncognitoUpdate) {
 // Tests that events are dispatched to the correct profile for split mode
 // extensions.
 IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, IncognitoSplit) {
-  ExtensionTestMessageListener listener_ready("regular ready", false);
-  ExtensionTestMessageListener incognito_ready("incognito ready", false);
+  ExtensionTestMessageListener listener_ready("regular ready");
+  ExtensionTestMessageListener incognito_ready("incognito ready");
 
   // Open an incognito browser.
   // Note: It is important that we create incognito profile before loading
@@ -823,7 +810,7 @@ IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType, IncognitoSplit) {
 }
 
 IN_PROC_BROWSER_TEST_F(BrowserActionApiTest, CloseBackgroundPage) {
-  ExtensionTestMessageListener listener("ready", /*will_reply=*/false);
+  ExtensionTestMessageListener listener("ready");
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("browser_action/close_background")));
   const Extension* extension = GetSingleLoadedExtension();
@@ -954,17 +941,14 @@ IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType,
   EXPECT_TRUE(tab);
 
   // Verify that the browser action turned the background color red.
-  const std::string script =
-      "window.domAutomationController.send(document.body.style."
-      "backgroundColor);";
-  std::string result;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(tab, script, &result));
-  EXPECT_EQ(result, "red");
+  const std::string script = "document.body.style.backgroundColor;";
+  EXPECT_EQ(content::EvalJs(tab, script), "red");
 }
 
 IN_PROC_BROWSER_TEST_P(BrowserActionApiTestWithContextType,
                        WithRectangularIcon) {
-  ExtensionTestMessageListener ready_listener("ready", true);
+  ExtensionTestMessageListener ready_listener("ready",
+                                              ReplyBehavior::kWillReply);
 
   const Extension* extension = LoadExtension(
       test_data_dir_.AppendASCII("browser_action").AppendASCII("rect_icon"));
@@ -1018,18 +1002,19 @@ IN_PROC_BROWSER_TEST_F(BrowserActionApiTest,
   content::VideoPictureInPictureWindowController* window_controller =
       content::PictureInPictureWindowController::
           GetOrCreateVideoPictureInPictureController(web_contents);
-  ASSERT_TRUE(window_controller->GetWindowForTesting());
-  EXPECT_FALSE(window_controller->GetWindowForTesting()->IsVisible());
+  EXPECT_FALSE(window_controller->GetWindowForTesting());
 
   // Click on the browser action icon to enter Picture-in-Picture.
   ResultCatcher catcher;
   GetBrowserActionsBar()->Press(extension->id());
   EXPECT_TRUE(catcher.GetNextResult());
+  ASSERT_TRUE(window_controller->GetWindowForTesting());
   EXPECT_TRUE(window_controller->GetWindowForTesting()->IsVisible());
 
   // Click on the browser action icon to exit Picture-in-Picture.
   GetBrowserActionsBar()->Press(extension->id());
   EXPECT_TRUE(catcher.GetNextResult());
+  ASSERT_TRUE(window_controller->GetWindowForTesting());
   EXPECT_FALSE(window_controller->GetWindowForTesting()->IsVisible());
 }
 
