@@ -1,10 +1,14 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <memory>
 
+#include "base/files/file_util.h"
+#include "base/memory/raw_ptr.h"
 #include "base/run_loop.h"
+#include "base/scoped_observation.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/test/test_timeouts.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
@@ -17,29 +21,27 @@
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/extensions/extension_action_test_helper.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
-#include "chrome/browser/ui/toolbar/toolbar_actions_bar.h"
 #include "chrome/browser/ui/toolbar/toolbar_actions_model.h"
-#include "chrome/browser/ui/ui_features.h"
 #include "chrome/test/base/interactive_test_utils.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/sessions/content/session_tab_helper.h"
 #include "components/zoom/test/zoom_test_utils.h"
 #include "components/zoom/zoom_controller.h"
 #include "content/public/browser/host_zoom_map.h"
-#include "content/public/browser/notification_details.h"
-#include "content/public/browser/notification_service.h"
-#include "content/public/browser/notification_types.h"
-#include "content/public/browser/render_view_host.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/download_test_observer.h"
+#include "content/public/test/fenced_frame_test_util.h"
+#include "content/public/test/test_navigation_observer.h"
 #include "extensions/browser/extension_action.h"
 #include "extensions/browser/extension_action_manager.h"
 #include "extensions/browser/extension_host.h"
+#include "extensions/browser/extension_host_registry.h"
+#include "extensions/browser/extension_host_test_helper.h"
 #include "extensions/browser/extension_registry.h"
-#include "extensions/browser/notification_types.h"
 #include "extensions/common/extension.h"
 #include "extensions/common/extension_set.h"
+#include "extensions/common/mojom/view_type.mojom.h"
 #include "extensions/common/permissions/permissions_data.h"
 #include "extensions/test/extension_test_message_listener.h"
 #include "extensions/test/result_catcher.h"
@@ -51,34 +53,51 @@
 #include "ui/gfx/scrollbar_size.h"
 #include "ui/views/widget/widget.h"
 
-#if defined(OS_WIN)
+#if !BUILDFLAG(IS_CHROMEOS)
+#include "chrome/browser/download/bubble/download_bubble_ui_controller.h"
+#include "chrome/browser/download/bubble/download_display_controller.h"
+#include "chrome/browser/ui/download/download_display.h"
+#endif
+
+#if BUILDFLAG(IS_WIN)
 #include "ui/views/win/hwnd_util.h"
 #endif
 
 namespace extensions {
 namespace {
 
+#if !BUILDFLAG(IS_CHROMEOS)
+bool IsDownloadSurfaceVisible(BrowserWindow* window) {
+  return window->GetDownloadBubbleUIController()
+      ->GetDownloadDisplayController()
+      ->download_display_for_testing()
+      ->IsShowingDetails();
+}
+#endif
+
 // Helper to ensure all extension hosts are destroyed during the test. If a host
 // is still alive, the Profile can not be destroyed in
-// BrowserProcessImpl::StartTearDown(). TODO(tapted): The existence of this
-// helper is probably a bug. Extension hosts do not currently block shutdown the
-// way a browser tab does. Maybe they should. See http://crbug.com/729476.
-class PopupHostWatcher : public content::NotificationObserver {
+// BrowserProcessImpl::StartTearDown().
+// TODO(tapted): The existence of this helper is probably a bug. Extension
+// hosts do not currently block shutdown the way a browser tab does. Maybe they
+// should. See http://crbug.com/729476.
+class PopupHostWatcher : public ExtensionHostRegistry::Observer {
  public:
-  PopupHostWatcher() {
-    registrar_.Add(this, NOTIFICATION_EXTENSION_HOST_CREATED,
-                   content::NotificationService::AllSources());
-    registrar_.Add(this, NOTIFICATION_EXTENSION_HOST_DESTROYED,
-                   content::NotificationService::AllSources());
+  explicit PopupHostWatcher(content::BrowserContext* browser_context) {
+    host_registry_observation_.Observe(
+        ExtensionHostRegistry::Get(browser_context));
   }
 
+  PopupHostWatcher(const PopupHostWatcher&) = delete;
+  PopupHostWatcher& operator=(const PopupHostWatcher&) = delete;
+
   void Wait() {
     if (created_ == destroyed_)
       return;
 
     base::RunLoop run_loop;
     quit_closure_ = run_loop.QuitClosure();
-    base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+    base::SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
         FROM_HERE, quit_closure_, TestTimeouts::action_timeout());
     run_loop.Run();
   }
@@ -86,29 +105,40 @@ class PopupHostWatcher : public content::NotificationObserver {
   int created() const { return created_; }
   int destroyed() const { return destroyed_; }
 
-  // NotificationObserver:
-  void Observe(int type,
-               const content::NotificationSource& source,
-               const content::NotificationDetails& details) override {
+  // ExtensionHostRegistry::Observer:
+  void OnExtensionHostRenderProcessReady(
+      content::BrowserContext* browser_context,
+      ExtensionHost* host) override {
     // Only track lifetimes for popup window ExtensionHost instances.
-    const ExtensionHost* host =
-        content::Details<const ExtensionHost>(details).ptr();
-    DCHECK(host);
-    if (host->extension_host_type() != VIEW_TYPE_EXTENSION_POPUP)
+    if (host->extension_host_type() != mojom::ViewType::kExtensionPopup)
       return;
 
-    ++(type == NOTIFICATION_EXTENSION_HOST_CREATED ? created_ : destroyed_);
+    ++created_;
+    QuitIfSatisfied();
+  }
+
+  void OnExtensionHostDestroyed(content::BrowserContext* browser_context,
+                                ExtensionHost* host) override {
+    // Only track lifetimes for popup window ExtensionHost instances.
+    if (host->extension_host_type() != mojom::ViewType::kExtensionPopup)
+      return;
+
+    ++destroyed_;
+    QuitIfSatisfied();
+  }
+
+ private:
+  void QuitIfSatisfied() {
     if (!quit_closure_.is_null() && created_ == destroyed_)
       quit_closure_.Run();
   }
 
- private:
-  content::NotificationRegistrar registrar_;
   base::RepeatingClosure quit_closure_;
   int created_ = 0;
   int destroyed_ = 0;
-
-  DISALLOW_COPY_AND_ASSIGN(PopupHostWatcher);
+  base::ScopedObservation<ExtensionHostRegistry,
+                          ExtensionHostRegistry::Observer>
+      host_registry_observation_{this};
 };
 
 // chrome.browserAction API tests that interact with the UI in such a way that
@@ -117,12 +147,17 @@ class PopupHostWatcher : public content::NotificationObserver {
 class BrowserActionInteractiveTest : public ExtensionApiTest {
  public:
   BrowserActionInteractiveTest() {}
+
+  BrowserActionInteractiveTest(const BrowserActionInteractiveTest&) = delete;
+  BrowserActionInteractiveTest& operator=(const BrowserActionInteractiveTest&) =
+      delete;
+
   ~BrowserActionInteractiveTest() override {}
 
   // BrowserTestBase:
   void SetUpOnMainThread() override {
-    host_watcher_ = std::make_unique<PopupHostWatcher>();
     ExtensionApiTest::SetUpOnMainThread();
+    host_watcher_ = std::make_unique<PopupHostWatcher>(profile());
     host_resolver()->AddRule("*", "127.0.0.1");
     EXPECT_TRUE(ui_test_utils::BringBrowserWindowToFront(browser()));
   }
@@ -132,9 +167,11 @@ class BrowserActionInteractiveTest : public ExtensionApiTest {
     // called after this. But relying on the window close to close the
     // extension host can cause flakes. See http://crbug.com/729476.
     // Waiting here requires individual tests to ensure their popup has closed.
-    ExtensionApiTest::TearDownOnMainThread();
     host_watcher_->Wait();
     EXPECT_EQ(host_watcher_->created(), host_watcher_->destroyed());
+    // Destroy the PopupHostWatcher to ensure it stops watching the profile.
+    host_watcher_.reset();
+    ExtensionApiTest::TearDownOnMainThread();
   }
 
  protected:
@@ -151,34 +188,30 @@ class BrowserActionInteractiveTest : public ExtensionApiTest {
   // method will create a listener and reply to the extension before returning
   // to avoid leaking an API function while waiting for a reply.
   void OpenPopupViaAPI(bool will_reply) {
-    // Setup the notification observer to wait for the popup to finish loading.
-    content::WindowedNotificationObserver frame_observer(
-        content::NOTIFICATION_LOAD_COMPLETED_MAIN_FRAME,
-        content::NotificationService::AllSources());
+    // Setup the observer to wait for the popup to finish loading.
+    content::CreateAndLoadWebContentsObserver frame_observer;
     std::unique_ptr<ExtensionTestMessageListener> listener;
     if (!will_reply)
-      listener = std::make_unique<ExtensionTestMessageListener>("ready", false);
+      listener = std::make_unique<ExtensionTestMessageListener>("ready");
     // Show first popup in first window and expect it to have loaded.
-    ASSERT_TRUE(RunExtensionSubtest("browser_action/open_popup",
-                                    "open_popup_succeeds.html")) << message_;
+    ASSERT_TRUE(RunExtensionTest("browser_action/open_popup",
+                                 {.extension_url = "open_popup_succeeds.html"}))
+        << message_;
     if (listener)
       EXPECT_TRUE(listener->WaitUntilSatisfied());
     frame_observer.Wait();
     EnsurePopupActive();
   }
 
-  // Open an extension popup by clicking the browser action button.
-  content::WebContents* OpenPopupViaToolbar() {
-    content::WindowedNotificationObserver popup_observer(
-        content::NOTIFICATION_LOAD_COMPLETED_MAIN_FRAME,
-        content::NotificationService::AllSources());
-    ExtensionActionTestHelper::Create(browser())->Press(0);
-    popup_observer.Wait();
+  // Open an extension popup by clicking the browser action button associated
+  // with `id`.
+  content::WebContents* OpenPopupViaToolbar(const std::string& id) {
+    EXPECT_FALSE(id.empty());
+    content::CreateAndLoadWebContentsObserver popup_observer;
+    ExtensionActionTestHelper::Create(browser())->Press(id);
+    content::WebContents* popup = popup_observer.Wait();
     EnsurePopupActive();
-    const auto& source =
-        static_cast<const content::Source<content::WebContents>&>(
-            popup_observer.source());
-    return source.ptr();
+    return popup;
   }
 
   // Close the popup window directly.
@@ -189,11 +222,10 @@ class BrowserActionInteractiveTest : public ExtensionApiTest {
   // Trigger a focus loss to close the popup.
   void ClosePopupViaFocusLoss() {
     EXPECT_TRUE(ExtensionActionTestHelper::Create(browser())->HasPopup());
-    content::WindowedNotificationObserver observer(
-        extensions::NOTIFICATION_EXTENSION_HOST_DESTROYED,
-        content::NotificationService::AllSources());
 
-#if defined(OS_MAC)
+    ExtensionHostTestHelper host_helper(profile());
+
+#if BUILDFLAG(IS_MAC)
     // ClickOnView() in an inactive window is not robust on Mac. The click does
     // not guarantee window activation on trybots. So activate the browser
     // explicitly, thus causing the bubble to lose focus and dismiss itself.
@@ -211,7 +243,7 @@ class BrowserActionInteractiveTest : public ExtensionApiTest {
 
     // Wait for the notification to achieve a consistent state and verify that
     // the popup was properly torn down.
-    observer.Wait();
+    host_helper.WaitForHostDestroyed();
     base::RunLoop().RunUntilIdle();
   }
 
@@ -219,17 +251,16 @@ class BrowserActionInteractiveTest : public ExtensionApiTest {
 
  private:
   std::unique_ptr<PopupHostWatcher> host_watcher_;
-
-  DISALLOW_COPY_AND_ASSIGN(BrowserActionInteractiveTest);
 };
 
 // Tests opening a popup using the chrome.browserAction.openPopup API. This test
 // opens a popup in the starting window, closes the popup, creates a new window
 // and opens a popup in the new window. Both popups should succeed in opening.
-IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, TestOpenPopup) {
+// TODO(crbug.com/1233996): Test flaking frequently.
+IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, DISABLED_TestOpenPopup) {
   auto browserActionBar = ExtensionActionTestHelper::Create(browser());
   // Setup extension message listener to wait for javascript to finish running.
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   {
     OpenPopupViaAPI(true);
     EXPECT_TRUE(browserActionBar->HasPopup());
@@ -237,29 +268,26 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, TestOpenPopup) {
   }
 
   EXPECT_TRUE(listener.WaitUntilSatisfied());
-  Browser* new_browser = NULL;
+  Browser* new_browser = nullptr;
   {
-    content::WindowedNotificationObserver frame_observer(
-        content::NOTIFICATION_LOAD_COMPLETED_MAIN_FRAME,
-        content::NotificationService::AllSources());
+    content::CreateAndLoadWebContentsObserver frame_observer;
     // Open a new window.
-    new_browser = chrome::FindBrowserWithWebContents(browser()->OpenURL(
+    new_browser = chrome::FindBrowserWithTab(browser()->OpenURL(
         content::OpenURLParams(GURL("about:blank"), content::Referrer(),
                                WindowOpenDisposition::NEW_WINDOW,
                                ui::PAGE_TRANSITION_TYPED, false)));
-    // Hide all the buttons to test that it opens even when the browser action
-    // is in the overflow bucket.
-    ToolbarActionsModel::Get(profile())->SetVisibleIconCount(0);
+    // Pin the extension to test that it opens when the action is on the
+    // toolbar.
+    ToolbarActionsModel::Get(profile())->SetActionVisibility(
+        last_loaded_extension_id(), true);
     frame_observer.Wait();
   }
 
-  EXPECT_TRUE(new_browser != NULL);
+  EXPECT_TRUE(new_browser != nullptr);
 
   ResultCatcher catcher;
   {
-    content::WindowedNotificationObserver frame_observer(
-        content::NOTIFICATION_LOAD_COMPLETED_MAIN_FRAME,
-        content::NotificationService::AllSources());
+    content::CreateAndLoadWebContentsObserver frame_observer;
     // Show second popup in new window.
     listener.Reply("show another");
     frame_observer.Wait();
@@ -271,17 +299,18 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, TestOpenPopup) {
 
 // Tests opening a popup in an incognito window.
 IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, TestOpenPopupIncognito) {
-  content::WindowedNotificationObserver frame_observer(
-      content::NOTIFICATION_LOAD_COMPLETED_MAIN_FRAME,
-      content::NotificationService::AllSources());
-  ASSERT_TRUE(RunExtensionSubtest("browser_action/open_popup",
-                                  "open_popup_succeeds.html",
-                                  kFlagEnableIncognito, kFlagUseIncognito))
+  // The creation of the incognito window is the first WebContents.
+  content::CreateAndLoadWebContentsObserver frame_observer(
+      /*num_expected_contents=*/2);
+  ASSERT_TRUE(RunExtensionTest(
+      "browser_action/open_popup",
+      {.extension_url = "open_popup_succeeds.html", .open_in_incognito = true},
+      {.allow_in_incognito = true}))
       << message_;
   frame_observer.Wait();
   // Non-Aura Linux uses a singleton for the popup, so it looks like all windows
   // have popups if there is any popup open.
-#if !((defined(OS_LINUX) || defined(OS_CHROMEOS)) && !defined(USE_AURA))
+#if !((BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)) && !defined(USE_AURA))
   // Starting window does not have a popup.
   EXPECT_FALSE(ExtensionActionTestHelper::Create(browser())->HasPopup());
 #endif
@@ -302,7 +331,7 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest,
                         .AppendASCII("open_popup_background"),
                     {.allow_in_incognito = true});
   ASSERT_TRUE(extension);
-  ExtensionTestMessageListener listener(false);
+  ExtensionTestMessageListener listener;
   listener.set_extension_id(extension->id());
 
   Browser* incognito_browser =
@@ -319,20 +348,23 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest,
 IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest,
                        TestOpenPopupDoesNotCloseOtherPopups) {
   // Load a first extension that can open a popup.
-  ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII(
-      "browser_action/popup")));
-  const Extension* extension = GetSingleLoadedExtension();
-  ASSERT_TRUE(extension) << message_;
+  const Extension* first_extension =
+      LoadExtension(test_data_dir_.AppendASCII("browser_action/popup"));
+  ASSERT_TRUE(first_extension) << message_;
 
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   // Load the test extension which will do nothing except notifyPass() to
   // return control here.
-  ASSERT_TRUE(RunExtensionSubtest("browser_action/open_popup",
-                                  "open_popup_fails.html")) << message_;
+  ASSERT_TRUE(RunExtensionTest("browser_action/open_popup",
+                               {.extension_url = "open_popup_fails.html"}))
+      << message_;
   EXPECT_TRUE(listener.WaitUntilSatisfied());
-  OpenPopupViaToolbar();
+
+  // Open a popup with the first extension.
+  OpenPopupViaToolbar(first_extension->id());
   ResultCatcher catcher;
-  // Return control to javascript to validate that opening a popup fails now.
+  // Now, try to open a popup with the second extension. It should fail since
+  // there's an active popup.
   listener.Reply("show another");
   ASSERT_TRUE(catcher.GetNextResult()) << message_;
   EXPECT_TRUE(ClosePopup());
@@ -352,7 +384,7 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest,
                        sessions::SessionTabHelper::IdForTab(
                            browser()->tab_strip_model()->GetActiveWebContents())
                            .id(),
-                       APIPermission::kTab));
+                       mojom::APIPermissionID::kTab));
   EXPECT_TRUE(ClosePopup());
 }
 
@@ -369,7 +401,7 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, FocusLossClosesPopup2) {
       "browser_action/popup")));
   const Extension* extension = GetSingleLoadedExtension();
   ASSERT_TRUE(extension) << message_;
-  OpenPopupViaToolbar();
+  OpenPopupViaToolbar(extension->id());
   ClosePopupViaFocusLoss();
 }
 
@@ -382,13 +414,12 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, TabSwitchClosesPopup) {
             browser()->tab_strip_model()->GetActiveWebContents());
   OpenPopupViaAPI(false);
 
-  content::WindowedNotificationObserver observer(
-      extensions::NOTIFICATION_EXTENSION_HOST_DESTROYED,
-      content::NotificationService::AllSources());
+  ExtensionHostTestHelper host_helper(profile());
   // Change active tabs, the extension popup should close.
   browser()->tab_strip_model()->ActivateTabAt(
-      0, {TabStripModel::GestureType::kOther});
-  observer.Wait();
+      0, TabStripUserGestureDetails(
+             TabStripUserGestureDetails::GestureType::kOther));
+  host_helper.WaitForHostDestroyed();
 
   EXPECT_FALSE(ExtensionActionTestHelper::Create(browser())->HasPopup());
 }
@@ -422,8 +453,8 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, PopupZoomsIndependently) {
   ASSERT_TRUE(extension) << message_;
 
   // Navigate to one of the extension's pages in a tab.
-  ui_test_utils::NavigateToURL(browser(),
-                               extension->GetResourceURL("popup.html"));
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(
+      browser(), extension->GetResourceURL("popup.html")));
   content::WebContents* tab_contents =
       browser()->tab_strip_model()->GetActiveWebContents();
 
@@ -440,13 +471,10 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, PopupZoomsIndependently) {
   zoom_change_watcher.Wait();
 
   // Open the extension's popup.
-  content::WindowedNotificationObserver popup_observer(
-      NOTIFICATION_EXTENSION_HOST_CREATED,
-      content::NotificationService::AllSources());
-  OpenPopupViaToolbar();
-  popup_observer.Wait();
-  ExtensionHost* extension_host =
-      content::Details<ExtensionHost>(popup_observer.details()).ptr();
+  ExtensionHostTestHelper host_helper(profile(), extension->id());
+  OpenPopupViaToolbar(extension->id());
+  ExtensionHost* extension_host = host_helper.WaitForRenderProcessReady();
+  ASSERT_TRUE(extension_host);
   content::WebContents* popup_contents = extension_host->host_contents();
 
   // The popup should not use the per-origin zoom level that was set by zooming
@@ -486,10 +514,13 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, PopupZoomsIndependently) {
 class BrowserActionInteractiveViewsTest : public BrowserActionInteractiveTest {
  public:
   BrowserActionInteractiveViewsTest() = default;
-  ~BrowserActionInteractiveViewsTest() override = default;
 
- private:
-  DISALLOW_COPY_AND_ASSIGN(BrowserActionInteractiveViewsTest);
+  BrowserActionInteractiveViewsTest(const BrowserActionInteractiveViewsTest&) =
+      delete;
+  BrowserActionInteractiveViewsTest& operator=(
+      const BrowserActionInteractiveViewsTest&) = delete;
+
+  ~BrowserActionInteractiveViewsTest() override = default;
 };
 
 // Test closing the browser while inspecting an extension popup with dev tools.
@@ -502,10 +533,10 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveViewsTest,
   ASSERT_TRUE(extension) << message_;
 
   // Open an extension popup by clicking the browser action button.
-  content::WindowedNotificationObserver frame_observer(
-      content::NOTIFICATION_LOAD_COMPLETED_MAIN_FRAME,
-      content::NotificationService::AllSources());
-  ExtensionActionTestHelper::Create(browser())->InspectPopup(0);
+  // This creates two WebContents: the popup and the DevTools window.
+  content::CreateAndLoadWebContentsObserver frame_observer(
+      /*num_expected_contents=*/2);
+  ExtensionActionTestHelper::Create(browser())->InspectPopup(extension->id());
   frame_observer.Wait();
   EXPECT_TRUE(ExtensionActionTestHelper::Create(browser())->HasPopup());
 
@@ -513,13 +544,13 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveViewsTest,
   chrome::CloseWindow(browser());
 }
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 // Forcibly closing a browser HWND with a popup should not cause a crash.
 IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, DestroyHWNDDoesNotCrash) {
   OpenPopupViaAPI(false);
   auto test_util = ExtensionActionTestHelper::Create(browser());
   const gfx::NativeView popup_view = test_util->GetPopupNativeView();
-  EXPECT_NE(static_cast<gfx::NativeView>(nullptr), popup_view);
+  EXPECT_NE(gfx::NativeView(), popup_view);
   const HWND popup_hwnd = views::HWNDForNativeView(popup_view);
   EXPECT_EQ(TRUE, ::IsWindow(popup_hwnd));
   const HWND browser_hwnd =
@@ -537,7 +568,7 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, DestroyHWNDDoesNotCrash) {
   EXPECT_EQ(FALSE, ::IsWindow(browser_hwnd));
   EXPECT_EQ(FALSE, ::IsWindow(popup_hwnd));
 }
-#endif  // OS_WIN
+#endif  // BUILDFLAG(IS_WIN)
 
 class MainFrameSizeWaiter : public content::WebContentsObserver {
  public:
@@ -556,7 +587,7 @@ class MainFrameSizeWaiter : public content::WebContentsObserver {
     return web_contents()->GetContainerBounds().size();
   }
 
-  void MainFrameWasResized(bool width_changed) override {
+  void PrimaryMainFrameWasResized(bool width_changed) override {
     if (current_size() == size_to_wait_for_)
       run_loop_.Quit();
   }
@@ -565,9 +596,20 @@ class MainFrameSizeWaiter : public content::WebContentsObserver {
   base::RunLoop run_loop_;
 };
 
-IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, BrowserActionPopup) {
-  base::AutoReset<bool> disable_toolbar_animations(
-      &ToolbarActionsBar::disable_animations_for_testing_, true);
+// TODO(crbug.com/1249851): Test crashes on Windows
+#if BUILDFLAG(IS_WIN)
+#define MAYBE_BrowserActionPopup DISABLED_BrowserActionPopup
+#elif BUILDFLAG(IS_LINUX) && \
+    (defined(THREAD_SANITIZER) || defined(ADDRESS_SANITIZER))
+// TODO(crbug.com/1269076): Test is flaky for linux tsan and asan builds
+#define MAYBE_BrowserActionPopup DISABLED_BrowserActionPopup
+#elif BUILDFLAG(IS_MAC)
+// TODO(crbug.com/1269076): Test is flaky on Mac as well.
+#define MAYBE_BrowserActionPopup DISABLED_BrowserActionPopup
+#else
+#define MAYBE_BrowserActionPopup BrowserActionPopup
+#endif
+IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, MAYBE_BrowserActionPopup) {
   ASSERT_TRUE(
       LoadExtension(test_data_dir_.AppendASCII("browser_action/popup")));
   const Extension* extension = GetSingleLoadedExtension();
@@ -586,21 +628,20 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, BrowserActionPopup) {
   ASSERT_GT(minSize.width() + kGrowFactor * 2, maxSize.width());
 
   // Simulate a click on the browser action and verify the size of the resulting
-  // popup.
-  const gfx::Size kExpectedSizes[] = {minSize, middleSize, maxSize};
-  for (size_t i = 0; i < base::size(kExpectedSizes); i++) {
+  // popup. It is important to do minSize last, because all browser actions
+  // start at minSize and later increase in size. The MainFrameSizeWaiter won't
+  // wait for the popup.js code to run in the minSize case, which can prevent it
+  // from setting and storing the size for the next iteration, resulting in test
+  // flakiness.
+  const gfx::Size kExpectedSizes[] = {maxSize, middleSize, minSize};
+  for (size_t i = 0; i < std::size(kExpectedSizes); i++) {
     content::WebContentsAddedObserver popup_observer;
-    actions_bar->Press(0);
+    actions_bar->Press(extension->id());
     content::WebContents* popup = popup_observer.GetWebContents();
-
-    if (base::FeatureList::IsEnabled(features::kExtensionsToolbarMenu)) {
-      actions_bar->WaitForExtensionsContainerLayout();
-    } else {
-      RunScheduledLayouts();
-    }
+    actions_bar->WaitForExtensionsContainerLayout();
 
     gfx::Size max_available_size =
-        actions_bar->GetMaxAvailableSizeToFitBubbleOnScreen(0);
+        actions_bar->GetMaxAvailableSizeToFitBubbleOnScreen(extension->id());
 
     // Take the screen boundaries into account for calculating the size of the
     // displayed popup
@@ -613,7 +654,7 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, BrowserActionPopup) {
     // scrollbars are overlaid, appear on hover and don't increase the height
     // or width of the popup.
     const int kScrollbarAdjustment =
-#if defined(OS_MAC)
+#if BUILDFLAG(IS_MAC)
         0;
 #else
         gfx::scrollbar_size();
@@ -649,11 +690,11 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest,
   ASSERT_TRUE(extension) << message_;
 
   content::DownloadTestObserverTerminal downloads_observer(
-      content::BrowserContext::GetDownloadManager(browser()->profile()), 1,
+      browser()->profile()->GetDownloadManager(), 1,
       content::DownloadTestObserver::ON_DANGEROUS_DOWNLOAD_FAIL);
 
   // Simulate a click on the browser action to open the popup.
-  content::WebContents* popup = OpenPopupViaToolbar();
+  content::WebContents* popup = OpenPopupViaToolbar(extension->id());
   ASSERT_TRUE(popup);
   content::ExecuteScriptAsync(popup, "run_tests()");
 
@@ -683,7 +724,9 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, OpenPopupOnPopup) {
   // that's what we check when we try to open the popup.
   // TODO(crbug.com/1115237): Now that this is an interactive test, is this
   // ifdef still necessary?
-#if !defined(OS_MAC)
+#if !BUILDFLAG(IS_MAC)
+  ui_test_utils::BrowserActivationWaiter waiter(popup_browser);
+  waiter.WaitForActivation();
   EXPECT_TRUE(popup_browser->window()->IsActive());
 #endif
   EXPECT_FALSE(browser()->window()->IsActive());
@@ -693,7 +736,7 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest, OpenPopupOnPopup) {
 
   // Load up the extension, which will call chrome.browserAction.openPopup()
   // when it is loaded and verify that the popup didn't open.
-  ExtensionTestMessageListener listener("ready", true);
+  ExtensionTestMessageListener listener("ready", ReplyBehavior::kWillReply);
   EXPECT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("browser_action/open_popup_on_reply")));
   EXPECT_TRUE(listener.WaitUntilSatisfied());
@@ -723,7 +766,7 @@ class RenderFrameChangedWatcher : public content::WebContentsObserver {
 
  private:
   base::RunLoop run_loop_;
-  content::RenderFrameHost* created_frame_;
+  raw_ptr<content::RenderFrameHost> created_frame_;
 };
 
 // Test that a browser action popup with a web iframe works correctly. The
@@ -732,14 +775,13 @@ class RenderFrameChangedWatcher : public content::WebContentsObserver {
 IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest,
                        BrowserActionPopupWithIframe) {
   ASSERT_TRUE(embedded_test_server()->Start());
-
   ASSERT_TRUE(LoadExtension(
       test_data_dir_.AppendASCII("browser_action/popup_with_iframe")));
   const Extension* extension = GetSingleLoadedExtension();
   ASSERT_TRUE(extension) << message_;
 
   // Simulate a click on the browser action to open the popup.
-  ASSERT_TRUE(OpenPopupViaToolbar());
+  ASSERT_TRUE(OpenPopupViaToolbar(extension->id()));
 
   // Find the RenderFrameHost associated with the iframe in the popup.
   content::RenderFrameHost* frame_host = nullptr;
@@ -765,7 +807,7 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest,
       content::WebContents::FromRenderFrameHost(frame_host));
   GURL foo_url(embedded_test_server()->GetURL("foo.com", "/popup_iframe.html"));
   std::string script = "location.href = '" + foo_url.spec() + "'";
-  EXPECT_TRUE(ExecuteScript(frame_host, script));
+  EXPECT_TRUE(ExecJs(frame_host, script));
 
   frame_host = watcher.WaitAndReturnNewFrame();
 
@@ -778,6 +820,72 @@ IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveTest,
   EXPECT_TRUE(ClosePopup());
 }
 
+class BrowserActionInteractiveFencedFrameTest
+    : public BrowserActionInteractiveTest {
+ public:
+  ~BrowserActionInteractiveFencedFrameTest() override = default;
+
+  content::test::FencedFrameTestHelper& fenced_frame_test_helper() {
+    return fenced_frame_test_helper_;
+  }
+
+ private:
+  content::test::FencedFrameTestHelper fenced_frame_test_helper_;
+};
+
+IN_PROC_BROWSER_TEST_F(BrowserActionInteractiveFencedFrameTest,
+                       BrowserActionPopupWithFencedFrame) {
+  net::EmbeddedTestServer https_server(net::EmbeddedTestServer::TYPE_HTTPS);
+  https_server.SetSSLConfig(net::EmbeddedTestServer::CERT_TEST_NAMES);
+  https_server.ServeFilesFromSourceDirectory("chrome/test/data");
+  ASSERT_TRUE(https_server.Start());
+
+  ASSERT_TRUE(LoadExtension(
+      test_data_dir_.AppendASCII("browser_action/popup_with_fencedframe")));
+  const Extension* extension = GetSingleLoadedExtension();
+  ASSERT_TRUE(extension) << message_;
+
+  // Simulate a click on the browser action to open the popup.
+  ASSERT_TRUE(OpenPopupViaToolbar(extension->id()));
+
+  // Find a primary main frame associated in the popup.
+  extensions::ProcessManager* manager =
+      extensions::ProcessManager::Get(browser()->profile());
+  std::set<content::RenderFrameHost*> hosts =
+      manager->GetRenderFrameHostsForExtension(extension->id());
+  const auto& it = base::ranges::find_if(
+      hosts, &content::RenderFrameHost::IsInPrimaryMainFrame);
+  content::RenderFrameHost* primary_render_frame_host =
+      (it != hosts.end()) ? *it : nullptr;
+  ASSERT_TRUE(primary_render_frame_host);
+
+  // Navigate the popup's fenced frame to a (cross-site) web page via its
+  // parent, and wait for that page to send a message, which will ensure that
+  // the page has loaded.
+  GURL foo_url(https_server.GetURL("a.test", "/popup_fencedframe.html"));
+
+  content::TestNavigationObserver observer(
+      content::WebContents::FromRenderFrameHost(primary_render_frame_host));
+  std::string script =
+      "document.querySelector('fencedframe').config = new FencedFrameConfig('" +
+      foo_url.spec() + "')";
+  EXPECT_TRUE(ExecJs(primary_render_frame_host, script));
+  observer.WaitForNavigationFinished();
+
+  content::RenderFrameHost* fenced_frame_render_frame_host =
+      fenced_frame_test_helper().GetMostRecentlyAddedFencedFrame(
+          primary_render_frame_host);
+  ASSERT_TRUE(fenced_frame_render_frame_host);
+
+  // Confirm that the new page (popup_fencedframe.html) is actually loaded.
+  content::DOMMessageQueue dom_message_queue(fenced_frame_render_frame_host);
+  std::string json;
+  EXPECT_TRUE(dom_message_queue.WaitForMessage(&json));
+  EXPECT_EQ("\"DONE\"", json);
+
+  EXPECT_TRUE(ClosePopup());
+}
+
 class NavigatingExtensionPopupInteractiveTest
     : public BrowserActionInteractiveTest {
  public:
@@ -838,30 +946,28 @@ class NavigatingExtensionPopupInteractiveTest
       std::unique_ptr<ExtensionActionTestHelper> action_bar_helper =
           ExtensionActionTestHelper::Create(browser());
       ASSERT_EQ(2, action_bar_helper->NumberOfBrowserActions());
-      ASSERT_EQ(popup_extension().id(), action_bar_helper->GetExtensionId(0));
-      ASSERT_EQ(other_extension().id(), action_bar_helper->GetExtensionId(1));
+      EXPECT_TRUE(action_bar_helper->HasAction(popup_extension().id()));
+      EXPECT_TRUE(action_bar_helper->HasAction(other_extension().id()));
     }
 
     // Simulate a click on the browser action to open the popup.
-    content::WebContents* popup = OpenPopupViaToolbar();
+    content::WebContents* popup = OpenPopupViaToolbar(popup_extension().id());
     ASSERT_TRUE(popup);
     GURL popup_url = popup_extension().GetResourceURL("popup.html");
     EXPECT_EQ(popup_url, popup->GetLastCommittedURL());
 
-    // Note that the |setTimeout| call below is needed to make sure
-    // ExecuteScriptAndExtractBool returns *after* a scheduled navigation has
-    // already started.
-    std::string script_to_execute =
-        navigation_starting_script +
-        "setTimeout(\n"
-        "    function() { window.domAutomationController.send(true); },\n"
-        "    0);\n";
+    // Note that the |setTimeout| call below is needed to make sure EvalJs
+    // returns *after* a scheduled navigation has already started.
+    std::string script_to_execute = navigation_starting_script +
+                                    "new Promise(resolve => {\n"
+                                    "  setTimeout(\n"
+                                    "    function() { resolve(true); },\n"
+                                    "    0);\n"
+                                    "});\n";
 
     // Try to navigate the pop-up.
-    bool ignored_script_result = false;
     content::WebContentsDestroyedWatcher popup_destruction_watcher(popup);
-    EXPECT_TRUE(ExecuteScriptAndExtractBool(popup, script_to_execute,
-                                            &ignored_script_result));
+    EXPECT_TRUE(ExecJs(popup, script_to_execute));
     popup = popup_destruction_watcher.web_contents();
 
     // Verify if the popup navigation succeeded or failed as expected.
@@ -909,8 +1015,8 @@ class NavigatingExtensionPopupInteractiveTest
     }
   }
 
-  const Extension* popup_extension_;
-  const Extension* other_extension_;
+  raw_ptr<const Extension, DanglingUntriaged> popup_extension_;
+  raw_ptr<const Extension, DanglingUntriaged> other_extension_;
 };
 
 // Tests that an extension pop-up cannot be navigated to a web page.
@@ -961,7 +1067,7 @@ IN_PROC_BROWSER_TEST_F(NavigatingExtensionPopupInteractiveTest,
                        DownloadViaPost) {
   // Setup monitoring of the downloads.
   content::DownloadTestObserverTerminal downloads_observer(
-      content::BrowserContext::GetDownloadManager(browser()->profile()),
+      browser()->profile()->GetDownloadManager(),
       1,  // == wait_count (only waiting for "download-test3.gif").
       content::DownloadTestObserver::ON_DANGEROUS_DOWNLOAD_FAIL);
 
@@ -985,10 +1091,11 @@ IN_PROC_BROWSER_TEST_F(NavigatingExtensionPopupInteractiveTest,
       downloads_directory.AppendASCII("download-test3-attachment.gif")));
 
   // The test verification below is applicable only to scenarios where the
-  // download shelf is supported - on ChromeOS, instead of the download shelf,
-  // there is a download notification in the right-bottom corner of the screen.
-#if !BUILDFLAG(IS_CHROMEOS_ASH)
-  EXPECT_TRUE(browser()->window()->IsDownloadShelfVisible());
+  // download surface is supported - on ChromeOS, instead of the download
+  // surface, there is a download notification in the right-bottom corner of the
+  // screen.
+#if !BUILDFLAG(IS_CHROMEOS)
+  EXPECT_TRUE(IsDownloadSurfaceVisible(browser()->window()));
 #endif
 }
 
@@ -996,7 +1103,7 @@ IN_PROC_BROWSER_TEST_F(NavigatingExtensionPopupInteractiveTest,
                        DownloadViaGet) {
   // Setup monitoring of the downloads.
   content::DownloadTestObserverTerminal downloads_observer(
-      content::BrowserContext::GetDownloadManager(browser()->profile()),
+      browser()->profile()->GetDownloadManager(),
       1,  // == wait_count (only waiting for "download-test3.gif").
       content::DownloadTestObserver::ON_DANGEROUS_DOWNLOAD_FAIL);
 
@@ -1020,10 +1127,11 @@ IN_PROC_BROWSER_TEST_F(NavigatingExtensionPopupInteractiveTest,
       downloads_directory.AppendASCII("download-test3-attachment.gif")));
 
   // The test verification below is applicable only to scenarios where the
-  // download shelf is supported - on ChromeOS, instead of the download shelf,
-  // there is a download notification in the right-bottom corner of the screen.
-#if !BUILDFLAG(IS_CHROMEOS_ASH)
-  EXPECT_TRUE(browser()->window()->IsDownloadShelfVisible());
+  // download surface is supported - on ChromeOS, instead of the download
+  // surface, there is a download notification in the right-bottom corner of the
+  // screen.
+#if !BUILDFLAG(IS_CHROMEOS)
+  EXPECT_TRUE(IsDownloadSurfaceVisible(browser()->window()));
 #endif
 }
 
