@@ -1,23 +1,22 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/api/extension_action/extension_action_api.h"
 
 #include <stddef.h>
+
 #include <memory>
 #include <utility>
 
-#include "base/bind.h"
+#include "base/functional/bind.h"
 #include "base/lazy_instance.h"
 #include "base/location.h"
 #include "base/metrics/histogram_functions.h"
-#include "base/metrics/histogram_macros.h"
-#include "base/single_thread_task_runner.h"
-#include "base/stl_util.h"
+#include "base/observer_list.h"
+#include "base/ranges/algorithm.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/threading/thread_task_runner_handle.h"
-#include "base/values.h"
+#include "base/task/single_thread_task_runner.h"
 #include "chrome/browser/extensions/extension_action_runner.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/extensions/extension_ui_util.h"
@@ -28,8 +27,9 @@
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/extensions/extensions_container.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
+#include "chrome/browser/ui/toolbar/toolbar_actions_model.h"
 #include "components/sessions/content/session_tab_helper.h"
-#include "content/public/browser/notification_service.h"
+#include "content/public/common/color_parser.h"
 #include "extensions/browser/api/declarative_net_request/constants.h"
 #include "extensions/browser/api/declarative_net_request/utils.h"
 #include "extensions/browser/event_router.h"
@@ -38,13 +38,17 @@
 #include "extensions/browser/extension_prefs.h"
 #include "extensions/browser/extension_registry.h"
 #include "extensions/browser/extension_util.h"
-#include "extensions/browser/notification_types.h"
 #include "extensions/common/api/extension_action/action_info.h"
+#include "extensions/common/constants.h"
 #include "extensions/common/error_utils.h"
 #include "extensions/common/feature_switch.h"
 #include "extensions/common/image_util.h"
+#include "extensions/common/manifest_constants.h"
+#include "extensions/common/mojom/context_type.mojom.h"
+#include "extensions/common/mojom/view_type.mojom.h"
 #include "ui/gfx/image/image.h"
 #include "ui/gfx/image/image_skia.h"
+#include "url/origin.h"
 
 using content::WebContents;
 
@@ -59,11 +63,105 @@ const char kNoTabError[] = "No tab with id: *.";
 const char kOpenPopupError[] =
     "Failed to show popup either because there is an existing popup or another "
     "error occurred.";
-const char kInvalidColorError[] =
-    "The color specification could not be parsed.";
+const char kFailedToOpenPopupGenericError[] = "Failed to open popup.";
+constexpr char kNoActiveWindowFound[] =
+    "Could not find an active browser window.";
+constexpr char kNoActivePopup[] =
+    "Extension does not have a popup on the active tab.";
 
 bool g_report_error_for_invisible_icon = false;
 
+// Returns the browser that was last active in the given `profile`, optionally
+// also checking the incognito profile.
+Browser* FindLastActiveBrowserWindow(Profile* profile,
+                                     bool check_incognito_profile) {
+  Browser* browser = chrome::FindLastActiveWithProfile(profile);
+
+  if (browser && browser->window()->IsActive())
+    return browser;  // Found an active browser.
+
+  // It's possible that the last active browser actually corresponds to the
+  // associated incognito profile, and this won't be returned by
+  // FindLastActiveWithProfile(). If the extension can operate incognito, then
+  // check the last active incognito, too.
+  if (check_incognito_profile && profile->HasPrimaryOTRProfile()) {
+    Profile* incognito_profile =
+        profile->GetPrimaryOTRProfile(/*create_if_needed=*/false);
+    DCHECK(incognito_profile);
+    Browser* incognito_browser =
+        chrome::FindLastActiveWithProfile(incognito_profile);
+    if (incognito_browser->window()->IsActive())
+      return incognito_browser;
+  }
+
+  return nullptr;
+}
+
+// Returns true if the color values provided could be parsed into a color
+// object out param.
+bool ParseColor(const base::Value& color_value, SkColor& color) {
+  if (color_value.is_string())
+    return content::ParseCssColorString(color_value.GetString(), &color);
+
+  if (!color_value.is_list())
+    return false;
+
+  const base::Value::List& color_list = color_value.GetList();
+  if (color_list.size() != 4 ||
+      base::ranges::any_of(color_list,
+                           [](const auto& color) { return !color.is_int(); })) {
+    return false;
+  }
+
+  color = SkColorSetARGB(color_list[3].GetInt(), color_list[0].GetInt(),
+                         color_list[1].GetInt(), color_list[2].GetInt());
+  return true;
+}
+
+// Returns true if the given `extension` has an active popup on the active tab
+// of `browser`.
+bool HasPopupOnActiveTab(Browser* browser,
+                         content::BrowserContext* browser_context,
+                         const Extension& extension) {
+  content::WebContents* web_contents =
+      browser->tab_strip_model()->GetActiveWebContents();
+  ExtensionAction* extension_action =
+      ExtensionActionManager::Get(browser_context)
+          ->GetExtensionAction(extension);
+  DCHECK(extension_action);
+  int tab_id = ExtensionTabUtil::GetTabId(web_contents);
+
+  return extension_action->HasPopup(tab_id) &&
+         extension_action->GetIsVisibleIgnoringDeclarative(tab_id);
+}
+
+// Attempts to open `extension`'s popup in the given `browser`. Returns true on
+// success; otherwise, populates `error` and returns false.
+bool OpenPopupInBrowser(Browser& browser,
+                        const Extension& extension,
+                        std::string* error,
+                        ShowPopupCallback callback) {
+  if (!browser.SupportsWindowFeature(Browser::FEATURE_TOOLBAR) ||
+      !browser.window()->IsToolbarVisible()) {
+    *error = "Browser window has no toolbar.";
+    return false;
+  }
+
+  ExtensionsContainer* extensions_container =
+      browser.window()->GetExtensionsContainer();
+  // The ExtensionsContainer could be null if, e.g., this is a popup window with
+  // no toolbar.
+  // TODO(devlin): Is that still possible, given the checks above?
+  if (!extensions_container ||
+      !extensions_container->ShowToolbarActionPopupForAPICall(
+          extension.id(), std::move(callback))) {
+    *error = kFailedToOpenPopupGenericError;
+    return false;
+  }
+
+  return true;
+}
+
 }  // namespace
 
 //
@@ -114,28 +212,6 @@ void ExtensionActionAPI::RemoveObserver(Observer* observer) {
   observers_.RemoveObserver(observer);
 }
 
-bool ExtensionActionAPI::ShowExtensionActionPopupForAPICall(
-    const Extension* extension,
-    Browser* browser) {
-  ExtensionAction* extension_action =
-      ExtensionActionManager::Get(browser_context_)->GetExtensionAction(
-          *extension);
-  if (!extension_action)
-    return false;
-
-  // Don't support showing action popups in a popup window.
-  if (!browser->SupportsWindowFeature(Browser::FEATURE_TOOLBAR))
-    return false;
-
-  ExtensionsContainer* extensions_container =
-      browser->window()->GetExtensionsContainer();
-  // The ExtensionsContainer could be null if, e.g., this is a popup window with
-  // no toolbar.
-  return extensions_container &&
-         extensions_container->ShowToolbarActionPopupForAPICall(
-             extension->id());
-}
-
 void ExtensionActionAPI::NotifyChange(ExtensionAction* extension_action,
                                       content::WebContents* web_contents,
                                       content::BrowserContext* context) {
@@ -148,7 +224,7 @@ void ExtensionActionAPI::DispatchExtensionActionClicked(
     WebContents* web_contents,
     const Extension* extension) {
   events::HistogramValue histogram_value = events::UNKNOWN;
-  const char* event_name = NULL;
+  const char* event_name = nullptr;
   switch (extension_action.action_type()) {
     case ActionInfo::TYPE_ACTION:
       histogram_value = events::ACTION_ON_CLICKED;
@@ -165,18 +241,18 @@ void ExtensionActionAPI::DispatchExtensionActionClicked(
   }
 
   if (event_name) {
-    std::unique_ptr<base::ListValue> args(new base::ListValue());
+    base::Value::List args;
     // The action APIs (browserAction, pageAction, action) are only available
     // to blessed extension contexts. As such, we deterministically know that
     // the right context type here is blessed.
-    constexpr Feature::Context context_type =
-        Feature::BLESSED_EXTENSION_CONTEXT;
+    constexpr mojom::ContextType context_type =
+        mojom::ContextType::kPrivilegedExtension;
     ExtensionTabUtil::ScrubTabBehavior scrub_tab_behavior =
         ExtensionTabUtil::GetScrubTabBehavior(extension, context_type,
                                               web_contents);
-    args->Append(ExtensionTabUtil::CreateTabObject(
-                     web_contents, scrub_tab_behavior, extension)
-                     ->ToValue());
+    args.Append(ExtensionTabUtil::CreateTabObject(web_contents,
+                                                  scrub_tab_behavior, extension)
+                    .ToValue());
 
     DispatchEventToExtension(web_contents->GetBrowserContext(),
                              extension_action.extension_id(), histogram_value,
@@ -219,7 +295,7 @@ void ExtensionActionAPI::DispatchEventToExtension(
     const std::string& extension_id,
     events::HistogramValue histogram_value,
     const std::string& event_name,
-    std::unique_ptr<base::ListValue> event_args) {
+    base::Value::List event_args) {
   if (!EventRouter::Get(context))
     return;
 
@@ -264,10 +340,14 @@ ExtensionFunction::ResponseAction ExtensionActionFunction::Run() {
 
   // Find the WebContents that contains this tab id if one is required.
   if (tab_id_ != ExtensionAction::kDefaultTabId) {
+    content::WebContents* contents_out_param = nullptr;
     ExtensionTabUtil::GetTabById(tab_id_, browser_context(),
-                                 include_incognito_information(), &contents_);
-    if (!contents_)
+                                 include_incognito_information(),
+                                 &contents_out_param);
+    if (!contents_out_param) {
       return RespondNow(Error(kNoTabError, base::NumberToString(tab_id_)));
+    }
+    contents_ = contents_out_param;
   } else {
     // Page actions do not have a default tabId.
     EXTENSION_FUNCTION_VALIDATE(extension_action_->action_type() !=
@@ -281,27 +361,27 @@ bool ExtensionActionFunction::ExtractDataFromArguments() {
   // The tabId might appear in details (if it exists), as the first
   // argument besides the action type (depends on the function), or be omitted
   // entirely.
-  base::Value* first_arg = NULL;
-  if (!args_->Get(0, &first_arg))
+  if (args().empty())
     return true;
 
-  switch (first_arg->type()) {
+  base::Value& first_arg = mutable_args()[0];
+
+  switch (first_arg.type()) {
     case base::Value::Type::INTEGER:
-      CHECK(first_arg->GetAsInteger(&tab_id_));
+      tab_id_ = first_arg.GetInt();
       break;
 
-    case base::Value::Type::DICTIONARY: {
+    case base::Value::Type::DICT: {
       // Found the details argument.
-      details_ = static_cast<base::DictionaryValue*>(first_arg);
+      details_ = &first_arg.GetDict();
       // Still need to check for the tabId within details.
-      base::Value* tab_id_value = NULL;
-      if (details_->Get("tabId", &tab_id_value)) {
+      if (base::Value* tab_id_value = details_->Find("tabId")) {
         switch (tab_id_value->type()) {
           case base::Value::Type::NONE:
             // OK; tabId is optional, leave it default.
             return true;
           case base::Value::Type::INTEGER:
-            CHECK(tab_id_value->GetAsInteger(&tab_id_));
+            tab_id_ = tab_id_value->GetInt();
             return true;
           default:
             // Boom.
@@ -347,6 +427,12 @@ ExtensionActionHideFunction::RunExtensionAction() {
   return RespondNow(NoArguments());
 }
 
+ExtensionFunction::ResponseAction
+ActionIsEnabledFunction::RunExtensionAction() {
+  return RespondNow(WithArguments(
+      extension_action_->GetIsVisibleIgnoringDeclarative(tab_id_)));
+}
+
 // static
 void ExtensionActionSetIconFunction::SetReportErrorForInvisibleIconForTesting(
     bool value) {
@@ -355,51 +441,18 @@ void ExtensionActionSetIconFunction::SetReportErrorForInvisibleIconForTesting(
 
 ExtensionFunction::ResponseAction
 ExtensionActionSetIconFunction::RunExtensionAction() {
-  // TODO(devlin): Temporary logging to track down https://crbug.com/1087948.
-  // Remove this (and the redundant `if (!x) { VALIDATE(x); }`) checks after
-  // the bug is fixed.
-  // Don't reorder or remove values.
-  enum class FailureType {
-    kFailedToParseDetails = 0,
-    kFailedToDecodeCanvas = 1,
-    kFailedToUnpickleCanvas = 2,
-    kNoImageDataOrIconIndex = 3,
-    kMaxValue = kNoImageDataOrIconIndex,
-  };
-
-  auto log_set_icon_failure = [](FailureType type) {
-    base::UmaHistogramEnumeration("Extensions.ActionSetIconFailureType", type);
-  };
-
-  if (!details_) {
-    log_set_icon_failure(FailureType::kFailedToParseDetails);
-    EXTENSION_FUNCTION_VALIDATE(details_);
-  }
+  EXTENSION_FUNCTION_VALIDATE(details_);
 
   // setIcon can take a variant argument: either a dictionary of canvas
   // ImageData, or an icon index.
-  base::DictionaryValue* canvas_set = NULL;
-  int icon_index;
-  if (details_->GetDictionary("imageData", &canvas_set)) {
+  base::Value::Dict* canvas_set = details_->FindDict("imageData");
+  if (canvas_set) {
     gfx::ImageSkia icon;
 
     ExtensionAction::IconParseResult parse_result =
         ExtensionAction::ParseIconFromCanvasDictionary(*canvas_set, &icon);
-
-    if (parse_result != ExtensionAction::IconParseResult::kSuccess) {
-      switch (parse_result) {
-        case ExtensionAction::IconParseResult::kDecodeFailure:
-          log_set_icon_failure(FailureType::kFailedToDecodeCanvas);
-          break;
-        case ExtensionAction::IconParseResult::kUnpickleFailure:
-          log_set_icon_failure(FailureType::kFailedToUnpickleCanvas);
-          break;
-        case ExtensionAction::IconParseResult::kSuccess:
-          NOTREACHED();
-          break;
-      }
-      EXTENSION_FUNCTION_VALIDATE(false);
-    }
+    EXTENSION_FUNCTION_VALIDATE(parse_result ==
+                                ExtensionAction::IconParseResult::kSuccess);
 
     if (icon.isNull())
       return RespondNow(Error("Icon invalid."));
@@ -407,26 +460,17 @@ ExtensionActionSetIconFunction::RunExtensionAction() {
     gfx::Image icon_image(icon);
     const SkBitmap bitmap = icon_image.AsBitmap();
     const bool is_visible = image_util::IsIconSufficientlyVisible(bitmap);
-    UMA_HISTOGRAM_BOOLEAN("Extensions.DynamicExtensionActionIconWasVisible",
-                          is_visible);
-    const bool is_visible_rendered =
-        extensions::ui_util::IsRenderedIconSufficientlyVisibleForBrowserContext(
-            bitmap, browser_context());
-    UMA_HISTOGRAM_BOOLEAN(
-        "Extensions.DynamicExtensionActionIconWasVisibleRendered",
-        is_visible_rendered);
-
     if (!is_visible && g_report_error_for_invisible_icon)
       return RespondNow(Error("Icon not sufficiently visible."));
 
     extension_action_->SetIcon(tab_id_, icon_image);
-  } else if (details_->GetInteger("iconIndex", &icon_index)) {
+  } else if (details_->FindInt("iconIndex")) {
     // Obsolete argument: ignore it.
     return RespondNow(NoArguments());
   } else {
-    log_set_icon_failure(FailureType::kNoImageDataOrIconIndex);
     EXTENSION_FUNCTION_VALIDATE(false);
   }
+
   NotifyChange();
   return RespondNow(NoArguments());
 }
@@ -434,9 +478,9 @@ ExtensionActionSetIconFunction::RunExtensionAction() {
 ExtensionFunction::ResponseAction
 ExtensionActionSetTitleFunction::RunExtensionAction() {
   EXTENSION_FUNCTION_VALIDATE(details_);
-  std::string title;
-  EXTENSION_FUNCTION_VALIDATE(details_->GetString("title", &title));
-  extension_action_->SetTitle(tab_id_, title);
+  const std::string* title = details_->FindString("title");
+  EXTENSION_FUNCTION_VALIDATE(title);
+  extension_action_->SetTitle(tab_id_, *title);
   NotifyChange();
   return RespondNow(NoArguments());
 }
@@ -444,12 +488,23 @@ ExtensionActionSetTitleFunction::RunExtensionAction() {
 ExtensionFunction::ResponseAction
 ExtensionActionSetPopupFunction::RunExtensionAction() {
   EXTENSION_FUNCTION_VALIDATE(details_);
-  std::string popup_string;
-  EXTENSION_FUNCTION_VALIDATE(details_->GetString("popup", &popup_string));
-
+  std::string* popup_string = details_->FindString("popup");
+  EXTENSION_FUNCTION_VALIDATE(popup_string);
   GURL popup_url;
-  if (!popup_string.empty())
-    popup_url = extension()->GetResourceURL(popup_string);
+
+  // If an empty string is passed, remove the explicitly set popup. Setting it
+  // back to an empty string (URL) will cause it to fall back to the default set
+  // in the manifest.
+  if (!popup_string->empty()) {
+    popup_url = extension()->GetResourceURL(*popup_string);
+    // Validate popup is same-origin (only for this extension). We do not
+    // validate the file exists (like we do in manifest validation) because an
+    // extension could potentially intercept the request with a service worker
+    // and dynamically provide content.
+    if (!extension()->origin().IsSameOriginWith(popup_url)) {
+      return RespondNow(Error(manifest_errors::kInvalidExtensionOriginPopup));
+    }
+  }
 
   extension_action_->SetPopupUrl(tab_id_, popup_url);
   NotifyChange();
@@ -460,9 +515,9 @@ ExtensionFunction::ResponseAction
 ExtensionActionSetBadgeTextFunction::RunExtensionAction() {
   EXTENSION_FUNCTION_VALIDATE(details_);
 
-  std::string badge_text;
-  if (details_->GetString("text", &badge_text))
-    extension_action_->SetBadgeText(tab_id_, badge_text);
+  std::string* badge_text = details_->FindString("text");
+  if (badge_text)
+    extension_action_->SetBadgeText(tab_id_, *badge_text);
   else
     extension_action_->ClearBadgeText(tab_id_);
 
@@ -473,43 +528,41 @@ ExtensionActionSetBadgeTextFunction::RunExtensionAction() {
 ExtensionFunction::ResponseAction
 ExtensionActionSetBadgeBackgroundColorFunction::RunExtensionAction() {
   EXTENSION_FUNCTION_VALIDATE(details_);
-  base::Value* color_value = NULL;
-  EXTENSION_FUNCTION_VALIDATE(details_->Get("color", &color_value));
+  base::Value* color_value = details_->Find("color");
+  EXTENSION_FUNCTION_VALIDATE(color_value);
   SkColor color = 0;
-  if (color_value->is_list()) {
-    base::ListValue* list = NULL;
-    EXTENSION_FUNCTION_VALIDATE(details_->GetList("color", &list));
-    EXTENSION_FUNCTION_VALIDATE(list->GetSize() == 4);
-
-    int color_array[4] = {0};
-    for (size_t i = 0; i < base::size(color_array); ++i) {
-      EXTENSION_FUNCTION_VALIDATE(list->GetInteger(i, &color_array[i]));
-    }
-
-    color = SkColorSetARGB(color_array[3], color_array[0],
-                           color_array[1], color_array[2]);
-  } else if (color_value->is_string()) {
-    std::string color_string;
-    EXTENSION_FUNCTION_VALIDATE(details_->GetString("color", &color_string));
-    if (!image_util::ParseCssColorString(color_string, &color))
-      return RespondNow(Error(kInvalidColorError));
-  }
-
+  if (!ParseColor(*color_value, color))
+    return RespondNow(Error(extension_misc::kInvalidColorError));
   extension_action_->SetBadgeBackgroundColor(tab_id_, color);
   NotifyChange();
   return RespondNow(NoArguments());
 }
 
+ExtensionFunction::ResponseAction
+ActionSetBadgeTextColorFunction::RunExtensionAction() {
+  EXTENSION_FUNCTION_VALIDATE(details_);
+  base::Value* color_value = details_->Find("color");
+  EXTENSION_FUNCTION_VALIDATE(color_value);
+  SkColor color = 0;
+  if (!ParseColor(*color_value, color))
+    return RespondNow(Error(extension_misc::kInvalidColorError));
+
+  if (SkColorGetA(color) == SK_AlphaTRANSPARENT)
+    return RespondNow(Error(extension_misc::kInvalidColorError));
+  extension_action_->SetBadgeTextColor(tab_id_, color);
+  NotifyChange();
+  return RespondNow(NoArguments());
+}
+
 ExtensionFunction::ResponseAction
 ExtensionActionGetTitleFunction::RunExtensionAction() {
-  return RespondNow(
-      OneArgument(base::Value(extension_action_->GetTitle(tab_id_))));
+  return RespondNow(WithArguments(extension_action_->GetTitle(tab_id_)));
 }
 
 ExtensionFunction::ResponseAction
 ExtensionActionGetPopupFunction::RunExtensionAction() {
   return RespondNow(
-      OneArgument(base::Value(extension_action_->GetPopupUrl(tab_id_).spec())));
+      WithArguments(extension_action_->GetPopupUrl(tab_id_).spec()));
 }
 
 ExtensionFunction::ResponseAction
@@ -526,72 +579,187 @@ ExtensionActionGetBadgeTextFunction::RunExtensionAction() {
   if (is_dnr_action_count_active &&
       !declarative_net_request::HasDNRFeedbackPermission(extension(),
                                                          tab_id_)) {
-    return RespondNow(OneArgument(base::Value(
-        std::move(declarative_net_request::kActionCountPlaceholderBadgeText))));
+    return RespondNow(WithArguments(
+        std::move(declarative_net_request::kActionCountPlaceholderBadgeText)));
   }
 
-  return RespondNow(OneArgument(
-      base::Value(extension_action_->GetDisplayBadgeText(tab_id_))));
+  return RespondNow(
+      WithArguments(extension_action_->GetDisplayBadgeText(tab_id_)));
 }
 
 ExtensionFunction::ResponseAction
 ExtensionActionGetBadgeBackgroundColorFunction::RunExtensionAction() {
-  std::unique_ptr<base::ListValue> list(new base::ListValue());
+  base::Value::List list;
   SkColor color = extension_action_->GetBadgeBackgroundColor(tab_id_);
-  list->AppendInteger(static_cast<int>(SkColorGetR(color)));
-  list->AppendInteger(static_cast<int>(SkColorGetG(color)));
-  list->AppendInteger(static_cast<int>(SkColorGetB(color)));
-  list->AppendInteger(static_cast<int>(SkColorGetA(color)));
-  return RespondNow(
-      OneArgument(base::Value::FromUniquePtrValue(std::move(list))));
+  list.Append(static_cast<int>(SkColorGetR(color)));
+  list.Append(static_cast<int>(SkColorGetG(color)));
+  list.Append(static_cast<int>(SkColorGetB(color)));
+  list.Append(static_cast<int>(SkColorGetA(color)));
+  return RespondNow(WithArguments(std::move(list)));
+}
+
+ExtensionFunction::ResponseAction
+ActionGetBadgeTextColorFunction::RunExtensionAction() {
+  base::Value::List list;
+  SkColor color = extension_action_->GetBadgeTextColor(tab_id_);
+  list.Append(static_cast<int>(SkColorGetR(color)));
+  list.Append(static_cast<int>(SkColorGetG(color)));
+  list.Append(static_cast<int>(SkColorGetB(color)));
+  list.Append(static_cast<int>(SkColorGetA(color)));
+  return RespondNow(WithArguments(std::move(list)));
+}
+
+ActionGetUserSettingsFunction::ActionGetUserSettingsFunction() = default;
+ActionGetUserSettingsFunction::~ActionGetUserSettingsFunction() = default;
+
+ExtensionFunction::ResponseAction ActionGetUserSettingsFunction::Run() {
+  DCHECK(extension());
+  ExtensionActionManager* const action_manager =
+      ExtensionActionManager::Get(browser_context());
+  ExtensionAction* const action =
+      action_manager->GetExtensionAction(*extension());
+
+  // This API is only available to extensions with the "action" key in the
+  // manifest, so they should always have an action.
+  DCHECK(action);
+  DCHECK_EQ(ActionInfo::TYPE_ACTION, action->action_type());
+
+  const bool is_pinned =
+      ToolbarActionsModel::Get(Profile::FromBrowserContext(browser_context()))
+          ->IsActionPinned(extension_id());
+
+  // TODO(devlin): Today, no action APIs are compiled. Unfortunately, this
+  // means we miss out on the compiled types, which would be rather helpful
+  // here.
+  base::Value::Dict ui_settings;
+  ui_settings.Set("isOnToolbar", is_pinned);
+
+  return RespondNow(WithArguments(std::move(ui_settings)));
+}
+
+ActionOpenPopupFunction::ActionOpenPopupFunction() = default;
+ActionOpenPopupFunction::~ActionOpenPopupFunction() = default;
+
+ExtensionFunction::ResponseAction ActionOpenPopupFunction::Run() {
+  // Unfortunately, the action API types aren't compiled. However, the bindings
+  // should still valid the form of the arguments.
+  EXTENSION_FUNCTION_VALIDATE(args().size() == 1u);
+  EXTENSION_FUNCTION_VALIDATE(extension());
+  const base::Value& options = args()[0];
+
+  // TODO(https://crbug.com/1245093): Support specifying the tab ID? This is
+  // kind of racy (because really what the extension probably cares about is
+  // the document ID; tab ID persists across pages, whereas document ID would
+  // detect things like navigations).
+  int window_id = extension_misc::kCurrentWindowId;
+  if (options.is_dict()) {
+    const base::Value* window_value = options.GetDict().Find("windowId");
+    if (window_value) {
+      EXTENSION_FUNCTION_VALIDATE(window_value->is_int());
+      window_id = window_value->GetInt();
+    }
+  }
+
+  Browser* browser = nullptr;
+  Profile* profile = Profile::FromBrowserContext(browser_context());
+  std::string error;
+  if (window_id == extension_misc::kCurrentWindowId) {
+    browser =
+        FindLastActiveBrowserWindow(profile, include_incognito_information());
+    if (!browser)
+      error = kNoActiveWindowFound;
+  } else {
+    browser = ExtensionTabUtil::GetBrowserInProfileWithId(
+        profile, window_id, include_incognito_information(), &error);
+  }
+
+  if (!browser) {
+    DCHECK(!error.empty());
+    return RespondNow(Error(std::move(error)));
+  }
+
+  if (!HasPopupOnActiveTab(browser, browser_context(), *extension()))
+    return RespondNow(Error(kNoActivePopup));
+
+  if (!OpenPopupInBrowser(
+          *browser, *extension(), &error,
+          base::BindOnce(&ActionOpenPopupFunction::OnShowPopupComplete,
+                         this))) {
+    DCHECK(!error.empty());
+    return RespondNow(Error(std::move(error)));
+  }
+
+  // The function responds in OnShowPopupComplete(). Note that the function is
+  // kept alive by the ref-count owned by the ShowPopupCallback.
+  return RespondLater();
+}
+
+void ActionOpenPopupFunction::OnShowPopupComplete(ExtensionHost* popup_host) {
+  DCHECK(!did_respond());
+
+  if (popup_host) {
+    // TODO(https://crbug.com/1245093): Return the tab for which the extension
+    // popup was shown?
+    DCHECK(popup_host->document_element_available());
+    Respond(NoArguments());
+  } else {
+    // NOTE(devlin): We could have the callback pass more information here about
+    // why the popup didn't open (e.g., another active popup vs popup closing
+    // before display, as may happen if the window closes), but it's not clear
+    // whether that would be significantly helpful to developers and it may
+    // leak other information about the user's browser.
+    Respond(Error(kFailedToOpenPopupGenericError));
+  }
 }
 
 BrowserActionOpenPopupFunction::BrowserActionOpenPopupFunction() = default;
+BrowserActionOpenPopupFunction::~BrowserActionOpenPopupFunction() = default;
 
 ExtensionFunction::ResponseAction BrowserActionOpenPopupFunction::Run() {
   // We only allow the popup in the active window.
   Profile* profile = Profile::FromBrowserContext(browser_context());
-  Browser* browser = chrome::FindLastActiveWithProfile(profile);
-  // It's possible that the last active browser actually corresponds to the
-  // associated incognito profile, and this won't be returned by
-  // FindLastActiveWithProfile. If the browser we found isn't active and the
-  // extension can operate incognito, then check the last active incognito, too.
-  if ((!browser || !browser->window()->IsActive()) &&
-      util::IsIncognitoEnabled(extension()->id(), profile) &&
-      profile->HasPrimaryOTRProfile()) {
-    browser =
-        chrome::FindLastActiveWithProfile(profile->GetPrimaryOTRProfile());
-  }
+  Browser* browser =
+      FindLastActiveBrowserWindow(profile, include_incognito_information());
 
-  // If there's no active browser, or the Toolbar isn't visible, abort.
-  // Otherwise, try to open a popup in the active browser.
-  // TODO(justinlin): Remove toolbar check when http://crbug.com/308645 is
-  // fixed.
-  if (!browser || !browser->window()->IsActive() ||
-      !browser->window()->IsToolbarVisible() ||
-      !ExtensionActionAPI::Get(profile)->ShowExtensionActionPopupForAPICall(
-          extension_.get(), browser)) {
-    return RespondNow(Error(kOpenPopupError));
+  if (!browser)
+    return RespondNow(Error(kNoActiveWindowFound));
+
+  if (!HasPopupOnActiveTab(browser, browser_context(), *extension()))
+    return RespondNow(Error(kNoActivePopup));
+
+  std::string error;
+  if (!OpenPopupInBrowser(*browser, *extension(), &error,
+                          ShowPopupCallback())) {
+    DCHECK(!error.empty());
+    return RespondNow(Error(std::move(error)));
   }
 
   // Even if this is for an incognito window, we want to use the normal profile.
   // If the extension is spanning, then extension hosts are created with the
   // original profile, and if it's split, then we know the api call came from
   // the right profile.
-  registrar_.Add(this, NOTIFICATION_EXTENSION_HOST_DID_STOP_FIRST_LOAD,
-                 content::Source<Profile>(profile));
+  host_registry_observation_.Observe(ExtensionHostRegistry::Get(profile));
 
   // Set a timeout for waiting for the notification that the popup is loaded.
   // Waiting is required so that the popup view can be retrieved by the custom
   // bindings for the response callback. It's also needed to keep this function
   // instance around until a notification is observed.
-  base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+  base::SingleThreadTaskRunner::GetCurrentDefault()->PostDelayedTask(
       FROM_HERE,
       base::BindOnce(&BrowserActionOpenPopupFunction::OpenPopupTimedOut, this),
-      base::TimeDelta::FromSeconds(10));
+      base::Seconds(10));
   return RespondLater();
 }
 
+void BrowserActionOpenPopupFunction::OnBrowserContextShutdown() {
+  // No point in responding at this point (the context is gone). However, we
+  // need to explicitly remove the ExtensionHostRegistry observation, since the
+  // ExtensionHostRegistry's lifetime is tied to the BrowserContext. Otherwise,
+  // this would cause a UAF when the observation is destructed as part of this
+  // instance's destruction.
+  host_registry_observation_.Reset();
+}
+
 void BrowserActionOpenPopupFunction::OpenPopupTimedOut() {
   if (did_respond())
     return;
@@ -600,21 +768,18 @@ void BrowserActionOpenPopupFunction::OpenPopupTimedOut() {
   Respond(Error(kOpenPopupError));
 }
 
-void BrowserActionOpenPopupFunction::Observe(
-    int type,
-    const content::NotificationSource& source,
-    const content::NotificationDetails& details) {
-  DCHECK_EQ(NOTIFICATION_EXTENSION_HOST_DID_STOP_FIRST_LOAD, type);
+void BrowserActionOpenPopupFunction::OnExtensionHostCompletedFirstLoad(
+    content::BrowserContext* browser_context,
+    ExtensionHost* host) {
   if (did_respond())
     return;
 
-  ExtensionHost* host = content::Details<ExtensionHost>(details).ptr();
-  if (host->extension_host_type() != VIEW_TYPE_EXTENSION_POPUP ||
+  if (host->extension_host_type() != mojom::ViewType::kExtensionPopup ||
       host->extension()->id() != extension_->id())
     return;
 
   Respond(NoArguments());
-  registrar_.RemoveAll();
+  host_registry_observation_.Reset();
 }
 
 }  // namespace extensions
