@@ -1,20 +1,21 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/api/language_settings_private/language_settings_private_api.h"
 
-#include <algorithm>
 #include <map>
 #include <memory>
+#include <set>
 #include <string>
-#include <unordered_set>
 #include <utility>
 #include <vector>
 
 #include "base/containers/contains.h"
 #include "base/containers/flat_set.h"
-#include "base/strings/string16.h"
+#include "base/feature_list.h"
+#include "base/ranges/algorithm.h"
+#include "base/strings/string_piece.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
@@ -49,57 +50,42 @@
 #if BUILDFLAG(IS_CHROMEOS_ASH)
 #include "ash/constants/ash_features.h"
 #include "chrome/grit/generated_resources.h"
-#include "ui/base/ime/chromeos/component_extension_ime_manager.h"
-#include "ui/base/ime/chromeos/extension_ime_util.h"
-#include "ui/base/ime/chromeos/input_method_manager.h"
-#include "ui/base/ime/chromeos/input_method_util.h"
+#include "ui/base/ime/ash/component_extension_ime_manager.h"
+#include "ui/base/ime/ash/extension_ime_util.h"
+#include "ui/base/ime/ash/input_method_descriptor.h"
+#include "ui/base/ime/ash/input_method_manager.h"
+#include "ui/base/ime/ash/input_method_util.h"
 #endif
 
 namespace extensions {
 
 namespace language_settings_private = api::language_settings_private;
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-using chromeos::input_method::InputMethodDescriptor;
-using chromeos::input_method::InputMethodDescriptors;
-using chromeos::input_method::InputMethodManager;
-using chromeos::input_method::InputMethodUtil;
-
 namespace {
 
-// Returns the set of IDs of all enabled IMEs.
-std::unordered_set<std::string> GetEnabledIMEs(
-    scoped_refptr<InputMethodManager::State> ime_state) {
-  const std::vector<std::string>& ime_ids(ime_state->GetActiveInputMethodIds());
-  return std::unordered_set<std::string>(ime_ids.begin(), ime_ids.end());
-}
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+using ::ash::input_method::InputMethodDescriptor;
+using ::ash::input_method::InputMethodDescriptors;
+using ::ash::input_method::InputMethodManager;
+using ::ash::input_method::InputMethodUtil;
 
-// Returns the set of IDs of all allowed IMEs.
-std::unordered_set<std::string> GetAllowedIMEs(
-    scoped_refptr<InputMethodManager::State> ime_state) {
-  const std::vector<std::string>& ime_ids(ime_state->GetAllowedInputMethods());
-  return std::unordered_set<std::string>(ime_ids.begin(), ime_ids.end());
-}
+// Number of IMEs that are needed to automatically enable the IME menu option.
+const size_t kNumImesToAutoEnableImeMenu = 2;
 
 // Returns the set of IDs of enabled IMEs for the given pref.
-std::unordered_set<std::string> GetIMEsFromPref(PrefService* prefs,
-                                                const char* pref_name) {
-  std::vector<std::string> enabled_imes =
-      base::SplitString(prefs->GetString(pref_name), ",", base::TRIM_WHITESPACE,
-                        base::SPLIT_WANT_NONEMPTY);
-  return std::unordered_set<std::string>(enabled_imes.begin(),
-                                         enabled_imes.end());
+base::flat_set<std::string> GetIMEsFromPref(PrefService* prefs,
+                                            const char* pref_name) {
+  return base::SplitString(prefs->GetString(pref_name), ",",
+                           base::TRIM_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
 }
 
 // Returns the set of allowed UI locales.
-std::unordered_set<std::string> GetAllowedLanguages(PrefService* prefs) {
-  std::unordered_set<std::string> allowed_languages;
-  for (const base::Value& locale_value :
-       prefs->GetList(prefs::kAllowedLanguages)->GetList()) {
-    allowed_languages.insert(locale_value.GetString());
-  }
-
-  return allowed_languages;
+base::flat_set<std::string> GetAllowedLanguages(PrefService* prefs) {
+  const auto& allowed_languages_values =
+      prefs->GetList(prefs::kAllowedLanguages);
+  return base::MakeFlatSet<std::string>(
+      allowed_languages_values, {},
+      [](const auto& locale_value) { return locale_value.GetString(); });
 }
 
 // Sorts the input methods by the order of their associated languages. For
@@ -110,36 +96,33 @@ std::unordered_set<std::string> GetAllowedLanguages(PrefService* prefs) {
 std::vector<std::string> GetSortedComponentIMEs(
     InputMethodManager* manager,
     scoped_refptr<InputMethodManager::State> ime_state,
-    const std::unordered_set<std::string>& component_ime_set,
+    const base::flat_set<std::string>& component_ime_set,
     PrefService* prefs) {
   std::vector<std::string> enabled_languages =
       base::SplitString(prefs->GetString(language::prefs::kPreferredLanguages),
                         ",", base::TRIM_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
 
   // Duplicate set for membership testing.
-  std::unordered_set<std::string> available_component_imes(component_ime_set);
+  std::set<std::string> available_component_imes(component_ime_set.begin(),
+                                                 component_ime_set.end());
   std::vector<std::string> component_ime_list;
 
   for (const auto& language_code : enabled_languages) {
     // Get all input methods for this language.
     std::vector<std::string> input_method_ids;
     manager->GetInputMethodUtil()->GetInputMethodIdsFromLanguageCode(
-        language_code, chromeos::input_method::kAllInputMethods,
-        &input_method_ids);
+        language_code, ash::input_method::kAllInputMethods, &input_method_ids);
     // Append the enabled ones to the new list. Also remove them from the set
     // so they aren't duplicated for other languages.
     for (const auto& input_method_id : input_method_ids) {
-      if (available_component_imes.count(input_method_id)) {
+      if (base::Contains(available_component_imes, input_method_id)) {
         component_ime_list.push_back(input_method_id);
         available_component_imes.erase(input_method_id);
       }
     }
   }
-  if (base::FeatureList::IsEnabled(
-          chromeos::features::kLanguageSettingsUpdate)) {
-    for (const auto& input_method_id : available_component_imes) {
-      component_ime_list.push_back(input_method_id);
-    }
+  for (const auto& input_method_id : available_component_imes) {
+    component_ime_list.push_back(input_method_id);
   }
 
   return component_ime_list;
@@ -148,46 +131,51 @@ std::vector<std::string> GetSortedComponentIMEs(
 // Sorts the third-party IMEs by the order of their associated languages.
 std::vector<std::string> GetSortedThirdPartyIMEs(
     scoped_refptr<InputMethodManager::State> ime_state,
-    const std::unordered_set<std::string>& third_party_ime_set,
+    const base::flat_set<std::string>& third_party_ime_set,
     PrefService* prefs) {
   std::vector<std::string> ime_list;
   std::string preferred_languages =
       prefs->GetString(language::prefs::kPreferredLanguages);
-  std::vector<std::string> enabled_languages =
-      base::SplitString(preferred_languages, ",", base::TRIM_WHITESPACE,
-                        base::SPLIT_WANT_NONEMPTY);
+  std::vector<base::StringPiece> enabled_languages =
+      base::SplitStringPiece(preferred_languages, ",", base::TRIM_WHITESPACE,
+                             base::SPLIT_WANT_NONEMPTY);
 
   // Add the fake language for ARC IMEs at the very last of the list. Unlike
   // Chrome OS IMEs, these ARC ones are not associated with any (real) language.
-  enabled_languages.push_back(chromeos::extension_ime_util::kArcImeLanguage);
+  enabled_languages.push_back(ash::extension_ime_util::kArcImeLanguage);
 
   InputMethodDescriptors descriptors;
   ime_state->GetInputMethodExtensions(&descriptors);
 
   // Filter out the IMEs not in |third_party_ime_set|.
-  auto it = descriptors.begin();
-  while (it != descriptors.end()) {
-    if (third_party_ime_set.count(it->id()) == 0)
-      it = descriptors.erase(it);
-    else
-      it++;
-  }
+      base::EraseIf(descriptors,
+          [&third_party_ime_set](const InputMethodDescriptor& descriptor) {
+            return !third_party_ime_set.contains(descriptor.id());
+          });
+
+  // A set of the elements of |ime_list|.
+  std::set<std::string> ime_set;
 
   // For each language, add any candidate IMEs that support it.
   for (const auto& language : enabled_languages) {
-    auto it = descriptors.begin();
-    while (it != descriptors.end() && descriptors.size()) {
-      if (third_party_ime_set.count(it->id()) &&
-          base::Contains(it->language_codes(), language)) {
-        ime_list.push_back(it->id());
-        // Remove the added descriptor from the candidate list.
-        it = descriptors.erase(it);
-      } else {
-        it++;
+    for (const InputMethodDescriptor& descriptor : descriptors) {
+      const std::string& id = descriptor.id();
+      if (!base::Contains(ime_set, id) &&
+          base::Contains(descriptor.language_codes(), language)) {
+        ime_list.push_back(id);
+        ime_set.insert(id);
       }
     }
   }
 
+  // Add the rest of the third party IMEs
+  for (const InputMethodDescriptor& descriptor : descriptors) {
+    const std::string& id = descriptor.id();
+    if (!base::Contains(ime_set, id)) {
+      ime_list.push_back(id);
+    }
+  }
+
   return ime_list;
 }
 
@@ -202,13 +190,19 @@ std::vector<std::string> GetInputMethodTags(
   return tags;
 }
 
-}  // namespace
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
+std::unique_ptr<translate::TranslatePrefs>
+CreateTranslatePrefsForBrowserContext(
+    content::BrowserContext* browser_context) {
+  return ChromeTranslateClient::CreateTranslatePrefs(
+      Profile::FromBrowserContext(browser_context)->GetPrefs());
+}
+
+}  // namespace
+
 LanguageSettingsPrivateGetLanguageListFunction::
-    LanguageSettingsPrivateGetLanguageListFunction()
-    : chrome_details_(this),
-      language_list_(std::make_unique<base::ListValue>()) {}
+    LanguageSettingsPrivateGetLanguageListFunction() = default;
 
 LanguageSettingsPrivateGetLanguageListFunction::
     ~LanguageSettingsPrivateGetLanguageListFunction() = default;
@@ -218,8 +212,7 @@ LanguageSettingsPrivateGetLanguageListFunction::Run() {
   // Collect the language codes from the supported accept-languages.
   const std::string app_locale = g_browser_process->GetApplicationLocale();
   const std::unique_ptr<translate::TranslatePrefs> translate_prefs =
-      ChromeTranslateClient::CreateTranslatePrefs(
-          chrome_details_.GetProfile()->GetPrefs());
+      CreateTranslatePrefsForBrowserContext(browser_context());
 
   std::vector<translate::TranslateLanguageInfo> languages;
   translate::TranslatePrefs::GetLanguageInfoList(
@@ -232,10 +225,10 @@ LanguageSettingsPrivateGetLanguageListFunction::Run() {
       std::move(spellcheck_languages));
 
   // Build the language list.
-  language_list_->Clear();
+  language_list_.clear();
 #if BUILDFLAG(IS_CHROMEOS_ASH)
-  const std::unordered_set<std::string> allowed_ui_locales(
-      GetAllowedLanguages(chrome_details_.GetProfile()->GetPrefs()));
+  const base::flat_set<std::string> allowed_ui_locales(GetAllowedLanguages(
+      Profile::FromBrowserContext(browser_context())->GetPrefs()));
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
   for (const auto& entry : languages) {
     language_settings_private::Language language;
@@ -246,24 +239,23 @@ LanguageSettingsPrivateGetLanguageListFunction::Run() {
 
     // Set optional fields only if they differ from the default.
     if (base::Contains(spellcheck_language_set, entry.code)) {
-      language.supports_spellcheck.reset(new bool(true));
+      language.supports_spellcheck = true;
     }
     if (entry.supports_translate) {
-      language.supports_translate.reset(new bool(true));
+      language.supports_translate = true;
     }
 
-    std::string temp_locale = entry.code;
-    if (language::ConvertToActualUILocale(&temp_locale)) {
-      language.supports_ui.reset(new bool(true));
+    if (l10n_util::IsUserFacingUILocale(entry.code)) {
+      language.supports_ui = true;
     }
 #if BUILDFLAG(IS_CHROMEOS_ASH)
     if (!allowed_ui_locales.empty() &&
-        allowed_ui_locales.count(language.code) == 0) {
-      language.is_prohibited_language.reset(new bool(true));
+        !base::Contains(allowed_ui_locales, language.code)) {
+      language.is_prohibited_language = true;
     }
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
-    language_list_->Append(language.ToValue());
+    language_list_.Append(language.ToValue());
   }
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
@@ -272,14 +264,14 @@ LanguageSettingsPrivateGetLanguageListFunction::Run() {
   // drop-down menu doesn't list the fake language.
   {
     language_settings_private::Language language;
-    language.code = chromeos::extension_ime_util::kArcImeLanguage;
+    language.code = ash::extension_ime_util::kArcImeLanguage;
     language.display_name =
         l10n_util::GetStringUTF8(IDS_SETTINGS_LANGUAGES_KEYBOARD_APPS);
-    language_list_->Append(language.ToValue());
+    language_list_.Append(base::Value(language.ToValue()));
   }
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   if (spellcheck::UseBrowserSpellChecker()) {
     if (!base::FeatureList::IsEnabled(
             spellcheck::kWinDelaySpellcheckServiceInit)) {
@@ -297,18 +289,16 @@ LanguageSettingsPrivateGetLanguageListFunction::Run() {
       return RespondLater();
     }
   }
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
-  return RespondNow(
-      OneArgument(base::Value::FromUniquePtrValue(std::move(language_list_))));
+  return RespondNow(WithArguments(std::move(language_list_)));
 }
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 void LanguageSettingsPrivateGetLanguageListFunction::
     OnDictionariesInitialized() {
   UpdateSupportedPlatformDictionaries();
-  Respond(
-      OneArgument(base::Value::FromUniquePtrValue(std::move(language_list_))));
+  Respond(WithArguments(std::move(language_list_)));
   // Matches the AddRef in Run().
   Release();
 }
@@ -317,17 +307,18 @@ void LanguageSettingsPrivateGetLanguageListFunction::
     UpdateSupportedPlatformDictionaries() {
   SpellcheckService* service =
       SpellcheckServiceFactory::GetForContext(browser_context());
-  for (auto& language_val : language_list_->GetList()) {
-    if (service->UsesWindowsDictionary(*language_val.FindStringKey("code"))) {
-      language_val.SetBoolKey("supportsSpellcheck", new bool(true));
+  for (auto& language_val : language_list_) {
+    base::Value::Dict& language_val_dict = language_val.GetDict();
+    const std::string* str = language_val_dict.FindString("code");
+    if (str && service->UsesWindowsDictionary(*str)) {
+      language_val_dict.Set("supportsSpellcheck", true);
     }
   }
 }
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
 LanguageSettingsPrivateEnableLanguageFunction::
-    LanguageSettingsPrivateEnableLanguageFunction()
-    : chrome_details_(this) {}
+    LanguageSettingsPrivateEnableLanguageFunction() = default;
 
 LanguageSettingsPrivateEnableLanguageFunction::
     ~LanguageSettingsPrivateEnableLanguageFunction() = default;
@@ -335,32 +326,25 @@ LanguageSettingsPrivateEnableLanguageFunction::
 ExtensionFunction::ResponseAction
 LanguageSettingsPrivateEnableLanguageFunction::Run() {
   const auto parameters =
-      language_settings_private::EnableLanguage::Params::Create(*args_);
-  EXTENSION_FUNCTION_VALIDATE(parameters.get());
+      language_settings_private::EnableLanguage::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(parameters);
   const std::string& language_code = parameters->language_code;
 
   std::unique_ptr<translate::TranslatePrefs> translate_prefs =
-      ChromeTranslateClient::CreateTranslatePrefs(
-          chrome_details_.GetProfile()->GetPrefs());
+      CreateTranslatePrefsForBrowserContext(browser_context());
 
   std::vector<std::string> languages;
   translate_prefs->GetLanguageList(&languages);
   std::string chrome_language = language_code;
   language::ToChromeLanguageSynonym(&chrome_language);
 
-  if (base::Contains(languages, chrome_language)) {
-    LOG(ERROR) << "Language " << chrome_language << " already enabled";
-    return RespondNow(NoArguments());
-  }
-
   translate_prefs->AddToLanguageList(language_code, /*force_blocked=*/false);
 
   return RespondNow(NoArguments());
 }
 
 LanguageSettingsPrivateDisableLanguageFunction::
-    LanguageSettingsPrivateDisableLanguageFunction()
-    : chrome_details_(this) {}
+    LanguageSettingsPrivateDisableLanguageFunction() = default;
 
 LanguageSettingsPrivateDisableLanguageFunction::
     ~LanguageSettingsPrivateDisableLanguageFunction() = default;
@@ -368,35 +352,25 @@ LanguageSettingsPrivateDisableLanguageFunction::
 ExtensionFunction::ResponseAction
 LanguageSettingsPrivateDisableLanguageFunction::Run() {
   const auto parameters =
-      language_settings_private::DisableLanguage::Params::Create(*args_);
-  EXTENSION_FUNCTION_VALIDATE(parameters.get());
+      language_settings_private::DisableLanguage::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(parameters);
   const std::string& language_code = parameters->language_code;
 
   std::unique_ptr<translate::TranslatePrefs> translate_prefs =
-      ChromeTranslateClient::CreateTranslatePrefs(
-          chrome_details_.GetProfile()->GetPrefs());
+      CreateTranslatePrefsForBrowserContext(browser_context());
 
   std::vector<std::string> languages;
   translate_prefs->GetLanguageList(&languages);
   std::string chrome_language = language_code;
   language::ToChromeLanguageSynonym(&chrome_language);
 
-  if (!base::Contains(languages, chrome_language)) {
-    LOG(ERROR) << "Language " << chrome_language << " not enabled";
-    return RespondNow(NoArguments());
-  }
-
   translate_prefs->RemoveFromLanguageList(language_code);
-  if (language_code == translate_prefs->GetRecentTargetLanguage()) {
-    translate_prefs->ResetRecentTargetLanguage();
-  }
 
   return RespondNow(NoArguments());
 }
 
 LanguageSettingsPrivateSetEnableTranslationForLanguageFunction::
-    LanguageSettingsPrivateSetEnableTranslationForLanguageFunction()
-    : chrome_details_(this) {}
+    LanguageSettingsPrivateSetEnableTranslationForLanguageFunction() = default;
 
 LanguageSettingsPrivateSetEnableTranslationForLanguageFunction::
     ~LanguageSettingsPrivateSetEnableTranslationForLanguageFunction() = default;
@@ -404,15 +378,14 @@ LanguageSettingsPrivateSetEnableTranslationForLanguageFunction::
 ExtensionFunction::ResponseAction
 LanguageSettingsPrivateSetEnableTranslationForLanguageFunction::Run() {
   const auto parameters = language_settings_private::
-      SetEnableTranslationForLanguage::Params::Create(*args_);
-  EXTENSION_FUNCTION_VALIDATE(parameters.get());
+      SetEnableTranslationForLanguage::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(parameters);
   const std::string& language_code = parameters->language_code;
   // True if translation enabled, false if disabled.
   const bool enable = parameters->enable;
 
   std::unique_ptr<translate::TranslatePrefs> translate_prefs =
-      ChromeTranslateClient::CreateTranslatePrefs(
-          chrome_details_.GetProfile()->GetPrefs());
+      CreateTranslatePrefsForBrowserContext(browser_context());
 
   if (enable) {
     translate_prefs->UnblockLanguage(language_code);
@@ -423,9 +396,72 @@ LanguageSettingsPrivateSetEnableTranslationForLanguageFunction::Run() {
   return RespondNow(NoArguments());
 }
 
+LanguageSettingsPrivateGetAlwaysTranslateLanguagesFunction::
+    LanguageSettingsPrivateGetAlwaysTranslateLanguagesFunction() = default;
+
+LanguageSettingsPrivateGetAlwaysTranslateLanguagesFunction::
+    ~LanguageSettingsPrivateGetAlwaysTranslateLanguagesFunction() = default;
+
+ExtensionFunction::ResponseAction
+LanguageSettingsPrivateGetAlwaysTranslateLanguagesFunction::Run() {
+  const std::unique_ptr<translate::TranslatePrefs> translate_prefs =
+      CreateTranslatePrefsForBrowserContext(browser_context());
+
+  std::vector<std::string> languages =
+      translate_prefs->GetAlwaysTranslateLanguages();
+
+  base::Value::List always_translate_languages;
+  for (const auto& entry : languages) {
+    always_translate_languages.Append(entry);
+  }
+
+  return RespondNow(WithArguments(std::move(always_translate_languages)));
+}
+
+LanguageSettingsPrivateSetLanguageAlwaysTranslateStateFunction::
+    LanguageSettingsPrivateSetLanguageAlwaysTranslateStateFunction() = default;
+
+LanguageSettingsPrivateSetLanguageAlwaysTranslateStateFunction::
+    ~LanguageSettingsPrivateSetLanguageAlwaysTranslateStateFunction() = default;
+
+ExtensionFunction::ResponseAction
+LanguageSettingsPrivateSetLanguageAlwaysTranslateStateFunction::Run() {
+  const auto params = language_settings_private::
+      SetLanguageAlwaysTranslateState::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
+
+  const std::unique_ptr<translate::TranslatePrefs> translate_prefs =
+      CreateTranslatePrefsForBrowserContext(browser_context());
+
+  translate_prefs->SetLanguageAlwaysTranslateState(params->language_code,
+                                                   params->always_translate);
+
+  return RespondNow(NoArguments());
+}
+
+LanguageSettingsPrivateGetNeverTranslateLanguagesFunction::
+    LanguageSettingsPrivateGetNeverTranslateLanguagesFunction() = default;
+
+LanguageSettingsPrivateGetNeverTranslateLanguagesFunction::
+    ~LanguageSettingsPrivateGetNeverTranslateLanguagesFunction() = default;
+
+ExtensionFunction::ResponseAction
+LanguageSettingsPrivateGetNeverTranslateLanguagesFunction::Run() {
+  const std::unique_ptr<translate::TranslatePrefs> translate_prefs =
+      CreateTranslatePrefsForBrowserContext(browser_context());
+
+  std::vector<std::string> languages =
+      translate_prefs->GetNeverTranslateLanguages();
+
+  base::Value::List never_translate_languages;
+  for (auto& entry : languages) {
+    never_translate_languages.Append(std::move(entry));
+  }
+  return RespondNow(WithArguments(std::move(never_translate_languages)));
+}
+
 LanguageSettingsPrivateMoveLanguageFunction::
-    LanguageSettingsPrivateMoveLanguageFunction()
-    : chrome_details_(this) {}
+    LanguageSettingsPrivateMoveLanguageFunction() = default;
 
 LanguageSettingsPrivateMoveLanguageFunction::
     ~LanguageSettingsPrivateMoveLanguageFunction() = default;
@@ -433,8 +469,8 @@ LanguageSettingsPrivateMoveLanguageFunction::
 ExtensionFunction::ResponseAction
 LanguageSettingsPrivateMoveLanguageFunction::Run() {
   const auto parameters =
-      language_settings_private::MoveLanguage::Params::Create(*args_);
-  EXTENSION_FUNCTION_VALIDATE(parameters.get());
+      language_settings_private::MoveLanguage::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(parameters);
 
   const std::string app_locale = g_browser_process->GetApplicationLocale();
   std::vector<std::string> supported_language_codes;
@@ -444,26 +480,25 @@ LanguageSettingsPrivateMoveLanguageFunction::Run() {
   const language_settings_private::MoveType move_type = parameters->move_type;
 
   std::unique_ptr<translate::TranslatePrefs> translate_prefs =
-      ChromeTranslateClient::CreateTranslatePrefs(
-          chrome_details_.GetProfile()->GetPrefs());
+      CreateTranslatePrefsForBrowserContext(browser_context());
 
   translate::TranslatePrefs::RearrangeSpecifier where =
       translate::TranslatePrefs::kNone;
   switch (move_type) {
-    case language_settings_private::MOVE_TYPE_TOP:
+    case language_settings_private::MoveType::kTop:
       where = translate::TranslatePrefs::kTop;
       break;
 
-    case language_settings_private::MOVE_TYPE_UP:
+    case language_settings_private::MoveType::kUp:
       where = translate::TranslatePrefs::kUp;
       break;
 
-    case language_settings_private::MOVE_TYPE_DOWN:
+    case language_settings_private::MoveType::kDown:
       where = translate::TranslatePrefs::kDown;
       break;
 
-    case language_settings_private::MOVE_TYPE_NONE:
-    case language_settings_private::MOVE_TYPE_LAST:
+    case language_settings_private::MoveType::kNone:
+    case language_settings_private::MoveType::kMaxValue:
       NOTREACHED();
   }
 
@@ -505,8 +540,7 @@ LanguageSettingsPrivateGetSpellcheckWordsFunction::Run() {
   SpellcheckCustomDictionary* dictionary = service->GetCustomDictionary();
 
   if (dictionary->IsLoaded())
-    return RespondNow(
-        OneArgument(base::Value::FromUniquePtrValue(GetSpellcheckWords())));
+    return RespondNow(WithArguments(GetSpellcheckWords()));
 
   dictionary->AddObserver(this);
   AddRef();  // Balanced in OnCustomDictionaryLoaded().
@@ -518,7 +552,7 @@ void LanguageSettingsPrivateGetSpellcheckWordsFunction::
   SpellcheckService* service =
       SpellcheckServiceFactory::GetForContext(browser_context());
   service->GetCustomDictionary()->RemoveObserver(this);
-  Respond(OneArgument(base::Value::FromUniquePtrValue(GetSpellcheckWords())));
+  Respond(WithArguments(GetSpellcheckWords()));
   Release();
 }
 
@@ -530,7 +564,7 @@ void LanguageSettingsPrivateGetSpellcheckWordsFunction::
                   "OnCustomDictionaryLoaded()";
 }
 
-std::unique_ptr<base::ListValue>
+base::Value::List
 LanguageSettingsPrivateGetSpellcheckWordsFunction::GetSpellcheckWords() const {
   SpellcheckService* service =
       SpellcheckServiceFactory::GetForContext(browser_context());
@@ -538,10 +572,11 @@ LanguageSettingsPrivateGetSpellcheckWordsFunction::GetSpellcheckWords() const {
   DCHECK(dictionary->IsLoaded());
 
   // TODO(michaelpg): Sort using app locale.
-  std::unique_ptr<base::ListValue> word_list(new base::ListValue());
+  base::Value::List word_list;
   const std::set<std::string>& words = dictionary->GetWords();
+  word_list.reserve(words.size());
   for (const std::string& word : words)
-    word_list->AppendString(word);
+    word_list.Append(word);
   return word_list;
 }
 
@@ -554,8 +589,8 @@ LanguageSettingsPrivateAddSpellcheckWordFunction::
 ExtensionFunction::ResponseAction
 LanguageSettingsPrivateAddSpellcheckWordFunction::Run() {
   const auto params =
-      language_settings_private::AddSpellcheckWord::Params::Create(*args_);
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+      language_settings_private::AddSpellcheckWord::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   SpellcheckService* service =
       SpellcheckServiceFactory::GetForContext(browser_context());
@@ -568,7 +603,7 @@ LanguageSettingsPrivateAddSpellcheckWordFunction::Run() {
   }
 #endif
 
-  return RespondNow(OneArgument(base::Value(success)));
+  return RespondNow(WithArguments(success));
 }
 
 LanguageSettingsPrivateRemoveSpellcheckWordFunction::
@@ -580,8 +615,8 @@ LanguageSettingsPrivateRemoveSpellcheckWordFunction::
 ExtensionFunction::ResponseAction
 LanguageSettingsPrivateRemoveSpellcheckWordFunction::Run() {
   const auto params =
-      language_settings_private::RemoveSpellcheckWord::Params::Create(*args_);
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+      language_settings_private::RemoveSpellcheckWord::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   SpellcheckService* service =
       SpellcheckServiceFactory::GetForContext(browser_context());
@@ -594,24 +629,50 @@ LanguageSettingsPrivateRemoveSpellcheckWordFunction::Run() {
   }
 #endif
 
-  return RespondNow(OneArgument(base::Value(success)));
+  return RespondNow(WithArguments(success));
 }
 
 LanguageSettingsPrivateGetTranslateTargetLanguageFunction::
-    LanguageSettingsPrivateGetTranslateTargetLanguageFunction()
-    : chrome_details_(this) {}
+    LanguageSettingsPrivateGetTranslateTargetLanguageFunction() = default;
 
 LanguageSettingsPrivateGetTranslateTargetLanguageFunction::
     ~LanguageSettingsPrivateGetTranslateTargetLanguageFunction() = default;
 
 ExtensionFunction::ResponseAction
 LanguageSettingsPrivateGetTranslateTargetLanguageFunction::Run() {
-  Profile* profile = chrome_details_.GetProfile();
   language::LanguageModel* language_model =
-      LanguageModelManagerFactory::GetForBrowserContext(profile)
+      LanguageModelManagerFactory::GetForBrowserContext(browser_context())
           ->GetPrimaryModel();
-  return RespondNow(OneArgument(base::Value(TranslateService::GetTargetLanguage(
-      profile->GetPrefs(), language_model))));
+  return RespondNow(WithArguments(TranslateService::GetTargetLanguage(
+      Profile::FromBrowserContext(browser_context())->GetPrefs(),
+      language_model)));
+}
+
+LanguageSettingsPrivateSetTranslateTargetLanguageFunction::
+    LanguageSettingsPrivateSetTranslateTargetLanguageFunction() = default;
+
+LanguageSettingsPrivateSetTranslateTargetLanguageFunction::
+    ~LanguageSettingsPrivateSetTranslateTargetLanguageFunction() = default;
+
+ExtensionFunction::ResponseAction
+LanguageSettingsPrivateSetTranslateTargetLanguageFunction::Run() {
+  const auto parameters =
+      language_settings_private::SetTranslateTargetLanguage::Params::Create(
+          args());
+  EXTENSION_FUNCTION_VALIDATE(parameters);
+  const std::string& language_code = parameters->language_code;
+
+  std::unique_ptr<translate::TranslatePrefs> translate_prefs =
+      CreateTranslatePrefsForBrowserContext(browser_context());
+
+  std::string chrome_language = language_code;
+
+  if (language_code == translate_prefs->GetRecentTargetLanguage()) {
+    return RespondNow(NoArguments());
+  }
+  translate_prefs->SetRecentTargetLanguage(language_code);
+
+  return RespondNow(NoArguments());
 }
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
@@ -627,22 +688,20 @@ void PopulateInputMethodListFromDescriptors(
   if (!ime_state.get())
     return;
 
-  const std::unordered_set<std::string> active_ids(GetEnabledIMEs(ime_state));
-  const std::unordered_set<std::string> allowed_ids(GetAllowedIMEs(ime_state));
+  const base::flat_set<std::string> enabled_ids(
+      ime_state->GetEnabledInputMethodIds());
+  const base::flat_set<std::string> allowed_ids(
+      ime_state->GetAllowedInputMethodIds());
 
-  // Collator used to sort display names in the given locale.
   UErrorCode error = U_ZERO_ERROR;
-  const std::string app_locale = g_browser_process->GetApplicationLocale();
   std::unique_ptr<icu::Collator> collator(
-      icu::Collator::createInstance(icu::Locale(app_locale.c_str()), error));
-  if (U_FAILURE(error)) {
-    collator.reset();
-  }
+      icu::Collator::createInstance(error));  // use current ICU locale
+  DCHECK(U_SUCCESS(error));
 
   // Map of sorted [display name -> input methods].
-  std::map<base::string16, language_settings_private::InputMethod,
-           l10n_util::StringComparator<base::string16>>
-      input_map(l10n_util::StringComparator<base::string16>(collator.get()));
+  std::map<std::u16string, language_settings_private::InputMethod,
+           l10n_util::StringComparator<std::u16string>>
+      input_map(l10n_util::StringComparator<std::u16string>(collator.get()));
 
   for (const auto& descriptor : descriptors) {
     language_settings_private::InputMethod input_method;
@@ -650,15 +709,12 @@ void PopulateInputMethodListFromDescriptors(
     input_method.display_name = util->GetLocalizedDisplayName(descriptor);
     input_method.language_codes = descriptor.language_codes();
     input_method.tags = GetInputMethodTags(&input_method);
-    if (active_ids.count(input_method.id) > 0)
-      input_method.enabled.reset(new bool(true));
+    if (base::Contains(enabled_ids, input_method.id))
+      input_method.enabled = true;
     if (descriptor.options_page_url().is_valid())
-      input_method.has_options_page.reset(new bool(true));
-    if (!allowed_ids.empty() &&
-        (util->IsKeyboardLayout(input_method.id) ||
-         chromeos::extension_ime_util::IsArcIME(input_method.id)) &&
-        allowed_ids.count(input_method.id) == 0) {
-      input_method.is_prohibited_by_policy.reset(new bool(true));
+      input_method.has_options_page = true;
+    if (!allowed_ids.empty() && !base::Contains(allowed_ids, input_method.id)) {
+      input_method.is_prohibited_by_policy = true;
     }
     input_map[base::UTF8ToUTF16(util->GetLocalizedDisplayName(descriptor))] =
         std::move(input_method);
@@ -685,7 +741,7 @@ LanguageSettingsPrivateGetInputMethodListsFunction::Run() {
   language_settings_private::InputMethodLists input_method_lists;
   InputMethodManager* manager = InputMethodManager::Get();
 
-  chromeos::ComponentExtensionIMEManager* component_extension_manager =
+  ash::ComponentExtensionIMEManager* component_extension_manager =
       manager->GetComponentExtensionIMEManager();
   PopulateInputMethodListFromDescriptors(
       component_extension_manager->GetAllIMEAsInputMethodDescriptor(),
@@ -700,14 +756,12 @@ LanguageSettingsPrivateGetInputMethodListsFunction::Run() {
         ext_ime_descriptors, &input_method_lists.third_party_extension_imes);
   }
 
-  return RespondNow(OneArgument(
-      base::Value::FromUniquePtrValue(input_method_lists.ToValue())));
+  return RespondNow(WithArguments(input_method_lists.ToValue()));
 #endif
 }
 
 LanguageSettingsPrivateAddInputMethodFunction::
-    LanguageSettingsPrivateAddInputMethodFunction()
-    : chrome_details_(this) {}
+    LanguageSettingsPrivateAddInputMethodFunction() = default;
 
 LanguageSettingsPrivateAddInputMethodFunction::
     ~LanguageSettingsPrivateAddInputMethodFunction() = default;
@@ -718,8 +772,8 @@ LanguageSettingsPrivateAddInputMethodFunction::Run() {
   EXTENSION_FUNCTION_VALIDATE(false);
 #else
   const auto params =
-      language_settings_private::AddInputMethod::Params::Create(*args_);
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+      language_settings_private::AddInputMethod::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   InputMethodManager* manager = InputMethodManager::Get();
   scoped_refptr<InputMethodManager::State> ime_state =
@@ -729,16 +783,16 @@ LanguageSettingsPrivateAddInputMethodFunction::Run() {
 
   std::string new_input_method_id = params->input_method_id;
   bool is_component_extension_ime =
-      chromeos::extension_ime_util::IsComponentExtensionIME(
-          new_input_method_id);
+      ash::extension_ime_util::IsComponentExtensionIME(new_input_method_id);
 
-  PrefService* prefs = chrome_details_.GetProfile()->GetPrefs();
+  PrefService* prefs =
+      Profile::FromBrowserContext(browser_context())->GetPrefs();
   const char* pref_name = is_component_extension_ime
                               ? prefs::kLanguagePreloadEngines
                               : prefs::kLanguageEnabledImes;
 
   // Get the input methods we are adding to.
-  std::unordered_set<std::string> input_method_set(
+  base::flat_set<std::string> input_method_set(
       GetIMEsFromPref(prefs, pref_name));
 
   // Add the new input method.
@@ -756,13 +810,30 @@ LanguageSettingsPrivateAddInputMethodFunction::Run() {
 
   std::string input_methods = base::JoinString(input_method_list, ",");
   prefs->SetString(pref_name, input_methods);
+
+  // We want to automatically enable "Show input options in shelf" when the user
+  // has multiple input methods.
+  // We don't want to repeatedly enable it every time the user adds an input
+  // method, as a user may want to intentionally turn it off - so we only enable
+  // it once the user reaches two input methods.
+  // As pref_name and input_method_set only refer to the preference related to
+  // the list of IMEs for which this newly-added IME is in, we need the other
+  // IME list to calculate the total number of IMEs.
+  const char* other_ime_list_pref_name = is_component_extension_ime
+                                             ? prefs::kLanguageEnabledImes
+                                             : prefs::kLanguagePreloadEngines;
+  base::flat_set<std::string> other_input_method_set(
+      GetIMEsFromPref(prefs, other_ime_list_pref_name));
+  if (input_method_set.size() + other_input_method_set.size() ==
+      kNumImesToAutoEnableImeMenu) {
+    prefs->SetBoolean(prefs::kLanguageImeMenuActivated, true);
+  }
 #endif
   return RespondNow(NoArguments());
 }
 
 LanguageSettingsPrivateRemoveInputMethodFunction::
-    LanguageSettingsPrivateRemoveInputMethodFunction()
-    : chrome_details_(this) {}
+    LanguageSettingsPrivateRemoveInputMethodFunction() = default;
 
 LanguageSettingsPrivateRemoveInputMethodFunction::
     ~LanguageSettingsPrivateRemoveInputMethodFunction() = default;
@@ -773,8 +844,8 @@ LanguageSettingsPrivateRemoveInputMethodFunction::Run() {
   EXTENSION_FUNCTION_VALIDATE(false);
 #else
   const auto params =
-      language_settings_private::RemoveInputMethod::Params::Create(*args_);
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+      language_settings_private::RemoveInputMethod::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   InputMethodManager* manager = InputMethodManager::Get();
   scoped_refptr<InputMethodManager::State> ime_state =
@@ -784,10 +855,11 @@ LanguageSettingsPrivateRemoveInputMethodFunction::Run() {
 
   std::string input_method_id = params->input_method_id;
   bool is_component_extension_ime =
-      chromeos::extension_ime_util::IsComponentExtensionIME(input_method_id);
+      ash::extension_ime_util::IsComponentExtensionIME(input_method_id);
 
   // Use the pref for the corresponding input method type.
-  PrefService* prefs = chrome_details_.GetProfile()->GetPrefs();
+  PrefService* prefs =
+      Profile::FromBrowserContext(browser_context())->GetPrefs();
   const char* pref_name = is_component_extension_ime
                               ? prefs::kLanguagePreloadEngines
                               : prefs::kLanguageEnabledImes;
@@ -797,8 +869,7 @@ LanguageSettingsPrivateRemoveInputMethodFunction::Run() {
       input_method_ids, ",", base::TRIM_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
 
   // Find and remove the matching input method id.
-  const auto& pos = std::find(input_method_list.begin(),
-                              input_method_list.end(), input_method_id);
+  const auto& pos = base::ranges::find(input_method_list, input_method_id);
   if (pos != input_method_list.end()) {
     input_method_list.erase(pos);
     prefs->SetString(pref_name, base::JoinString(input_method_list, ","));
@@ -817,8 +888,8 @@ ExtensionFunction::ResponseAction
 LanguageSettingsPrivateRetryDownloadDictionaryFunction::Run() {
   const auto parameters =
       language_settings_private::RetryDownloadDictionary::Params::Create(
-          *args_);
-  EXTENSION_FUNCTION_VALIDATE(parameters.get());
+          args());
+  EXTENSION_FUNCTION_VALIDATE(parameters);
 
   LanguageSettingsPrivateDelegate* delegate =
       LanguageSettingsPrivateDelegateFactory::GetForBrowserContext(
