@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,11 +7,10 @@
 #include <memory>
 #include <utility>
 
-#include "base/bind.h"
-#include "base/macros.h"
-#include "base/strings/strcat.h"
+#include "base/functional/bind.h"
+#include "base/memory/raw_ptr.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/task/post_task.h"
+#include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
 #include "chrome/browser/apps/app_service/app_launch_params.h"
 #include "chrome/browser/apps/app_service/app_service_proxy.h"
@@ -32,17 +31,24 @@
 #include "chrome/browser/ui/tab_helpers.h"
 #include "chrome/browser/ui/web_applications/web_app_dialog_utils.h"
 #include "chrome/browser/ui/webui/extensions/extension_icon_source.h"
-#include "chrome/browser/web_applications/components/app_registrar.h"
-#include "chrome/browser/web_applications/components/install_manager.h"
-#include "chrome/browser/web_applications/components/web_app_constants.h"
-#include "chrome/browser/web_applications/components/web_app_helpers.h"
-#include "chrome/browser/web_applications/components/web_app_install_utils.h"
-#include "chrome/browser/web_applications/components/web_app_provider_base.h"
-#include "chrome/browser/web_applications/components/web_app_utils.h"
-#include "chrome/browser/web_applications/components/web_application_info.h"
+#include "chrome/browser/web_applications/extension_status_utils.h"
+#include "chrome/browser/web_applications/mojom/user_display_mode.mojom.h"
+#include "chrome/browser/web_applications/web_app_command_scheduler.h"
+#include "chrome/browser/web_applications/web_app_helpers.h"
+#include "chrome/browser/web_applications/web_app_install_info.h"
+#include "chrome/browser/web_applications/web_app_install_manager.h"
+#include "chrome/browser/web_applications/web_app_install_params.h"
+#include "chrome/browser/web_applications/web_app_install_utils.h"
+#include "chrome/browser/web_applications/web_app_provider.h"
+#include "chrome/browser/web_applications/web_app_registrar.h"
+#include "chrome/browser/web_applications/web_app_utils.h"
+#include "chrome/common/chrome_features.h"
 #include "chrome/common/extensions/extension_metrics.h"
 #include "chrome/common/extensions/manifest_handlers/app_launch_info.h"
 #include "components/favicon/core/favicon_service.h"
+#include "components/services/app_service/public/cpp/app_launch_util.h"
+#include "components/supervised_user/core/common/buildflags.h"
+#include "components/webapps/browser/install_result_code.h"
 #include "components/webapps/browser/installable/installable_manager.h"
 #include "components/webapps/browser/installable/installable_metrics.h"
 #include "content/public/browser/browser_context.h"
@@ -55,70 +61,21 @@
 #include "extensions/browser/extension_system.h"
 #include "extensions/common/api/management.h"
 #include "extensions/common/extension.h"
-#include "services/data_decoder/public/cpp/data_decoder.h"
+#include "extensions/common/extension_urls.h"
+#include "extensions/common/mojom/context_type.mojom.h"
 #include "third_party/blink/public/mojom/manifest/display_mode.mojom.h"
+#include "third_party/blink/public/mojom/window_features/window_features.mojom.h"
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
 #include "chrome/browser/ash/login/demo_mode/demo_session.h"
-#include "chrome/browser/chromeos/arc/arc_util.h"
-#include "chrome/browser/ui/app_list/arc/arc_app_utils.h"
-#include "components/arc/arc_service_manager.h"
-#include "components/arc/arc_util.h"
-#include "components/arc/mojom/intent_helper.mojom.h"
-#include "components/arc/session/arc_bridge_service.h"
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
-#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
-// TODO(https://crbug.com/1060801): Here and elsewhere, possibly switch build
-// flag to #if defined(OS_CHROMEOS)
-#include "chrome/browser/supervised_user/supervised_user_service.h"
-#include "chrome/browser/supervised_user/supervised_user_service_factory.h"
-#endif
-
 namespace {
-
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-const char kPlayIntentPrefix[] =
-    "https://play.google.com/store/apps/details?id=";
-const char kChromeWebStoreReferrer[] = "&referrer=chrome_web_store";
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
-
-using InstallAndroidAppCallback =
-    extensions::ManagementAPIDelegate::InstallAndroidAppCallback;
-using AndroidAppInstallStatusCallback =
-    extensions::ManagementAPIDelegate::AndroidAppInstallStatusCallback;
 using InstallOrLaunchWebAppCallback =
     extensions::ManagementAPIDelegate::InstallOrLaunchWebAppCallback;
 using InstallOrLaunchWebAppResult =
     extensions::ManagementAPIDelegate::InstallOrLaunchWebAppResult;
-using InstallableCheckResult = web_app::InstallManager::InstallableCheckResult;
-
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-void OnDidCheckForIntentToPlayStore(const std::string& intent,
-                                    InstallAndroidAppCallback callback,
-                                    bool installable) {
-  if (!installable) {
-    std::move(callback).Run(false);
-    return;
-  }
-
-  auto* arc_service_manager = arc::ArcServiceManager::Get();
-  if (!arc_service_manager) {
-    std::move(callback).Run(false);
-    return;
-  }
-
-  auto* instance = ARC_GET_INSTANCE_FOR_METHOD(
-      arc_service_manager->arc_bridge_service()->intent_helper(), HandleUrl);
-  if (!instance) {
-    std::move(callback).Run(false);
-    return;
-  }
-
-  instance->HandleUrl(intent, arc::kPlayStorePackage);
-  std::move(callback).Run(true);
-}
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+using InstallableCheckResult = web_app::InstallableCheckResult;
 
 class ManagementSetEnabledFunctionInstallPromptDelegate
     : public extensions::InstallPromptDelegate {
@@ -141,11 +98,22 @@ class ManagementSetEnabledFunctionInstallPromptDelegate
         std::make_unique<ExtensionInstallPrompt::Prompt>(type),
         ExtensionInstallPrompt::GetDefaultShowDialogCallback());
   }
+
+  ManagementSetEnabledFunctionInstallPromptDelegate(
+      const ManagementSetEnabledFunctionInstallPromptDelegate&) = delete;
+  ManagementSetEnabledFunctionInstallPromptDelegate& operator=(
+      const ManagementSetEnabledFunctionInstallPromptDelegate&) = delete;
+
   ~ManagementSetEnabledFunctionInstallPromptDelegate() override {}
 
  private:
-  void OnInstallPromptDone(ExtensionInstallPrompt::Result result) {
-    std::move(callback_).Run(result ==
+  void OnInstallPromptDone(
+      ExtensionInstallPrompt::DoneCallbackPayload payload) {
+    // This dialog doesn't support the "withhold permissions" checkbox.
+    DCHECK_NE(
+        payload.result,
+        ExtensionInstallPrompt::Result::ACCEPTED_WITH_WITHHELD_PERMISSIONS);
+    std::move(callback_).Run(payload.result ==
                              ExtensionInstallPrompt::Result::ACCEPTED);
   }
 
@@ -156,8 +124,6 @@ class ManagementSetEnabledFunctionInstallPromptDelegate
 
   base::WeakPtrFactory<ManagementSetEnabledFunctionInstallPromptDelegate>
       weak_factory_{this};
-
-  DISALLOW_COPY_AND_ASSIGN(ManagementSetEnabledFunctionInstallPromptDelegate);
 };
 
 class ManagementUninstallFunctionUninstallDialogDelegate
@@ -171,17 +137,20 @@ class ManagementUninstallFunctionUninstallDialogDelegate
       : function_(function) {
     ChromeExtensionFunctionDetails details(function);
     extension_uninstall_dialog_ = extensions::ExtensionUninstallDialog::Create(
-        details.GetProfile(), details.GetNativeWindowForUI(), this);
+        Profile::FromBrowserContext(function->browser_context()),
+        details.GetNativeWindowForUI(), this);
     bool uninstall_from_webstore =
-        function->extension() &&
-        function->extension()->id() == extensions::kWebStoreAppId;
+        (function->extension() &&
+         function->extension()->id() == extensions::kWebStoreAppId) ||
+        function->source_url().DomainIs(
+            extension_urls::GetNewWebstoreLaunchURL().host());
     extensions::UninstallSource source;
     extensions::UninstallReason reason;
     if (uninstall_from_webstore) {
       source = extensions::UNINSTALL_SOURCE_CHROME_WEBSTORE;
       reason = extensions::UNINSTALL_REASON_CHROME_WEBSTORE;
     } else if (function->source_context_type() ==
-               extensions::Feature::WEBUI_CONTEXT) {
+               extensions::mojom::ContextType::kWebUi) {
       source = extensions::UNINSTALL_SOURCE_CHROME_EXTENSIONS_PAGE;
       // TODO: Update this to a new reason; it shouldn't be lumped in with
       // other uninstalls if it's from the chrome://extensions page.
@@ -199,34 +168,41 @@ class ManagementUninstallFunctionUninstallDialogDelegate
     }
   }
 
+  ManagementUninstallFunctionUninstallDialogDelegate(
+      const ManagementUninstallFunctionUninstallDialogDelegate&) = delete;
+  ManagementUninstallFunctionUninstallDialogDelegate& operator=(
+      const ManagementUninstallFunctionUninstallDialogDelegate&) = delete;
+
   ~ManagementUninstallFunctionUninstallDialogDelegate() override {}
 
   // ExtensionUninstallDialog::Delegate implementation.
   void OnExtensionUninstallDialogClosed(bool did_start_uninstall,
-                                        const base::string16& error) override {
+                                        const std::u16string& error) override {
     function_->OnExtensionUninstallDialogClosed(did_start_uninstall, error);
   }
 
  private:
-  extensions::ManagementUninstallFunctionBase* function_;
+  raw_ptr<extensions::ManagementUninstallFunctionBase> function_;
   std::unique_ptr<extensions::ExtensionUninstallDialog>
       extension_uninstall_dialog_;
-
-  DISALLOW_COPY_AND_ASSIGN(ManagementUninstallFunctionUninstallDialogDelegate);
 };
 
 void OnGenerateAppForLinkCompleted(
     extensions::ManagementGenerateAppForLinkFunction* function,
-    const web_app::AppId& app_id,
-    web_app::InstallResultCode code) {
+    const webapps::AppId& app_id,
+    webapps::InstallResultCode code) {
   const bool install_success =
-      code == web_app::InstallResultCode::kSuccessNewInstall;
+      code == webapps::InstallResultCode::kSuccessNewInstall;
   function->FinishCreateWebApp(app_id, install_success);
 }
 
 class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
  public:
   ChromeAppForLinkDelegate() {}
+
+  ChromeAppForLinkDelegate(const ChromeAppForLinkDelegate&) = delete;
+  ChromeAppForLinkDelegate& operator=(const ChromeAppForLinkDelegate&) = delete;
+
   ~ChromeAppForLinkDelegate() override {}
 
   void OnFaviconForApp(
@@ -235,23 +211,33 @@ class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
       const std::string& title,
       const GURL& launch_url,
       const favicon_base::FaviconImageResult& image_result) {
-    auto web_app_info = std::make_unique<WebApplicationInfo>();
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+    // Avoid accessing the WebAppProvider when web apps are enabled in Lacros
+    // (and thus disabled in Ash).
+    if (web_app::IsWebAppsCrosapiEnabled()) {
+      function->FinishCreateWebApp(std::string(),
+                                   /*install_success=*/false);
+      return;
+    }
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+
+    auto web_app_info = std::make_unique<web_app::WebAppInstallInfo>();
     web_app_info->title = base::UTF8ToUTF16(title);
     web_app_info->start_url = launch_url;
     web_app_info->display_mode = web_app::DisplayMode::kBrowser;
-    web_app_info->open_as_window = false;
+    web_app_info->user_display_mode = web_app::mojom::UserDisplayMode::kBrowser;
 
     if (!image_result.image.IsEmpty()) {
-      web_app_info->icon_bitmaps_any[image_result.image.Width()] =
+      web_app_info->icon_bitmaps.any[image_result.image.Width()] =
           image_result.image.AsBitmap();
     }
 
-    auto* provider = web_app::WebAppProviderBase::GetProviderBase(
+    auto* provider = web_app::WebAppProvider::GetForWebApps(
         Profile::FromBrowserContext(context));
-    DCHECK(provider);
 
-    provider->install_manager().InstallWebAppFromInfo(
-        std::move(web_app_info), web_app::ForInstallableSite::kNo,
+    provider->scheduler().InstallFromInfo(
+        std::move(web_app_info),
+        /*overwrite_existing_manifest_fields=*/false,
         webapps::WebappInstallSource::MANAGEMENT_API,
         base::BindOnce(OnGenerateAppForLinkCompleted,
                        base::RetainedRef(function)));
@@ -260,28 +246,26 @@ class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
   extensions::api::management::ExtensionInfo CreateExtensionInfoFromWebApp(
       const std::string& app_id,
       content::BrowserContext* context) override {
-    auto* provider = web_app::WebAppProviderBase::GetProviderBase(
+    auto* provider = web_app::WebAppProvider::GetForWebApps(
         Profile::FromBrowserContext(context));
     DCHECK(provider);
-    const web_app::AppRegistrar& registrar = provider->registrar();
+    const web_app::WebAppRegistrar& registrar = provider->registrar_unsafe();
 
     extensions::api::management::ExtensionInfo info;
     info.id = app_id;
     info.name = registrar.GetAppShortName(app_id);
     info.enabled = registrar.IsLocallyInstalled(app_id);
     info.install_type =
-        extensions::api::management::EXTENSION_INSTALL_TYPE_OTHER;
+        extensions::api::management::ExtensionInstallType::kOther;
     info.is_app = true;
-    info.type = extensions::api::management::EXTENSION_TYPE_HOSTED_APP;
-    info.app_launch_url =
-        std::make_unique<std::string>(registrar.GetAppStartUrl(app_id).spec());
+    info.type = extensions::api::management::ExtensionType::kHostedApp;
+    info.app_launch_url = registrar.GetAppStartUrl(app_id).spec();
 
-    info.icons =
-        std::make_unique<std::vector<extensions::api::management::IconInfo>>();
-    std::vector<WebApplicationIconInfo> icon_infos =
+    info.icons.emplace();
+    std::vector<apps::IconInfo> manifest_icons =
         registrar.GetAppIconInfos(app_id);
-    info.icons->reserve(icon_infos.size());
-    for (const WebApplicationIconInfo& web_app_icon_info : icon_infos) {
+    info.icons->reserve(manifest_icons.size());
+    for (const apps::IconInfo& web_app_icon_info : manifest_icons) {
       extensions::api::management::IconInfo icon_info;
       if (web_app_icon_info.square_size_px)
         icon_info.size = *web_app_icon_info.square_size_px;
@@ -292,21 +276,23 @@ class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
     switch (registrar.GetAppDisplayMode(app_id)) {
       case web_app::DisplayMode::kBrowser:
         info.launch_type =
-            extensions::api::management::LAUNCH_TYPE_OPEN_AS_REGULAR_TAB;
+            extensions::api::management::LaunchType::kOpenAsRegularTab;
         break;
       case web_app::DisplayMode::kMinimalUi:
       case web_app::DisplayMode::kStandalone:
         info.launch_type =
-            extensions::api::management::LAUNCH_TYPE_OPEN_AS_WINDOW;
+            extensions::api::management::LaunchType::kOpenAsWindow;
         break;
       case web_app::DisplayMode::kFullscreen:
         info.launch_type =
-            extensions::api::management::LAUNCH_TYPE_OPEN_FULL_SCREEN;
+            extensions::api::management::LaunchType::kOpenFullScreen;
         break;
-      // This mode is not supported by the extension app backend.
+      // These modes are not supported by the extension app backend.
       case web_app::DisplayMode::kWindowControlsOverlay:
+      case web_app::DisplayMode::kTabbed:
+      case web_app::DisplayMode::kBorderless:
       case web_app::DisplayMode::kUndefined:
-        info.launch_type = extensions::api::management::LAUNCH_TYPE_NONE;
+        info.launch_type = extensions::api::management::LaunchType::kNone;
         break;
     }
 
@@ -315,34 +301,39 @@ class ChromeAppForLinkDelegate : public extensions::AppForLinkDelegate {
 
   // Used for favicon loading tasks.
   base::CancelableTaskTracker cancelable_task_tracker_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(ChromeAppForLinkDelegate);
 };
 
-void LaunchWebApp(const web_app::AppId& app_id, Profile* profile) {
+void LaunchWebApp(const webapps::AppId& app_id, Profile* profile) {
   // Look at prefs to find the right launch container. If the user has not set a
   // preference, the default launch value will be returned.
   // TODO(crbug.com/1003602): Make AppLaunchParams launch container Optional or
   // add a "default" launch container enum value.
-  auto* provider = web_app::WebAppProviderBase::GetProviderBase(profile);
+  auto* provider = web_app::WebAppProvider::GetForWebApps(profile);
   DCHECK(provider);
-  blink::mojom::DisplayMode display_mode =
-      provider->registrar().GetAppUserDisplayMode(app_id);
-  auto launch_container = apps::mojom::LaunchContainer::kLaunchContainerWindow;
-  if (display_mode == blink::mojom::DisplayMode::kBrowser)
-    launch_container = apps::mojom::LaunchContainer::kLaunchContainerTab;
+  std::optional<web_app::mojom::UserDisplayMode> display_mode =
+      provider->registrar_unsafe().GetAppUserDisplayMode(app_id);
+  auto launch_container = apps::LaunchContainer::kLaunchContainerWindow;
+  if (display_mode == web_app::mojom::UserDisplayMode::kBrowser) {
+    launch_container = apps::LaunchContainer::kLaunchContainerTab;
+  }
 
-  apps::AppServiceProxyFactory::GetForProfile(profile)
-      ->BrowserAppLauncher()
-      ->LaunchAppWithParams(apps::AppLaunchParams(
-          app_id, launch_container, WindowOpenDisposition::NEW_FOREGROUND_TAB,
-          apps::mojom::AppLaunchSource::kSourceManagementApi));
+  if (!apps::AppServiceProxyFactory::IsAppServiceAvailableForProfile(profile)) {
+    // If the profile doesn't have an App Service Proxy available, that means
+    // this extension has been explicitly permitted to run in an incognito
+    // context. Treat this as if the extension is running in the original
+    // profile, so it is allowed to access apps in the original profile.
+    profile = profile->GetOriginalProfile();
+  }
+
+  apps::AppServiceProxyFactory::GetForProfile(profile)->LaunchAppWithParams(
+      apps::AppLaunchParams(app_id, launch_container,
+                            WindowOpenDisposition::NEW_FOREGROUND_TAB,
+                            apps::LaunchSource::kFromManagementApi));
 }
 
 void OnWebAppInstallCompleted(InstallOrLaunchWebAppCallback callback,
-                              const web_app::AppId& app_id,
-                              web_app::InstallResultCode code) {
+                              const webapps::AppId& app_id,
+                              webapps::InstallResultCode code) {
   InstallOrLaunchWebAppResult result =
       IsSuccess(code) ? InstallOrLaunchWebAppResult::kSuccess
                       : InstallOrLaunchWebAppResult::kUnknownError;
@@ -350,15 +341,18 @@ void OnWebAppInstallCompleted(InstallOrLaunchWebAppCallback callback,
 }
 
 void OnWebAppInstallabilityChecked(
-    Profile* profile,
+    base::WeakPtr<Profile> profile,
     InstallOrLaunchWebAppCallback callback,
     std::unique_ptr<content::WebContents> web_contents,
     InstallableCheckResult result,
-    base::Optional<web_app::AppId> app_id) {
+    std::optional<webapps::AppId> app_id) {
+  if (!profile) {
+    return;
+  }
   switch (result) {
     case InstallableCheckResult::kAlreadyInstalled:
       DCHECK(app_id);
-      LaunchWebApp(*app_id, profile);
+      LaunchWebApp(*app_id, profile.get());
       std::move(callback).Run(InstallOrLaunchWebAppResult::kSuccess);
       return;
     case InstallableCheckResult::kNotInstallable:
@@ -366,29 +360,41 @@ void OnWebAppInstallabilityChecked(
       return;
     case InstallableCheckResult::kInstallable:
       content::WebContents* containing_contents = web_contents.get();
-      chrome::ScopedTabbedBrowserDisplayer displayer(profile);
-      const GURL& url = web_contents->GetURL();
-      chrome::AddWebContents(
-          displayer.browser(), nullptr, std::move(web_contents), url,
-          WindowOpenDisposition::NEW_FOREGROUND_TAB, gfx::Rect());
+      chrome::ScopedTabbedBrowserDisplayer displayer(profile.get());
+      const GURL& url = web_contents->GetLastCommittedURL();
+      chrome::AddWebContents(displayer.browser(), nullptr,
+                             std::move(web_contents), url,
+                             WindowOpenDisposition::NEW_FOREGROUND_TAB,
+                             blink::mojom::WindowFeatures());
       web_app::CreateWebAppFromManifest(
-          containing_contents, /*bypass_service_worker_check=*/true,
-          webapps::WebappInstallSource::MANAGEMENT_API,
+          containing_contents, webapps::WebappInstallSource::MANAGEMENT_API,
           base::BindOnce(&OnWebAppInstallCompleted, std::move(callback)));
       return;
   }
   NOTREACHED();
 }
 
+#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
+extensions::SupervisedUserExtensionsDelegate*
+GetSupervisedUserExtensionsDelegateFromContext(
+    content::BrowserContext* context) {
+  extensions::SupervisedUserExtensionsDelegate*
+      supervised_user_extensions_delegate =
+          extensions::ManagementAPI::GetFactoryInstance()
+              ->Get(context)
+              ->GetSupervisedUserExtensionsDelegate();
+  CHECK(supervised_user_extensions_delegate);
+  return supervised_user_extensions_delegate;
+}
+#endif  // BUILDFLAG(ENABLE_SUPERVISED_USERS)
+
 }  // namespace
 
-ChromeManagementAPIDelegate::ChromeManagementAPIDelegate() {
-}
+ChromeManagementAPIDelegate::ChromeManagementAPIDelegate() = default;
 
-ChromeManagementAPIDelegate::~ChromeManagementAPIDelegate() {
-}
+ChromeManagementAPIDelegate::~ChromeManagementAPIDelegate() = default;
 
-void ChromeManagementAPIDelegate::LaunchAppFunctionDelegate(
+bool ChromeManagementAPIDelegate::LaunchAppFunctionDelegate(
     const extensions::Extension* extension,
     content::BrowserContext* context) const {
   // Look at prefs to find the right launch container.
@@ -396,23 +402,36 @@ void ChromeManagementAPIDelegate::LaunchAppFunctionDelegate(
   // returned.
   // TODO(crbug.com/1003602): Make AppLaunchParams launch container Optional or
   // add a "default" launch container enum value.
-  extensions::LaunchContainer launch_container =
+  apps::LaunchContainer launch_container =
       GetLaunchContainer(extensions::ExtensionPrefs::Get(context), extension);
   Profile* profile = Profile::FromBrowserContext(context);
-  apps::AppServiceProxyFactory::GetForProfile(profile)
-      ->BrowserAppLauncher()
-      ->LaunchAppWithParams(apps::AppLaunchParams(
-          extension->id(), launch_container,
-          WindowOpenDisposition::NEW_FOREGROUND_TAB,
-          apps::mojom::AppLaunchSource::kSourceManagementApi));
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX) || \
+    BUILDFLAG(IS_FUCHSIA)
+  if (extensions::IsExtensionUnsupportedDeprecatedApp(profile,
+                                                      extension->id())) {
+    return false;
+  }
+#endif
+  if (!apps::AppServiceProxyFactory::IsAppServiceAvailableForProfile(profile)) {
+    // If the profile doesn't have an App Service Proxy available, that means
+    // this extension has been explicitly permitted to run in an incognito
+    // context. Treat this as if the extension is running in the original
+    // profile, so it is allowed to access apps in the original profile.
+    profile = profile->GetOriginalProfile();
+  }
+  apps::AppServiceProxyFactory::GetForProfile(profile)->LaunchAppWithParams(
+      apps::AppLaunchParams(extension->id(), launch_container,
+                            WindowOpenDisposition::NEW_FOREGROUND_TAB,
+                            apps::LaunchSource::kFromManagementApi));
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
-  chromeos::DemoSession::RecordAppLaunchSourceIfInDemoMode(
-      chromeos::DemoSession::AppLaunchSource::kExtensionApi);
+  ash::DemoSession::RecordAppLaunchSourceIfInDemoMode(
+      ash::DemoSession::AppLaunchSource::kExtensionApi);
 #endif
 
   extensions::RecordAppLaunchType(extension_misc::APP_LAUNCH_EXTENSION_API,
                                   extension->GetType());
+  return true;
 }
 
 GURL ChromeManagementAPIDelegate::GetFullLaunchURL(
@@ -426,18 +445,6 @@ extensions::LaunchType ChromeManagementAPIDelegate::GetLaunchType(
   return extensions::GetLaunchType(prefs, extension);
 }
 
-void ChromeManagementAPIDelegate::
-    GetPermissionWarningsByManifestFunctionDelegate(
-        extensions::ManagementGetPermissionWarningsByManifestFunction* function,
-        const std::string& manifest_str) const {
-  data_decoder::DataDecoder::ParseJsonIsolated(
-      manifest_str,
-      base::BindOnce(
-          &extensions::ManagementGetPermissionWarningsByManifestFunction::
-              OnParse,
-          function));
-}
-
 std::unique_ptr<extensions::InstallPromptDelegate>
 ChromeManagementAPIDelegate::SetEnabledFunctionDelegate(
     content::WebContents* web_contents,
@@ -512,82 +519,39 @@ void ChromeManagementAPIDelegate::InstallOrLaunchReplacementWebApp(
     content::BrowserContext* context,
     const GURL& web_app_url,
     InstallOrLaunchWebAppCallback callback) const {
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+  // Avoid accessing the WebAppProvider when web apps are enabled in Lacros (and
+  // thus disabled in Ash).
+  if (web_app::IsWebAppsCrosapiEnabled()) {
+    std::move(callback).Run(InstallOrLaunchWebAppResult::kUnknownError);
+    return;
+  }
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+
   Profile* profile = Profile::FromBrowserContext(context);
-  auto* provider = web_app::WebAppProviderBase::GetProviderBase(profile);
+  auto* provider = web_app::WebAppProvider::GetForWebApps(profile);
   DCHECK(provider);
 
   // Launch the app if web_app_url happens to match start_url. If not, the app
   // could still be installed with different start_url.
-  if (provider->registrar().IsLocallyInstalled(web_app_url)) {
-    LaunchWebApp(web_app::GenerateAppIdFromURL(web_app_url), profile);
+  if (provider->registrar_unsafe().IsLocallyInstalled(web_app_url)) {
+    LaunchWebApp(
+        web_app::GenerateAppId(/*manifest_id=*/std::nullopt, web_app_url),
+        profile);
     std::move(callback).Run(InstallOrLaunchWebAppResult::kSuccess);
     return;
   }
 
-  provider->install_manager().LoadWebAppAndCheckManifest(
-      web_app_url, webapps::WebappInstallSource::MANAGEMENT_API,
-      base::BindOnce(&OnWebAppInstallabilityChecked, profile,
-                     std::move(callback)));
-}
+  std::unique_ptr<content::WebContents> web_contents =
+      content::WebContents::Create(content::WebContents::CreateParams(profile));
+  web_app::CreateWebAppInstallTabHelpers(web_contents.get());
 
-bool ChromeManagementAPIDelegate::CanContextInstallAndroidApps(
-    content::BrowserContext* context) const {
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-  return arc::IsArcAllowedForProfile(Profile::FromBrowserContext(context));
-#else
-  return false;
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
-}
-
-void ChromeManagementAPIDelegate::CheckAndroidAppInstallStatus(
-    const std::string& package_name,
-    AndroidAppInstallStatusCallback callback) const {
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-  auto* arc_service_manager = arc::ArcServiceManager::Get();
-  if (!arc_service_manager) {
-    std::move(callback).Run(false);
-    return;
-  }
-
-  auto* instance = ARC_GET_INSTANCE_FOR_METHOD(
-      arc_service_manager->arc_bridge_service()->app(), IsInstallable);
-  if (!instance) {
-    std::move(callback).Run(false);
-    return;
-  }
-
-  instance->IsInstallable(package_name, std::move(callback));
-#else
-  std::move(callback).Run(false);
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
-}
-
-void ChromeManagementAPIDelegate::InstallReplacementAndroidApp(
-    const std::string& package_name,
-    InstallAndroidAppCallback callback) const {
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-  std::string intent =
-      base::StrCat({kPlayIntentPrefix, package_name, kChromeWebStoreReferrer});
-
-  auto* arc_service_manager = arc::ArcServiceManager::Get();
-  if (!arc_service_manager) {
-    std::move(callback).Run(false);
-    return;
-  }
-
-  auto* instance = ARC_GET_INSTANCE_FOR_METHOD(
-      arc_service_manager->arc_bridge_service()->app(), IsInstallable);
-  if (!instance) {
-    std::move(callback).Run(false);
-    return;
-  }
-
-  instance->IsInstallable(
-      package_name, base::BindOnce(&OnDidCheckForIntentToPlayStore, intent,
-                                   std::move(callback)));
-#else
-  std::move(callback).Run(false);
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+  base::WeakPtr<content::WebContents> web_contents_ptr =
+      web_contents->GetWeakPtr();
+  provider->scheduler().FetchInstallabilityForChromeManagement(
+      web_app_url, web_contents_ptr,
+      base::BindOnce(&OnWebAppInstallabilityChecked, profile->GetWeakPtr(),
+                     std::move(callback), std::move(web_contents)));
 }
 
 void ChromeManagementAPIDelegate::EnableExtension(
@@ -601,11 +565,11 @@ void ChromeManagementAPIDelegate::EnableExtension(
   // We add approval for the extension here under the assumption that prior
   // to this point, the supervised child user has already been prompted
   // for, and received parent permission to install the extension.
-  SupervisedUserService* supervised_user_service =
-      SupervisedUserServiceFactory::GetForBrowserContext(context);
-  supervised_user_service->AddExtensionApproval(*extension);
-  supervised_user_service->RecordExtensionEnablementUmaMetrics(
-      /*enabled=*/true);
+  extensions::SupervisedUserExtensionsDelegate* extensions_delegate =
+      GetSupervisedUserExtensionsDelegateFromContext(context);
+
+  extensions_delegate->AddExtensionApproval(*extension);
+  extensions_delegate->RecordExtensionEnablementUmaMetrics(/*enabled=*/true);
 #endif  // BUILDFLAG(ENABLE_SUPERVISED_USERS)
 
   // If the extension was disabled for a permissions increase, the Management
@@ -622,10 +586,9 @@ void ChromeManagementAPIDelegate::DisableExtension(
     const std::string& extension_id,
     extensions::disable_reason::DisableReason disable_reason) const {
 #if BUILDFLAG(ENABLE_SUPERVISED_USERS)
-  SupervisedUserService* supervised_user_service =
-      SupervisedUserServiceFactory::GetForBrowserContext(context);
-  supervised_user_service->RecordExtensionEnablementUmaMetrics(
-      /*enabled=*/false);
+  extensions::SupervisedUserExtensionsDelegate* extensions_delegate =
+      GetSupervisedUserExtensionsDelegateFromContext(context);
+  extensions_delegate->RecordExtensionEnablementUmaMetrics(/*enabled=*/false);
 #endif  // BUILDFLAG(ENABLE_SUPERVISED_USERS)
   extensions::ExtensionSystem::Get(context)
       ->extension_service()
@@ -637,7 +600,7 @@ bool ChromeManagementAPIDelegate::UninstallExtension(
     content::BrowserContext* context,
     const std::string& transient_extension_id,
     extensions::UninstallReason reason,
-    base::string16* error) const {
+    std::u16string* error) const {
   return extensions::ExtensionSystem::Get(context)
       ->extension_service()
       ->UninstallExtension(transient_extension_id, reason, error);
