@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,12 +9,8 @@
 #include <utility>
 #include <vector>
 
-#include "base/bind.h"
-#include "base/callback.h"
 #include "base/command_line.h"
-#include "base/macros.h"
-#include "build/build_config.h"
-#include "chrome/browser/extensions/api/networking_cast_private/chrome_networking_cast_private_delegate.h"
+#include "base/functional/bind.h"
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "components/keyed_service/core/keyed_service.h"
 #include "components/onc/onc_constants.h"
@@ -46,7 +42,11 @@ class TestNetworkingPrivateDelegate : public NetworkingPrivateDelegate {
   explicit TestNetworkingPrivateDelegate(bool test_failure)
       : fail_(test_failure) {}
 
-  ~TestNetworkingPrivateDelegate() override {}
+  TestNetworkingPrivateDelegate(const TestNetworkingPrivateDelegate&) = delete;
+  TestNetworkingPrivateDelegate& operator=(
+      const TestNetworkingPrivateDelegate&) = delete;
+
+  ~TestNetworkingPrivateDelegate() override = default;
 
   // Asynchronous methods
   void GetProperties(const std::string& guid,
@@ -67,7 +67,7 @@ class TestNetworkingPrivateDelegate : public NetworkingPrivateDelegate {
   }
 
   void SetProperties(const std::string& guid,
-                     std::unique_ptr<base::DictionaryValue> properties,
+                     base::Value::Dict properties,
                      bool allow_set_shared_config,
                      VoidCallback success_callback,
                      FailureCallback failure_callback) override {
@@ -75,7 +75,7 @@ class TestNetworkingPrivateDelegate : public NetworkingPrivateDelegate {
   }
 
   void CreateNetwork(bool shared,
-                     std::unique_ptr<base::DictionaryValue> properties,
+                     base::Value::Dict properties,
                      StringCallback success_callback,
                      FailureCallback failure_callback) override {
     StringResult(std::move(success_callback), std::move(failure_callback),
@@ -98,12 +98,12 @@ class TestNetworkingPrivateDelegate : public NetworkingPrivateDelegate {
     if (fail_) {
       std::move(failure_callback).Run(kFailure);
     } else {
-      std::unique_ptr<base::ListValue> result(new base::ListValue);
-      std::unique_ptr<base::DictionaryValue> network(new base::DictionaryValue);
-      network->SetString(::onc::network_config::kType,
-                         ::onc::network_config::kEthernet);
-      network->SetString(::onc::network_config::kGUID, kGuid);
-      result->Append(std::move(network));
+      base::Value::List result;
+      base::Value::Dict network;
+      network.Set(::onc::network_config::kType,
+                  ::onc::network_config::kEthernet);
+      network.Set(::onc::network_config::kGUID, kGuid);
+      result.Append(std::move(network));
       std::move(success_callback).Run(std::move(result));
     }
   }
@@ -158,50 +158,49 @@ class TestNetworkingPrivateDelegate : public NetworkingPrivateDelegate {
     VoidResult(std::move(success_callback), std::move(failure_callback));
   }
 
-  // Synchronous methods
-  std::unique_ptr<base::ListValue> GetEnabledNetworkTypes() override {
-    std::unique_ptr<base::ListValue> result;
+  void GetEnabledNetworkTypes(EnabledNetworkTypesCallback callback) override {
+    base::Value::List result;
     if (!fail_) {
-      result.reset(new base::ListValue);
-      result->AppendString(::onc::network_config::kEthernet);
+      result.Append(::onc::network_config::kEthernet);
     }
-    return result;
+    std::move(callback).Run(std::move(result));
   }
 
-  std::unique_ptr<DeviceStateList> GetDeviceStateList() override {
-    std::unique_ptr<DeviceStateList> result;
-    if (fail_)
-      return result;
-    result.reset(new DeviceStateList);
-    std::unique_ptr<api::networking_private::DeviceStateProperties> properties(
-        new api::networking_private::DeviceStateProperties);
-    properties->type = api::networking_private::NETWORK_TYPE_ETHERNET;
-    properties->state = api::networking_private::DEVICE_STATE_TYPE_ENABLED;
-    result->push_back(std::move(properties));
-    return result;
+  void GetDeviceStateList(DeviceStateListCallback callback) override {
+    DeviceStateList result;
+    if (!fail_) {
+      api::networking_private::DeviceStateProperties& properties =
+          result.emplace_back();
+      properties.type = api::networking_private::NetworkType::kEthernet;
+      properties.state = api::networking_private::DeviceStateType::kEnabled;
+    }
+    std::move(callback).Run(std::move(result));
   }
 
-  std::unique_ptr<base::DictionaryValue> GetGlobalPolicy() override {
-    return std::make_unique<base::DictionaryValue>();
+  void GetGlobalPolicy(GetGlobalPolicyCallback callback) override {
+    std::move(callback).Run(base::Value::Dict());
   }
 
-  std::unique_ptr<base::DictionaryValue> GetCertificateLists() override {
-    return std::make_unique<base::DictionaryValue>();
+  void GetCertificateLists(GetCertificateListsCallback callback) override {
+    std::move(callback).Run(base::Value::Dict());
   }
 
-  bool EnableNetworkType(const std::string& type) override {
+  // Synchronous methods
+  void EnableNetworkType(const std::string& type,
+                         BoolCallback callback) override {
     enabled_[type] = true;
-    return !fail_;
+    std::move(callback).Run(!fail_);
   }
 
-  bool DisableNetworkType(const std::string& type) override {
+  void DisableNetworkType(const std::string& type,
+                          BoolCallback callback) override {
     disabled_[type] = true;
-    return !fail_;
+    std::move(callback).Run(!fail_);
   }
 
-  bool RequestScan(const std::string& type) override {
+  void RequestScan(const std::string& type, BoolCallback callback) override {
     scan_requested_.push_back(type);
-    return !fail_;
+    std::move(callback).Run(!fail_);
   }
 
   void set_fail(bool fail) { fail_ = fail; }
@@ -215,10 +214,9 @@ class TestNetworkingPrivateDelegate : public NetworkingPrivateDelegate {
     if (fail_) {
       std::move(failure_callback).Run(kFailure);
     } else {
-      std::unique_ptr<base::DictionaryValue> result(new base::DictionaryValue);
-      result->SetString(::onc::network_config::kGUID, guid);
-      result->SetString(::onc::network_config::kType,
-                        ::onc::network_config::kWiFi);
+      base::Value::Dict result;
+      result.Set(::onc::network_config::kGUID, guid);
+      result.Set(::onc::network_config::kType, ::onc::network_config::kWiFi);
       std::move(success_callback).Run(std::move(result));
     }
   }
@@ -253,14 +251,13 @@ class TestNetworkingPrivateDelegate : public NetworkingPrivateDelegate {
 
   void ValueResult(const std::string& guid, PropertiesCallback callback) {
     if (fail_) {
-      std::move(callback).Run(base::nullopt, kFailure);
+      std::move(callback).Run(std::nullopt, kFailure);
       return;
     }
-    base::Value result(base::Value::Type::DICTIONARY);
-    result.SetStringKey(::onc::network_config::kGUID, guid);
-    result.SetStringKey(::onc::network_config::kType,
-                        ::onc::network_config::kWiFi);
-    std::move(callback).Run(std::move(result), base::nullopt);
+    base::Value::Dict result;
+    result.Set(::onc::network_config::kGUID, guid);
+    result.Set(::onc::network_config::kType, ::onc::network_config::kWiFi);
+    std::move(callback).Run(std::move(result), std::nullopt);
   }
 
  private:
@@ -268,60 +265,17 @@ class TestNetworkingPrivateDelegate : public NetworkingPrivateDelegate {
   std::map<std::string, bool> enabled_;
   std::map<std::string, bool> disabled_;
   std::vector<std::string> scan_requested_;
-
-  DISALLOW_COPY_AND_ASSIGN(TestNetworkingPrivateDelegate);
-};
-
-class TestNetworkingCastPrivateDelegate
-    : public ChromeNetworkingCastPrivateDelegate {
- public:
-  explicit TestNetworkingCastPrivateDelegate(bool test_failure)
-      : fail_(test_failure) {}
-
-  ~TestNetworkingCastPrivateDelegate() override {}
-
-  void VerifyDestination(std::unique_ptr<Credentials> credentials,
-                         VerifiedCallback success_callback,
-                         FailureCallback failure_callback) override {
-    if (fail_) {
-      std::move(failure_callback).Run(kFailure);
-    } else {
-      std::move(success_callback).Run(true);
-    }
-  }
-
-  void VerifyAndEncryptData(const std::string& data,
-                            std::unique_ptr<Credentials> credentials,
-                            DataCallback success_callback,
-                            FailureCallback failure_callback) override {
-    if (fail_) {
-      std::move(failure_callback).Run(kFailure);
-    } else {
-      std::move(success_callback).Run("encrypted_data");
-    }
-  }
-
- private:
-  bool fail_;
-
-  DISALLOW_COPY_AND_ASSIGN(TestNetworkingCastPrivateDelegate);
 };
 
 class NetworkingPrivateApiTest : public ExtensionApiTest {
  public:
   NetworkingPrivateApiTest() = default;
+
+  NetworkingPrivateApiTest(const NetworkingPrivateApiTest&) = delete;
+  NetworkingPrivateApiTest& operator=(const NetworkingPrivateApiTest&) = delete;
+
   ~NetworkingPrivateApiTest() override = default;
 
-  void SetUp() override {
-    networking_cast_delegate_factory_ = base::BindRepeating(
-        &NetworkingPrivateApiTest::CreateTestNetworkingCastPrivateDelegate,
-        base::Unretained(this), test_failure_);
-    ChromeNetworkingCastPrivateDelegate::SetFactoryCallbackForTest(
-        &networking_cast_delegate_factory_);
-
-    ExtensionApiTest::SetUp();
-  }
-
   void SetUpCommandLine(base::CommandLine* command_line) override {
     ExtensionApiTest::SetUpCommandLine(command_line);
     // Allowlist the extension ID of the test extension.
@@ -339,12 +293,6 @@ class NetworkingPrivateApiTest : public ExtensionApiTest {
             base::Unretained(this), test_failure_));
   }
 
-  void TearDown() override {
-    ExtensionApiTest::TearDown();
-
-    ChromeNetworkingCastPrivateDelegate::SetFactoryCallbackForTest(nullptr);
-  }
-
   bool GetEnabled(const std::string& type) {
     return networking_private_delegate()->GetEnabled(type);
   }
@@ -359,16 +307,13 @@ class NetworkingPrivateApiTest : public ExtensionApiTest {
 
  protected:
   bool RunNetworkingSubtest(const std::string& subtest) {
-    return RunExtensionSubtest("networking_private", "main.html?" + subtest,
-                               kFlagNone, kFlagLoadAsComponent);
+    const std::string extension_url = "main.html?" + subtest;
+    return RunExtensionTest("networking_private",
+                            {.extension_url = extension_url.c_str()},
+                            {.load_as_component = true});
   }
 
  private:
-  std::unique_ptr<ChromeNetworkingCastPrivateDelegate>
-  CreateTestNetworkingCastPrivateDelegate(bool test_failure) {
-    return std::make_unique<TestNetworkingCastPrivateDelegate>(test_failure);
-  }
-
   std::unique_ptr<KeyedService> CreateTestNetworkingPrivateDelegate(
       bool test_failure,
       content::BrowserContext* /*context*/) {
@@ -385,12 +330,6 @@ class NetworkingPrivateApiTest : public ExtensionApiTest {
 
  protected:
   bool test_failure_ = false;
-
- private:
-  ChromeNetworkingCastPrivateDelegate::FactoryCallback
-      networking_cast_delegate_factory_;
-
-  DISALLOW_COPY_AND_ASSIGN(NetworkingPrivateApiTest);
 };
 
 }  // namespace
@@ -470,14 +409,6 @@ IN_PROC_BROWSER_TEST_F(NetworkingPrivateApiTest, StartActivate) {
   EXPECT_TRUE(RunNetworkingSubtest("startActivate")) << message_;
 }
 
-IN_PROC_BROWSER_TEST_F(NetworkingPrivateApiTest, VerifyDestination) {
-  EXPECT_TRUE(RunNetworkingSubtest("verifyDestination")) << message_;
-}
-
-IN_PROC_BROWSER_TEST_F(NetworkingPrivateApiTest, VerifyAndEncryptData) {
-  EXPECT_TRUE(RunNetworkingSubtest("verifyAndEncryptData")) << message_;
-}
-
 IN_PROC_BROWSER_TEST_F(NetworkingPrivateApiTest, GetCaptivePortalStatus) {
   EXPECT_TRUE(RunNetworkingSubtest("getCaptivePortalStatus")) << message_;
 }
@@ -505,10 +436,13 @@ class NetworkingPrivateApiTestFail : public NetworkingPrivateApiTest {
  public:
   NetworkingPrivateApiTestFail() { test_failure_ = true; }
 
+  NetworkingPrivateApiTestFail(const NetworkingPrivateApiTestFail&) = delete;
+  NetworkingPrivateApiTestFail& operator=(const NetworkingPrivateApiTestFail&) =
+      delete;
+
   ~NetworkingPrivateApiTestFail() override = default;
 
  protected:
-  DISALLOW_COPY_AND_ASSIGN(NetworkingPrivateApiTestFail);
 };
 
 }  // namespace
@@ -571,14 +505,6 @@ IN_PROC_BROWSER_TEST_F(NetworkingPrivateApiTestFail, StartActivate) {
   EXPECT_FALSE(RunNetworkingSubtest("startActivate")) << message_;
 }
 
-IN_PROC_BROWSER_TEST_F(NetworkingPrivateApiTestFail, VerifyDestination) {
-  EXPECT_FALSE(RunNetworkingSubtest("verifyDestination")) << message_;
-}
-
-IN_PROC_BROWSER_TEST_F(NetworkingPrivateApiTestFail, VerifyAndEncryptData) {
-  EXPECT_FALSE(RunNetworkingSubtest("verifyAndEncryptData")) << message_;
-}
-
 IN_PROC_BROWSER_TEST_F(NetworkingPrivateApiTestFail, GetCaptivePortalStatus) {
   EXPECT_FALSE(RunNetworkingSubtest("getCaptivePortalStatus")) << message_;
 }
