@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,19 +7,22 @@
 #include <memory>
 #include <string>
 
-#include "base/macros.h"
+#include "base/memory/raw_ptr.h"
 #include "base/values.h"
-#include "chrome/browser/extensions/api/content_settings/content_settings_service.h"
 #include "chrome/browser/extensions/api/preference/preference_api.h"
 #include "chrome/browser/extensions/extension_prefs_unittest.h"
 #include "chrome/test/base/testing_profile.h"
 #include "components/pref_registry/pref_registry_syncable.h"
 #include "components/prefs/mock_pref_change_callback.h"
+#include "extensions/browser/api/content_settings/content_settings_service.h"
 #include "extensions/browser/extension_prefs.h"
+#include "extensions/browser/extension_prefs_helper.h"
+#include "extensions/common/api/types.h"
 #include "extensions/common/extension.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using base::Value;
+using extensions::api::types::ChromeSettingScope;
 
 namespace extensions {
 
@@ -38,35 +41,6 @@ const char kDefaultPref4[] = "default pref 4";
 
 }  // namespace
 
-// An implementation of the PreferenceAPI which returns the ExtensionPrefs and
-// ExtensionPrefValueMap from the TestExtensionPrefs, rather than from a
-// profile (which we don't create in unittests).
-class TestPreferenceAPI : public PreferenceAPIBase {
- public:
-  explicit TestPreferenceAPI(TestExtensionPrefs* test_extension_prefs,
-                             ContentSettingsService* content_settings)
-      : test_extension_prefs_(test_extension_prefs),
-        content_settings_(content_settings) {}
-  ~TestPreferenceAPI() {}
-
- private:
-  // PreferenceAPIBase implementation.
-  ExtensionPrefs* extension_prefs() override {
-    return test_extension_prefs_->prefs();
-  }
-  ExtensionPrefValueMap* extension_pref_value_map() override {
-    return test_extension_prefs_->extension_pref_value_map();
-  }
-  scoped_refptr<ContentSettingsStore> content_settings_store() override {
-    return content_settings_->content_settings_store();
-  }
-
-  TestExtensionPrefs* test_extension_prefs_;
-  ContentSettingsService* content_settings_;
-
-  DISALLOW_COPY_AND_ASSIGN(TestPreferenceAPI);
-};
-
 class ExtensionControlledPrefsTest : public PrefsPrepopulatedTestBase {
  public:
   ExtensionControlledPrefsTest();
@@ -95,14 +69,14 @@ class ExtensionControlledPrefsTest : public PrefsPrepopulatedTestBase {
   void EnsureExtensionUninstalled(const std::string& extension_id);
 
   TestingProfile profile_;
-  ContentSettingsService* content_settings_;
-  TestPreferenceAPI test_preference_api_;
+  raw_ptr<ContentSettingsService> content_settings_;
+  ExtensionPrefsHelper prefs_helper_;
 };
 
 ExtensionControlledPrefsTest::ExtensionControlledPrefsTest()
     : PrefsPrepopulatedTestBase(),
       content_settings_(ContentSettingsService::Get(&profile_)),
-      test_preference_api_(&prefs_, content_settings_) {
+      prefs_helper_(prefs_.prefs(), prefs_.extension_pref_value_map()) {
   content_settings_->OnExtensionPrefsAvailable(prefs_.prefs());
 }
 
@@ -122,8 +96,8 @@ void ExtensionControlledPrefsTest::InstallExtensionControlledPref(
     const std::string& key,
     base::Value value) {
   EnsureExtensionInstalled(extension);
-  test_preference_api_.SetExtensionControlledPref(
-      extension->id(), key, kExtensionPrefsScopeRegular, std::move(value));
+  prefs_helper_.SetExtensionControlledPref(
+      extension->id(), key, ChromeSettingScope::kRegular, std::move(value));
 }
 
 void ExtensionControlledPrefsTest::InstallExtensionControlledPrefIncognito(
@@ -131,8 +105,8 @@ void ExtensionControlledPrefsTest::InstallExtensionControlledPrefIncognito(
     const std::string& key,
     base::Value value) {
   EnsureExtensionInstalled(extension);
-  test_preference_api_.SetExtensionControlledPref(
-      extension->id(), key, kExtensionPrefsScopeIncognitoPersistent,
+  prefs_helper_.SetExtensionControlledPref(
+      extension->id(), key, ChromeSettingScope::kIncognitoPersistent,
       std::move(value));
 }
 
@@ -141,8 +115,8 @@ void ExtensionControlledPrefsTest::
                                                        const std::string& key,
                                                        base::Value value) {
   EnsureExtensionInstalled(extension);
-  test_preference_api_.SetExtensionControlledPref(
-      extension->id(), key, kExtensionPrefsScopeIncognitoSessionOnly,
+  prefs_helper_.SetExtensionControlledPref(
+      extension->id(), key, ChromeSettingScope::kIncognitoSessionOnly,
       std::move(value));
 }
 
@@ -183,7 +157,8 @@ void ExtensionControlledPrefsTest::EnsureExtensionUninstalled(
       break;
     }
   }
-  prefs()->OnExtensionUninstalled(extension_id, Manifest::INTERNAL, false);
+  prefs()->OnExtensionUninstalled(extension_id,
+                                  mojom::ManifestLocation::kInternal, false);
 }
 
 class ControlledPrefsInstallOneExtension
@@ -273,7 +248,7 @@ class ControlledPrefsUninstallExtension : public ExtensionControlledPrefsTest {
         ContentSettingsPattern::FromString("http://[*.]example.com");
     store->SetExtensionContentSetting(
         extension1()->id(), pattern, pattern, ContentSettingsType::IMAGES,
-        CONTENT_SETTING_BLOCK, kExtensionPrefsScopeRegular);
+        CONTENT_SETTING_BLOCK, ChromeSettingScope::kRegular);
 
     UninstallExtension(extension1()->id());
   }
