@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,10 +9,8 @@
 #include <string>
 #include <utility>
 
-#include "base/macros.h"
+#include "base/memory/raw_ptr.h"
 #include "base/scoped_observation.h"
-#include "content/public/browser/notification_observer.h"
-#include "content/public/browser/notification_registrar.h"
 #include "extensions/browser/api/runtime/runtime_api.h"
 #include "extensions/browser/api/runtime/runtime_api_delegate.h"
 #include "extensions/browser/extension_registry.h"
@@ -21,24 +19,25 @@
 namespace base {
 class TickClock;
 class TimeTicks;
-}
+}  // namespace base
 
 namespace content {
 class BrowserContext;
-class NotificationDetails;
-class NotificationSource;
-}
+}  // namespace content
 
 namespace extensions {
 class RuntimeAPI;
 class UpdateObserver;
-}
+}  // namespace extensions
 
 class ChromeRuntimeAPIDelegate : public extensions::RuntimeAPIDelegate,
-                                 public content::NotificationObserver,
                                  public extensions::ExtensionRegistryObserver {
  public:
   explicit ChromeRuntimeAPIDelegate(content::BrowserContext* context);
+
+  ChromeRuntimeAPIDelegate(const ChromeRuntimeAPIDelegate&) = delete;
+  ChromeRuntimeAPIDelegate& operator=(const ChromeRuntimeAPIDelegate&) = delete;
+
   ~ChromeRuntimeAPIDelegate() override;
 
   // Sets a custom TickClock to use in tests.
@@ -59,23 +58,18 @@ class ChromeRuntimeAPIDelegate : public extensions::RuntimeAPIDelegate,
   bool OpenOptionsPage(const extensions::Extension* extension,
                        content::BrowserContext* browser_context) override;
 
-  // content::NotificationObserver implementation.
-  void Observe(int type,
-               const content::NotificationSource& source,
-               const content::NotificationDetails& details) override;
-
   // ExtensionRegistryObserver implementation.
   void OnExtensionInstalled(content::BrowserContext* browser_context,
                             const extensions::Extension* extension,
                             bool is_update) override;
 
+  void OnExtensionUpdateFound(const std::string& id,
+                              const base::Version& version);
   void UpdateCheckComplete(const std::string& extension_id);
   void CallUpdateCallbacks(const std::string& extension_id,
                            const UpdateCheckResult& result);
 
-  content::BrowserContext* browser_context_;
-
-  content::NotificationRegistrar registrar_;
+  raw_ptr<content::BrowserContext> browser_context_;
 
   // Whether the API registered with the ExtensionService to receive
   // update notifications.
@@ -84,7 +78,7 @@ class ChromeRuntimeAPIDelegate : public extensions::RuntimeAPIDelegate,
   // Map to prevent extensions from getting stuck in reload loops. Maps
   // extension id to the last time it was reloaded and the number of times
   // it was reloaded with not enough time in between reloads.
-  std::map<std::string, std::pair<base::TimeTicks, int> > last_reload_time_;
+  std::map<std::string, std::pair<base::TimeTicks, int>> last_reload_time_;
 
   // Information about update checks, keyed by extension id.
   struct UpdateCheckInfo;
@@ -93,8 +87,6 @@ class ChromeRuntimeAPIDelegate : public extensions::RuntimeAPIDelegate,
   base::ScopedObservation<extensions::ExtensionRegistry,
                           extensions::ExtensionRegistryObserver>
       extension_registry_observation_{this};
-
-  DISALLOW_COPY_AND_ASSIGN(ChromeRuntimeAPIDelegate);
 };
 
 #endif  // CHROME_BROWSER_EXTENSIONS_API_RUNTIME_CHROME_RUNTIME_API_DELEGATE_H_
