@@ -1,13 +1,14 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef CHROME_BROWSER_MEDIA_WEBRTC_MEDIA_AUTHORIZATION_WRAPPER_MAC_H_
 #define CHROME_BROWSER_MEDIA_WEBRTC_MEDIA_AUTHORIZATION_WRAPPER_MAC_H_
 
-#import <Foundation/NSString.h>
+#import <AVFoundation/AVFoundation.h>
+#import <Foundation/Foundation.h>
 
-#include "base/callback_forward.h"
+#include "base/functional/callback_forward.h"
 
 namespace system_media_permissions {
 
@@ -15,12 +16,10 @@ class MediaAuthorizationWrapper {
  public:
   virtual ~MediaAuthorizationWrapper() {}
 
-  // NB: NSInteger is used rather than AVAuthorizationStatus; when macOS 10.14
-  // is the minimum requirement for Chromium, switch types.
-  virtual NSInteger AuthorizationStatusForMediaType(NSString* media_type) = 0;
+  virtual AVAuthorizationStatus AuthorizationStatusForMediaType(
+      NSString* media_type) = 0;
   virtual void RequestAccessForMediaType(NSString* media_type,
-                                         base::RepeatingClosure callback,
-                                         const base::TaskTraits& traits) = 0;
+                                         base::OnceClosure callback) = 0;
 };
 
 }  // namespace system_media_permissions
