@@ -6,6 +6,7 @@
 #define CHROME_BROWSER_MEDIA_WEBRTC_WEBRTC_EVENT_LOG_MANAGER_REMOTE_H_
 
 #include <map>
+#include <optional>
 #include <set>
 #include <string>
 #include <vector>
@@ -19,7 +20,6 @@
 #include "chrome/browser/media/webrtc/webrtc_event_log_uploader.h"
 #include "components/upload_list/upload_list.h"
 #include "services/network/public/cpp/network_connection_tracker.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace webrtc_event_logging {
 
@@ -285,7 +285,7 @@ class WebRtcRemoteEventLogManager final
   // this check is not too expensive.
   // If a |browser_context_id| is provided, logs are only pruned for it.
   void PrunePendingLogs(
-      absl::optional<BrowserContextId> browser_context_id = absl::nullopt);
+      std::optional<BrowserContextId> browser_context_id = std::nullopt);
 
   // PrunePendingLogs() and schedule the next proactive pending logs prune.
   void RecurringlyPrunePendingLogs();
@@ -314,7 +314,7 @@ class WebRtcRemoteEventLogManager final
   void MaybeRemovePendingLogs(
       const base::Time& delete_begin,
       const base::Time& delete_end,
-      absl::optional<BrowserContextId> browser_context_id,
+      std::optional<BrowserContextId> browser_context_id,
       bool is_cache_clear);
 
   // Remove all history files associated with |browser_context_id| which were
@@ -348,7 +348,7 @@ class WebRtcRemoteEventLogManager final
   //   can match the filter.
   bool MatchesFilter(BrowserContextId log_browser_context_id,
                      const base::Time& log_last_modification,
-                     absl::optional<BrowserContextId> filter_browser_context_id,
+                     std::optional<BrowserContextId> filter_browser_context_id,
                      const base::Time& filter_range_begin,
                      const base::Time& filter_range_end) const;
 
