@@ -21,7 +21,7 @@
 #include "base/threading/scoped_blocking_call.h"
 #include "build/build_config.h"
 #include "chrome/browser/spellchecker/spellcheck_service.h"
-#ifndef TOOLKIT_QT
+#if !BUILDFLAG(IS_QTWEBENGINE)
 #include "chrome/common/chrome_paths.h"
 #endif
 #include "components/spellcheck/browser/spellcheck_platform.h"
@@ -247,7 +247,6 @@ void SpellcheckHunspellDictionary::OnSimpleLoaderComplete(
 #if !BUILDFLAG(IS_ANDROID)
   // To prevent corrupted dictionary data from causing a renderer crash, scan
   // the dictionary data and verify it is sane before save it to a file.
-  // TODO(rlp): Adding metrics to RecordDictionaryCorruptionStats
   if (!hunspell::BDict::Verify(data->data(), data->size())) {
     // Let PostTaskAndReply caller send to InformListenersOfInitialization
     // through SaveDictionaryDataComplete().
@@ -379,7 +378,7 @@ SpellcheckHunspellDictionary::OpenDictionaryFile(base::TaskRunner* task_runner,
     dictionary.file.Initialize(dictionary.path,
                                base::File::FLAG_READ | base::File::FLAG_OPEN);
   } else {
-#ifndef TOOLKIT_QT
+#if !BUILDFLAG(IS_QTWEBENGINE)
     base::DeleteFile(dictionary.path);
 #endif
   }
@@ -412,7 +411,7 @@ void SpellcheckHunspellDictionary::InitializeDictionaryLocationComplete(
     DictionaryFile file) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   dictionary_file_ = std::move(file);
-#ifndef TOOLKIT_QT
+#if !BUILDFLAG(IS_QTWEBENGINE)
   if (!dictionary_file_.file.IsValid()) {
     // Notify browser tests that this dictionary is corrupted. Skip downloading
     // the dictionary in browser tests.
@@ -437,7 +436,7 @@ void SpellcheckHunspellDictionary::InitializeDictionaryLocationComplete(
       InformListenersOfDownloadFailure();
   else
       InformListenersOfInitialization();
-#endif
+#endif  // !BUILDFLAG(IS_QTWEBENGINE)
 }
 #endif  // !BUILDFLAG(IS_ANDROID)
 
