@@ -1,31 +1,33 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "base/callback_helpers.h"
 #include "base/command_line.h"
-#include "base/macros.h"
+#include "base/functional/callback_helpers.h"
+#include "base/memory/raw_ptr.h"
+#include "base/strings/strcat.h"
+#include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/bind.h"
 #include "chrome/app/chrome_command_ids.h"
-#include "chrome/browser/chrome_notification_types.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_commands.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
+#include "chrome/common/chrome_features.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/test/base/in_process_browser_test.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "content/public/browser/back_forward_cache.h"
 #include "content/public/browser/navigation_entry.h"
 #include "content/public/browser/notification_service.h"
-#include "content/public/browser/notification_types.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/browser_test_utils.h"
+#include "content/public/test/prerender_test_util.h"
 #include "content/public/test/test_navigation_observer.h"
 #include "content/public/test/test_utils.h"
 #include "content/public/test/url_loader_interceptor.h"
@@ -49,7 +51,12 @@ const char kTestMedia[] = "/media/pink_noise_140ms.wav";
 
 class ViewSourceTest : public InProcessBrowserTest {
  public:
-  ViewSourceTest() {}
+  ViewSourceTest() {
+    feature_list_.InitAndDisableFeature(features::kHttpsUpgrades);
+  }
+
+  ViewSourceTest(const ViewSourceTest&) = delete;
+  ViewSourceTest& operator=(const ViewSourceTest&) = delete;
 
  protected:
   void SetUpOnMainThread() override {
@@ -57,13 +64,20 @@ class ViewSourceTest : public InProcessBrowserTest {
     host_resolver()->AddRule("*", "127.0.0.1");
   }
 
+  base::test::ScopedFeatureList* feature_list() { return &feature_list_; }
+
  private:
-  DISALLOW_COPY_AND_ASSIGN(ViewSourceTest);
+  base::test::ScopedFeatureList feature_list_;
 };
 
-class ViewSourceFeaturePolicyTest : public ViewSourceTest {
+class ViewSourcePermissionsPolicyTest : public ViewSourceTest {
  public:
-  ViewSourceFeaturePolicyTest() : ViewSourceTest() {}
+  ViewSourcePermissionsPolicyTest() : ViewSourceTest() {}
+
+  ViewSourcePermissionsPolicyTest(const ViewSourcePermissionsPolicyTest&) =
+      delete;
+  ViewSourcePermissionsPolicyTest& operator=(
+      const ViewSourcePermissionsPolicyTest&) = delete;
 
  protected:
   void SetUpOnMainThread() override {
@@ -75,9 +89,6 @@ class ViewSourceFeaturePolicyTest : public ViewSourceTest {
     command_line->AppendSwitch(
         switches::kEnableExperimentalWebPlatformFeatures);
   }
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(ViewSourceFeaturePolicyTest);
 };
 
 // This test renders a page in view-source and then checks to see if the title
@@ -90,11 +101,11 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, DoesBrowserRenderInViewSource) {
   // First we navigate to our view-source test page.
   GURL url(content::kViewSourceScheme + std::string(":") +
            embedded_test_server()->GetURL(kTestHtml).spec());
-  ui_test_utils::NavigateToURL(browser(), url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
 
   // Check that the title didn't get set.  It should not be there (because we
   // are in view-source mode).
-  EXPECT_NE(base::ASCIIToUTF16("foo"),
+  EXPECT_NE(u"foo",
             browser()->tab_strip_model()->GetActiveWebContents()->GetTitle());
 }
 
@@ -107,12 +118,12 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, DoesBrowserConsumeViewSourcePrefix) {
 
   // First we navigate to google.html.
   GURL url(embedded_test_server()->GetURL(kTestHtml));
-  ui_test_utils::NavigateToURL(browser(), url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
 
   // Then we navigate to the same url but with the "view-source:" prefix.
   GURL url_viewsource(content::kViewSourceScheme + std::string(":") +
                       url.spec());
-  ui_test_utils::NavigateToURL(browser(), url_viewsource);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url_viewsource));
 
   // The URL should still be prefixed with "view-source:".
   EXPECT_EQ(url_viewsource.spec(),
@@ -126,7 +137,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, ViewSourceInMenuEnabledOnANormalPage) {
   ASSERT_TRUE(embedded_test_server()->Start());
 
   GURL url(embedded_test_server()->GetURL(kTestHtml));
-  ui_test_utils::NavigateToURL(browser(), url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
 
   EXPECT_TRUE(chrome::CanViewSource(browser()));
 }
@@ -137,7 +148,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, ViewSourceInMenuDisabledOnAMediaPage) {
   ASSERT_TRUE(embedded_test_server()->Start());
 
   GURL url(embedded_test_server()->GetURL(kTestMedia));
-  ui_test_utils::NavigateToURL(browser(), url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
 
   const char* mime_type = browser()->tab_strip_model()->GetActiveWebContents()->
       GetContentsMimeType().c_str();
@@ -154,7 +165,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
 
   GURL url_viewsource(content::kViewSourceScheme + std::string(":") +
                       embedded_test_server()->GetURL(kTestHtml).spec());
-  ui_test_utils::NavigateToURL(browser(), url_viewsource);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url_viewsource));
 
   EXPECT_FALSE(chrome::CanViewSource(browser()));
 }
@@ -168,19 +179,16 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, DISABLED_TestViewSourceReload) {
   GURL url_viewsource(content::kViewSourceScheme + std::string(":") +
                       embedded_test_server()->GetURL(kTestHtml).spec());
 
-  content::WindowedNotificationObserver observer(
-      content::NOTIFICATION_LOAD_STOP,
-      content::NotificationService::AllSources());
-  ui_test_utils::NavigateToURL(browser(), url_viewsource);
+  content::LoadStopObserver observer(
+      browser()->tab_strip_model()->GetActiveWebContents());
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url_viewsource));
   observer.Wait();
 
-  ASSERT_TRUE(
-      content::ExecuteScript(browser()->tab_strip_model()->GetWebContentsAt(0),
-                             "window.location.reload();"));
+  ASSERT_TRUE(content::ExecJs(browser()->tab_strip_model()->GetWebContentsAt(0),
+                              "window.location.reload();"));
 
-  content::WindowedNotificationObserver observer2(
-      content::NOTIFICATION_LOAD_STOP,
-      content::NotificationService::AllSources());
+  content::LoadStopObserver observer2(
+      browser()->tab_strip_model()->GetWebContentsAt(0));
   observer2.Wait();
   ASSERT_TRUE(browser()->tab_strip_model()->GetWebContentsAt(0)->
                   GetController().GetActiveEntry()->IsViewSourceMode());
@@ -194,7 +202,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
 
   GURL url_viewsource(content::kViewSourceScheme + std::string(":") +
                       embedded_test_server()->GetURL(kTestHtml).spec());
-  ui_test_utils::NavigateToURL(browser(), url_viewsource);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url_viewsource));
   EXPECT_FALSE(chrome::CanViewSource(browser()));
   EXPECT_EQ(1, browser()->tab_strip_model()->count());
 
@@ -202,20 +210,22 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
   // the original tab is navigated cross-process. This is required for the
   // original bug to reproduce.
   {
-    GURL url = embedded_test_server()->GetURL("/title1.html");
+    GURL url = embedded_test_server()->GetURL("a.com", "/title1.html");
     ui_test_utils::UrlLoadObserver load_complete(
         url, content::NotificationService::AllSources());
-    EXPECT_TRUE(content::ExecuteScript(
-        browser()->tab_strip_model()->GetActiveWebContents(),
-        "window.open('" + url.spec() + "');"));
+    EXPECT_TRUE(
+        content::ExecJs(browser()->tab_strip_model()->GetActiveWebContents(),
+                        "window.open('" + url.spec() + "');"));
     load_complete.Wait();
     EXPECT_EQ(2, browser()->tab_strip_model()->count());
   }
 
   // Switch back to the first tab and navigate it cross-process.
   browser()->tab_strip_model()->ActivateTabAt(
-      0, {TabStripModel::GestureType::kOther});
-  ui_test_utils::NavigateToURL(browser(), GURL(chrome::kChromeUIVersionURL));
+      0, TabStripUserGestureDetails(
+             TabStripUserGestureDetails::GestureType::kOther));
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(
+      browser(), embedded_test_server()->GetURL("b.com", "/title1.html")));
   EXPECT_TRUE(chrome::CanViewSource(browser()));
 
   // Navigate back in session history to ensure view-source mode is still
@@ -230,12 +240,9 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
   // Check whether the page is in view-source mode or not by checking if an
   // expected element on the page exists or not. In view-source mode it
   // should not be found.
-  bool result = false;
-  EXPECT_TRUE(content::ExecuteScriptAndExtractBool(
-      browser()->tab_strip_model()->GetActiveWebContents(),
-      "domAutomationController.send(document.getElementById('bar') === null);",
-      &result));
-  EXPECT_TRUE(result);
+  EXPECT_EQ(true, content::EvalJs(
+                      browser()->tab_strip_model()->GetActiveWebContents(),
+                      "document.getElementById('bar') === null;"));
   EXPECT_FALSE(chrome::CanViewSource(browser()));
 }
 
@@ -247,21 +254,22 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, CrossSiteSubframe) {
   ASSERT_TRUE(embedded_test_server()->Start());
   GURL main_url(
       embedded_test_server()->GetURL("a.com", "/iframe_cross_site.html"));
-  ui_test_utils::NavigateToURL(browser(), main_url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), main_url));
 
   // Grab the original frames.
   content::WebContents* original_contents =
       browser()->tab_strip_model()->GetActiveWebContents();
   content::RenderFrameHost* original_main_frame =
-      original_contents->GetMainFrame();
-  ASSERT_LE(2u, original_contents->GetAllFrames().size());
+      original_contents->GetPrimaryMainFrame();
   content::RenderFrameHost* original_child_frame =
-      original_contents->GetAllFrames()[1];
+      ChildFrameAt(original_main_frame, 0);
+  ASSERT_TRUE(original_child_frame);
 
   // Do a sanity check that in this particular test page the main frame and the
   // subframe are cross-site.
-  EXPECT_NE(original_main_frame->GetLastCommittedURL().GetOrigin(),
-            original_child_frame->GetLastCommittedURL().GetOrigin());
+  EXPECT_NE(
+      original_main_frame->GetLastCommittedURL().DeprecatedGetOriginAsURL(),
+      original_child_frame->GetLastCommittedURL().DeprecatedGetOriginAsURL());
   if (content::AreAllSitesIsolatedForTesting()) {
     EXPECT_NE(original_main_frame->GetSiteInstance(),
               original_child_frame->GetSiteInstance());
@@ -279,7 +287,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, CrossSiteSubframe) {
   content::WebContents* view_source_contents =
       view_source_contents_observer.GetWebContents();
   content::RenderFrameHost* view_source_frame =
-      view_source_contents->GetMainFrame();
+      view_source_contents->GetPrimaryMainFrame();
   EXPECT_TRUE(WaitForLoadStop(view_source_contents));
 
   // Verify that the last committed URL is the same in the original and the
@@ -300,17 +308,15 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, CrossSiteSubframe) {
             view_source_frame->GetSiteInstance());
 
   // Verify the contents of the view-source tab (should match title1.html).
-  std::string source_text;
   std::string view_source_extraction_script = R"(
       output = "";
       document.querySelectorAll(".line-content").forEach(function(elem) {
           output += elem.innerText;
       });
-      domAutomationController.send(output); )";
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      view_source_contents, view_source_extraction_script, &source_text));
-  EXPECT_EQ("<html><head></head><body>This page has no title.</body></html>",
-            source_text);
+      output; )";
+  EXPECT_EQ(
+      "<html><head></head><body>This page has no title.</body></html>",
+      content::EvalJs(view_source_contents, view_source_extraction_script));
 
   // Verify the title is derived from the subframe URL.
   GURL original_url = original_child_frame->GetLastCommittedURL();
@@ -329,21 +335,21 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, HttpPostInMainframe) {
   ASSERT_TRUE(embedded_test_server()->Start());
   GURL form_url(embedded_test_server()->GetURL(
       "a.com", "/form_that_posts_to_echoall.html"));
-  ui_test_utils::NavigateToURL(browser(), form_url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), form_url));
   content::WebContents* original_contents =
       browser()->tab_strip_model()->GetActiveWebContents();
   content::RenderFrameHost* original_main_frame =
-      original_contents->GetMainFrame();
+      original_contents->GetPrimaryMainFrame();
 
   // Submit the form and verify that we arrived at the expected location.
   content::TestNavigationObserver form_post_observer(original_contents, 1);
-  EXPECT_TRUE(ExecuteScript(original_main_frame,
-                            "document.getElementById('form').submit();"));
+  EXPECT_TRUE(
+      ExecJs(original_main_frame, "document.getElementById('form').submit();"));
   form_post_observer.Wait();
   GURL target_url(embedded_test_server()->GetURL("a.com", "/echoall"));
 
   content::RenderFrameHost* current_main_frame =
-      original_contents->GetMainFrame();
+      original_contents->GetPrimaryMainFrame();
   if (content::CanSameSiteMainFrameNavigationsChangeRenderFrameHosts()) {
     // When ProactivelySwapBrowsingInstance or RenderDocument is enabled on
     // same-site main frame navigations, the form submission above will result
@@ -355,12 +361,11 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, HttpPostInMainframe) {
   EXPECT_EQ(target_url, current_main_frame->GetLastCommittedURL());
 
   // Extract the response nonce.
-  std::string response_nonce;
-  std::string response_nonce_extraction_script = R"(
-      domAutomationController.send(
-          document.getElementById('response-nonce').innerText); )";
-  EXPECT_TRUE(ExecuteScriptAndExtractString(
-      current_main_frame, response_nonce_extraction_script, &response_nonce));
+  std::string response_nonce_extraction_script =
+      "document.getElementById('response-nonce').innerText;";
+  std::string response_nonce =
+      EvalJs(current_main_frame, response_nonce_extraction_script)
+          .ExtractString();
 
   // Open view-source mode tab for the main frame.  This tries to mimic the
   // behavior of RenderViewContextMenu::ExecuteCommand when it handles
@@ -376,15 +381,15 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, HttpPostInMainframe) {
   // 2) the sources should contain the original response-nonce
   //    (i.e. no new network request should be made - the data should be
   //    retrieved from the cache)
-  std::string source_text;
   std::string view_source_extraction_script = R"(
       output = "";
       document.querySelectorAll(".line-content").forEach(function(elem) {
           output += elem.innerText;
       });
-      domAutomationController.send(output); )";
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      view_source_contents, view_source_extraction_script, &source_text));
+      output; )";
+  std::string source_text =
+      content::EvalJs(view_source_contents, view_source_extraction_script)
+          .ExtractString();
   EXPECT_THAT(source_text,
               HasSubstr("<h1>Request Body:</h1><pre>text=value</pre>"));
   EXPECT_THAT(source_text,
@@ -404,7 +409,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, HttpPostInMainframe) {
   // Verify that the original contents and the view-source contents are in a
   // different process - see https://crbug.com/699493.
   EXPECT_NE(current_main_frame->GetSiteInstance(),
-            view_source_contents->GetMainFrame()->GetSiteInstance());
+            view_source_contents->GetPrimaryMainFrame()->GetSiteInstance());
 
   // Verify the title of view-source is derived from the URL (not from the title
   // of the original contents).
@@ -436,7 +441,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
   // Navigate to a URL, it doesn't matter which, just need the tab to have a
   // committed entry other than about:blank or the NTP.
   GURL url(embedded_test_server()->GetURL(kTestHtml));
-  ui_test_utils::NavigateToURL(browser(), url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
   EXPECT_EQ(0, browser()->tab_strip_model()->active_index());
 
   // Duplicate the tab. The newly created tab should be active.
@@ -452,7 +457,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
   EXPECT_EQ(GURL(), browser()
                         ->tab_strip_model()
                         ->GetActiveWebContents()
-                        ->GetMainFrame()
+                        ->GetPrimaryMainFrame()
                         ->GetLastCommittedURL());
 
   // Open a view source tab, and watch for its main network request.
@@ -461,7 +466,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
   browser()
       ->tab_strip_model()
       ->GetActiveWebContents()
-      ->GetMainFrame()
+      ->GetPrimaryMainFrame()
       ->ViewSource();
   content::WebContents* view_source_contents =
       view_source_contents_observer.GetWebContents();
@@ -469,9 +474,15 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
   GURL view_source_url(content::kViewSourceScheme + std::string(":") +
                        url.spec());
   EXPECT_EQ(view_source_url, view_source_contents->GetLastCommittedURL());
+  // Make sure that the navigation type reported is "back_forward" on the
+  // duplicated tab.
+  EXPECT_EQ(
+      "back_forward",
+      content::EvalJs(browser()->tab_strip_model()->GetActiveWebContents(),
+                      "performance.getEntriesByType('navigation')[0].type"));
 
   // Verify the request for the view-source tab had the correct IsolationInfo.
-  base::Optional<network::ResourceRequest> request =
+  std::optional<network::ResourceRequest> request =
       loader_monitor.GetRequestInfo(url);
   ASSERT_TRUE(request);
   ASSERT_TRUE(request->trusted_params);
@@ -479,8 +490,7 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest,
   EXPECT_TRUE(request->trusted_params->isolation_info.IsEqualForTesting(
       net::IsolationInfo::Create(net::IsolationInfo::RequestType::kMainFrame,
                                  origin, origin,
-                                 net::SiteForCookies::FromOrigin(origin),
-                                 std::set<net::SchemefulSite>())));
+                                 net::SiteForCookies::FromOrigin(origin))));
 }
 
 class ViewSourceWithSplitCacheTest
@@ -488,15 +498,23 @@ class ViewSourceWithSplitCacheTest
       public ::testing::WithParamInterface<bool> {
  public:
   void SetUp() override {
+    std::vector<base::test::FeatureRef> enabled_features;
+    std::vector<base::test::FeatureRef> disabled_features;
+
     bool split_cache_by_network_isolation_key = GetParam();
-    feature_list_.InitWithFeatureState(
-        net::features::kSplitCacheByNetworkIsolationKey,
-        split_cache_by_network_isolation_key);
+    if (split_cache_by_network_isolation_key) {
+      enabled_features.push_back(
+          net::features::kSplitCacheByNetworkIsolationKey);
+    } else {
+      disabled_features.push_back(
+          net::features::kSplitCacheByNetworkIsolationKey);
+    }
+    disabled_features.push_back(features::kHttpsUpgrades);
+    feature_list()->Reset();
+    feature_list()->InitWithFeatures(enabled_features, disabled_features);
+
     ViewSourceTest::SetUp();
   }
-
- private:
-  base::test::ScopedFeatureList feature_list_;
 };
 
 // Tests that "View Source" works fine for *subframes* shown via HTTP POST.
@@ -507,7 +525,7 @@ IN_PROC_BROWSER_TEST_P(ViewSourceWithSplitCacheTest, HttpPostInSubframe) {
   ASSERT_TRUE(embedded_test_server()->Start());
   GURL main_url(
       embedded_test_server()->GetURL("a.com", "/iframe_cross_site.html"));
-  ui_test_utils::NavigateToURL(browser(), main_url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), main_url));
   content::WebContents* original_contents =
       browser()->tab_strip_model()->GetActiveWebContents();
 
@@ -516,25 +534,26 @@ IN_PROC_BROWSER_TEST_P(ViewSourceWithSplitCacheTest, HttpPostInSubframe) {
       "b.com", "/form_that_posts_to_echoall.html"));
   EXPECT_TRUE(
       content::NavigateIframeToURL(original_contents, "frame1", form_url));
-  EXPECT_LE(2u, original_contents->GetAllFrames().size());
   content::RenderFrameHost* original_child_frame =
-      original_contents->GetAllFrames()[1];
+      ChildFrameAt(original_contents, 0);
+  ASSERT_TRUE(original_child_frame);
 
   // Submit the form and verify that we arrived at the expected location.
   content::TestNavigationObserver form_post_observer(original_contents, 1);
-  EXPECT_TRUE(ExecuteScript(original_child_frame,
-                            "document.getElementById('form').submit();"));
+  EXPECT_TRUE(ExecJs(original_child_frame,
+                     "document.getElementById('form').submit();"));
   form_post_observer.Wait();
+  original_child_frame = ChildFrameAt(original_contents, 0);
+
   GURL target_url(embedded_test_server()->GetURL("b.com", "/echoall"));
   EXPECT_EQ(target_url, original_child_frame->GetLastCommittedURL());
 
   // Extract the response nonce.
-  std::string response_nonce;
-  std::string response_nonce_extraction_script = R"(
-      domAutomationController.send(
-          document.getElementById('response-nonce').innerText); )";
-  EXPECT_TRUE(ExecuteScriptAndExtractString(
-      original_child_frame, response_nonce_extraction_script, &response_nonce));
+  std::string response_nonce_extraction_script =
+      "document.getElementById('response-nonce').innerText;";
+  std::string response_nonce =
+      EvalJs(original_child_frame, response_nonce_extraction_script)
+          .ExtractString();
 
   // Open view-source mode tab for the subframe.  This tries to mimic the
   // behavior of RenderViewContextMenu::ExecuteCommand when it handles
@@ -550,15 +569,15 @@ IN_PROC_BROWSER_TEST_P(ViewSourceWithSplitCacheTest, HttpPostInSubframe) {
   // 2) the sources should contain the original response-nonce
   //    (i.e. no new network request should be made - the data should be
   //    retrieved from the cache)
-  std::string source_text;
   std::string view_source_extraction_script = R"(
       output = "";
       document.querySelectorAll(".line-content").forEach(function(elem) {
           output += elem.innerText;
       });
-      domAutomationController.send(output); )";
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      view_source_contents, view_source_extraction_script, &source_text));
+      output; )";
+  std::string source_text =
+      content::EvalJs(view_source_contents, view_source_extraction_script)
+          .ExtractString();
   EXPECT_THAT(source_text,
               HasSubstr("<h1>Request Body:</h1><pre>text=value</pre>"));
   EXPECT_THAT(source_text,
@@ -577,9 +596,9 @@ IN_PROC_BROWSER_TEST_P(ViewSourceWithSplitCacheTest, HttpPostInSubframe) {
 
   // Verify that view-source opens in a new process - https://crbug.com/699493.
   EXPECT_NE(original_child_frame->GetSiteInstance(),
-            view_source_contents->GetMainFrame()->GetSiteInstance());
+            view_source_contents->GetPrimaryMainFrame()->GetSiteInstance());
   EXPECT_NE(original_contents->GetSiteInstance(),
-            view_source_contents->GetMainFrame()->GetSiteInstance());
+            view_source_contents->GetPrimaryMainFrame()->GetSiteInstance());
 
   // Verify the title is derived from the URL.
   GURL original_url = original_child_frame->GetLastCommittedURL();
@@ -617,7 +636,7 @@ IN_PROC_BROWSER_TEST_P(ViewSourceWithSplitCacheEnabledTest,
 
   // 1. Navigate to page a.com/title1.html
   GURL main_url(embedded_test_server()->GetURL("a.com", "/title1.html"));
-  ui_test_utils::NavigateToURL(browser(), main_url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), main_url));
 
   content::WebContents* original_contents =
       browser()->tab_strip_model()->GetActiveWebContents();
@@ -632,14 +651,14 @@ IN_PROC_BROWSER_TEST_P(ViewSourceWithSplitCacheEnabledTest,
         "document.body.appendChild(frame);",
         subframe_url.c_str());
     content::TestNavigationObserver navigation_observer(original_contents);
-    original_contents->GetMainFrame()->ExecuteJavaScriptForTests(
+    original_contents->GetPrimaryMainFrame()->ExecuteJavaScriptForTests(
         base::ASCIIToUTF16(create_frame_script), base::NullCallback());
     navigation_observer.Wait();
   }
 
   // 3. View-source the subframe
   content::WebContentsAddedObserver view_source_contents_observer;
-  original_contents->GetAllFrames()[1]->ViewSource();
+  ChildFrameAt(original_contents, 0)->ViewSource();
   content::WebContents* view_source_contents =
       view_source_contents_observer.GetWebContents();
   EXPECT_TRUE(WaitForLoadStop(view_source_contents));
@@ -649,11 +668,12 @@ IN_PROC_BROWSER_TEST_P(ViewSourceWithSplitCacheEnabledTest,
   // |view_source_contents| to make sure it will not be put into the back
   // forward cache.
   view_source_contents->GetController().GetBackForwardCache().DisableForTesting(
-      content::BackForwardCache::TEST_ASSUMES_NO_CACHING);
+      content::BackForwardCache::TEST_REQUIRES_NO_CACHING);
 
   // 4. Navigate the view-source page to a c.com/title1.html
-  ui_test_utils::NavigateToURL(
-      browser(), GURL(embedded_test_server()->GetURL("c.com", "/title1.html")));
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(
+      browser(),
+      GURL(embedded_test_server()->GetURL("c.com", "/title1.html"))));
 
   base::RunLoop cache_status_waiter;
   content::URLLoaderInterceptor interceptor(
@@ -695,23 +715,18 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, NavigationOmitsReferrer) {
 
   GURL url(content::kViewSourceScheme + std::string(":") +
            embedded_test_server()->GetURL(kTestNavigationHtml).spec());
-  ui_test_utils::NavigateToURL(browser(), url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
 
   // Click the first link in the view-source markup.
   content::WebContentsAddedObserver nav_observer;
-  EXPECT_TRUE(content::ExecuteScript(
-      browser()->tab_strip_model()->GetActiveWebContents(),
-      "document.getElementsByTagName('A')[0].click();"));
+  EXPECT_TRUE(
+      content::ExecJs(browser()->tab_strip_model()->GetActiveWebContents(),
+                      "document.getElementsByTagName('A')[0].click();"));
   content::WebContents* new_contents = nav_observer.GetWebContents();
   EXPECT_TRUE(WaitForLoadStop(new_contents));
 
   // Validate that no referrer was sent.
-  std::string response_text;
-  std::string response_text_extraction_script =
-      "domAutomationController.send(document.body.innerText);";
-  EXPECT_TRUE(ExecuteScriptAndExtractString(
-      new_contents, response_text_extraction_script, &response_text));
-  EXPECT_EQ("None", response_text);
+  EXPECT_EQ("None", EvalJs(new_contents, "document.body.innerText;"));
 }
 
 // Verify that JavaScript URIs are sanitized to about:blank.
@@ -720,44 +735,97 @@ IN_PROC_BROWSER_TEST_F(ViewSourceTest, JavaScriptURISanitized) {
 
   GURL url(content::kViewSourceScheme + std::string(":") +
            embedded_test_server()->GetURL(kTestNavigationHtml).spec());
-  ui_test_utils::NavigateToURL(browser(), url);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
 
   // Get the href of the second link in the view-source markup.
-  std::string link_href;
-  std::string link_href_extraction_script = R"(
-      domAutomationController.send(
-          document.getElementsByTagName('A')[1].href);)";
+  std::string link_href_extraction_script =
+      "document.getElementsByTagName('A')[1].href;";
 
-  EXPECT_TRUE(content::ExecuteScriptAndExtractString(
-      browser()->tab_strip_model()->GetActiveWebContents(),
-      link_href_extraction_script, &link_href));
-  EXPECT_EQ("about:blank", link_href);
+  EXPECT_EQ(
+      "about:blank",
+      content::EvalJs(browser()->tab_strip_model()->GetActiveWebContents(),
+                      link_href_extraction_script));
 }
 
 // This test verifies that 'view-source' documents are not affected by vertical
 // scroll (see https://crbug.com/898688).
-IN_PROC_BROWSER_TEST_F(ViewSourceFeaturePolicyTest,
+IN_PROC_BROWSER_TEST_F(ViewSourcePermissionsPolicyTest,
                        ViewSourceNotAffectedByHeaderPolicy) {
   ASSERT_TRUE(embedded_test_server()->Start());
   const std::string k_verify_feature = R"(
       var all_features = document.featurePolicy.allowedFeatures();
       var vs = all_features.find((f) => f === 'vertical-scroll');
       console.log(vs);
-      domAutomationController.send("" + vs);)";
+      "" + vs;)";
   // Sanity-check: 'vertical-scroll' is disabled in the actual page (set by the
   // mock headers).
   GURL url(embedded_test_server()->GetURL(kTestHtml));
-  ui_test_utils::NavigateToURL(browser(), url);
-  std::string response;
-  ASSERT_TRUE(ExecuteScriptAndExtractString(
-      browser()->tab_strip_model()->GetActiveWebContents(), k_verify_feature,
-      &response));
-  EXPECT_EQ("undefined", response);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), url));
+  EXPECT_EQ("undefined",
+            EvalJs(browser()->tab_strip_model()->GetActiveWebContents(),
+                   k_verify_feature));
   // Ensure the policy is enabled in the view-source version.
-  ui_test_utils::NavigateToURL(browser(), GURL(content::kViewSourceScheme +
-                                               std::string(":") + url.spec()));
-  ASSERT_TRUE(ExecuteScriptAndExtractString(
-      browser()->tab_strip_model()->GetActiveWebContents(), k_verify_feature,
-      &response));
-  EXPECT_EQ("vertical-scroll", response);
+  ASSERT_TRUE(ui_test_utils::NavigateToURL(
+      browser(),
+      GURL(content::kViewSourceScheme + std::string(":") + url.spec())));
+  EXPECT_EQ("vertical-scroll",
+            EvalJs(browser()->tab_strip_model()->GetActiveWebContents(),
+                   k_verify_feature));
 }
+
+namespace {
+
+class ViewSourcePrerenderTest : public ViewSourceTest {
+ protected:
+  content::test::PrerenderTestHelper& prerender_test_helper() {
+    return prerender_test_helper_;
+  }
+
+  content::WebContents* target() const { return target_; }
+  void set_target(content::WebContents* target) { target_ = target; }
+
+  void SetUp() override {
+    prerender_test_helper().RegisterServerRequestMonitor(
+        embedded_test_server());
+    ViewSourceTest::SetUp();
+  }
+
+ private:
+  content::test::PrerenderTestHelper prerender_test_helper_{
+      base::BindRepeating(&ViewSourcePrerenderTest::target,
+                          base::Unretained(this))};
+
+  // The WebContents which is expected to request prerendering.
+  raw_ptr<content::WebContents, AcrossTasksDanglingUntriaged> target_ = nullptr;
+};
+
+// A frame in a prerendered page should be able to have its source viewed, like
+// any other. There is currently no UI for this, but in principle it should
+// work.
+IN_PROC_BROWSER_TEST_F(ViewSourcePrerenderTest, ViewSourceForPrerender) {
+  ASSERT_TRUE(embedded_test_server()->Start());
+  GURL referrer_url = embedded_test_server()->GetURL("/title1.html");
+  GURL prerender_url = embedded_test_server()->GetURL("/title2.html");
+  content::RenderFrameHost* referrer_frame =
+      ui_test_utils::NavigateToURL(browser(), referrer_url);
+  set_target(content::WebContents::FromRenderFrameHost(referrer_frame));
+
+  prerender_test_helper().AddPrerender(prerender_url);
+  int host_id = prerender_test_helper().GetHostForUrl(prerender_url);
+  content::RenderFrameHost* prerender_frame =
+      prerender_test_helper().GetPrerenderedMainFrameHost(host_id);
+  EXPECT_TRUE(prerender_frame);
+
+  content::WebContentsAddedObserver view_source_contents_observer;
+  prerender_frame->ViewSource();
+  content::WebContents* view_source_contents =
+      view_source_contents_observer.GetWebContents();
+  EXPECT_TRUE(WaitForLoadStop(view_source_contents));
+  EXPECT_EQ(view_source_contents->GetLastCommittedURL(),
+            GURL(base::StrCat(
+                {content::kViewSourceScheme, ":", prerender_url.spec()})));
+  EXPECT_THAT(base::UTF16ToUTF8(view_source_contents->GetTitle()),
+              HasSubstr(content::kViewSourceScheme));
+}
+
+}  // namespace
