@@ -10,6 +10,7 @@
 #include "chrome/browser/ui/webui/ash/multidevice_setup/multidevice_setup_localized_strings_provider.h"
 #include "chrome/grit/generated_resources.h"
 #include "components/login/localized_values_builder.h"
+#include "components/strings/grit/components_strings.h"
 
 namespace ash {
 
@@ -21,6 +22,12 @@ MultiDeviceSetupScreenHandler::~MultiDeviceSetupScreenHandler() = default;
 void MultiDeviceSetupScreenHandler::DeclareLocalizedValues(
     ::login::LocalizedValuesBuilder* builder) {
   multidevice_setup::AddLocalizedValuesToBuilder(builder);
+  builder->Add("arcOverlayClose", IDS_ARC_OOBE_TERMS_POPUP_HELP_CLOSE_BUTTON);
+
+  builder->Add("accept", IDS_MULTIDEVICE_SETUP_ACCEPT_LABEL);
+  builder->Add("cancel", IDS_CANCEL);
+  builder->Add("done", IDS_DONE);
+  builder->Add("noThanks", IDS_NO_THANKS);
 }
 
 void MultiDeviceSetupScreenHandler::Show() {
