@@ -1,4 +1,4 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,9 @@
 
 #include <map>
 #include <set>
+#include <string>
 #include <utility>
+#include <vector>
 
 #include "base/containers/contains.h"
 #include "base/json/json_reader.h"
@@ -136,6 +138,7 @@ class TestChangeStream : public discards::mojom::GraphChangeStream {
 
 class DiscardsGraphDumpImplTest : public testing::Test {
  public:
+  void SetUp() override { graph_.SetUp(); }
   void TearDown() override { graph_.TearDown(); }
 
  protected:
@@ -145,25 +148,35 @@ class DiscardsGraphDumpImplTest : public testing::Test {
 class TestNodeDataDescriber : public performance_manager::NodeDataDescriber {
  public:
   // NodeDataDescriber implementations:
-  base::Value DescribeFrameNodeData(
+  base::Value::Dict DescribeFrameNodeData(
       const performance_manager::FrameNode* node) const override {
-    return base::Value("frame");
+    base::Value::Dict dict;
+    dict.Set("type", "frame");
+    return dict;
   }
-  base::Value DescribePageNodeData(
+  base::Value::Dict DescribePageNodeData(
       const performance_manager::PageNode* node) const override {
-    return base::Value("page");
+    base::Value::Dict dict;
+    dict.Set("type", "page");
+    return dict;
   }
-  base::Value DescribeProcessNodeData(
+  base::Value::Dict DescribeProcessNodeData(
       const performance_manager::ProcessNode* node) const override {
-    return base::Value("process");
+    base::Value::Dict dict;
+    dict.Set("type", "process");
+    return dict;
   }
-  base::Value DescribeSystemNodeData(
+  base::Value::Dict DescribeSystemNodeData(
       const performance_manager::SystemNode* node) const override {
-    return base::Value("system");
+    base::Value::Dict dict;
+    dict.Set("type", "system");
+    return dict;
   }
-  base::Value DescribeWorkerNodeData(
+  base::Value::Dict DescribeWorkerNodeData(
       const performance_manager::WorkerNode* node) const override {
-    return base::Value("worker");
+    base::Value::Dict dict;
+    dict.Set("type", "worker");
+    return dict;
   }
 };
 
@@ -192,7 +205,7 @@ TEST_F(DiscardsGraphDumpImplTest, ChangeStream) {
   mock_graph.other_page->OnMainFrameNavigationCommitted(
       false, now, next_navigation_id++, kExampleUrl, kHtmlMimeType);
 
-  auto* main_frame = mock_graph.page->GetMainFrameNodeImpl();
+  auto* main_frame = mock_graph.page->main_frame_node();
   main_frame->OnNavigationCommitted(kExampleUrl, /* same_document */ false);
 
   std::unique_ptr<DiscardsGraphDumpImpl> impl =
@@ -212,26 +225,32 @@ TEST_F(DiscardsGraphDumpImplTest, ChangeStream) {
   task_environment.RunUntilIdle();
 
   // Validate that the initial graph state dump is complete. Note that there is
-  // an update for each node as part of the initial state dump.
-  EXPECT_EQ(8u, change_stream.num_changes());
-  EXPECT_EQ(8u, change_stream.id_set().size());
+  // an update for each node as part of the initial state dump, except the
+  // system node.
+  size_t expected_changes =
+      graph_.GetAllFrameNodes().size() + graph_.GetAllPageNodes().size() +
+      graph_.GetAllProcessNodes().size() + graph_.GetAllWorkerNodes().size();
+  EXPECT_EQ(expected_changes, change_stream.num_changes());
+  EXPECT_EQ(expected_changes, change_stream.id_set().size());
 
-  EXPECT_EQ(2u, change_stream.process_map().size());
+  EXPECT_EQ(graph_.GetAllProcessNodes().size(),
+            change_stream.process_map().size());
   for (const auto& kv : change_stream.process_map()) {
     const auto* process_info = kv.second.get();
     EXPECT_NE(0u, process_info->id);
-    EXPECT_EQ(base::JSONReader::Read("{\"test\":\"process\"}"),
+    EXPECT_EQ(base::JSONReader::Read("{\"test\":{\"type\":\"process\"}}"),
               base::JSONReader::Read(process_info->description_json));
   }
 
-  EXPECT_EQ(3u, change_stream.frame_map().size());
+  EXPECT_EQ(graph_.GetAllFrameNodes().size(), change_stream.frame_map().size());
   for (const auto& kv : change_stream.frame_map()) {
-    EXPECT_EQ(base::JSONReader::Read("{\"test\":\"frame\"}"),
+    EXPECT_EQ(base::JSONReader::Read("{\"test\":{\"type\":\"frame\"}}"),
               base::JSONReader::Read(kv.second->description_json));
   }
-  EXPECT_EQ(1u, change_stream.worker_map().size());
+  EXPECT_EQ(graph_.GetAllWorkerNodes().size(),
+            change_stream.worker_map().size());
   for (const auto& kv : change_stream.worker_map()) {
-    EXPECT_EQ(base::JSONReader::Read("{\"test\":\"worker\"}"),
+    EXPECT_EQ(base::JSONReader::Read("{\"test\":{\"type\":\"worker\"}}"),
               base::JSONReader::Read(kv.second->description_json));
   }
 
@@ -256,12 +275,12 @@ TEST_F(DiscardsGraphDumpImplTest, ChangeStream) {
   // Make sure we have one top-level frame per page.
   EXPECT_EQ(change_stream.page_map().size(), top_level_frames);
 
-  EXPECT_EQ(2u, change_stream.page_map().size());
+  EXPECT_EQ(graph_.GetAllPageNodes().size(), change_stream.page_map().size());
   for (const auto& kv : change_stream.page_map()) {
     const auto& page = kv.second;
     EXPECT_NE(0u, page->id);
     EXPECT_EQ(kExampleUrl, page->main_frame_url);
-    EXPECT_EQ(base::JSONReader::Read("{\"test\":\"page\"}"),
+    EXPECT_EQ(base::JSONReader::Read("{\"test\":{\"type\":\"page\"}}"),
               base::JSONReader::Read(kv.second->description_json));
   }
 
@@ -277,7 +296,8 @@ TEST_F(DiscardsGraphDumpImplTest, ChangeStream) {
   task_environment.RunUntilIdle();
 
   // Main frame navigation results in a notification for the url.
-  EXPECT_EQ(9u, change_stream.num_changes());
+  expected_changes += 1;
+  EXPECT_EQ(expected_changes, change_stream.num_changes());
   EXPECT_FALSE(base::Contains(change_stream.id_set(), child_frame_id));
 
   const auto main_page_it = change_stream.page_map().find(
@@ -310,10 +330,12 @@ TEST_F(DiscardsGraphDumpImplTest, ChangeStream) {
               // Check that the descriptions make sense.
               for (auto kv : node_descriptions_json) {
                 keys_received.push_back(kv.first);
-                absl::optional<base::Value> v =
+                std::optional<base::Value> v =
                     base::JSONReader::Read(kv.second);
                 EXPECT_TRUE(v->is_dict());
-                std::string* str = v->FindStringKey("test");
+                base::Value::Dict* dict = v->GetDict().FindDict("test");
+                EXPECT_TRUE(dict);
+                std::string* str = dict->FindString("type");
                 EXPECT_TRUE(str);
                 if (str) {
                   EXPECT_TRUE(*str == "frame" || *str == "page" ||
