@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,8 +8,8 @@
 #include <string>
 #include <utility>
 
-#include "base/bind.h"
 #include "base/feature_list.h"
+#include "base/functional/bind.h"
 #include "base/i18n/rtl.h"
 #include "base/memory/ref_counted_memory.h"
 #include "base/strings/utf_string_conversions.h"
@@ -64,9 +64,7 @@ NewTabUI::NewTabUI(content::WebUI* web_ui) : content::WebUIController(web_ui) {
   // The title should be "New Tab" for regular mode and guest mode, while it
   // should be "New Incognito Tab" for incognito mode.
   const int title_resource_id =
-      base::FeatureList::IsEnabled(
-          features::kUpdateHistoryEntryPointsInIncognito) &&
-              profile->IsOffTheRecord() && !profile->IsGuestSession()
+      profile->IsOffTheRecord() && !profile->IsGuestSession()
           ? IDS_NEW_INCOGNITO_TAB_TITLE
           : IDS_NEW_TAB_TITLE;
   web_ui->OverrideTitle(l10n_util::GetStringUTF16(title_resource_id));
@@ -189,8 +187,7 @@ void NewTabUI::NewTabHTMLSource::StartDataRequest(
   std::move(callback).Run(html_bytes.get());
 }
 
-std::string NewTabUI::NewTabHTMLSource::GetMimeType(
-    const std::string& resource) {
+std::string NewTabUI::NewTabHTMLSource::GetMimeType(const GURL&) {
   return "text/html";
 }
 
