@@ -1,24 +1,26 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/webui/print_preview/pdf_printer_handler.h"
 
-#include "base/json/json_reader.h"
+#include <optional>
+
 #include "base/run_loop.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/test/values_test_util.h"
 #include "build/build_config.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/test/base/browser_with_test_window_test.h"
 #include "chrome/test/base/scoped_browser_locale.h"
 #include "components/url_formatter/url_formatter.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 #include "url/gurl.h"
 
-#if defined(OS_MAC)
+#if BUILDFLAG(IS_MAC)
 #include "chrome/common/printing/printer_capabilities_mac.h"
 #include "printing/backend/print_backend.h"
+#include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/size.h"
 #endif
 
@@ -43,53 +45,96 @@ const char kPdfPrinterCapability[] =
                 }
               ]
             },
+            "dpi": {
+               "option": [ {
+                  "horizontal_dpi": 300,
+                  "is_default": true,
+                  "vertical_dpi": 300
+               } ]
+            },
             "media_size":{
               "option":[
                 {
                   "height_microns":1189000,
+                  "imageable_area_bottom_microns": 0,
+                  "imageable_area_left_microns": 0,
+                  "imageable_area_right_microns": 841000,
+                  "imageable_area_top_microns": 1189000,
                   "name":"ISO_A0",
-                  "width_microns":841000
+                  "width_microns":841000,
                 },
                 {
                   "height_microns":841000,
+                  "imageable_area_bottom_microns": 0,
+                  "imageable_area_left_microns": 0,
+                  "imageable_area_right_microns": 594000,
+                  "imageable_area_top_microns": 841000,
                   "name":"ISO_A1",
-                  "width_microns":594000
+                  "width_microns":594000,
                 },
                 {
                   "height_microns":594000,
+                  "imageable_area_bottom_microns": 0,
+                  "imageable_area_left_microns": 0,
+                  "imageable_area_right_microns": 420000,
+                  "imageable_area_top_microns": 594000,
                   "name":"ISO_A2",
-                  "width_microns":420000
+                  "width_microns":420000,
                 },
                 {
                   "height_microns":420000,
+                  "imageable_area_bottom_microns": 0,
+                  "imageable_area_left_microns": 0,
+                  "imageable_area_right_microns": 297000,
+                  "imageable_area_top_microns": 420000,
                   "name":"ISO_A3",
-                  "width_microns":297000
+                  "width_microns":297000,
                 },
                 {
                   "height_microns":297000,
+                  "imageable_area_bottom_microns": 0,
+                  "imageable_area_left_microns": 0,
+                  "imageable_area_right_microns": 210000,
+                  "imageable_area_top_microns": 297000,
                   "name":"ISO_A4",
-                  "width_microns":210000
+                  "width_microns":210000,
                 },
                 {
                   "height_microns":210000,
+                  "imageable_area_bottom_microns": 0,
+                  "imageable_area_left_microns": 0,
+                  "imageable_area_right_microns": 148000,
+                  "imageable_area_top_microns": 210000,
                   "name":"ISO_A5",
-                  "width_microns":148000
+                  "width_microns":148000,
                 },
                 {
                   "height_microns":355600,
+                  "imageable_area_bottom_microns": 0,
+                  "imageable_area_left_microns": 0,
+                  "imageable_area_right_microns": 215900,
+                  "imageable_area_top_microns": 355600,
                   "name":"NA_LEGAL",
-                  "width_microns":215900
+                  "width_microns":215900,
                 },
                 {
                   "height_microns":279400,
+                  "imageable_area_bottom_microns": 0,
+                  "imageable_area_left_microns": 0,
+                  "imageable_area_right_microns": 215900,
+                  "imageable_area_top_microns": 279400,
                   "is_default":true,
                   "name":"NA_LETTER",
-                  "width_microns":215900
+                  "width_microns":215900,
                 },
                 {
                   "height_microns":431800,
+                  "imageable_area_bottom_microns": 0,
+                  "imageable_area_left_microns": 0,
+                  "imageable_area_right_microns": 279400,
+                  "imageable_area_top_microns": 431800,
                   "name":"NA_LEDGER",
-                  "width_microns":279400
+                  "width_microns":279400,
                 }
               ]
             },
@@ -116,19 +161,30 @@ const char kPdfPrinterCapability[] =
 // Used as a callback to StartGetCapability() in tests.
 // Records values returned by StartGetCapability().
 void RecordCapability(base::OnceClosure done_closure,
-                      base::Value* capability_out,
-                      base::Value capability) {
+                      base::Value::Dict* capability_out,
+                      base::Value::Dict capability) {
   *capability_out = std::move(capability);
   std::move(done_closure).Run();
 }
 
-#if defined(OS_MAC)
-base::Value GetValueFromCustomPaper(
+#if BUILDFLAG(IS_MAC)
+base::Value::Dict GetValueFromCustomPaper(
     const PrinterSemanticCapsAndDefaults::Paper& paper) {
-  base::Value paper_value(base::Value::Type::DICTIONARY);
-  paper_value.SetStringKey("custom_display_name", paper.display_name);
-  paper_value.SetIntKey("height_microns", paper.size_um.height());
-  paper_value.SetIntKey("width_microns", paper.size_um.width());
+  base::Value::Dict paper_value;
+  paper_value.Set("custom_display_name", paper.display_name());
+  paper_value.Set("height_microns", paper.size_um().height());
+  paper_value.Set("width_microns", paper.size_um().width());
+  int imageable_area_left_microns = paper.printable_area_um().x();
+  int imageable_area_bottom_microns = paper.printable_area_um().y();
+  int imageable_area_right_microns =
+      paper.printable_area_um().x() + paper.printable_area_um().width();
+  int imageable_area_top_microns =
+      paper.printable_area_um().y() + paper.printable_area_um().height();
+  paper_value.Set("imageable_area_left_microns", imageable_area_left_microns);
+  paper_value.Set("imageable_area_bottom_microns",
+                  imageable_area_bottom_microns);
+  paper_value.Set("imageable_area_right_microns", imageable_area_right_microns);
+  paper_value.Set("imageable_area_top_microns", imageable_area_top_microns);
   return paper_value;
 }
 #endif
@@ -155,9 +211,9 @@ class PdfPrinterHandlerGetCapabilityTest : public BrowserWithTestWindowTest {
   }
 
  protected:
-  base::Value StartGetCapabilityAndWaitForResults() {
+  base::Value::Dict StartGetCapabilityAndWaitForResults() {
     base::RunLoop run_loop;
-    base::Value capability;
+    base::Value::Dict capability;
     pdf_printer_handler_->StartGetCapability(
         kPdfDeviceName,
         base::BindOnce(&RecordCapability, run_loop.QuitClosure(), &capability));
@@ -262,44 +318,47 @@ TEST_F(PdfPrinterHandlerTest, GetFileName) {
 }
 
 TEST_F(PdfPrinterHandlerGetCapabilityTest, GetCapability) {
-  absl::optional<base::Value> expected_capability =
-      base::JSONReader::Read(kPdfPrinterCapability);
-  ASSERT_TRUE(expected_capability.has_value());
-
-  base::Value capability = StartGetCapabilityAndWaitForResults();
-  EXPECT_EQ(expected_capability.value(), capability);
+  base::Value expected_capability =
+      base::test::ParseJson(kPdfPrinterCapability);
+  ASSERT_TRUE(expected_capability.is_dict());
+  base::Value::Dict capability = StartGetCapabilityAndWaitForResults();
+  EXPECT_EQ(expected_capability.GetDict(), capability);
 }
 
-#if defined(OS_MAC)
+#if BUILDFLAG(IS_MAC)
 TEST_F(PdfPrinterHandlerGetCapabilityTest,
        GetMacCustomPaperSizesInCapabilities) {
   constexpr char kPaperOptionPath[] = "capabilities.printer.media_size.option";
+  // The first Paper has an arbitrary valid printable area. The rest have
+  // printable area that match the size.
   static const PrinterSemanticCapsAndDefaults::Papers kTestPapers = {
-      {"printer1", "", gfx::Size(101600, 127000)},
-      {"printer2", "", gfx::Size(76200, 152400)},
-      {"printer3", "", gfx::Size(330200, 863600)},
-      {"printer4", "", gfx::Size(101600, 50800)},
+      {"printer1", "", gfx::Size(101600, 127000),
+       gfx::Rect(120, 120, 101480, 126880)},
+      {"printer2", "", gfx::Size(76200, 152400),
+       gfx::Rect(0, 0, 76200, 152400)},
+      {"printer3", "", gfx::Size(330200, 863600),
+       gfx::Rect(0, 0, 330200, 863600)},
+      {"printer4", "", gfx::Size(101600, 50800),
+       gfx::Rect(0, 0, 101600, 50800)},
   };
 
-  absl::optional<base::Value> expected_capability =
-      base::JSONReader::Read(kPdfPrinterCapability);
-  ASSERT_TRUE(expected_capability.has_value());
-  ASSERT_TRUE(expected_capability.value().is_dict());
+  base::Value expected_capability =
+      base::test::ParseJson(kPdfPrinterCapability);
+  ASSERT_TRUE(expected_capability.is_dict());
 
-  base::Value* expected_paper_options =
-      expected_capability.value().FindListPath(kPaperOptionPath);
+  base::Value::List* expected_paper_options =
+      expected_capability.GetDict().FindListByDottedPath(kPaperOptionPath);
   ASSERT_TRUE(expected_paper_options);
 
-  for (const PrinterSemanticCapsAndDefaults::Paper& paper : kTestPapers) {
+  for (const PrinterSemanticCapsAndDefaults::Paper& paper : kTestPapers)
     expected_paper_options->Append(GetValueFromCustomPaper(paper));
-  }
 
   SetMacCustomPaperSizesForTesting(kTestPapers);
 
-  base::Value capability = StartGetCapabilityAndWaitForResults();
-  ASSERT_TRUE(capability.is_dict());
+  base::Value::Dict capability = StartGetCapabilityAndWaitForResults();
 
-  base::Value* paper_options = capability.FindListPath(kPaperOptionPath);
+  const base::Value::List* paper_options =
+      capability.FindListByDottedPath(kPaperOptionPath);
   ASSERT_TRUE(paper_options);
   EXPECT_EQ(*expected_paper_options, *paper_options);
 }
