@@ -7,6 +7,7 @@
 
 #include <stdint.h>
 
+#include "components/cdm/common/clearkey_drm_delegate_android.h"
 #include "components/cdm/common/widevine_drm_delegate_android.h"
 #include "media/base/android/media_drm_bridge_client.h"
 
@@ -26,6 +27,8 @@ class ChromeMediaDrmBridgeClient : public media::MediaDrmBridgeClient {
       const std::vector<uint8_t>& scheme_uuid) override;
 
   cdm::WidevineDrmDelegateAndroid widevine_delegate_;
+
+  cdm::ClearKeyDrmDelegateAndroid clearkey_delegate_;
 };
 
 #endif  // CHROME_COMMON_MEDIA_CHROME_MEDIA_DRM_BRIDGE_CLIENT_H_
