@@ -6,12 +6,12 @@
 #define CHROME_COMMON_PROFILER_THREAD_PROFILER_CONFIGURATION_H_
 
 #include <initializer_list>
+#include <optional>
 #include <string>
 
 #include "base/no_destructor.h"
 #include "base/profiler/stack_sampling_profiler.h"
-#include "components/metrics/call_stack_profile_params.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
+#include "components/metrics/call_stacks/call_stack_profile_params.h"
 #include "third_party/abseil-cpp/absl/types/variant.h"
 
 namespace base {
@@ -51,11 +51,19 @@ class ThreadProfilerConfiguration {
   bool GetSyntheticFieldTrial(std::string* trial_name,
                               std::string* group_name) const;
 
+  // True if profiler should be enabled for the child process.
+  bool IsProfilerEnabledForChildProcess(
+      metrics::CallStackProfileParams::Process child_process) const;
+
   // Add a command line switch that instructs the child process to run the
   // profiler. This should only be called from the browser process.
   void AppendCommandLineSwitchForChildProcess(
       base::CommandLine* command_line) const;
 
+#if BUILDFLAG(IS_ANDROID)
+  bool IsJavaNameHashingEnabled() const;
+#endif  // BUILDFLAG(IS_ANDROID)
+
  private:
   friend base::NoDestructor<ThreadProfilerConfiguration>;
 
@@ -73,14 +81,30 @@ class ThreadProfilerConfiguration {
     // kProfileDisabled group).
     kProfileControl,
 
+#if BUILDFLAG(IS_ANDROID)
+    // Enabled within the experiment (and paired with equal-sized
+    // kProfileDisabled and kProfileControl groups). Java names will be
+    // hashed within this group.
+    kProfileEnabledWithJavaNameHashing,
+#endif  // BUILDFLAG(IS_ANDROID)
+
     // Enabled outside of the experiment.
     kProfileEnabled,
   };
 
-  // The configuration state for the browser process. If !has_value() profiling
-  // is disabled and no variations state is reported. Otherwise profiling is
-  // enabled based on the VariationGroup and the variation state is reported.
-  using BrowserProcessConfiguration = absl::optional<VariationGroup>;
+  struct BrowserProcessConfiguration {
+    // The configuration state for the browser process. If !has_value()
+    // profiling is disabled and no variations state is reported. Otherwise
+    // profiling is enabled based on the VariationGroup and the variation state
+    // is reported.
+    std::optional<VariationGroup> variation_group;
+
+    // In pick-single-type-of-process-to-sample mode, only a single process
+    // type will be profiled when profiling is enabled. If !has_value(), the
+    // profiling will be enabled for as many processes as possible.
+    std::optional<metrics::CallStackProfileParams::Process>
+        process_type_to_sample;
+  };
 
   // The configuration state in child processes.
   enum ChildProcessConfiguration {
@@ -103,7 +127,14 @@ class ThreadProfilerConfiguration {
 
   // True if the profiler is to be enabled for |variation_group|.
   static bool EnableForVariationGroup(
-      absl::optional<VariationGroup> variation_group);
+      std::optional<VariationGroup> variation_group);
+
+  // True if the given process is picked to enable profiling. In pick-single-
+  // type-of-process-to-sample mode, only one type of process is picked to
+  // have profiling enabled so that the user impact can be minimized.
+  static bool IsProcessGloballyEnabled(
+      const ThreadProfilerConfiguration::BrowserProcessConfiguration& config,
+      metrics::CallStackProfileParams::Process process);
 
   // Randomly chooses a variation from the weighted variations. Weights are
   // expected to sum to 100 as a sanity check.
