@@ -1,10 +1,11 @@
-// Copyright (c) 2011 The Chromium Authors. All rights reserved.
+// Copyright 2011 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <memory>
 #include <tuple>
 
-#include "base/bind.h"
+#include "base/functional/bind.h"
 #include "base/run_loop.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/scoped_feature_list.h"
@@ -12,11 +13,14 @@
 #include "build/build_config.h"
 #include "chrome/test/base/chrome_render_view_test.h"
 #include "components/autofill/content/renderer/autofill_agent.h"
+#include "components/autofill/content/renderer/autofill_agent_test_api.h"
 #include "components/autofill/content/renderer/focus_test_utils.h"
+#include "components/autofill/content/renderer/form_autofill_util.h"
+#include "components/autofill/content/renderer/form_tracker_test_api.h"
 #include "components/autofill/core/common/autofill_features.h"
 #include "components/autofill/core/common/form_data.h"
+#include "components/autofill/core/common/mojom/autofill_types.mojom-shared.h"
 #include "content/public/renderer/render_frame.h"
-#include "content/public/renderer/render_view.h"
 #include "mojo/public/cpp/bindings/associated_receiver_set.h"
 #include "mojo/public/cpp/bindings/pending_associated_receiver.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -43,7 +47,7 @@ class FakeContentAutofillDriver : public mojom::AutofillDriver {
  public:
   FakeContentAutofillDriver() = default;
 
-  ~FakeContentAutofillDriver() override {}
+  ~FakeContentAutofillDriver() override = default;
 
   void BindReceiver(
       mojo::PendingAssociatedReceiver<mojom::AutofillDriver> receiver) {
@@ -67,14 +71,15 @@ class FakeContentAutofillDriver : public mojom::AutofillDriver {
  private:
   // mojom::AutofillDriver:
   void SetFormToBeProbablySubmitted(
-      const base::Optional<FormData>& form) override {}
+      const std::optional<FormData>& form) override {}
 
-  void FormsSeen(const std::vector<FormData>& forms) override {}
+  void FormsSeen(const std::vector<FormData>& updated_forms,
+                 const std::vector<FormRendererId>& removed_forms) override {}
 
   void FormSubmitted(const FormData& form,
                      bool known_success,
                      SubmissionSource source) override {
-    form_submitted_.reset(new FormData(form));
+    form_submitted_ = std::make_unique<FormData>(form);
     known_success_ = known_success;
     submission_source_ = source;
   }
@@ -94,11 +99,16 @@ class FakeContentAutofillDriver : public mojom::AutofillDriver {
     select_control_changed_ = std::make_unique<FormFieldData>(field);
   }
 
-  void QueryFormFieldAutofill(int32_t id,
-                              const FormData& form,
-                              const FormFieldData& field,
-                              const gfx::RectF& bounding_box,
-                              bool autoselect_first_field) override {}
+  void JavaScriptChangedAutofilledValue(
+      const FormData& form,
+      const FormFieldData& field,
+      const std::u16string& old_value) override {}
+
+  void AskForValuesToFill(
+      const FormData& form,
+      const FormFieldData& field,
+      const gfx::RectF& bounding_box,
+      AutofillSuggestionTriggerSource trigger_source) override {}
 
   void HidePopup() override {}
 
@@ -114,11 +124,10 @@ class FakeContentAutofillDriver : public mojom::AutofillDriver {
   void DidFillAutofillFormData(const FormData& form,
                                base::TimeTicks timestamp) override {}
 
-  void DidPreviewAutofillFormData() override {}
-
   void DidEndTextFieldEditing() override {}
 
-  void SelectFieldOptionsDidChange(const autofill::FormData& form) override {}
+  void SelectOrSelectListFieldOptionsDidChange(
+      const autofill::FormData& form) override {}
 
   // Records whether FocusNoLongerOnForm() get called.
   bool did_unfocus_form_{false};
@@ -153,9 +162,9 @@ void VerifyReceivedRendererMessages(
   // The tuple also includes a timestamp, which is ignored.
   const FormData& submitted_form = *(fake_driver.form_submitted());
   ASSERT_LE(2U, submitted_form.fields.size());
-  EXPECT_EQ(base::ASCIIToUTF16("fname"), submitted_form.fields[0].name);
+  EXPECT_EQ(u"fname", submitted_form.fields[0].name);
   EXPECT_EQ(base::UTF8ToUTF16(fname), submitted_form.fields[0].value);
-  EXPECT_EQ(base::ASCIIToUTF16("lname"), submitted_form.fields[1].name);
+  EXPECT_EQ(u"lname", submitted_form.fields[1].name);
   EXPECT_EQ(expect_known_success, fake_driver.known_success());
   EXPECT_EQ(expect_submission_source,
             mojo::ConvertTo<SubmissionSource>(fake_driver.submission_source()));
@@ -171,7 +180,7 @@ void VerifyReceivedAddressRendererMessages(
   // The tuple also includes a timestamp, which is ignored.
   const FormData& submitted_form = *(fake_driver.form_submitted());
   ASSERT_LE(1U, submitted_form.fields.size());
-  EXPECT_EQ(base::ASCIIToUTF16("address"), submitted_form.fields[0].name);
+  EXPECT_EQ(u"address", submitted_form.fields[0].name);
   EXPECT_EQ(base::UTF8ToUTF16(address), submitted_form.fields[0].value);
   EXPECT_EQ(expect_known_success, fake_driver.known_success());
   EXPECT_EQ(expect_submission_source,
@@ -186,11 +195,9 @@ void VerifyNoSubmitMessagesReceived(
   EXPECT_EQ(nullptr, fake_driver.form_submitted());
 }
 
-// Simulates receiving a message from the browser to fill a form.
-void SimulateOnFillForm(autofill::AutofillAgent* autofill_agent,
-                        blink::WebLocalFrame* main_frame) {
+FormData CreateAutofillFormData(blink::WebLocalFrame* main_frame) {
   FormData data;
-  data.name = base::ASCIIToUTF16("name");
+  data.name = u"name";
   data.url = GURL("http://example.com/");
   data.action = GURL("http://example.com/blade.php");
   data.is_form_tag = true;  // Default value.
@@ -199,39 +206,58 @@ void SimulateOnFillForm(autofill::AutofillAgent* autofill_agent,
   WebFormControlElement fname_element =
       document.GetElementById(WebString::FromUTF8("fname"))
           .To<WebFormControlElement>();
-  ASSERT_FALSE(fname_element.IsNull());
   WebFormControlElement lname_element =
       document.GetElementById(WebString::FromUTF8("lname"))
           .To<WebFormControlElement>();
 
   FormFieldData field_data;
-  field_data.name = base::ASCIIToUTF16("fname");
-  field_data.value = base::ASCIIToUTF16("John");
+  field_data.name = u"fname";
+  field_data.value = u"John";
   field_data.is_autofilled = true;
-  field_data.unique_renderer_id =
-      FieldRendererId(fname_element.UniqueRendererFormControlId());
+  field_data.unique_renderer_id = form_util::GetFieldRendererId(fname_element);
   data.fields.push_back(field_data);
 
-  field_data.name = base::ASCIIToUTF16("lname");
-  field_data.value = base::ASCIIToUTF16("Smith");
-  field_data.is_autofilled = true;
   if (!lname_element.IsNull()) {
+    field_data.name = u"lname";
+    field_data.value = u"Smith";
+    field_data.is_autofilled = true;
     field_data.unique_renderer_id =
-        FieldRendererId(lname_element.UniqueRendererFormControlId());
+        form_util::GetFieldRendererId(lname_element);
+    data.fields.push_back(field_data);
   }
-  data.fields.push_back(field_data);
 
+  return data;
+}
+
+void SimulateFillForm(const FormData& form_data,
+                      autofill::AutofillAgent* autofill_agent,
+                      blink::WebLocalFrame* main_frame) {
+  WebDocument document = main_frame->GetDocument();
+  WebFormControlElement fname_element =
+      document.GetElementById(WebString::FromUTF8("fname"))
+          .To<WebFormControlElement>();
+
+  ASSERT_FALSE(fname_element.IsNull());
   // This call is necessary to setup the autofill agent appropriate for the
   // user selection; simulates the menu actually popping up.
-  autofill_agent->FormControlElementClicked(fname_element.To<WebInputElement>(),
-                                            false);
+  autofill_agent->FormControlElementClicked(
+      fname_element.To<WebInputElement>());
 
-  autofill_agent->FillForm(0, data);
+  autofill_agent->ApplyFormAction(
+      mojom::ActionType::kFill, mojom::ActionPersistence::kFill,
+      form_data.unique_renderer_id, form_data.fields);
+}
+
+// Simulates receiving a message from the browser to fill a form.
+void SimulateFillForm(autofill::AutofillAgent* autofill_agent,
+                      blink::WebLocalFrame* main_frame) {
+  FormData data = CreateAutofillFormData(main_frame);
+  SimulateFillForm(data, autofill_agent, main_frame);
 }
 
 // Simulates receiving a message from the browser to fill a form with an
 // additional non-autofillable field.
-void SimulateOnFillFormWithNonFillableFields(
+void SimulateFillFormWithNonFillableFields(
     autofill::AutofillAgent* autofill_agent,
     blink::WebLocalFrame* main_frame) {
   WebDocument document = main_frame->GetDocument();
@@ -248,49 +274,48 @@ void SimulateOnFillFormWithNonFillableFields(
           .To<WebFormControlElement>();
   ASSERT_FALSE(lname_element.IsNull());
 
-  FormData data;
-  data.name = base::ASCIIToUTF16("name");
-  data.url = GURL("http://example.com/");
-  data.action = GURL("http://example.com/blade.php");
-  data.is_form_tag = true;  // Default value.
+  FormData form;
+  form.name = u"name";
+  form.url = GURL("http://example.com/");
+  form.action = GURL("http://example.com/blade.php");
+  form.is_form_tag = true;  // Default value.
 
-  FormFieldData field_data;
-  field_data.name = base::ASCIIToUTF16("fname");
-  field_data.value = base::ASCIIToUTF16("John");
-  field_data.is_autofilled = true;
-  field_data.unique_renderer_id =
-      FieldRendererId(fname_element.UniqueRendererFormControlId());
-  data.fields.push_back(field_data);
+  FormFieldData field;
+  field.name = u"fname";
+  field.value = u"John";
+  field.is_autofilled = true;
+  field.unique_renderer_id = form_util::GetFieldRendererId(fname_element);
+  form.fields.push_back(field);
 
-  field_data.name = base::ASCIIToUTF16("lname");
-  field_data.value = base::ASCIIToUTF16("Smith");
-  field_data.is_autofilled = true;
-  field_data.unique_renderer_id =
-      FieldRendererId(lname_element.UniqueRendererFormControlId());
-  data.fields.push_back(field_data);
+  field.name = u"lname";
+  field.value = u"Smith";
+  field.is_autofilled = true;
+  field.unique_renderer_id = form_util::GetFieldRendererId(lname_element);
+  form.fields.push_back(field);
 
   // Additional non-autofillable field.
-  field_data.name = base::ASCIIToUTF16("mname");
-  field_data.value = base::ASCIIToUTF16("James");
-  field_data.is_autofilled = false;
-  field_data.unique_renderer_id =
-      FieldRendererId(mname_element.UniqueRendererFormControlId());
-  data.fields.push_back(field_data);
+  field.name = u"mname";
+  field.value = u"James";
+  field.is_autofilled = false;
+  field.unique_renderer_id = form_util::GetFieldRendererId(mname_element);
+  form.fields.push_back(field);
 
   // This call is necessary to setup the autofill agent appropriate for the
   // user selection; simulates the menu actually popping up.
-  autofill_agent->FormControlElementClicked(fname_element.To<WebInputElement>(),
-                                            false);
+  autofill_agent->FormControlElementClicked(
+      fname_element.To<WebInputElement>());
 
-  autofill_agent->FillForm(0, data);
+  autofill_agent->ApplyFormAction(mojom::ActionType::kFill,
+                                  mojom::ActionPersistence::kFill,
+                                  form.unique_renderer_id, form.fields);
 }
 
-}  // end namespace
-
 class FormAutocompleteTest : public ChromeRenderViewTest {
  public:
-  FormAutocompleteTest() {}
-  ~FormAutocompleteTest() override {}
+  FormAutocompleteTest() = default;
+  FormAutocompleteTest(const FormAutocompleteTest&) = delete;
+  FormAutocompleteTest& operator=(const FormAutocompleteTest&) = delete;
+  ~FormAutocompleteTest() override = default;
 
  protected:
   void SetUp() override {
@@ -299,7 +324,7 @@ class FormAutocompleteTest : public ChromeRenderViewTest {
     // We only use the fake driver for main frame
     // because our test cases only involve the main frame.
     blink::AssociatedInterfaceProvider* remote_interfaces =
-        view_->GetMainRenderFrame()->GetRemoteAssociatedInterfaces();
+        GetMainRenderFrame()->GetRemoteAssociatedInterfaces();
     remote_interfaces->OverrideBinderForTesting(
         mojom::AutofillDriver::Name_,
         base::BindRepeating(&FormAutocompleteTest::BindAutofillDriver,
@@ -330,9 +355,6 @@ class FormAutocompleteTest : public ChromeRenderViewTest {
 
   FakeContentAutofillDriver fake_driver_;
   std::unique_ptr<test::FocusTestUtils> focus_test_utils_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(FormAutocompleteTest);
 };
 
 // Tests that submitting a form generates FormSubmitted message with the form
@@ -445,7 +467,7 @@ TEST_F(FormAutocompleteTest,
 // compare field data within the forms.
 // TODO(kolos) Re-enable when the implementation of IsFormVisible is on-par
 // for these platforms.
-#if defined(OS_MAC)
+#if BUILDFLAG(IS_MAC)
 #define MAYBE_NoLongerVisibleBothNoActions DISABLED_NoLongerVisibleBothNoActions
 #else
 #define MAYBE_NoLongerVisibleBothNoActions NoLongerVisibleBothNoActions
@@ -570,7 +592,7 @@ TEST_F(FormAutocompleteTest, AjaxSucceeded_FilledFormIsInvisible) {
       "<input name='lname'/></form></html>");
 
   // Simulate filling a form using Autofill.
-  SimulateOnFillForm(autofill_agent_, GetMainFrame());
+  SimulateFillForm(autofill_agent_, GetMainFrame());
 
   // Simulate user input since ajax request doesn't fire submission message
   // if there is no user input.
@@ -601,7 +623,7 @@ TEST_F(FormAutocompleteTest, AjaxSucceeded_FilledFormStillVisible) {
       "<input name='lname' value='Deckard'/></form></html>");
 
   // Simulate filling a form using Autofill.
-  SimulateOnFillForm(autofill_agent_, GetMainFrame());
+  SimulateFillForm(autofill_agent_, GetMainFrame());
 
   // Form still visible.
 
@@ -629,7 +651,7 @@ TEST_F(FormAutocompleteTest, VerifyFocusAndBlurEventsAfterAutofill) {
   focus_test_utils_->FocusElement("fname");
 
   // Simulate filling the form using Autofill.
-  SimulateOnFillFormWithNonFillableFields(autofill_agent_, GetMainFrame());
+  SimulateFillFormWithNonFillableFields(autofill_agent_, GetMainFrame());
   base::RunLoop().RunUntilIdle();
 
   // Expected Result in order:
@@ -657,7 +679,7 @@ TEST_F(FormAutocompleteTest,
   focus_test_utils_->FocusElement("fname");
 
   // Simulate filling the form using Autofill.
-  SimulateOnFillForm(autofill_agent_, GetMainFrame());
+  SimulateFillForm(autofill_agent_, GetMainFrame());
   base::RunLoop().RunUntilIdle();
 
   // Expected Result in order:
@@ -685,10 +707,71 @@ TEST_F(FormAutocompleteTest,
   focus_test_utils_->FocusElement("fname");
 
   // Simulate filling the form using Autofill.
-  SimulateOnFillForm(autofill_agent_, GetMainFrame());
+  SimulateFillForm(autofill_agent_, GetMainFrame());
   base::RunLoop().RunUntilIdle();
 
-  EXPECT_EQ(GetFocusLog(), "");
+  // Expected Result in order:
+  // * Change fname
+  EXPECT_EQ(GetFocusLog(), "c0");
+}
+
+// Tests that a field is added to the form between the times of triggering
+// and executing the filling.
+TEST_F(FormAutocompleteTest, VerifyFocusAndBlurEventAfterElementAdded) {
+  // Load a form.
+  LoadHTML(
+      "<html><form id='myForm'>"
+      "<label>First Name:</label><input id='fname' name='0'/><br/>"
+      "<label>Last Name:</label> <input id='lname' name='1'/><br/>"
+      "</form></html>");
+
+  focus_test_utils_->SetUpFocusLogging();
+  focus_test_utils_->FocusElement("fname");
+
+  // Simulate filling the form using Autofill.
+  FormData data = CreateAutofillFormData(GetMainFrame());
+  // Simulate that the form was modified between parsing and executing the fill.
+  // The element is inserted at the beginning of the form to verify that
+  // everything works correctly even if unique_renderer_ids of the <input>
+  // elements are not in ascending order.
+  ExecuteJavaScriptForTests(
+      "document.getElementById('fname').insertAdjacentHTML('beforebegin', "
+      "'<label>Zip code:</label><input id=\"zip_code\"/>');");
+  SimulateFillForm(data, autofill_agent_, GetMainFrame());
+  base::RunLoop().RunUntilIdle();
+
+  // Expected Result in order:
+  // * Change fname
+  // * Blur fname
+  // * Focus lname
+  // * Change lname
+  // * Blur lname
+  // * Focus fname
+  EXPECT_EQ(GetFocusLog(), "c0b0f1c1b1f0");
+}
+
+// Tests that a field is removed from the form between the times of
+// triggering and executing the filling.
+TEST_F(FormAutocompleteTest, VerifyFocusAndBlurEventAfterElementRemoved) {
+  // Load a form.
+  LoadHTML(
+      "<html><form id='myForm'>"
+      "<label>First Name:</label><input id='fname' name='0'/><br/>"
+      "<label>Last Name:</label> <input id='lname' name='1'/><br/>"
+      "</form></html>");
+
+  focus_test_utils_->SetUpFocusLogging();
+  focus_test_utils_->FocusElement("fname");
+
+  // Simulate filling the form using Autofill.
+  FormData data = CreateAutofillFormData(GetMainFrame());
+  ExecuteJavaScriptForTests("document.getElementById('lname').remove()");
+  SimulateFillForm(data, autofill_agent_, GetMainFrame());
+  base::RunLoop().RunUntilIdle();
+
+  // Expected Result in order:
+  // * Change fname
+  EXPECT_EQ(GetFocusLog(), "c0");
 }
 
 // Tests that completing an Ajax request without a form present will still
@@ -723,32 +806,6 @@ TEST_F(FormAutocompleteTest, AjaxSucceeded_FormlessElements) {
                                  SubmissionSource::XHR_SUCCEEDED);
 }
 
-// Unit test for CollectFormlessElements.
-TEST_F(FormAutocompleteTest, CollectFormlessElements) {
-  LoadHTML(
-      "<html><title>Checkout</title></head>"
-      "<input type='text' name='text_input'/>"
-      "<input type='checkbox' name='check_input'/>"
-      "<input type='number' name='number_input'/>"
-      "<select name='select_input'/>"
-      "  <option value='option_1'></option>"
-      "  <option value='option_2'></option>"
-      "</select>"
-      "<form><input type='text' name='excluded'/></form>"
-      "</html>");
-
-  FormData result;
-  autofill_agent_->CollectFormlessElements(&result);
-
-  // Asserting size 4 also ensures that 'excluded' field inside <form> is not
-  // collected.
-  ASSERT_EQ(4U, result.fields.size());
-  EXPECT_EQ(base::ASCIIToUTF16("text_input"), result.fields[0].name);
-  EXPECT_EQ(base::ASCIIToUTF16("check_input"), result.fields[1].name);
-  EXPECT_EQ(base::ASCIIToUTF16("number_input"), result.fields[2].name);
-  EXPECT_EQ(base::ASCIIToUTF16("select_input"), result.fields[3].name);
-}
-
 // Unit test for AutofillAgent::AcceptDataListSuggestion.
 TEST_F(FormAutocompleteTest, AcceptDataListSuggestion) {
   LoadHTML(
@@ -767,8 +824,7 @@ TEST_F(FormAutocompleteTest, AcceptDataListSuggestion) {
   WebDocument document = GetMainFrame()->GetDocument();
 
   // Each case tests a different field value with the same suggestion.
-  const base::string16 kSuggestion =
-      base::ASCIIToUTF16("suggestion@example.com");
+  const std::u16string kSuggestion = u"suggestion@example.com";
   struct TestCase {
     std::string id;
     std::string expected;
@@ -791,14 +847,13 @@ TEST_F(FormAutocompleteTest, AcceptDataListSuggestion) {
   for (const auto& c : cases) {
     WebElement element = document.GetElementById(WebString::FromUTF8(c.id));
     ASSERT_FALSE(element.IsNull());
-    WebInputElement* input_element = blink::ToWebInputElement(&element);
-    ASSERT_TRUE(input_element);
+    WebInputElement input_element = element.To<WebInputElement>();
     // Select this element in |autofill_agent_|.
-    autofill_agent_->FormControlElementClicked(element.To<WebInputElement>(),
-                                               false);
+    autofill_agent_->FormControlElementClicked(input_element);
 
-    autofill_agent_->AcceptDataListSuggestion(kSuggestion);
-    EXPECT_EQ(c.expected, input_element->Value().Utf8()) << "Case id: " << c.id;
+    autofill_agent_->AcceptDataListSuggestion(
+        form_util::GetFieldRendererId(input_element), kSuggestion);
+    EXPECT_EQ(c.expected, input_element.Value().Utf8()) << "Case id: " << c.id;
   }
 }
 
@@ -1015,9 +1070,9 @@ TEST_F(FormAutocompleteTest, FormSubmittedByDOMMutationAfterXHR) {
   ExecuteJavaScriptForTests(hide_elements.c_str());
   base::RunLoop().RunUntilIdle();
 
-  VerifyReceivedAddressRendererMessages(
-      fake_driver_, "City", true /* expect_known_success */,
-      SubmissionSource::DOM_MUTATION_AFTER_XHR);
+  VerifyReceivedAddressRendererMessages(fake_driver_, "City",
+                                        true /* expect_known_success */,
+                                        SubmissionSource::XHR_SUCCEEDED);
 }
 
 TEST_F(FormAutocompleteTest, FormSubmittedBySameDocumentNavigation) {
@@ -1036,8 +1091,8 @@ TEST_F(FormAutocompleteTest, FormSubmittedBySameDocumentNavigation) {
   ExecuteJavaScriptForTests(hide_elements.c_str());
 
   // Simulate same document navigation.
-  autofill_agent_->form_tracker_for_testing()
-      ->DidFinishSameDocumentNavigation();
+  test_api(test_api(*autofill_agent_).form_tracker())
+      .DidFinishSameDocumentNavigation();
   base::RunLoop().RunUntilIdle();
 
   VerifyReceivedAddressRendererMessages(
@@ -1065,8 +1120,8 @@ TEST_F(FormAutocompleteTest, FormSubmittedByProbablyFormSubmitted) {
   ExecuteJavaScriptForTests(hide_elements.c_str());
 
   // Simulate navigation.
-  autofill_agent_->form_tracker_for_testing()
-      ->FireProbablyFormSubmittedForTesting();
+  test_api(test_api(*autofill_agent_).form_tracker())
+      .FireProbablyFormSubmitted();
 
   base::RunLoop().RunUntilIdle();
 
@@ -1098,8 +1153,10 @@ TEST_F(FormAutocompleteTest, SelectControlChanged) {
 
   const FormFieldData* field = fake_driver_.select_control_changed();
   ASSERT_TRUE(field);
-  EXPECT_EQ(base::ASCIIToUTF16("color"), field->name);
-  EXPECT_EQ(base::ASCIIToUTF16("blue"), field->value);
+  EXPECT_EQ(u"color", field->name);
+  EXPECT_EQ(u"blue", field->value);
 }
 
+}  // namespace
+
 }  // namespace autofill
