@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,11 +7,10 @@
 #include <string>
 #include <vector>
 
-#include "base/bind.h"
 #include "base/command_line.h"
+#include "base/functional/bind.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/threading/thread_task_runner_handle.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/common/chrome_switches.h"
@@ -20,10 +19,9 @@
 #include "chrome/test/base/in_process_browser_test.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/network_session_configurator/common/network_switches.h"
-#include "content/public/browser/browser_task_traits.h"
+#include "content/public/browser/browser_thread.h"
 #include "content/public/common/content_constants.h"
 #include "content/public/renderer/render_frame.h"
-#include "content/public/renderer/render_view.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/browser_test_utils.h"
 #include "content/public/test/mock_render_thread.h"
@@ -39,6 +37,11 @@
 #include "third_party/blink/public/web/web_plugin_params.h"
 #include "url/gurl.h"
 
+#if BUILDFLAG(ENABLE_EXTENSIONS)
+#include "chrome/common/extensions/extension_test_util.h"
+#include "extensions/common/extensions_client.h"
+#endif
+
 using ChromeContentRendererClientSearchBoxTest = ChromeRenderViewTest;
 
 const char kHtmlWithIframe[] ="<iframe srcdoc=\"Nothing here\"></iframe>";
@@ -57,8 +60,9 @@ TEST_F(ChromeContentRendererClientSearchBoxTest, RewriteThumbnailURL) {
 
   // Create a thumbnail URL containing the correct render frame ID and an
   // arbitrary instant restricted ID.
-  GURL thumbnail_url(base::StringPrintf("chrome-search:/thumb/%i/1",
-                                        render_frame->GetRoutingID()));
+  GURL thumbnail_url(base::StringPrintf(
+      "chrome-search:/thumb/%s/1",
+      render_frame->GetWebFrame()->GetLocalFrameToken().ToString().c_str()));
 
   GURL result;
   // Make sure the SearchBox rewrites a thumbnail request from the main frame.
@@ -172,9 +176,8 @@ IN_PROC_BROWSER_TEST_P(ChromeContentRendererClientBrowserTest,
       browser()->tab_strip_model()->GetActiveWebContents();
 
   GURL video_url = https_server()->GetURL(GetParam().host, GetParam().path);
-  EXPECT_TRUE(ExecuteScript(web_contents, "appendEmbedToDOM('" +
-                                              video_url.spec() + "','" +
-                                              GetParam().type + "');"));
+  EXPECT_TRUE(ExecJs(web_contents, "appendEmbedToDOM('" + video_url.spec() +
+                                       "','" + GetParam().type + "');"));
   WaitForYouTubeRequest();
 }
 
@@ -186,12 +189,32 @@ IN_PROC_BROWSER_TEST_P(ChromeContentRendererClientBrowserTest,
      browser()->tab_strip_model()->GetActiveWebContents();
 
   GURL video_url = https_server()->GetURL(GetParam().host, GetParam().path);
-  EXPECT_TRUE(ExecuteScript(web_contents, "appendDataEmbedToDOM('" +
-                                              video_url.spec() + "','" +
-                                              GetParam().type + "');"));
+  EXPECT_TRUE(ExecJs(web_contents, "appendDataEmbedToDOM('" + video_url.spec() +
+                                       "','" + GetParam().type + "');"));
   WaitForYouTubeRequest();
 }
 
 INSTANTIATE_TEST_SUITE_P(FlashEmbeds,
                          ChromeContentRendererClientBrowserTest,
                          ::testing::ValuesIn(kFlashEmbedsTestData));
+
+#if BUILDFLAG(ENABLE_EXTENSIONS)
+IN_PROC_BROWSER_TEST_F(ChromeContentRendererClientBrowserTest,
+                       AvailabilityMapCreated) {
+  auto* extensions_client = extensions::ExtensionsClient::Get();
+  ASSERT_TRUE(extensions_client);
+
+  // ChromeContentRendererClient initializes the ExtensionClient with the
+  // FeatureDelegatedAvailabilityMap, which will maintain ownership of the map.
+  // Verify that the map is created correctly.
+  {
+    const auto& map =
+        extensions_client->GetFeatureDelegatedAvailabilityCheckMap();
+    EXPECT_EQ(5u, map.size());
+    for (const auto* feature :
+         extension_test_util::GetExpectedDelegatedFeaturesForTest()) {
+      EXPECT_EQ(1u, map.count(feature));
+    }
+  }
+}
+#endif
