@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,22 +9,30 @@
 
 #include <limits>
 #include <map>
+#include <set>
 #include <utility>
 
-#include "base/bind.h"
 #include "base/command_line.h"
+#include "base/functional/bind.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/no_destructor.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/synchronization/lock.h"
+#include "base/time/time.h"
+#include "base/trace_event/trace_event.h"
+#include "build/build_config.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_isolated_world_ids.h"
 #include "chrome/common/chrome_switches.h"
+#include "chrome/common/draggable_regions.mojom.h"
 #include "chrome/common/open_search_description_document_handler.mojom.h"
+#include "chrome/common/webui_url_constants.h"
 #include "chrome/renderer/chrome_content_settings_agent_delegate.h"
+#include "chrome/renderer/companion/visual_query/visual_query_classifier_agent.h"
 #include "chrome/renderer/media/media_feeds.h"
 #include "components/crash/core/common/crash_key.h"
+#include "components/lens/lens_metadata.mojom.h"
 #include "components/no_state_prefetch/renderer/no_state_prefetch_helper.h"
 #include "components/offline_pages/buildflags/buildflags.h"
 #include "components/optimization_guide/content/renderer/page_text_agent.h"
@@ -32,15 +40,16 @@
 #include "components/translate/core/common/translate_util.h"
 #include "components/web_cache/renderer/web_cache_impl.h"
 #include "content/public/common/bindings_policy.h"
+#include "content/public/common/content_features.h"
 #include "content/public/renderer/render_frame.h"
+#include "content/public/renderer/render_frame_visitor.h"
 #include "content/public/renderer/render_thread.h"
-#include "content/public/renderer/render_view.h"
 #include "content/public/renderer/window_features_converter.h"
-#include "extensions/common/constants.h"
 #include "printing/buildflags/buildflags.h"
 #include "services/service_manager/public/cpp/binder_registry.h"
 #include "skia/ext/image_operations.h"
 #include "third_party/blink/public/common/associated_interfaces/associated_interface_provider.h"
+#include "third_party/blink/public/common/browser_interface_broker_proxy.h"
 #include "third_party/blink/public/platform/web_url_request.h"
 #include "third_party/blink/public/web/web_console_message.h"
 #include "third_party/blink/public/web/web_document.h"
@@ -51,18 +60,23 @@
 #include "third_party/blink/public/web/web_node.h"
 #include "third_party/blink/public/web/web_security_policy.h"
 #include "third_party/blink/public/web/web_view.h"
+#include "third_party/libwebp/src/src/webp/decode.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/gfx/codec/jpeg_codec.h"
 #include "ui/gfx/codec/png_codec.h"
+#include "ui/gfx/codec/webp_codec.h"
 #include "ui/gfx/geometry/size_f.h"
 #include "url/gurl.h"
 
-#if !defined(OS_ANDROID)
+#if !BUILDFLAG(IS_ANDROID)
+#include "chrome/renderer/accessibility/read_anything_app_controller.h"
 #include "chrome/renderer/searchbox/searchbox_extension.h"
-#endif  // !defined(OS_ANDROID)
+#include "ui/accessibility/accessibility_features.h"
+#endif  // !BUILDFLAG(IS_ANDROID)
 
 #if BUILDFLAG(SAFE_BROWSING_AVAILABLE)
 #include "components/safe_browsing/content/renderer/phishing_classifier/phishing_classifier_delegate.h"
+#include "components/safe_browsing/content/renderer/phishing_classifier/phishing_image_embedder_delegate.h"
 #endif
 
 #if BUILDFLAG(ENABLE_OFFLINE_PAGES)
@@ -90,8 +104,7 @@ static const char kTranslateCaptureText[] = "Translate.CaptureText";
 
 // For a page that auto-refreshes, we still show the bubble, if
 // the refresh delay is less than this value (in seconds).
-static constexpr base::TimeDelta kLocationChangeInterval =
-    base::TimeDelta::FromSeconds(10);
+static constexpr base::TimeDelta kLocationChangeInterval = base::Seconds(10);
 
 // For the context menu, we want to keep transparency as is instead of
 // replacing transparent pixels with black ones
@@ -102,14 +115,15 @@ namespace {
 const char kGifExtension[] = ".gif";
 const char kPngExtension[] = ".png";
 const char kJpgExtension[] = ".jpg";
+const char kWebpExtension[] = ".webp";
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 base::Lock& GetFrameHeaderMapLock() {
   static base::NoDestructor<base::Lock> s;
   return *s;
 }
 
-using FrameHeaderMap = std::map<int, std::string>;
+using FrameHeaderMap = std::map<blink::LocalFrameToken, std::string>;
 
 FrameHeaderMap& GetFrameHeaderMap() {
   GetFrameHeaderMapLock().AssertAcquired();
@@ -118,6 +132,52 @@ FrameHeaderMap& GetFrameHeaderMap() {
 }
 #endif
 
+// Renderers can handle multiple pages, especially in low-memory conditions.
+// Record crash keys for a few origins, in the hope of finding more culprit
+// origins for OOM crashes. Keys are recorded here and not via
+// ChromeContentClient::SetActiveURL() because that method is only invoked in
+// response to IPC messages and most OOMs do not occur in response to an IPC.
+// https://crbug.com/1310046
+void UpdateLoadedOriginCrashKeys() {
+  // Capture the origin for each RenderFrame.
+  struct Visitor : public content::RenderFrameVisitor {
+    bool Visit(RenderFrame* render_frame) override {
+      if (render_frame) {
+        WebLocalFrame* web_frame = render_frame->GetWebFrame();
+        if (web_frame) {
+          frame_count_++;
+          origins_.insert(web_frame->GetSecurityOrigin().ToString().Utf8());
+        }
+      }
+      return true;  // Keep going.
+    }
+    int frame_count_ = 0;
+    std::set<std::string> origins_;  // Use set to collapse duplicate origins.
+  } visitor;
+  RenderFrame::ForEach(&visitor);
+
+  static crash_reporter::CrashKeyString<8> frame_count("web-frame-count");
+  frame_count.Set(base::NumberToString(visitor.frame_count_));
+
+  // Record 3 recently-loaded origins in crash keys (which 3 is arbitrary).
+  using ArrayItemKey = crash_reporter::CrashKeyString<64>;
+  static ArrayItemKey crash_keys[] = {
+      {"loaded-origin-0", ArrayItemKey::Tag::kArray},
+      {"loaded-origin-1", ArrayItemKey::Tag::kArray},
+      {"loaded-origin-2", ArrayItemKey::Tag::kArray},
+  };
+  for (auto& crash_key : crash_keys) {
+    if (!visitor.origins_.empty()) {
+      auto origin_it = visitor.origins_.begin();
+      crash_key.Set(*origin_it);
+      visitor.origins_.erase(origin_it);
+    } else {
+      // If there are fewer than 3 origins, clear the remaining keys.
+      crash_key.Clear();
+    }
+  }
+}
+
 }  // namespace
 
 ChromeRenderFrameObserver::ChromeRenderFrameObserver(
@@ -127,8 +187,8 @@ ChromeRenderFrameObserver::ChromeRenderFrameObserver(
       translate_agent_(nullptr),
       page_text_agent_(new optimization_guide::PageTextAgent(render_frame)),
       web_cache_impl_(web_cache_impl) {
-  render_frame->GetAssociatedInterfaceRegistry()->AddInterface(
-      base::BindRepeating(
+  render_frame->GetAssociatedInterfaceRegistry()
+      ->AddInterface<chrome::mojom::ChromeRenderFrame>(base::BindRepeating(
           &ChromeRenderFrameObserver::OnRenderFrameObserverRequest,
           base::Unretained(this)));
 
@@ -137,30 +197,24 @@ ChromeRenderFrameObserver::ChromeRenderFrameObserver(
     return;
 
 #if BUILDFLAG(SAFE_BROWSING_AVAILABLE)
-  const base::CommandLine& command_line =
-      *base::CommandLine::ForCurrentProcess();
-  if (!command_line.HasSwitch(switches::kDisableClientSidePhishingDetection))
-    SetClientSidePhishingDetection();
+  SetClientSidePhishingDetection();
 #endif
-  if (!translate::IsSubFrameTranslationEnabled()) {
-    translate_agent_ =
-        new translate::TranslateAgent(render_frame, ISOLATED_WORLD_ID_TRANSLATE,
-                                      extensions::kExtensionScheme);
-  }
+
+#if !BUILDFLAG(IS_ANDROID)
+  SetVisualQueryClassifierAgent();
+#endif
+  translate_agent_ =
+      new translate::TranslateAgent(render_frame, ISOLATED_WORLD_ID_TRANSLATE);
 }
 
-ChromeRenderFrameObserver::~ChromeRenderFrameObserver() {
-#if defined(OS_ANDROID)
-  base::AutoLock auto_lock(GetFrameHeaderMapLock());
-  GetFrameHeaderMap().erase(routing_id());
-#endif
-}
+ChromeRenderFrameObserver::~ChromeRenderFrameObserver() = default;
 
-#if defined(OS_ANDROID)
-std::string ChromeRenderFrameObserver::GetCCTClientHeader(int render_frame_id) {
+#if BUILDFLAG(IS_ANDROID)
+std::string ChromeRenderFrameObserver::GetCCTClientHeader(
+    const blink::LocalFrameToken& frame_token) {
   base::AutoLock auto_lock(GetFrameHeaderMapLock());
   auto frame_map = GetFrameHeaderMap();
-  auto iter = frame_map.find(render_frame_id);
+  auto iter = frame_map.find(frame_token);
   return iter == frame_map.end() ? std::string() : iter->second;
 }
 #endif
@@ -195,15 +249,15 @@ void ChromeRenderFrameObserver::ReadyToCommitNavigation(
 void ChromeRenderFrameObserver::DidFinishLoad() {
   WebLocalFrame* frame = render_frame()->GetWebFrame();
   // Don't do anything for subframes.
-  if (frame->Parent())
+  if (frame->Parent() || frame->IsInFencedFrameTree())
     return;
 
   GURL osdd_url = frame->GetDocument().OpenSearchDescriptionURL();
   if (!osdd_url.is_empty()) {
-    mojo::AssociatedRemote<chrome::mojom::OpenSearchDescriptionDocumentHandler>
+    mojo::Remote<chrome::mojom::OpenSearchDescriptionDocumentHandler>
         osdd_handler;
-    render_frame()->GetRemoteAssociatedInterfaces()->GetInterface(
-        &osdd_handler);
+    render_frame()->GetBrowserInterfaceBroker()->GetInterface(
+        osdd_handler.BindNewPipeAndPassReceiver());
     osdd_handler->PageHasOpenSearchDescriptionDocument(
         frame->GetDocument().Url(), osdd_url);
   }
@@ -239,6 +293,9 @@ void ChromeRenderFrameObserver::DidCreateNewDocument() {
 
 void ChromeRenderFrameObserver::DidCommitProvisionalLoad(
     ui::PageTransition transition) {
+  // Update crash keys on any frame transition, not just the main frame.
+  UpdateLoadedOriginCrashKeys();
+
   WebLocalFrame* frame = render_frame()->GetWebFrame();
 
   // Don't do anything for subframes.
@@ -246,10 +303,9 @@ void ChromeRenderFrameObserver::DidCommitProvisionalLoad(
     return;
 
   static crash_reporter::CrashKeyString<8> view_count_key("view-count");
-  view_count_key.Set(
-      base::NumberToString(content::RenderView::GetRenderViewCount()));
+  view_count_key.Set(base::NumberToString(blink::WebView::GetWebViewCount()));
 
-#if !defined(OS_ANDROID)
+#if !BUILDFLAG(IS_ANDROID)
   if (render_frame()->GetEnabledBindings() &
       content::kWebUIBindingsPolicyMask) {
     for (const auto& script : webui_javascript_)
@@ -260,12 +316,22 @@ void ChromeRenderFrameObserver::DidCommitProvisionalLoad(
 }
 
 void ChromeRenderFrameObserver::DidClearWindowObject() {
-#if !defined(OS_ANDROID)
+#if !BUILDFLAG(IS_ANDROID)
   const base::CommandLine& command_line =
       *base::CommandLine::ForCurrentProcess();
   if (command_line.HasSwitch(switches::kInstantProcess))
     SearchBoxExtension::Install(render_frame()->GetWebFrame());
-#endif  // !defined(OS_ANDROID)
+
+  // Install ReadAnythingAppController on render frames with the Read Anything
+  // url, which is chrome-untrusted. ReadAnythingAppController installs v8
+  // bindings in the chrome.readingMode namespace which are consumed by
+  // read_anything/app.ts, the resource of the Read Anything WebUI.
+  if (features::IsReadAnythingEnabled() &&
+      render_frame()->GetWebFrame()->GetDocument().Url() ==
+          chrome::kChromeUIUntrustedReadAnythingSidePanelURL) {
+    ReadAnythingAppController::Install(render_frame());
+  }
+#endif  // !BUILDFLAG(IS_ANDROID)
 }
 
 void ChromeRenderFrameObserver::DidMeaningfulLayout(
@@ -277,15 +343,50 @@ void ChromeRenderFrameObserver::OnDestruct() {
   delete this;
 }
 
+void ChromeRenderFrameObserver::WillDetach(blink::DetachReason detach_reason) {
+#if BUILDFLAG(IS_ANDROID)
+  base::AutoLock auto_lock(GetFrameHeaderMapLock());
+  GetFrameHeaderMap().erase(
+      render_frame()->GetWebFrame()->GetLocalFrameToken());
+#endif
+}
+
+void ChromeRenderFrameObserver::DraggableRegionsChanged() {
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_LINUX) || \
+    BUILDFLAG(IS_CHROMEOS)
+  // Only the main frame is allowed to control draggable regions, to avoid other
+  // frames manipulate the regions in the browser process.
+  if (!render_frame()->IsMainFrame())
+    return;
+
+  blink::WebVector<blink::WebDraggableRegion> web_regions =
+      render_frame()->GetWebFrame()->GetDocument().DraggableRegions();
+  auto regions = std::vector<chrome::mojom::DraggableRegionPtr>();
+  for (blink::WebDraggableRegion& web_region : web_regions) {
+    render_frame()->ConvertViewportToWindow(&web_region.bounds);
+
+    auto region = chrome::mojom::DraggableRegion::New();
+    region->bounds = web_region.bounds;
+    region->draggable = web_region.draggable;
+    regions.emplace_back(std::move(region));
+  }
+
+  mojo::Remote<chrome::mojom::DraggableRegions> remote;
+  render_frame()->GetBrowserInterfaceBroker()->GetInterface(
+      remote.BindNewPipeAndPassReceiver());
+  remote->UpdateDraggableRegions(std::move(regions));
+#endif
+}
+
 void ChromeRenderFrameObserver::SetWindowFeatures(
     blink::mojom::WindowFeaturesPtr window_features) {
-  render_frame()->GetRenderView()->GetWebView()->SetWindowFeatures(
+  render_frame()->GetWebView()->SetWindowFeatures(
       content::ConvertMojoWindowFeaturesToWebWindowFeatures(*window_features));
 }
 
 void ChromeRenderFrameObserver::ExecuteWebUIJavaScript(
-    const base::string16& javascript) {
-#if !defined(OS_ANDROID)
+    const std::u16string& javascript) {
+#if !BUILDFLAG(IS_ANDROID)
   webui_javascript_.push_back(javascript);
 #endif
 }
@@ -294,32 +395,67 @@ void ChromeRenderFrameObserver::RequestImageForContextNode(
     int32_t thumbnail_min_area_pixels,
     const gfx::Size& thumbnail_max_size_pixels,
     chrome::mojom::ImageFormat image_format,
+    int32_t quality,
     RequestImageForContextNodeCallback callback) {
   WebNode context_node = render_frame()->GetWebFrame()->ContextMenuImageNode();
   std::vector<uint8_t> image_data;
   gfx::Size original_size;
   std::string image_extension;
+  std::vector<lens::mojom::LatencyLogPtr> latency_logs;
+
+  // Map for converting between multiple mojom ImageFormat structures to
+  // prevent a circular dependency (go/lens-logging-in-chromium)
+  // TODO(shivpatel): add default value UNKNOWN for if ImageFormat is updated
+  const std::map<chrome::mojom::ImageFormat, lens::mojom::ImageFormat>
+      image_format_conversion = {
+          {chrome::mojom::ImageFormat::ORIGINAL,
+           lens::mojom::ImageFormat::ORIGINAL},
+          {chrome::mojom::ImageFormat::PNG, lens::mojom::ImageFormat::PNG},
+          {chrome::mojom::ImageFormat::JPEG, lens::mojom::ImageFormat::JPEG},
+          {chrome::mojom::ImageFormat::WEBP, lens::mojom::ImageFormat::WEBP},
+      };
 
   if (context_node.IsNull() || !context_node.IsElementNode()) {
-    std::move(callback).Run(image_data, original_size, image_extension);
+    // The downscaled size is the original size, since no downscaling was
+    // required.
+    std::move(callback).Run(image_data, original_size,
+                            /*downscaled_size=*/original_size, image_extension,
+                            std::move(latency_logs));
     return;
   }
 
   WebElement web_element = context_node.To<WebElement>();
   original_size = web_element.GetImageSize();
   image_extension = "." + web_element.ImageExtension();
-  if (!NeedsEncodeImage(image_extension, image_format) &&
-      !NeedsDownscale(original_size, thumbnail_min_area_pixels,
-                      thumbnail_max_size_pixels)) {
+  bool needs_downscale = NeedsDownscale(
+      original_size, thumbnail_min_area_pixels, thumbnail_max_size_pixels);
+  bool needs_encode = NeedsEncodeImage(image_extension, image_format) ||
+                      IsAnimatedWebp(web_element.CopyOfImageData());
+  if (!needs_encode && !needs_downscale) {
     image_data = web_element.CopyOfImageData();
+    // The downscaled size is the original size, since no downscaling was
+    // required.
     std::move(callback).Run(std::move(image_data), original_size,
-                            image_extension);
+                            /*downscaled_size=*/original_size, image_extension,
+                            std::move(latency_logs));
     return;
   }
-
   SkBitmap image = web_element.ImageContents();
+  if (needs_downscale) {
+    latency_logs.push_back(lens::mojom::LatencyLog::New(
+        lens::mojom::Phase::DOWNSCALE_START, original_size, gfx::Size(),
+        image_format_conversion.at(image_format), base::Time::Now(),
+        /*encoded_size_bytes=*/0));
+  }
   SkBitmap thumbnail =
       Downscale(image, thumbnail_min_area_pixels, thumbnail_max_size_pixels);
+  gfx::Size downscaled_size = gfx::Size(thumbnail.width(), thumbnail.height());
+  if (needs_downscale) {
+    latency_logs.push_back(lens::mojom::LatencyLog::New(
+        lens::mojom::Phase::DOWNSCALE_END, original_size, downscaled_size,
+        image_format_conversion.at(image_format), base::Time::Now(),
+        /*encoded_size_bytes=*/0));
+  }
 
   SkBitmap bitmap;
   if (thumbnail.colorType() == kN32_SkColorType) {
@@ -331,9 +467,7 @@ void ChromeRenderFrameObserver::RequestImageForContextNode(
     }
   }
 
-  constexpr int kDefaultQuality = 90;
   std::vector<unsigned char> data;
-
   if (image_format == chrome::mojom::ImageFormat::ORIGINAL) {
     // ORIGINAL will only fall back to here if the image needs to downscale.
     // Let's PNG downscale to PNG and JEPG downscale to JPEG.
@@ -344,6 +478,12 @@ void ChromeRenderFrameObserver::RequestImageForContextNode(
     }
   }
 
+  if (needs_encode) {
+    latency_logs.push_back(lens::mojom::LatencyLog::New(
+        lens::mojom::Phase::ENCODE_START, original_size, downscaled_size,
+        image_format_conversion.at(image_format), base::Time::Now(),
+        /*encoded_size_bytes=*/0));
+  }
   switch (image_format) {
     case chrome::mojom::ImageFormat::PNG:
       if (gfx::PNGCodec::EncodeBGRASkBitmap(
@@ -352,16 +492,44 @@ void ChromeRenderFrameObserver::RequestImageForContextNode(
         image_extension = kPngExtension;
       }
       break;
+    case chrome::mojom::ImageFormat::WEBP:
+      if (gfx::WebpCodec::Encode(bitmap, quality, &data)) {
+        image_data.swap(data);
+        image_extension = kWebpExtension;
+      }
+      break;
     case chrome::mojom::ImageFormat::ORIGINAL:
     // Any format other than PNG and JPEG fall back to here.
     case chrome::mojom::ImageFormat::JPEG:
-      if (gfx::JPEGCodec::Encode(bitmap, kDefaultQuality, &data)) {
+      if (gfx::JPEGCodec::Encode(bitmap, quality, &data)) {
         image_data.swap(data);
         image_extension = kJpgExtension;
       }
       break;
   }
-  std::move(callback).Run(image_data, original_size, image_extension);
+  if (needs_encode) {
+    latency_logs.push_back(lens::mojom::LatencyLog::New(
+        lens::mojom::Phase::ENCODE_END, original_size, downscaled_size,
+        image_format_conversion.at(image_format), base::Time::Now(),
+        sizeof(uint8_t) * image_data.size()));
+  }
+
+  std::move(callback).Run(image_data, original_size, downscaled_size,
+                          image_extension, std::move(latency_logs));
+}
+
+void ChromeRenderFrameObserver::RequestBitmapForContextNode(
+    RequestBitmapForContextNodeCallback callback) {
+  WebNode context_node = render_frame()->GetWebFrame()->ContextMenuImageNode();
+  SkBitmap image;
+  if (context_node.IsNull() || !context_node.IsElementNode()) {
+    std::move(callback).Run(image);
+    return;
+  }
+
+  WebElement web_element = context_node.To<WebElement>();
+  image = web_element.ImageContents();
+  std::move(callback).Run(image);
 }
 
 void ChromeRenderFrameObserver::RequestReloadImageForContextNode() {
@@ -374,10 +542,14 @@ void ChromeRenderFrameObserver::RequestReloadImageForContextNode() {
   }
 }
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 void ChromeRenderFrameObserver::SetCCTClientHeader(const std::string& header) {
+  auto* web_frame = render_frame()->GetWebFrame();
+  if (!web_frame) {
+    return;
+  }
   base::AutoLock auto_lock(GetFrameHeaderMapLock());
-  GetFrameHeaderMap()[routing_id()] = header;
+  GetFrameHeaderMap()[web_frame->GetLocalFrameToken()] = header;
 }
 #endif
 
@@ -404,10 +576,24 @@ void ChromeRenderFrameObserver::LoadBlockedPlugins(
 #endif  // BUILDFLAG(ENABLE_PLUGINS)
 }
 
+void ChromeRenderFrameObserver::SetSupportsAppRegion(bool supports_app_region) {
+  render_frame()->GetWebView()->SetSupportsAppRegion(supports_app_region);
+}
+
 void ChromeRenderFrameObserver::SetClientSidePhishingDetection() {
 #if BUILDFLAG(SAFE_BROWSING_AVAILABLE)
   phishing_classifier_ = safe_browsing::PhishingClassifierDelegate::Create(
       render_frame(), nullptr);
+  phishing_image_embedder_ =
+      safe_browsing::PhishingImageEmbedderDelegate::Create(render_frame());
+#endif
+}
+
+void ChromeRenderFrameObserver::SetVisualQueryClassifierAgent() {
+#if !BUILDFLAG(IS_ANDROID)
+  visual_classifier_ =
+      companion::visual_query::VisualQueryClassifierAgent::Create(
+          render_frame());
 #endif
 }
 
@@ -471,7 +657,7 @@ bool ChromeRenderFrameObserver::ShouldCapturePageTextForTranslateOrPhishing(
   bool should_capture_for_phishing = false;
 
 #if BUILDFLAG(SAFE_BROWSING_AVAILABLE)
-  should_capture_for_phishing = !!phishing_classifier_;
+  should_capture_for_phishing = phishing_classifier_->is_ready();
 #endif
 
   return should_capture_for_translate || should_capture_for_phishing;
@@ -493,7 +679,7 @@ void ChromeRenderFrameObserver::CapturePageText(
   }
   DCHECK_GT(capture_max_size, 0U);
 
-  base::string16 contents;
+  std::u16string contents;
   {
     SCOPED_UMA_HISTOGRAM_TIMER(kTranslateCaptureText);
     TRACE_EVENT0("renderer", "ChromeRenderFrameObserver::CapturePageText");
@@ -504,9 +690,9 @@ void ChromeRenderFrameObserver::CapturePageText(
   }
 
   // Language detection should run only once. Parsing finishes before the page
-  // loads, so let's pick that timing.
+  // loads, so attempt detection here first.
   if (translate_agent_ &&
-      layout_type == blink::WebMeaningfulLayout::kFinishedParsing) {
+      (layout_type == blink::WebMeaningfulLayout::kFinishedParsing)) {
     translate_agent_->PageCaptured(contents);
   }
 
@@ -520,6 +706,10 @@ void ChromeRenderFrameObserver::CapturePageText(
     phishing_classifier_->PageCaptured(
         &contents, layout_type == blink::WebMeaningfulLayout::kFinishedParsing);
   }
+  if (phishing_image_embedder_) {
+    phishing_image_embedder_->PageCaptured(
+        layout_type == blink::WebMeaningfulLayout::kFinishedParsing);
+  }
 #endif
 }
 
@@ -573,20 +763,28 @@ bool ChromeRenderFrameObserver::NeedsEncodeImage(
   switch (image_format) {
     case chrome::mojom::ImageFormat::PNG:
       return !base::EqualsCaseInsensitiveASCII(image_extension, kPngExtension);
-      break;
+    case chrome::mojom::ImageFormat::WEBP:
+      return !base::EqualsCaseInsensitiveASCII(image_extension, kWebpExtension);
     case chrome::mojom::ImageFormat::JPEG:
       return !base::EqualsCaseInsensitiveASCII(image_extension, kJpgExtension);
-      break;
     case chrome::mojom::ImageFormat::ORIGINAL:
       return !base::EqualsCaseInsensitiveASCII(image_extension,
                                                kGifExtension) &&
              !base::EqualsCaseInsensitiveASCII(image_extension,
                                                kJpgExtension) &&
              !base::EqualsCaseInsensitiveASCII(image_extension, kPngExtension);
-      break;
   }
 
   // Should never hit this code since all cases were handled above.
   NOTREACHED();
   return true;
 }
+
+// static
+bool ChromeRenderFrameObserver::IsAnimatedWebp(
+    const std::vector<uint8_t>& image_data) {
+  WebPBitstreamFeatures features{};
+  VP8StatusCode status =
+      WebPGetFeatures(image_data.data(), image_data.size(), &features);
+  return status == VP8_STATUS_OK && features.has_animation;
+}
