@@ -6,13 +6,14 @@ var verbose = 0;
 
 // Aliexpress uses 'US $12.34' format in the price.
 // Macy's uses "$12.34 to 56.78" format.
-var priceCleanupPrefix =
-  'sale price|price|sale|with offer|only|our price|now|starting at';
+var priceCleanupPrefix = 'total price|sale price|price|sale|' +
+    'with offer|only|our price|now|starting at';
 var priceCleanupPostfix = '(/(each|set))';
 var priceRegexTemplate = '((reg|regular|orig|from|' + priceCleanupPrefix +
     ')\\s+)?' +
     '(\\d+\\s*/\\s*)?(US(D)?\\s*)?' +
-    '\\$[\\d.,]+(\\s+(to|-|–)\\s+(\\$)?[\\d.,]+)?' + priceCleanupPostfix + '?';
+    '\\$\\s*[\\d.,]+(\\s+(to|-|–)\\s+(\\$)?[\\d.,]+)?' +
+    priceCleanupPostfix + '?';
 var priceRegexFull = new RegExp('^' + priceRegexTemplate + '( ea)?$', 'i');
 var priceRegex = new RegExp(priceRegexTemplate, 'i');
 var priceCleanupRegex = new RegExp(
@@ -227,7 +228,8 @@ function extractUrl(item) {
   if (document.URL.includes("ae.com")
       || document.URL.includes("kiehls.com")
       || document.URL.includes("discounttiredirect.com")
-      || document.URL.includes("shutterfly.com")) {
+      || document.URL.includes("shutterfly.com")
+      || document.URL.includes("bkstr.com")) {
     return "";
   }
   let anchors;
@@ -513,9 +515,10 @@ function choosePrice(priceArray) {
 }
 
 function extractPrice(item) {
+  const hostname = new URL(document.baseURI).hostname;
   // shein.com shows price by one element per digit and it's challenging
   // to decide based on textContent.
-  if (document.URL.includes("shein.com")) {
+  if (hostname.endsWith("shein.com")) {
     return "";
   }
   // Etsy mobile
@@ -534,14 +537,15 @@ function extractPrice(item) {
   for (const price of item.querySelectorAll(
     'span, b, p, div, h3, td, li, em, strong, ins')) {
     let candidate = price.innerText.trim();
-    if (window.location.hostname.endsWith("urbanoutfitters.com")
-        || window.location.hostname.endsWith("freepeople.com")) {
+    if (hostname.endsWith("urbanoutfitters.com") ||
+        hostname.endsWith("freepeople.com")) {
       priceParts = candidate.split("\n");
       if (priceParts.length >= 2){
         candidate = priceParts[1];
       }
-    } else if (window.location.hostname.endsWith("thecompanystore.com")
-        || window.location.hostname.endsWith("childrensplace.com")) {
+    } else if (hostname.endsWith("thecompanystore.com") ||
+        hostname.endsWith("childrensplace.com") ||
+        hostname.endsWith("chewy.com")) {
       candidate = candidate.split("\n")[0];
     }
     if (!candidate.match(priceRegexFull))
