@@ -7,8 +7,7 @@
 
 #include "base/component_export.h"
 
-namespace account_manager {
-namespace prefs {
+namespace account_manager::prefs {
 
 COMPONENT_EXPORT(ACCOUNT_MANAGER_CORE)
 extern const char kSecondaryGoogleAccountSigninAllowed[];
@@ -19,10 +18,6 @@ extern const char kAccountAppsAvailability[];
 COMPONENT_EXPORT(ACCOUNT_MANAGER_CORE)
 extern const char kIsAvailableInArcKey[];
 
-COMPONENT_EXPORT(ACCOUNT_MANAGER_CORE)
-extern const char kSecondaryGoogleAccountUsage[];
-
-}  // namespace prefs
-}  // namespace account_manager
+}  // namespace account_manager::prefs
 
 #endif  // COMPONENTS_ACCOUNT_MANAGER_CORE_PREF_NAMES_H_
