@@ -3,6 +3,7 @@
 // found in the LICENSE file.
 
 #include <memory>
+#include <string_view>
 #include <utility>
 
 #include "base/functional/bind.h"
@@ -10,14 +11,17 @@
 #include "base/memory/raw_ptr.h"
 #include "base/test/scoped_feature_list.h"
 #include "base/test/task_environment.h"
+#include "components/autofill/content/browser/content_autofill_driver.h"
 #include "components/autofill/content/browser/content_autofill_driver_factory.h"
 #include "components/autofill/content/browser/content_autofill_driver_factory_test_api.h"
+#include "components/autofill/content/common/mojom/autofill_agent.mojom.h"
 #include "components/autofill/core/browser/test_autofill_client.h"
 #include "components/autofill/core/browser/test_autofill_driver.h"
 #include "components/autofill/core/common/autofill_features.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/content_features.h"
+#include "content/public/test/back_forward_cache_util.h"
 #include "content/public/test/navigation_simulator.h"
 #include "content/public/test/test_renderer_host.h"
 #include "mojo/public/cpp/bindings/associated_receiver_set.h"
@@ -30,14 +34,29 @@
 using testing::_;
 using testing::AtLeast;
 using testing::Between;
+using testing::Ref;
 
 namespace autofill {
 
 namespace {
 
-class MockAutofillClient : public TestAutofillClient {
+class MockContentAutofillDriverFactoryObserver
+    : public ContentAutofillDriverFactory::Observer {
  public:
-  MOCK_METHOD(void, HideAutofillPopup, (PopupHidingReason), (override));
+  MOCK_METHOD(void,
+              OnContentAutofillDriverFactoryDestroyed,
+              (ContentAutofillDriverFactory & factory),
+              (override));
+  MOCK_METHOD(void,
+              OnContentAutofillDriverCreated,
+              (ContentAutofillDriverFactory & factory,
+               ContentAutofillDriver& driver),
+              (override));
+  MOCK_METHOD(void,
+              OnContentAutofillDriverWillBeDeleted,
+              (ContentAutofillDriverFactory & factory,
+               ContentAutofillDriver& driver),
+              (override));
 };
 
 class MockAutofillAgent : public mojom::AutofillAgent {
@@ -47,14 +66,30 @@ class MockAutofillAgent : public mojom::AutofillAgent {
                              std::move(handle)));
   }
 
-  MOCK_METHOD(void, TriggerReparse, (), (override));
+  MOCK_METHOD(void, TriggerFormExtraction, (), (override));
   MOCK_METHOD(void,
-              TriggerReparseWithResponse,
+              TriggerFormExtractionWithResponse,
               (base::OnceCallback<void(bool)>),
               (override));
+  MOCK_METHOD(
+      void,
+      ExtractForm,
+      (FormRendererId form,
+       base::OnceCallback<void(const std::optional<FormData>&)> callback),
+      (override));
   MOCK_METHOD(void,
-              FillOrPreviewForm,
-              (const FormData& form, mojom::RendererFormDataAction action),
+              ApplyFormAction,
+              (mojom::ActionType action_type,
+               mojom::ActionPersistence action_persistence,
+               FormRendererId form_renderer_id,
+               const std::vector<FormFieldData>& fields),
+              (override));
+  MOCK_METHOD(void,
+              ApplyFieldAction,
+              (mojom::ActionPersistence action_persistence,
+               mojom::TextReplacement text_replacement,
+               FieldRendererId field,
+               const std::u16string& value),
               (override));
   MOCK_METHOD(void,
               FieldTypePredictionsAvailable,
@@ -63,26 +98,19 @@ class MockAutofillAgent : public mojom::AutofillAgent {
   MOCK_METHOD(void, ClearSection, (), (override));
   MOCK_METHOD(void, ClearPreviewedForm, (), (override));
   MOCK_METHOD(void,
-              FillFieldWithValue,
-              (FieldRendererId field, const std::u16string& value),
-              (override));
-  MOCK_METHOD(void,
-              PreviewFieldWithValue,
-              (FieldRendererId field, const ::std::u16string& value),
+              TriggerSuggestions,
+              (FieldRendererId field_id,
+               AutofillSuggestionTriggerSource trigger_source),
               (override));
   MOCK_METHOD(void,
               SetSuggestionAvailability,
-              (FieldRendererId field, mojom::AutofillState type),
+              (FieldRendererId field,
+               mojom::AutofillSuggestionAvailability suggestion_availability),
               (override));
   MOCK_METHOD(void,
               AcceptDataListSuggestion,
               (FieldRendererId field, const ::std::u16string& value),
               (override));
-  MOCK_METHOD(void,
-              FillPasswordSuggestion,
-              (const ::std::u16string& username,
-               const ::std::u16string& password),
-              (override));
   MOCK_METHOD(void,
               PreviewPasswordSuggestion,
               (const ::std::u16string& username,
@@ -98,8 +126,8 @@ class MockAutofillAgent : public mojom::AutofillAgent {
   MOCK_METHOD(void, SetQueryPasswordSuggestion, (bool query), (override));
   MOCK_METHOD(void, EnableHeavyFormDataScraping, (), (override));
   MOCK_METHOD(void,
-              SetFieldsEligibleForManualFilling,
-              (const std::vector<FieldRendererId>& fields),
+              GetPotentialLastFourCombinationsForStandaloneCvc,
+              (base::OnceCallback<void(const std::vector<std::string>&)>),
               (override));
 
  private:
@@ -119,7 +147,7 @@ class ContentAutofillDriverFactoryTest
   void SetUp() override {
     content::RenderViewHostTestHarness::SetUp();
 
-    client_ = std::make_unique<MockAutofillClient>();
+    client_ = std::make_unique<TestAutofillClient>();
     client_->set_channel_for_testing(channel_);
 
     agent_ = std::make_unique<MockAutofillAgent>();
@@ -132,8 +160,7 @@ class ContentAutofillDriverFactoryTest
 
     factory_ = ContentAutofillDriverFactoryTestApi::Create(
         web_contents(), client_.get(),
-        base::BindRepeating(&autofill::BrowserDriverInitHook, client_.get(),
-                            "en-US"));
+        base::BindRepeating(&BrowserDriverInitHook, client_.get(), "en-US"));
   }
 
   void TearDown() override {
@@ -143,56 +170,53 @@ class ContentAutofillDriverFactoryTest
     content::RenderViewHostTestHarness::TearDown();
   }
 
-  void NavigateMainFrame(base::StringPiece url) {
-    // One call of HideAutofillPopup() comes from ContentAutofillDriverFactory.
-    // A second one may come from BrowserAutofillManager::Reset().
-    EXPECT_CALL(*client_, HideAutofillPopup(PopupHidingReason::kNavigation))
-        .Times(Between(1, 2));
+  void NavigateMainFrame(std::string_view url) {
     content::NavigationSimulator::CreateBrowserInitiated(GURL(url),
                                                          web_contents())
         ->Commit();
   }
 
-  ContentAutofillDriverFactoryTestApi factory_test_api() {
-    return ContentAutofillDriverFactoryTestApi(factory_.get());
-  }
-
  protected:
   version_info::Channel channel_;
   std::unique_ptr<MockAutofillAgent> agent_;
-  std::unique_ptr<MockAutofillClient> client_;
+  std::unique_ptr<TestAutofillClient> client_;
   std::unique_ptr<ContentAutofillDriverFactory> factory_;
 };
 
 TEST_F(ContentAutofillDriverFactoryTest, MainDriver) {
   NavigateMainFrame("https://a.com/");
-  ContentAutofillDriver* main_driver = factory_test_api().GetDriver(main_rfh());
+  ContentAutofillDriver* main_driver =
+      test_api(*factory_).GetDriver(main_rfh());
   EXPECT_TRUE(main_driver);
-  EXPECT_EQ(factory_test_api().num_drivers(), 1u);
+  EXPECT_EQ(test_api(*factory_).num_drivers(), 1u);
   EXPECT_EQ(factory_->DriverForFrame(main_rfh()), main_driver);
-  EXPECT_EQ(factory_test_api().num_drivers(), 1u);
+  EXPECT_EQ(test_api(*factory_).num_drivers(), 1u);
   EXPECT_EQ(factory_->DriverForFrame(main_rfh()), main_driver);
-  EXPECT_EQ(factory_test_api().num_drivers(), 1u);
+  EXPECT_EQ(test_api(*factory_).num_drivers(), 1u);
 }
 
 // Test case with two frames: the main frame and one child frame.
 class ContentAutofillDriverFactoryTest_WithTwoFrames
     : public ContentAutofillDriverFactoryTest {
  public:
-  void NavigateChildFrame(base::StringPiece url) {
+  void NavigateChildFrame(std::string_view url) {
     CHECK(main_rfh());
-    if (!child_rfh_) {
-      child_rfh_ = content::RenderFrameHostTester::For(main_rfh())
-                       ->AppendChild(std::string("child"));
+    if (!child_rfh()) {
+      child_rfh_id_ = content::RenderFrameHostTester::For(main_rfh())
+                          ->AppendChild(std::string("child"))
+                          ->GetGlobalId();
     }
-    child_rfh_ = content::NavigationSimulator::NavigateAndCommitFromDocument(
-        GURL(url), child_rfh_);
+    child_rfh_id_ = content::NavigationSimulator::NavigateAndCommitFromDocument(
+                        GURL(url), child_rfh())
+                        ->GetGlobalId();
   }
 
-  content::RenderFrameHost* child_rfh() { return child_rfh_; }
+  content::RenderFrameHost* child_rfh() {
+    return content::RenderFrameHost::FromID(child_rfh_id_);
+  }
 
  private:
-  raw_ptr<content::RenderFrameHost> child_rfh_ = nullptr;
+  content::GlobalRenderFrameHostId child_rfh_id_;
 };
 
 TEST_F(ContentAutofillDriverFactoryTest_WithTwoFrames, TwoDrivers) {
@@ -206,15 +230,10 @@ TEST_F(ContentAutofillDriverFactoryTest_WithTwoFrames, TwoDrivers) {
   EXPECT_TRUE(child_driver);
   EXPECT_EQ(factory_->DriverForFrame(main_rfh()), main_driver);
   EXPECT_EQ(factory_->DriverForFrame(child_rfh()), child_driver);
-  EXPECT_EQ(factory_test_api().num_drivers(), 2u);
+  EXPECT_EQ(test_api(*factory_).num_drivers(), 2u);
   EXPECT_EQ(factory_->DriverForFrame(main_rfh()), main_driver);
   EXPECT_EQ(factory_->DriverForFrame(child_rfh()), child_driver);
-  EXPECT_EQ(factory_test_api().num_drivers(), 2u);
-  // TODO(crbug.com/1200511): Set the router's last source and target, and if
-  // the |child_driver| is destroyed, expect a call to
-  // AutofillManager::OnHidePopup(). For this to work, we need mock
-  // AutofillManagers instead of real BrowserAutofillManager, which are blocked
-  // by ContentAutofillDriver's use of the factory callback.
+  EXPECT_EQ(test_api(*factory_).num_drivers(), 2u);
 }
 
 // Test case with two frames, where the parameter selects one of them.
@@ -241,42 +260,32 @@ TEST_P(ContentAutofillDriverFactoryTest_WithTwoFrames_PickOne,
   ContentAutofillDriver* child_driver = factory_->DriverForFrame(child_rfh());
   EXPECT_TRUE(main_driver);
   EXPECT_TRUE(child_driver);
-  EXPECT_EQ(factory_test_api().num_drivers(), 2u);
+  EXPECT_EQ(test_api(*factory_).num_drivers(), 2u);
   factory_->RenderFrameDeleted(picked_rfh());
-  EXPECT_EQ(factory_test_api().num_drivers(), 1u);
+  EXPECT_EQ(test_api(*factory_).num_drivers(), 1u);
   if (picked_rfh() == main_rfh())
-    EXPECT_EQ(factory_test_api().GetDriver(child_rfh()), child_driver);
+    EXPECT_EQ(test_api(*factory_).GetDriver(child_rfh()), child_driver);
   else
-    EXPECT_EQ(factory_test_api().GetDriver(main_rfh()), main_driver);
+    EXPECT_EQ(test_api(*factory_).GetDriver(main_rfh()), main_driver);
 }
 
-// Tests that OnVisibilityChanged() hides the popup.
-TEST_F(ContentAutofillDriverFactoryTest, TabHidden) {
-  NavigateMainFrame("https://a.com/");
-  EXPECT_CALL(*client_, HideAutofillPopup(PopupHidingReason::kTabGone));
-  factory_->OnVisibilityChanged(content::Visibility::HIDDEN);
-}
-
-// Test case with one frame, with BFcache and AutofillAcrossIframes enabled or
-// disabled depending on the parameter.
+// Test case with one frame, with BFcache enabled or disabled depending on the
+// parameter.
 class ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes
     : public ContentAutofillDriverFactoryTest,
-      public ::testing::WithParamInterface<std::tuple<bool, bool>> {
+      public ::testing::WithParamInterface<std::tuple<bool>> {
  public:
   ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes() {
     std::vector<base::test::FeatureRef> enabled;
     // Allow BackForwardCache for all devices regardless of their memory.
-    std::vector<base::test::FeatureRef> disabled{
-        ::features::kBackForwardCacheMemoryControls};
-    (autofill_across_iframes() ? enabled : disabled)
-        .push_back(features::kAutofillAcrossIframes);
+    std::vector<base::test::FeatureRef> disabled =
+        content::GetDefaultDisabledBackForwardCacheFeaturesForTesting();
     (use_bfcache() ? enabled : disabled)
         .push_back(::features::kBackForwardCache);
     scoped_feature_list_.InitWithFeatures(enabled, disabled);
   }
 
   bool use_bfcache() { return std::get<0>(GetParam()); }
-  bool autofill_across_iframes() { return std::get<1>(GetParam()); }
 
  private:
   base::test::ScopedFeatureList scoped_feature_list_;
@@ -285,7 +294,7 @@ class ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes
 INSTANTIATE_TEST_SUITE_P(
     ContentAutofillDriverFactoryTest,
     ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes,
-    testing::Combine(testing::Bool(), testing::Bool()));
+    testing::Combine(testing::Bool()));
 
 // Tests that that a same-documentation navigation does not touch the factory's
 // router.
@@ -296,8 +305,8 @@ TEST_P(ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes,
   ContentAutofillDriver* orig_driver = factory_->DriverForFrame(orig_rfh);
   NavigateMainFrame("https://a.com/#same-site");
   ASSERT_EQ(orig_rfh, main_rfh());
-  EXPECT_EQ(factory_test_api().GetDriver(orig_rfh), orig_driver);
-  EXPECT_EQ(factory_test_api().num_drivers(), 1u);
+  EXPECT_EQ(test_api(*factory_).GetDriver(orig_rfh), orig_driver);
+  EXPECT_EQ(test_api(*factory_).num_drivers(), 1u);
 
   // TODO(crbug.com/1200511): Test that |router_| has been untouched. To this
   // end, call `orig_driver->FormsSeen({FormData{}})` above and then check
@@ -306,8 +315,8 @@ TEST_P(ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes,
   // by ContentAutofillDriver's use of the factory callback.
 }
 
-// Tests that that a driver survives a same-origin navigation but is reset
-// afterwards.
+// Tests that that a driver is 1:1 with RenderFrameHost, which might or might
+// not change after a same-origin navigation.
 TEST_P(ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes,
        SameOriginNavigation) {
   NavigateMainFrame("https://a.com/");
@@ -319,10 +328,15 @@ TEST_P(ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes,
   // of the factory callback.
 
   NavigateMainFrame("https://a.com/after-navigation");
-  EXPECT_EQ(factory_test_api().GetDriver(orig_rfh), orig_driver);
-  // If BFCache is enabled, there will be 2 drivers as the old document is still
-  // around.
-  EXPECT_EQ(factory_test_api().num_drivers(), use_bfcache() ? 2u : 1u);
+  // If the RenderFrameHost changed, a new driver for main_rfh() is created, and
+  // if BFCache is disabled the driver for |orig_rfh| has now been removed in
+  // ContentAutofillDriverFactory::RenderFrameDeleted().
+  if (use_bfcache()) {
+    EXPECT_EQ(test_api(*factory_).GetDriver(orig_rfh), orig_driver);
+  } else if (main_rfh() != orig_rfh) {
+    EXPECT_EQ(test_api(*factory_).GetDriver(orig_rfh), nullptr);
+  }
+  EXPECT_EQ(test_api(*factory_).num_drivers(), use_bfcache() ? 2u : 1u);
 }
 
 // Tests that that a driver is removed and replaced with a fresh one after a
@@ -335,8 +349,8 @@ TEST_P(ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes,
   ContentAutofillDriver* orig_driver = factory_->DriverForFrame(orig_rfh);
 
   ASSERT_EQ(orig_rfh, main_rfh());
-  EXPECT_EQ(factory_test_api().GetDriver(orig_rfh), orig_driver);
-  EXPECT_EQ(factory_test_api().num_drivers(), 1u);
+  EXPECT_EQ(test_api(*factory_).GetDriver(orig_rfh), orig_driver);
+  EXPECT_EQ(test_api(*factory_).num_drivers(), 1u);
 
   NavigateMainFrame("https://different-origin-after-navigation.com/");
 
@@ -346,43 +360,31 @@ TEST_P(ContentAutofillDriverFactoryTest_WithOrWithoutBfCacheAndIframes,
   // BFcache is enabled (or main_rfh() happens to have the same address as
   // |orig_rfh|).
   if (use_bfcache())
-    EXPECT_EQ(factory_test_api().GetDriver(orig_rfh), orig_driver);
+    EXPECT_EQ(test_api(*factory_).GetDriver(orig_rfh), orig_driver);
   else if (main_rfh() != orig_rfh)
-    EXPECT_EQ(factory_test_api().GetDriver(orig_rfh), nullptr);
-  EXPECT_NE(factory_test_api().GetDriver(main_rfh()), nullptr);
-  EXPECT_EQ(factory_test_api().num_drivers(), use_bfcache() ? 2u : 1u);
+    EXPECT_EQ(test_api(*factory_).GetDriver(orig_rfh), nullptr);
+  EXPECT_NE(test_api(*factory_).GetDriver(main_rfh()), nullptr);
+  EXPECT_EQ(test_api(*factory_).num_drivers(), use_bfcache() ? 2u : 1u);
 }
 
-// Fixture for testing that Autofill is enabled in fenced frames unless
-// AutofillEnableWithinFencedFrame is enabled. The bool parameter
-// enables/disables that feature.
+// Fixture for testing that Autofill is enabled in fenced frames.
 class ContentAutofillDriverFactoryTest_FencedFrames
-    : public ContentAutofillDriverFactoryTest,
-      public ::testing::WithParamInterface<bool> {
+    : public ContentAutofillDriverFactoryTest {
  public:
   ContentAutofillDriverFactoryTest_FencedFrames() {
-    std::vector<base::test::FeatureRefAndParams> enabled;
-    std::vector<base::test::FeatureRef> disabled;
-    enabled.push_back(
-        {blink::features::kFencedFrames, {{"implementation_type", "mparch"}}});
-    if (autofill_enabled_in_fencedframe()) {
-      enabled.push_back({blink::features::kFencedFramesAPIChanges, {}});
-      enabled.push_back({features::kAutofillEnableWithinFencedFrame, {}});
-    } else {
-      disabled.push_back(features::kAutofillEnableWithinFencedFrame);
-    }
-    scoped_feature_list_.InitWithFeaturesAndParameters(enabled, disabled);
+    std::vector<base::test::FeatureRefAndParams> enabled{
+        {blink::features::kFencedFrames, {{"implementation_type", "mparch"}}},
+        {blink::features::kFencedFramesAPIChanges, {}}};
+    scoped_feature_list_.InitWithFeaturesAndParameters(enabled, {});
   }
 
   ~ContentAutofillDriverFactoryTest_FencedFrames() override = default;
 
-  bool autofill_enabled_in_fencedframe() const { return GetParam(); }
-
  private:
   base::test::ScopedFeatureList scoped_feature_list_;
 };
 
-TEST_P(ContentAutofillDriverFactoryTest_FencedFrames,
+TEST_F(ContentAutofillDriverFactoryTest_FencedFrames,
        DisableAutofillWithinFencedFrame) {
   NavigateMainFrame("http://test.org");
   content::RenderFrameHost* fenced_frame_root =
@@ -391,19 +393,10 @@ TEST_P(ContentAutofillDriverFactoryTest_FencedFrames,
       content::RenderFrameHostTester::For(fenced_frame_root)
           ->AppendChild("iframe");
   EXPECT_NE(nullptr, factory_->DriverForFrame(main_rfh()));
-  if (autofill_enabled_in_fencedframe()) {
-    EXPECT_NE(nullptr, factory_->DriverForFrame(fenced_frame_root));
-    EXPECT_NE(nullptr, factory_->DriverForFrame(fenced_frame_subframe));
-  } else {
-    EXPECT_EQ(nullptr, factory_->DriverForFrame(fenced_frame_root));
-    EXPECT_EQ(nullptr, factory_->DriverForFrame(fenced_frame_subframe));
-  }
+  EXPECT_NE(nullptr, factory_->DriverForFrame(fenced_frame_root));
+  EXPECT_NE(nullptr, factory_->DriverForFrame(fenced_frame_subframe));
 }
 
-INSTANTIATE_TEST_SUITE_P(ContentAutofillDriverFactoryTest,
-                         ContentAutofillDriverFactoryTest_FencedFrames,
-                         testing::Bool());
-
 struct AgentSetupParam {
   version_info::Channel channel;
   bool heavy_scraping_enabled;
@@ -435,4 +428,49 @@ TEST_P(ContentAutofillDriverFactoryTest_AgentSetup,
   testing::Mock::VerifyAndClearExpectations(agent_.get());
 }
 
+// Tests the notifications of ContentAutofillDriverFactory::Observer.
+class ContentAutofillDriverFactoryTest_Observer
+    : public ContentAutofillDriverFactoryTest {
+ public:
+  void SetUp() override {
+    ContentAutofillDriverFactoryTest::SetUp();
+    factory_->AddObserver(&observer_);
+  }
+
+  void TearDown() override {
+    if (factory_) {
+      factory_->RemoveObserver(&observer_);
+    }
+    ContentAutofillDriverFactoryTest::TearDown();
+  }
+
+  MockContentAutofillDriverFactoryObserver observer_;
+};
+
+auto IsKnownDriver(ContentAutofillDriverFactory* factory) {
+  return testing::Truly([factory](ContentAutofillDriver& driver) {
+    return factory->DriverForFrame(driver.render_frame_host()) == &driver;
+  });
+}
+
+TEST_F(ContentAutofillDriverFactoryTest_Observer, FactoryDestroyed) {
+  EXPECT_CALL(observer_,
+              OnContentAutofillDriverFactoryDestroyed(Ref(*factory_)));
+  factory_.reset();
+}
+
+TEST_F(ContentAutofillDriverFactoryTest_Observer, DriverCreated) {
+  EXPECT_CALL(observer_, OnContentAutofillDriverCreated(
+                             Ref(*factory_), IsKnownDriver(factory_.get())));
+  NavigateMainFrame("https://a.com/");
+}
+
+TEST_F(ContentAutofillDriverFactoryTest_Observer, DriverDeleted) {
+  EXPECT_CALL(observer_, OnContentAutofillDriverCreated);
+  EXPECT_CALL(observer_, OnContentAutofillDriverWillBeDeleted(
+                             Ref(*factory_), IsKnownDriver(factory_.get())));
+  NavigateMainFrame("https://a.com/");
+  factory_->RenderFrameDeleted(main_rfh());
+}
+
 }  // namespace autofill
