@@ -12,6 +12,7 @@
 #include "base/command_line.h"
 #include "base/containers/contains.h"
 #include "base/functional/bind.h"
+#include "base/memory/raw_ptr.h"
 #include "base/ranges/algorithm.h"
 #include "base/task/single_thread_task_runner.h"
 #include "components/autofill/content/renderer/form_autofill_util.h"
@@ -22,14 +23,15 @@
 #include "components/autofill/core/common/password_form_generation_data.h"
 #include "components/autofill/core/common/password_generation_util.h"
 #include "components/autofill/core/common/signatures.h"
-#include "components/password_manager/core/common/password_manager_features.h"
 #include "content/public/renderer/render_frame.h"
 #include "google_apis/gaia/gaia_urls.h"
 #include "third_party/blink/public/common/associated_interfaces/associated_interface_provider.h"
 #include "third_party/blink/public/common/features.h"
+#include "third_party/blink/public/common/features_generated.h"
 #include "third_party/blink/public/platform/web_security_origin.h"
 #include "third_party/blink/public/platform/web_vector.h"
 #include "third_party/blink/public/web/web_document.h"
+#include "third_party/blink/public/web/web_form_control_element.h"
 #include "third_party/blink/public/web/web_form_element.h"
 #include "third_party/blink/public/web/web_input_element.h"
 #include "third_party/blink/public/web/web_local_frame.h"
@@ -65,7 +67,7 @@ FieldRendererId FindConfirmationPasswordFieldId(
   for (; iter != control_elements.end(); ++iter) {
     const WebInputElement input_element = iter->DynamicTo<WebInputElement>();
     if (!input_element.IsNull() && input_element.IsPasswordFieldForAutofill())
-      return FieldRendererId(input_element.UniqueRendererFormControlId());
+      return form_util::GetFieldRendererId(input_element);
   }
   return FieldRendererId();
 }
@@ -83,11 +85,17 @@ void CopyElementValueToOtherInputElements(
 
 void PreviewGeneratedValue(WebInputElement& input_element,
                            const blink::WebString& value) {
+  if (base::FeatureList::IsEnabled(blink::features::kPasswordStrongLabel)) {
+    input_element.SetShouldShowStrongPasswordLabel(true);
+  }
   input_element.SetShouldRevealPassword(true);
   input_element.SetSuggestedValue(value);
 }
 
 void ClearPreviewedValue(WebInputElement& input_element) {
+  if (base::FeatureList::IsEnabled(blink::features::kPasswordStrongLabel)) {
+    input_element.SetShouldShowStrongPasswordLabel(false);
+  }
   input_element.SetShouldRevealPassword(false);
   input_element.SetSuggestedValue(blink::WebString());
 }
@@ -159,7 +167,7 @@ class PasswordGenerationAgent::DeferringPasswordGenerationDriver
     DeferMsg(&mojom::PasswordGenerationDriver::GenerationElementLostFocus);
   }
 
-  PasswordGenerationAgent* agent_ = nullptr;
+  raw_ptr<PasswordGenerationAgent, ExperimentalRenderer> agent_ = nullptr;
   base::WeakPtrFactory<DeferringPasswordGenerationDriver> weak_ptr_factory_{
       this};
 };
@@ -262,7 +270,6 @@ void PasswordGenerationAgent::DidCommitProvisionalLoad(
     }
   }
   current_generation_item_.reset();
-  last_focused_password_element_.Reset();
   generation_enabled_fields_.clear();
 }
 
@@ -274,8 +281,6 @@ void PasswordGenerationAgent::DidChangeScrollOffset() {
 
 void PasswordGenerationAgent::OnDestruct() {
   receiver_.reset();
-  base::SingleThreadTaskRunner::GetCurrentDefault()->DeleteSoon(FROM_HERE,
-                                                                this);
 }
 
 void PasswordGenerationAgent::OnFieldAutofilled(
@@ -302,7 +307,7 @@ bool PasswordGenerationAgent::IsPrerendering() const {
 
 void PasswordGenerationAgent::PreviewGenerationSuggestion(
     const std::u16string& password) {
-  DCHECK(current_generation_item_);
+  CHECK(current_generation_item_);
 
   for (auto& password_field : current_generation_item_->password_elements_) {
     PreviewGeneratedValue(password_field,
@@ -331,10 +336,12 @@ bool PasswordGenerationAgent::DidClearGenerationSuggestion(
 
 void PasswordGenerationAgent::GeneratedPasswordAccepted(
     const std::u16string& password) {
-  // static cast is workaround for linker error.
-  DCHECK_LE(static_cast<size_t>(kMinimumLengthForEditedPassword),
-            password.size());
-  DCHECK(current_generation_item_);
+  // Check that the navigation in between didn't reset the state.
+  if (!current_generation_item_) {
+    return;
+  }
+  CHECK(!password.empty());
+  CHECK_LE(kMinimumLengthForEditedPassword, password.size());
   current_generation_item_->password_is_generated_ = true;
   current_generation_item_->password_edited_ = false;
   password_generation::LogPasswordGenerationEvent(
@@ -346,19 +353,24 @@ void PasswordGenerationAgent::GeneratedPasswordAccepted(
     password_element.SetAutofillValue(blink::WebString::FromUTF16(password));
     // setAutofillValue() above may have resulted in JavaScript closing the
     // frame.
-    if (!render_frame())
+    if (!render_frame()) {
       return;
+    }
+    // crbug.com/1467893: JS can clear the generated password. In this case
+    // consider filling unsuccessful and don't presave the password.
+    if (password_element.Value().IsEmpty()) {
+      return;
+    }
     password_agent_->TrackAutofilledElement(password_element);
-    // Advance focus to the next input field. We assume password fields in
-    // an account creation form are always adjacent.
-    render_frame()->GetWebView()->AdvanceFocus(false);
   }
+  CHECK(base::Contains(current_generation_item_->password_elements_,
+                       current_generation_item_->generation_element_));
 
   std::unique_ptr<FormData> presaved_form_data(CreateFormDataToPresave());
-  std::u16string generated_password =
+  const std::u16string generated_password =
       current_generation_item_->generation_element_.Value().Utf16();
   if (presaved_form_data) {
-    DCHECK_NE(std::u16string(), generated_password);
+    CHECK(!generated_password.empty());
     GetPasswordGenerationDriver().PresaveGeneratedPassword(*presaved_form_data,
                                                            generated_password);
   }
@@ -370,7 +382,22 @@ void PasswordGenerationAgent::GeneratedPasswordAccepted(
     // has changed. Without this we will overwrite the generated
     // password with an Autofilled password when saving.
     // https://crbug.com/493455
-    password_agent_->UpdateStateForTextChange(password_element);
+    password_agent_->autofill_agent().UpdateStateForTextChange(
+        password_element, FieldPropertiesFlags::kUserTyped);
+  }
+}
+
+void PasswordGenerationAgent::FocusNextFieldAfterPasswords() {
+  if (!current_generation_item_ || !render_frame()) {
+    return;
+  }
+
+  for (const WebInputElement& password_element :
+       current_generation_item_->password_elements_) {
+    if (password_element ==
+        password_agent_->focused_element().DynamicTo<WebInputElement>()) {
+      render_frame()->GetWebView()->AdvanceFocus(false);
+    }
   }
 }
 
@@ -397,8 +424,8 @@ void PasswordGenerationAgent::FoundFormEligibleForGeneration(
     if (doc.IsNull())
       return;
     WebFormControlElement new_password_input =
-        form_util::FindFormControlElementByUniqueRendererId(
-            doc, form.new_password_renderer_id);
+        form_util::FindFormControlByRendererId(doc,
+                                               form.new_password_renderer_id);
     if (!new_password_input.IsNull()) {
       // Mark the input element with renderer id
       // |form.new_password_renderer_id|.
@@ -422,51 +449,57 @@ void PasswordGenerationAgent::TriggeredGeneratePassword(
             current_generation_item_->generation_element_),
         current_generation_item_->generation_element_.MaxLength(),
         current_generation_item_->generation_element_.NameForAutofill().Utf16(),
-        current_generation_item_->generation_element_.Value().Utf16(),
-        FieldRendererId(current_generation_item_->generation_element_
-                            .UniqueRendererFormControlId()),
+        form_util::GetFieldRendererId(
+            current_generation_item_->generation_element_),
         is_generation_element_password_type,
         GetTextDirectionForElement(
             current_generation_item_->generation_element_),
-        current_generation_item_->form_data_);
+        current_generation_item_->form_data_,
+        current_generation_item_->generation_element_.Value().Utf16().empty());
     std::move(callback).Run(std::move(password_generation_ui_data));
     current_generation_item_->generation_popup_shown_ = true;
   } else {
-    std::move(callback).Run(absl::nullopt);
+    std::move(callback).Run(std::nullopt);
   }
 }
 
 bool PasswordGenerationAgent::SetUpTriggeredGeneration() {
-  if (last_focused_password_element_.IsNull() || !render_frame())
+  if (!render_frame()) {
     return false;
+  }
+  const WebInputElement last_focused_password_element =
+      password_agent_->focused_element().DynamicTo<WebInputElement>();
+  if (last_focused_password_element.IsNull() ||
+      last_focused_password_element.IsReadOnly() ||
+      !last_focused_password_element.IsPasswordFieldForAutofill()) {
+    return false;
+  }
 
-  FieldRendererId last_focused_password_element_id(
-      last_focused_password_element_.UniqueRendererFormControlId());
+  FieldRendererId last_focused_password_element_id =
+      form_util::GetFieldRendererId(last_focused_password_element);
 
   bool is_automatic_generation_available = base::Contains(
       generation_enabled_fields_, last_focused_password_element_id);
 
   if (!is_automatic_generation_available) {
-    WebFormElement form = last_focused_password_element_.Form();
-    std::vector<WebFormControlElement> control_elements;
-    if (!form.IsNull()) {
-      control_elements = form_util::ExtractAutofillableElementsInForm(form);
-    } else {
-      const WebLocalFrame& frame = *render_frame()->GetWebFrame();
-      blink::WebDocument doc = frame.GetDocument();
-      if (doc.IsNull())
-        return false;
-      control_elements = form_util::GetUnownedFormFieldElements(doc, nullptr);
+    blink::WebDocument document =
+        render_frame() ? render_frame()->GetWebFrame()->GetDocument()
+                       : WebDocument();
+    if (document.IsNull()) {
+      return false;
     }
+    WebFormElement form = last_focused_password_element.Form();
+    std::vector<WebFormControlElement> control_elements =
+        form_util::GetAutofillableFormControlElements(document, form);
 
     MaybeCreateCurrentGenerationItem(
-        last_focused_password_element_,
+        last_focused_password_element,
         FindConfirmationPasswordFieldId(control_elements,
-                                        last_focused_password_element_));
+                                        last_focused_password_element));
   } else {
     auto it = generation_enabled_fields_.find(last_focused_password_element_id);
     MaybeCreateCurrentGenerationItem(
-        last_focused_password_element_,
+        last_focused_password_element,
         it->second.confirmation_password_renderer_id);
   }
 
@@ -474,7 +507,7 @@ bool PasswordGenerationAgent::SetUpTriggeredGeneration() {
     return false;
 
   if (current_generation_item_->generation_element_ !=
-      last_focused_password_element_) {
+      last_focused_password_element) {
     return false;
   }
 
@@ -483,26 +516,12 @@ bool PasswordGenerationAgent::SetUpTriggeredGeneration() {
   return true;
 }
 
-bool PasswordGenerationAgent::FocusedNodeHasChanged(
-    const blink::WebNode& node) {
-  if (node.IsNull() || !node.IsElementNode()) {
-    return false;
-  }
+bool PasswordGenerationAgent::ShowPasswordGenerationSuggestions(
+    const WebInputElement& element) {
+  CHECK(!element.IsNull());
 
-  const blink::WebElement web_element = node.To<blink::WebElement>();
-  if (!web_element.GetDocument().GetFrame()) {
-    return false;
-  }
-
-  const WebInputElement element = web_element.DynamicTo<WebInputElement>();
-  if (element.IsNull())
-    return false;
-
-  if (element.IsPasswordFieldForAutofill())
-    last_focused_password_element_ = element;
-
-  auto it = generation_enabled_fields_.find(
-      FieldRendererId(element.UniqueRendererFormControlId()));
+  auto it =
+      generation_enabled_fields_.find(form_util::GetFieldRendererId(element));
   if (it != generation_enabled_fields_.end()) {
     MaybeCreateCurrentGenerationItem(
         element, it->second.confirmation_password_renderer_id);
@@ -513,33 +532,28 @@ bool PasswordGenerationAgent::FocusedNodeHasChanged(
   }
 
   if (current_generation_item_->password_is_generated_) {
-    if (current_generation_item_->generation_element_.Value().length() <
-        kMinimumLengthForEditedPassword) {
+    size_t password_length =
+        current_generation_item_->generation_element_.Value().length();
+    if (password_length < kMinimumLengthForEditedPassword) {
+      // Password is too short to be considered generated.
       PasswordNoLongerGenerated();
-      MaybeOfferAutomaticGeneration();
-      if (current_generation_item_->generation_element_.Value().IsEmpty())
+      if (password_length == 0) {
         current_generation_item_->generation_element_.SetShouldRevealPassword(
             false);
-    } else {
-      current_generation_item_->generation_element_.SetShouldRevealPassword(
-          true);
-      ShowEditingPopup();
+      }
+      return MaybeOfferAutomaticGeneration();
     }
+    current_generation_item_->generation_element_.SetShouldRevealPassword(true);
+    ShowEditingPopup();
     return true;
   }
 
-  // Assume that if the password field has less than or equal to
-  // `kMaximumCharsForGenerationOffer` characters, then the user is not finished
+  // Assume that if the password field has less than
+  // |kMaximumCharsForGenerationOffer| characters then the user is not finished
   // typing their password and display the password suggestion.
-  // With `kPasswordStrengthIndicator` enabled the decision to display the
-  // suggestion needs to be calculated in the browser process based on the
-  // strength of the typed password.
   if (!element.IsReadOnly() && element.IsEnabled() &&
-      (element.Value().length() <= kMaximumCharsForGenerationOffer ||
-       base::FeatureList::IsEnabled(
-           password_manager::features::kPasswordStrengthIndicator))) {
-    MaybeOfferAutomaticGeneration();
-    return true;
+      element.Value().length() <= kMaximumCharsForGenerationOffer) {
+    return MaybeOfferAutomaticGeneration();
   }
 
   return false;
@@ -564,12 +578,18 @@ bool PasswordGenerationAgent::TextDidChangeInTextField(
         current_generation_item_->password_is_generated_ && !element.IsNull() &&
         element.Form() ==
             current_generation_item_->generation_element_.Form()) {
-      std::unique_ptr<FormData> presaved_form_data(CreateFormDataToPresave());
-      std::u16string generated_password =
+      const std::u16string generated_password =
           current_generation_item_->generation_element_.Value().Utf16();
-      if (presaved_form_data) {
-        GetPasswordGenerationDriver().PresaveGeneratedPassword(
-            *presaved_form_data, generated_password);
+      if (generated_password.empty()) {
+        // JS cleared the generated password in the meantime. Consider the user
+        // left the generation state.
+        PasswordNoLongerGenerated();
+      } else {
+        std::unique_ptr<FormData> presaved_form_data(CreateFormDataToPresave());
+        if (presaved_form_data) {
+          GetPasswordGenerationDriver().PresaveGeneratedPassword(
+              *presaved_form_data, generated_password);
+        }
       }
     }
     return false;
@@ -581,9 +601,7 @@ bool PasswordGenerationAgent::TextDidChangeInTextField(
   }
 
   if (!current_generation_item_->password_is_generated_ &&
-      element.Value().length() > kMaximumCharsForGenerationOffer &&
-      !base::FeatureList::IsEnabled(
-          password_manager::features::kPasswordStrengthIndicator)) {
+      element.Value().length() > kMaximumCharsForGenerationOffer) {
     // User has rejected the feature and has started typing a password.
     GenerationRejectedByTyping();
   } else {
@@ -611,6 +629,7 @@ bool PasswordGenerationAgent::TextDidChangeInTextField(
       std::u16string generated_password =
           current_generation_item_->generation_element_.Value().Utf16();
       if (presaved_form_data) {
+        CHECK(!generated_password.empty());
         GetPasswordGenerationDriver().PresaveGeneratedPassword(
             *presaved_form_data, generated_password);
       }
@@ -619,17 +638,21 @@ bool PasswordGenerationAgent::TextDidChangeInTextField(
     // Notify `password_agent_` of text changes to the other confirmation
     // password fields.
     for (const auto& password_element :
-         current_generation_item_->password_elements_)
-      password_agent_->UpdateStateForTextChange(password_element);
+         current_generation_item_->password_elements_) {
+      password_agent_->autofill_agent().UpdateStateForTextChange(
+          password_element, FieldPropertiesFlags::kUserTyped);
+    }
   }
   return true;
 }
 
-void PasswordGenerationAgent::MaybeOfferAutomaticGeneration() {
+bool PasswordGenerationAgent::MaybeOfferAutomaticGeneration() {
   // TODO(crbug.com/852309): Add this check to the generation element class.
-  if (!current_generation_item_->is_manually_triggered_) {
-    AutomaticGenerationAvailable();
+  if (current_generation_item_->is_manually_triggered_) {
+    return false;
   }
+  AutomaticGenerationAvailable();
+  return true;
 }
 
 void PasswordGenerationAgent::AutomaticGenerationAvailable() {
@@ -649,12 +672,12 @@ void PasswordGenerationAgent::AutomaticGenerationAvailable() {
           current_generation_item_->generation_element_),
       current_generation_item_->generation_element_.MaxLength(),
       current_generation_item_->generation_element_.NameForAutofill().Utf16(),
-      current_generation_item_->generation_element_.Value().Utf16(),
-      FieldRendererId(current_generation_item_->generation_element_
-                          .UniqueRendererFormControlId()),
+      form_util::GetFieldRendererId(
+          current_generation_item_->generation_element_),
       is_generation_element_password_type,
       GetTextDirectionForElement(current_generation_item_->generation_element_),
-      current_generation_item_->form_data_);
+      current_generation_item_->form_data_,
+      current_generation_item_->generation_element_.Value().Utf16().empty());
   current_generation_item_->generation_popup_shown_ = true;
   GetPasswordGenerationDriver().AutomaticGenerationAvailable(
       password_generation_ui_data);
@@ -670,9 +693,9 @@ void PasswordGenerationAgent::ShowEditingPopup() {
   std::unique_ptr<FormData> form_data = CreateFormDataToPresave();
   DCHECK(form_data);
 
-  FieldRendererId generation_element_renderer_id(
-      current_generation_item_->generation_element_
-          .UniqueRendererFormControlId());
+  FieldRendererId generation_element_renderer_id =
+      form_util::GetFieldRendererId(
+          current_generation_item_->generation_element_);
   std::u16string password_value =
       current_generation_item_->generation_element_.Value().Utf16();
 
@@ -691,8 +714,10 @@ void PasswordGenerationAgent::PasswordNoLongerGenerated() {
   // Do not treat the password as generated, either here or in the browser.
   current_generation_item_->password_is_generated_ = false;
   current_generation_item_->password_edited_ = false;
-  for (WebInputElement& password : current_generation_item_->password_elements_)
+  for (WebInputElement& password :
+       current_generation_item_->password_elements_) {
     password.SetAutofillState(WebAutofillState::kNotFilled);
+  }
   password_generation::LogPasswordGenerationEvent(
       password_generation::PASSWORD_DELETED);
   // Clear all other password fields.
@@ -732,8 +757,8 @@ void PasswordGenerationAgent::MaybeCreateCurrentGenerationItem(
   std::vector<blink::WebInputElement> passwords = {generation_element};
 
   WebFormControlElement confirmation_password =
-      form_util::FindFormControlElementByUniqueRendererId(
-          generation_element.GetDocument(), confirmation_password_renderer_id);
+      form_util::FindFormControlByRendererId(generation_element.GetDocument(),
+                                             confirmation_password_renderer_id);
 
   if (!confirmation_password.IsNull()) {
     WebInputElement input = confirmation_password.DynamicTo<WebInputElement>();
