@@ -13,10 +13,8 @@ namespace autofill {
 
 class TestAutofillDataModel : public AutofillDataModel {
  public:
-  TestAutofillDataModel(const std::string& guid, const std::string& origin);
-  TestAutofillDataModel(const std::string& guid,
-                        size_t use_count,
-                        base::Time use_date);
+  explicit TestAutofillDataModel();
+  TestAutofillDataModel(size_t use_count, base::Time use_date);
 
   TestAutofillDataModel(const TestAutofillDataModel&) = delete;
   TestAutofillDataModel& operator=(const TestAutofillDataModel&) = delete;
@@ -24,11 +22,11 @@ class TestAutofillDataModel : public AutofillDataModel {
   ~TestAutofillDataModel() override;
 
  private:
-  std::u16string GetRawInfo(ServerFieldType type) const override;
-  void SetRawInfoWithVerificationStatus(ServerFieldType type,
+  std::u16string GetRawInfo(FieldType type) const override;
+  void SetRawInfoWithVerificationStatus(FieldType type,
                                         const std::u16string& value,
                                         VerificationStatus status) override;
-  void GetSupportedTypes(ServerFieldTypeSet* supported_types) const override;
+  void GetSupportedTypes(FieldTypeSet* supported_types) const override;
 };
 
 }  // namespace autofill
